/*******************************************************************************
 * Copyright (c) 2008 Matthew Hall and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Matthew Hall - initial API and implementation (bug 124684)
 *     IBM Corporation - through ListBinding.java
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.core.databinding.SetBinding;

import dwt.dwthelper.utils;

import java.util.Collections;
import java.util.Iterator;

import dwtx.core.databinding.observable.Diffs;
import dwtx.core.databinding.observable.set.IObservableSet;
import dwtx.core.databinding.observable.set.ISetChangeListener;
import dwtx.core.databinding.observable.set.SetChangeEvent;
import dwtx.core.databinding.observable.set.SetDiff;
import dwtx.core.databinding.observable.value.IObservableValue;
import dwtx.core.databinding.observable.value.WritableValue;
import dwtx.core.internal.databinding.BindingStatus;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.MultiStatus;
import dwtx.core.runtime.Status;

/**
 * @since 1.1
 * 
 */
public class SetBinding extends Binding {

    private UpdateSetStrategy targetToModel;
    private UpdateSetStrategy modelToTarget;
    private IObservableValue validationStatusObservable;
    private bool updatingTarget;
    private bool updatingModel;

    private ISetChangeListener targetChangeListener = new ISetChangeListener() {
        public void handleSetChange(SetChangeEvent event) {
            if (!updatingTarget) {
                doUpdate((IObservableSet) getTarget(),
                        (IObservableSet) getModel(), event.diff, targetToModel,
                        false, false);
            }
        }
    };

    private ISetChangeListener modelChangeListener = new ISetChangeListener() {
        public void handleSetChange(SetChangeEvent event) {
            if (!updatingModel) {
                doUpdate((IObservableSet) getModel(),
                        (IObservableSet) getTarget(), event.diff,
                        modelToTarget, false, false);
            }
        }
    };

    /**
     * @param target
     * @param model
     * @param modelToTargetStrategy
     * @param targetToModelStrategy
     */
    public SetBinding(IObservableSet target, IObservableSet model,
            UpdateSetStrategy targetToModelStrategy,
            UpdateSetStrategy modelToTargetStrategy) {
        super(target, model);
        this.targetToModel = targetToModelStrategy;
        this.modelToTarget = modelToTargetStrategy;
        if ((targetToModel.getUpdatePolicy() & UpdateSetStrategy.POLICY_UPDATE) !is 0) {
            target.addSetChangeListener(targetChangeListener);
        } else {
            targetChangeListener = null;
        }
        if ((modelToTarget.getUpdatePolicy() & UpdateSetStrategy.POLICY_UPDATE) !is 0) {
            model.addSetChangeListener(modelChangeListener);
        } else {
            modelChangeListener = null;
        }
    }

    public IObservableValue getValidationStatus() {
        return validationStatusObservable;
    }

    protected void preInit() {
        validationStatusObservable = new WritableValue(context
                .getValidationRealm(), Status.OK_STATUS, IStatus.class);
    }

    protected void postInit() {
        if (modelToTarget.getUpdatePolicy() is UpdateSetStrategy.POLICY_UPDATE) {
            updateModelToTarget();
        }
        if (targetToModel.getUpdatePolicy() !is UpdateSetStrategy.POLICY_NEVER) {
            validateTargetToModel();
        }
    }

    public void updateModelToTarget() {
        final IObservableSet modelSet = (IObservableSet) getModel();
        modelSet.getRealm().exec(new Runnable() {
            public void run() {
                SetDiff diff = Diffs.computeSetDiff(Collections.EMPTY_SET,
                        modelSet);
                doUpdate(modelSet, (IObservableSet) getTarget(), diff,
                        modelToTarget, true, true);
            }
        });
    }

    public void updateTargetToModel() {
        final IObservableSet targetSet = (IObservableSet) getTarget();
        targetSet.getRealm().exec(new Runnable() {
            public void run() {
                SetDiff diff = Diffs.computeSetDiff(Collections.EMPTY_SET,
                        targetSet);
                doUpdate(targetSet, (IObservableSet) getModel(), diff,
                        targetToModel, true, true);
            }
        });
    }

    public void validateModelToTarget() {
        // nothing for now
    }

    public void validateTargetToModel() {
        // nothing for now
    }

    /*
     * This method may be moved to UpdateSetStrategy in the future if clients
     * need more control over how the two sets are kept in sync.
     */
    private void doUpdate(final IObservableSet source,
            final IObservableSet destination, final SetDiff diff,
            final UpdateSetStrategy updateSetStrategy, final bool explicit,
            final bool clearDestination) {
        final int policy = updateSetStrategy.getUpdatePolicy();
        if (policy is UpdateSetStrategy.POLICY_NEVER)
            return;
        if (policy is UpdateSetStrategy.POLICY_ON_REQUEST && !explicit)
            return;
        destination.getRealm().exec(new Runnable() {
            public void run() {
                if (destination is getTarget()) {
                    updatingTarget = true;
                } else {
                    updatingModel = true;
                }
                MultiStatus multiStatus = BindingStatus.ok();

                try {
                    if (clearDestination) {
                        destination.clear();
                    }

                    for (Iterator iterator = diff.getRemovals().iterator(); iterator
                            .hasNext();) {
                        IStatus setterStatus = updateSetStrategy.doRemove(
                                destination, updateSetStrategy.convert(iterator
                                        .next()));

                        mergeStatus(multiStatus, setterStatus);
                        // TODO - at this point, the two sets
                        // will be out of sync if an error
                        // occurred...
                    }

                    for (Iterator iterator = diff.getAdditions().iterator(); iterator
                            .hasNext();) {
                        IStatus setterStatus = updateSetStrategy.doAdd(
                                destination, updateSetStrategy.convert(iterator
                                        .next()));

                        mergeStatus(multiStatus, setterStatus);
                        // TODO - at this point, the two sets
                        // will be out of sync if an error
                        // occurred...
                    }
                } finally {
                    validationStatusObservable.setValue(multiStatus);

                    if (destination is getTarget()) {
                        updatingTarget = false;
                    } else {
                        updatingModel = false;
                    }
                }
            }
        });
    }

    /**
     * Merges the provided <code>newStatus</code> into the
     * <code>multiStatus</code>.
     * 
     * @param multiStatus
     * @param newStatus
     */
    /* package */void mergeStatus(MultiStatus multiStatus, IStatus newStatus) {
        if (!newStatus.isOK()) {
            multiStatus.add(newStatus);
        }
    }

    public void dispose() {
        if (targetChangeListener !is null) {
            ((IObservableSet) getTarget())
                    .removeSetChangeListener(targetChangeListener);
            targetChangeListener = null;
        }
        if (modelChangeListener !is null) {
            ((IObservableSet) getModel())
                    .removeSetChangeListener(modelChangeListener);
            modelChangeListener = null;
        }
        super.dispose();
    }
}
