/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.expressions.PropertyTester;

import dwt.dwthelper.utils;

import org.osgi.framework.Bundle;

import dwtx.core.internal.expressions.PropertyTesterDescriptor;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.Platform;

/**
 * Abstract superclass of all property testers. Implementation classes of
 * the extension point <code>dwtx.core.expresssions.propertyTesters
 * </code> must extend <code>PropertyTester</code>.
 * <p>
 * A property tester implements the property tests enumerated in the property
 * tester extension point. For the following property test extension
 * <pre>
 *   &lt;propertyTester
 *       namespace="dwtx.jdt.core"
 *       id="dwtx.jdt.core.IPackageFragmentTester"
 *       properties="isDefaultPackage"
 *       type="dwtx.jdt.core.IPackageFragment"
 *       class="dwtx.demo.MyPackageFragmentTester"&gt;
 *     &lt;/propertyTester&gt;
 * </pre>
 * the corresponding implementation class looks like:
 * <pre>
 *   public class MyPackageFragmentTester {
 *       public bool test(Object receiver, String property, Object[] args, Object expectedValue) {
 *           IPackageFragment fragement= (IPackageFragment)receiver;
 *           if ("isDefaultPackage".equals(property)) { 
 *               return expectedValue is null
 *                  ? fragement.isDefaultPackage()
 *                  : fragement.isDefaultPackage() is ((Boolean)expectedValue).booleanValue();
 *           }
 *           Assert.isTrue(false);
 *           return false;
 *       }
 *   }
 * </pre>
 * The property can then be used in a test expression as follows:
 * <pre>
 *   &lt;instanceof value="dwtx.core.IPackageFragment"/&gt;
 *   &lt;test property="dwtx.jdt.core.isDefaultPackage"/&gt;
 * </pre>
 * </p>
 * <p>
 * There is no guarantee that the same instance of a property tester is used
 * to handle &lt;test property="..."/&gt; requests. So property testers 
 * should always be implemented in a stateless fashion.
 * </p>
 * @since 3.0 
 */
public abstract class PropertyTester implements IPropertyTester {
    
    private IConfigurationElement fConfigElement;
    private String fNamespace;
    private String fProperties;
    
    /**
     * Initialize the property tester with the given name space and property.
     * <p>
     * Note: this method is for internal use only. Clients must not call 
     * this method.
     * </p>
     * @param descriptor the descriptor object for this tester
     * 
     * @noreference This method is not intended to be referenced by clients.
     */
    public final void internalInitialize(PropertyTesterDescriptor descriptor) { 
        fProperties= descriptor.getProperties();
        fNamespace= descriptor.getNamespace();
        fConfigElement= descriptor.getConfigurationElement();
    }
    
    /**
     * Note: this method is for internal use only. Clients must not call 
     * this method.
     * 
     * @return the property tester descriptor
     * 
     * @noreference This method is not intended to be referenced by clients.
     */
    public final PropertyTesterDescriptor internalCreateDescriptor() {
        return new PropertyTesterDescriptor(fConfigElement, fNamespace, fProperties);
    }
    
    /**
     * {@inheritDoc}
     */
    public final bool handles(String namespace, String property) {
        return fNamespace.equals(namespace) && fProperties.indexOf("," + property + ",") !is -1;  //$NON-NLS-1$//$NON-NLS-2$
    }
    
    /**
     * {@inheritDoc}
     */
    public final bool isInstantiated() {
        return true;
    }
    
    /**
     * {@inheritDoc}
     */
    public bool isDeclaringPluginActive() {
        Bundle bundle= Platform.getBundle(fConfigElement.getContributor().getName());
        return bundle.getState() is Bundle.ACTIVE;      
    }
    
    /**
     * {@inheritDoc}
     */
    public final IPropertyTester instantiate() {
        return this;
    }
}
