/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.expressions.IterateExpression;

import dwt.dwthelper.utils;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Element;

import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.expressions.IIterable;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;

public class IterateExpression extends CompositeExpression {
    
    private static class IteratePool implements IEvaluationContext {
        
        private Iterator fIterator;
        private Object fDefaultVariable;
        private IEvaluationContext fParent;
        
        public IteratePool(IEvaluationContext parent, Iterator iterator) {
            Assert.isNotNull(parent);
            Assert.isNotNull(iterator);
            fParent= parent;
            fIterator= iterator;
        }
        public IEvaluationContext getParent() {
            return fParent;
        }
        public IEvaluationContext getRoot() {
            return fParent.getRoot();
        }
        public Object getDefaultVariable() {
            return fDefaultVariable;
        }
        public bool getAllowPluginActivation() {
            return fParent.getAllowPluginActivation();
        }
        public void setAllowPluginActivation(bool value) {
            fParent.setAllowPluginActivation(value);
        }
        public void addVariable(String name, Object value) {
            fParent.addVariable(name, value);
        }
        public Object removeVariable(String name) {
            return fParent.removeVariable(name);
        }
        public Object getVariable(String name) {
            return fParent.getVariable(name);
        }
        public Object resolveVariable(String name, Object[] args) throws CoreException {
            return fParent.resolveVariable(name, args);
        }
        public Object next() {
            fDefaultVariable= fIterator.next();
            return fDefaultVariable;
        }
        public bool hasNext() {
            return fIterator.hasNext();
        }
    }
    
    private static final String ATT_OPERATOR= "operator"; //$NON-NLS-1$
    private static final String ATT_IF_EMPTY= "ifEmpty"; //$NON-NLS-1$
    private static final int OR= 1;
    private static final int AND= 2;

    /**
     * The seed for the hash code for all iterate expressions.
     */
    private static final int HASH_INITIAL= IterateExpression.class.getName().hashCode();
    
    private int fOperator;
    private Boolean fEmptyResult;
    
    public IterateExpression(IConfigurationElement configElement) throws CoreException {
        String opValue= configElement.getAttribute(ATT_OPERATOR);
        initializeOperatorValue(opValue);
        initializeEmptyResultValue(configElement.getAttribute(ATT_IF_EMPTY));
    }

    public IterateExpression(Element element) throws CoreException {
        String opValue= element.getAttribute(ATT_OPERATOR);
        initializeOperatorValue(opValue.length() > 0 ? opValue : null);
        String ifEmpty= element.getAttribute(ATT_IF_EMPTY);
        initializeEmptyResultValue(ifEmpty.length() > 0 ? ifEmpty : null);
    }

    public IterateExpression(String opValue) throws CoreException {
        initializeOperatorValue(opValue);
    }
    
    public IterateExpression(String opValue, String ifEmpty) throws CoreException {
        initializeOperatorValue(opValue);
        initializeEmptyResultValue(ifEmpty);
    }
    
    private void initializeOperatorValue(String opValue) throws CoreException {
        if (opValue is null) {
            fOperator= AND;
        } else {
            Expressions.checkAttribute(ATT_OPERATOR, opValue, new String[] {"and", "or"});  //$NON-NLS-1$//$NON-NLS-2$
            if ("and".equals(opValue)) { //$NON-NLS-1$
                fOperator= AND;
            } else {
                fOperator= OR;
            }
        }
    }
    
    private void initializeEmptyResultValue(String value) {
        if (value is null) {
            fEmptyResult= null;
        } else {
            fEmptyResult= Boolean.valueOf(value);
        }
    }

    /* (non-Javadoc)
     * @see Expression#evaluate(IVariablePool)
     */
    public EvaluationResult evaluate(IEvaluationContext context) throws CoreException {
        Object var= context.getDefaultVariable();
        if (var instanceof Collection) {
            Collection col= (Collection)var; 
            switch (col.size()) {
                case 0:
                    if (fEmptyResult is null) {
                        return fOperator is AND ? EvaluationResult.TRUE : EvaluationResult.FALSE;
                    } else {
                        return fEmptyResult.booleanValue() ? EvaluationResult.TRUE : EvaluationResult.FALSE;
                    }
                case 1:
                    if (col instanceof List)
                        return evaluateAnd(new DefaultVariable(context, ((List)col).get(0)));
                    // fall through
                default:
                    IteratePool iter= new IteratePool(context, col.iterator());
                    EvaluationResult result= fOperator is AND ? EvaluationResult.TRUE : EvaluationResult.FALSE;
                    while (iter.hasNext()) {
                        iter.next();
                        switch(fOperator) {
                            case OR:
                                result= result.or(evaluateAnd(iter));
                                if (result is EvaluationResult.TRUE)
                                    return result;
                                break;
                            case AND:
                                result= result.and(evaluateAnd(iter));
                                if (result !is EvaluationResult.TRUE)
                                    return result;
                                break;
                        }
                    }
                    return result;
            }
        } else {
            IIterable iterable= Expressions.getAsIIterable(var, this);
            if (iterable is null)
                return EvaluationResult.NOT_LOADED;
            int count= 0;
            IteratePool iter= new IteratePool(context, iterable.iterator());
            EvaluationResult result= fOperator is AND ? EvaluationResult.TRUE : EvaluationResult.FALSE;
            while (iter.hasNext()) {
                iter.next();
                count++;
                switch(fOperator) {
                    case OR:
                        result= result.or(evaluateAnd(iter));
                        if (result is EvaluationResult.TRUE)
                            return result;
                        break;
                    case AND:
                        result= result.and(evaluateAnd(iter));
                        if (result !is EvaluationResult.TRUE)
                            return result;
                        break;
                }
            }
            if (count > 0) {
                return result;
            } else {
                if (fEmptyResult is null) {
                    return fOperator is AND ? EvaluationResult.TRUE : EvaluationResult.FALSE;
                } else {
                    return fEmptyResult.booleanValue() ? EvaluationResult.TRUE : EvaluationResult.FALSE;
                }
            }
        }
    }

    public void collectExpressionInfo(ExpressionInfo info) {
        // Although we access every single variable we only mark the default
        // variable as accessed since we don't have single variables for the
        // elements.
        info.markDefaultVariableAccessed();
        super.collectExpressionInfo(info);
    }

    public bool equals(final Object object) {
        if (!(object instanceof IterateExpression))
            return false;
        
        final IterateExpression that= (IterateExpression)object;
        return (this.fOperator is that.fOperator) && equals(this.fExpressions, that.fExpressions);
    } 

    protected int computeHashCode() {
        return HASH_INITIAL * HASH_FACTOR + hashCode(fExpressions)
            * HASH_FACTOR + fOperator;
    }
}
