/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.preferences.PreferencesOSGiUtils;

import dwt.dwthelper.utils;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Filter;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.service.packageadmin.PackageAdmin;
import org.osgi.util.tracker.ServiceTracker;

import dwtx.core.internal.preferences.exchange.ILegacyPreferences;
import dwtx.osgi.service.datalocation.Location;
import dwtx.osgi.service.debug.DebugOptions;

/**
 * This class contains a set of helper OSGI methods for the Preferences plugin.
 * The closeServices() method should be called before the plugin is stopped. 
 * 
 * @since dwtx.equinox.preferences 3.2
 */
public class PreferencesOSGiUtils {
    private ServiceTracker initTracker = null;
    private ServiceTracker debugTracker = null;
    private ServiceTracker bundleTracker = null;
    private ServiceTracker configurationLocationTracker = null;
    private ServiceTracker instanceLocationTracker = null;

    private static final PreferencesOSGiUtils singleton = new PreferencesOSGiUtils();

    public static PreferencesOSGiUtils getDefault() {
        return singleton;
    }

    /**
     * Private constructor to block instance creation.
     */
    private PreferencesOSGiUtils() {
        super();
    }

    void openServices() {
        BundleContext context = Activator.getContext();
        if (context is null) {
            if (EclipsePreferences.DEBUG_PREFERENCE_GENERAL)
                PrefsMessages.message("PreferencesOSGiUtils called before plugin started"); //$NON-NLS-1$
            return;
        }

        initTracker = new ServiceTracker(context, ILegacyPreferences.class.getName(), null);
        initTracker.open(true);

        debugTracker = new ServiceTracker(context, DebugOptions.class.getName(), null);
        debugTracker.open();

        bundleTracker = new ServiceTracker(context, PackageAdmin.class.getName(), null);
        bundleTracker.open();

        // locations

        Filter filter = null;
        try {
            filter = context.createFilter(Location.CONFIGURATION_FILTER);
        } catch (InvalidSyntaxException e) {
            // ignore this.  It should never happen as we have tested the above format.
        }
        configurationLocationTracker = new ServiceTracker(context, filter, null);
        configurationLocationTracker.open();

        try {
            filter = context.createFilter(Location.INSTANCE_FILTER);
        } catch (InvalidSyntaxException e) {
            // ignore this.  It should never happen as we have tested the above format.
        }
        instanceLocationTracker = new ServiceTracker(context, filter, null);
        instanceLocationTracker.open();
    }

    void closeServices() {
        if (initTracker !is null) {
            initTracker.close();
            initTracker = null;
        }
        if (debugTracker !is null) {
            debugTracker.close();
            debugTracker = null;
        }
        if (bundleTracker !is null) {
            bundleTracker.close();
            bundleTracker = null;
        }
        if (configurationLocationTracker !is null) {
            configurationLocationTracker.close();
            configurationLocationTracker = null;
        }
        if (instanceLocationTracker !is null) {
            instanceLocationTracker.close();
            instanceLocationTracker = null;
        }
    }

    public ILegacyPreferences getLegacyPreferences() {
        if (initTracker !is null)
            return (ILegacyPreferences) initTracker.getService();
        if (EclipsePreferences.DEBUG_PREFERENCE_GENERAL)
            PrefsMessages.message("Legacy preference tracker is not set"); //$NON-NLS-1$
        return null;
    }

    public bool getBooleanDebugOption(String option, bool defaultValue) {
        if (debugTracker is null) {
            if (EclipsePreferences.DEBUG_PREFERENCE_GENERAL)
                PrefsMessages.message("Debug tracker is not set"); //$NON-NLS-1$
            return defaultValue;
        }
        DebugOptions options = (DebugOptions) debugTracker.getService();
        if (options !is null) {
            String value = options.getOption(option);
            if (value !is null)
                return value.equalsIgnoreCase("true"); //$NON-NLS-1$
        }
        return defaultValue;
    }

    public Bundle getBundle(String bundleName) {
        if (bundleTracker is null) {
            if (EclipsePreferences.DEBUG_PREFERENCE_GENERAL)
                PrefsMessages.message("Bundle tracker is not set"); //$NON-NLS-1$
            return null;
        }
        PackageAdmin packageAdmin = (PackageAdmin) bundleTracker.getService();
        if (packageAdmin is null)
            return null;
        Bundle[] bundles = packageAdmin.getBundles(bundleName, null);
        if (bundles is null)
            return null;
        //Return the first bundle that is not installed or uninstalled
        for (int i = 0; i < bundles.length; i++) {
            if ((bundles[i].getState() & (Bundle.INSTALLED | Bundle.UNINSTALLED)) is 0) {
                return bundles[i];
            }
        }
        return null;
    }

    public Location getConfigurationLocation() {
        if (configurationLocationTracker !is null)
            return (Location) configurationLocationTracker.getService();
        return null;
    }

    public Location getInstanceLocation() {
        if (instanceLocationTracker !is null)
            return (Location) instanceLocationTracker.getService();
        return null;
    }
}
