/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.core.internal.registry.osgi.OSGIUtils;

import dwt.dwthelper.utils;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Filter;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.service.packageadmin.PackageAdmin;
import org.osgi.util.tracker.ServiceTracker;

import dwtx.core.internal.registry.RegistryMessages;
import dwtx.core.internal.runtime.RuntimeLog;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.osgi.service.datalocation.Location;
import dwtx.osgi.service.debug.DebugOptions;

/**
 * The class contains a set of helper methods for the runtime content plugin.
 * The closeServices() method should be called before the plugin is stopped. 
 * 
 * @since dwtx.equinox.registry 3.2
 */
public class OSGIUtils {
    private ServiceTracker debugTracker = null;
    private ServiceTracker bundleTracker = null;
    private ServiceTracker configurationLocationTracker = null;

    // OSGI system properties.  Copied from EclipseStarter
    public static final String PROP_CONFIG_AREA = "osgi.configuration.area"; //$NON-NLS-1$
    public static final String PROP_INSTANCE_AREA = "osgi.instance.area"; //$NON-NLS-1$

    private static final OSGIUtils singleton = new OSGIUtils();

    public static OSGIUtils getDefault() {
        return singleton;
    }

    /**
     * Private constructor to block instance creation.
     */
    private OSGIUtils() {
        super();
        initServices();
    }

    private void initServices() {
        BundleContext context = Activator.getContext();
        if (context is null) {
            RuntimeLog.log(new Status(IStatus.ERROR, RegistryMessages.OWNER_NAME, 0, RegistryMessages.bundle_not_activated, null));
            return;
        }

        debugTracker = new ServiceTracker(context, DebugOptions.class.getName(), null);
        debugTracker.open();

        bundleTracker = new ServiceTracker(context, PackageAdmin.class.getName(), null);
        bundleTracker.open();

        // locations
        final String FILTER_PREFIX = "(&(objectClass=dwtx.osgi.service.datalocation.Location)(type="; //$NON-NLS-1$
        Filter filter = null;
        try {
            filter = context.createFilter(FILTER_PREFIX + PROP_CONFIG_AREA + "))"); //$NON-NLS-1$
        } catch (InvalidSyntaxException e) {
            // ignore this.  It should never happen as we have tested the above format.
        }
        configurationLocationTracker = new ServiceTracker(context, filter, null);
        configurationLocationTracker.open();

    }

    void closeServices() {
        if (debugTracker !is null) {
            debugTracker.close();
            debugTracker = null;
        }
        if (bundleTracker !is null) {
            bundleTracker.close();
            bundleTracker = null;
        }
        if (configurationLocationTracker !is null) {
            configurationLocationTracker.close();
            configurationLocationTracker = null;
        }
    }

    public bool getBooleanDebugOption(String option, bool defaultValue) {
        if (debugTracker is null) {
            RuntimeLog.log(new Status(IStatus.ERROR, RegistryMessages.OWNER_NAME, 0, RegistryMessages.bundle_not_activated, null));
            return defaultValue;
        }
        DebugOptions options = (DebugOptions) debugTracker.getService();
        if (options !is null) {
            String value = options.getOption(option);
            if (value !is null)
                return value.equalsIgnoreCase("true"); //$NON-NLS-1$
        }
        return defaultValue;
    }

    public PackageAdmin getPackageAdmin() {
        if (bundleTracker is null) {
            RuntimeLog.log(new Status(IStatus.ERROR, RegistryMessages.OWNER_NAME, 0, RegistryMessages.bundle_not_activated, null));
            return null;
        }
        return (PackageAdmin) bundleTracker.getService();
    }

    public Bundle getBundle(String bundleName) {
        PackageAdmin packageAdmin = getPackageAdmin();
        if (packageAdmin is null)
            return null;
        Bundle[] bundles = packageAdmin.getBundles(bundleName, null);
        if (bundles is null)
            return null;
        //Return the first bundle that is not installed or uninstalled
        for (int i = 0; i < bundles.length; i++) {
            if ((bundles[i].getState() & (Bundle.INSTALLED | Bundle.UNINSTALLED)) is 0) {
                return bundles[i];
            }
        }
        return null;
    }

    public Bundle[] getFragments(Bundle bundle) {
        PackageAdmin packageAdmin = getPackageAdmin();
        if (packageAdmin is null)
            return null;
        return packageAdmin.getFragments(bundle);
    }

    public bool isFragment(Bundle bundle) {
        PackageAdmin packageAdmin = getPackageAdmin();
        if (packageAdmin is null)
            return false;
        return (packageAdmin.getBundleType(bundle) & PackageAdmin.BUNDLE_TYPE_FRAGMENT) > 0;
    }

    public Bundle[] getHosts(Bundle bundle) {
        PackageAdmin packageAdmin = getPackageAdmin();
        if (packageAdmin is null)
            return null;
        return packageAdmin.getHosts(bundle);
    }

    public Location getConfigurationLocation() {
        if (configurationLocationTracker is null)
            return null;
        return (Location) configurationLocationTracker.getService();
    }
}
