/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.core.runtime.internal.adaptor.EclipseAdaptorHook;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParserFactory;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.osgi.framework.ServiceFactory;
import org.osgi.framework.ServiceRegistration;

import dwtx.core.runtime.adaptor.LocationManager;
import dwtx.osgi.baseadaptor.BaseAdaptor;
import dwtx.osgi.baseadaptor.HookConfigurator;
import dwtx.osgi.baseadaptor.HookRegistry;
import dwtx.osgi.baseadaptor.hooks.AdaptorHook;
import dwtx.osgi.framework.adaptor.FrameworkAdaptor;
import dwtx.osgi.framework.console.CommandProvider;
import dwtx.osgi.framework.debug.Debug;
import dwtx.osgi.framework.debug.FrameworkDebugOptions;
import dwtx.osgi.framework.internal.core.Constants;
import dwtx.osgi.framework.log.FrameworkLog;
import dwtx.osgi.framework.log.FrameworkLogEntry;
import dwtx.osgi.internal.baseadaptor.AdaptorUtil;
import dwtx.osgi.service.datalocation.Location;
import dwtx.osgi.service.pluginconversion.PluginConverter;
import dwtx.osgi.service.resolver.PlatformAdmin;
import dwtx.osgi.service.urlconversion.URLConverter;

public class EclipseAdaptorHook implements AdaptorHook, HookConfigurator {
    /** The SAX factory name */
    public static final String SAXFACTORYNAME = "javax.xml.parsers.SAXParserFactory"; //$NON-NLS-1$
    /** The DOM factory name */
    public static final String DOMFACTORYNAME = "javax.xml.parsers.DocumentBuilderFactory"; //$NON-NLS-1$
    private static final String RUNTIME_ADAPTOR = FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME + "/eclipseadaptor"; //$NON-NLS-1$
    private static final String OPTION_CONVERTER = RUNTIME_ADAPTOR + "/converter/debug"; //$NON-NLS-1$
    private static final String OPTION_LOCATION = RUNTIME_ADAPTOR + "/debug/location"; //$NON-NLS-1$

    private BaseAdaptor adaptor;
    private bool noXML = false;
    private ArrayList registrations = new ArrayList(10);

    public void frameworkStart(BundleContext context) throws BundleException {
        registrations.clear();
        registerEndorsedXMLParser(context);
        Hashtable locationProperties = new Hashtable(1);
        Location location = LocationManager.getUserLocation();
        if (location !is null) {
            locationProperties.put("type", LocationManager.PROP_USER_AREA); //$NON-NLS-1$
            registrations.add(context.registerService(Location.class.getName(), location, locationProperties));
        }
        location = LocationManager.getInstanceLocation();
        if (location !is null) {
            locationProperties.put("type", LocationManager.PROP_INSTANCE_AREA); //$NON-NLS-1$
            registrations.add(context.registerService(Location.class.getName(), location, locationProperties));
        }
        location = LocationManager.getConfigurationLocation();
        if (location !is null) {
            locationProperties.put("type", LocationManager.PROP_CONFIG_AREA); //$NON-NLS-1$
            registrations.add(context.registerService(Location.class.getName(), location, locationProperties));
        }
        location = LocationManager.getInstallLocation();
        if (location !is null) {
            locationProperties.put("type", LocationManager.PROP_INSTALL_AREA); //$NON-NLS-1$
            registrations.add(context.registerService(Location.class.getName(), location, locationProperties));
        }

        location = LocationManager.getEclipseHomeLocation();
        if (location !is null) {
            locationProperties.put("type", LocationManager.PROP_HOME_LOCATION_AREA); //$NON-NLS-1$
            registrations.add(context.registerService(Location.class.getName(), location, locationProperties));
        }

        Dictionary urlProperties = new Hashtable();
        urlProperties.put("protocol", new String[] {"bundleentry", "bundleresource"}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        registrations.add(context.registerService(URLConverter.class.getName(), new URLConverterImpl(), urlProperties));

        registrations.add(AdaptorUtil.register(dwtx.osgi.service.environment.EnvironmentInfo.class.getName(), EclipseEnvironmentInfo.getDefault(), context));
        registrations.add(AdaptorUtil.register(PlatformAdmin.class.getName(), adaptor.getPlatformAdmin(), context));
        PluginConverter converter = PluginConverterImpl.getDefault();
        if (converter is null)
            converter = new PluginConverterImpl(adaptor, context);
        registrations.add(AdaptorUtil.register(PluginConverter.class.getName(), converter, context));
        registrations.add(AdaptorUtil.register(CommandProvider.class.getName(), new EclipseCommandProvider(context), context));
        registrations.add(AdaptorUtil.register(dwtx.osgi.service.localization.BundleLocalization.class.getName(), new BundleLocalizationImpl(), context));
    }

    private void registerEndorsedXMLParser(BundleContext bc) {
        try {
            Class.forName(SAXFACTORYNAME);
            registrations.add(bc.registerService(SAXFACTORYNAME, new SaxParsingService(), new Hashtable()));
            Class.forName(DOMFACTORYNAME);
            registrations.add(bc.registerService(DOMFACTORYNAME, new DomParsingService(), new Hashtable()));
        } catch (ClassNotFoundException e) {
            noXML = true;
            if (Debug.DEBUG && Debug.DEBUG_ENABLED) {
                String message = EclipseAdaptorMsg.ECLIPSE_ADAPTOR_ERROR_XML_SERVICE;
                adaptor.getFrameworkLog().log(new FrameworkLogEntry(FrameworkAdaptor.FRAMEWORK_SYMBOLICNAME, FrameworkLogEntry.ERROR, 0, message, 0, e, null));
            }
        }
    }

    private class SaxParsingService implements ServiceFactory {
        public Object getService(Bundle bundle, ServiceRegistration registration) {
            return SAXParserFactory.newInstance();
        }

        public void ungetService(Bundle bundle, ServiceRegistration registration, Object service) {
            // Do nothing.
        }
    }

    private class DomParsingService implements ServiceFactory {
        public Object getService(Bundle bundle, ServiceRegistration registration) {
            return DocumentBuilderFactory.newInstance();
        }

        public void ungetService(Bundle bundle, ServiceRegistration registration, Object service) {
            // Do nothing.
        }
    }

    public void frameworkStop(BundleContext context) throws BundleException {
        printStats();
        if (!noXML)
            PluginParser.releaseXMLParsing();
        // unregister services
        for (Iterator iRegistrations = registrations.iterator(); iRegistrations.hasNext();)
            ((ServiceRegistration) iRegistrations.next()).unregister();
        registrations.clear();
    }

    private void printStats() {
        FrameworkDebugOptions debugOptions = FrameworkDebugOptions.getDefault();
        if (debugOptions is null)
            return;
        String registryParsing = debugOptions.getOption("dwtx.core.runtime/registry/parsing/timing/value"); //$NON-NLS-1$
        if (registryParsing !is null)
            MessageHelper.debug("Time spent in registry parsing: " + registryParsing); //$NON-NLS-1$
        String packageAdminResolution = debugOptions.getOption("debug.packageadmin/timing/value"); //$NON-NLS-1$
        if (packageAdminResolution !is null)
            System.out.println("Time spent in package admin resolve: " + packageAdminResolution); //$NON-NLS-1$         
        String constraintResolution = debugOptions.getOption("dwtx.core.runtime.adaptor/resolver/timing/value"); //$NON-NLS-1$
        if (constraintResolution !is null)
            System.out.println("Time spent resolving the dependency system: " + constraintResolution); //$NON-NLS-1$ 
    }

    public void frameworkStopping(BundleContext context) {
        // do nothing
    }

    public void addProperties(Properties properties) {
        // default to not verify EE at install time
        if (properties.getProperty(Constants.ECLIPSE_EE_INSTALL_VERIFY) is null)
            properties.put(Constants.ECLIPSE_EE_INSTALL_VERIFY, "false"); //$NON-NLS-1$
        // default to enable compatibility boot delegation
        if (properties.getProperty(Constants.OSGI_COMPATIBILITY_BOOTDELEGATION) is null)
            properties.put(Constants.OSGI_COMPATIBILITY_BOOTDELEGATION, "true"); //$NON-NLS-1$
    }

    public URLConnection mapLocationToURLConnection(String location) throws IOException {
        // do nothing
        return null;
    }

    public void handleRuntimeError(Throwable error) {
        // do nothing
    }

    public bool matchDNChain(String pattern, String[] dnChain) {
        // do nothing
        return false;
    }

    public FrameworkLog createFrameworkLog() {
        // do nothing
        return null;
    }

    public void initialize(BaseAdaptor adaptor) {
        this.adaptor = adaptor;
        // EnvironmentInfo has to be initialized first to compute defaults for system context (see bug 88925)
        EclipseEnvironmentInfo.getDefault();
        setDebugOptions();
    }

    private void setDebugOptions() {
        FrameworkDebugOptions options = FrameworkDebugOptions.getDefault();
        // may be null if debugging is not enabled
        if (options is null)
            return;
        PluginConverterImpl.DEBUG = options.getBooleanOption(OPTION_CONVERTER, false);
        BasicLocation.DEBUG = options.getBooleanOption(OPTION_LOCATION, false);
    }

    public void addHooks(HookRegistry hookRegistry) {
        hookRegistry.addAdaptorHook(this);
    }

}
