/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.core.runtime.internal.adaptor.EclipseLogHook;

import dwt.dwthelper.utils;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URLConnection;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Properties;

import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

import dwtx.core.runtime.adaptor.EclipseLog;
import dwtx.core.runtime.adaptor.EclipseStarter;
import dwtx.core.runtime.adaptor.LocationManager;
import dwtx.osgi.baseadaptor.BaseAdaptor;
import dwtx.osgi.baseadaptor.HookConfigurator;
import dwtx.osgi.baseadaptor.HookRegistry;
import dwtx.osgi.baseadaptor.hooks.AdaptorHook;
import dwtx.osgi.framework.internal.core.Constants;
import dwtx.osgi.framework.internal.core.FrameworkProperties;
import dwtx.osgi.framework.log.FrameworkLog;
import dwtx.osgi.internal.baseadaptor.AdaptorUtil;
import dwtx.osgi.service.datalocation.Location;

public class EclipseLogHook implements HookConfigurator, AdaptorHook {
    // The eclipse log file extension */
    private static final String LOG_EXT = ".log"; //$NON-NLS-1$
    BaseAdaptor adaptor;

    public void addHooks(HookRegistry hookRegistry) {
        hookRegistry.addAdaptorHook(this);
    }

    public void initialize(BaseAdaptor adaptor) {
        this.adaptor = adaptor;
    }

    public void frameworkStart(BundleContext context) throws BundleException {
        AdaptorUtil.register(FrameworkLog.class.getName(), adaptor.getFrameworkLog(), context);
        registerPerformanceLog(context);
    }

    public void frameworkStop(BundleContext context) throws BundleException {
        // TODO should unregister service registered a frameworkStart
    }

    public void frameworkStopping(BundleContext context) {
        // do nothing

    }

    public void addProperties(Properties properties) {
        // do nothing
    }

    public URLConnection mapLocationToURLConnection(String location) throws IOException {
        // do nothing
        return null;
    }

    public void handleRuntimeError(Throwable error) {
        // TODO Auto-generated method stub

    }

    public bool matchDNChain(String pattern, String[] dnChain) {
        // do nothing
        return false;
    }

    public FrameworkLog createFrameworkLog() {
        FrameworkLog frameworkLog;
        String logFileProp = FrameworkProperties.getProperty(EclipseStarter.PROP_LOGFILE);
        if (logFileProp !is null) {
            frameworkLog = new EclipseLog(new File(logFileProp));
        } else {
            Location location = LocationManager.getConfigurationLocation();
            File configAreaDirectory = null;
            if (location !is null)
                // TODO assumes the URL is a file: url
                configAreaDirectory = new File(location.getURL().getFile());

            if (configAreaDirectory !is null) {
                String logFileName = Long.toString(System.currentTimeMillis()) + EclipseLogHook.LOG_EXT;
                File logFile = new File(configAreaDirectory, logFileName);
                FrameworkProperties.setProperty(EclipseStarter.PROP_LOGFILE, logFile.getAbsolutePath());
                frameworkLog = new EclipseLog(logFile);
            } else
                frameworkLog = new EclipseLog();
        }
        if ("true".equals(FrameworkProperties.getProperty(EclipseStarter.PROP_CONSOLE_LOG))) //$NON-NLS-1$
            frameworkLog.setConsoleLog(true);
        return frameworkLog;
    }

    private void registerPerformanceLog(BundleContext context) {
        Object service = createPerformanceLog();
        String serviceName = FrameworkLog.class.getName();
        Hashtable serviceProperties = new Hashtable(7);
        Dictionary headers = context.getBundle().getHeaders();

        serviceProperties.put(Constants.SERVICE_VENDOR, headers.get(Constants.BUNDLE_VENDOR));
        serviceProperties.put(Constants.SERVICE_RANKING, new Integer(Integer.MIN_VALUE));
        serviceProperties.put(Constants.SERVICE_PID, context.getBundle().getBundleId() + '.' + service.getClass().getName());
        serviceProperties.put(FrameworkLog.SERVICE_PERFORMANCE, Boolean.TRUE.toString());

        context.registerService(serviceName, service, serviceProperties);
    }

    private FrameworkLog createPerformanceLog() {
        String logFileProp = FrameworkProperties.getProperty(EclipseStarter.PROP_LOGFILE);
        if (logFileProp !is null) {
            int lastSlash = logFileProp.lastIndexOf(File.separatorChar);
            if (lastSlash > 0) {
                String logFile = logFileProp.substring(0, lastSlash + 1) + "performance.log"; //$NON-NLS-1$
                return new EclipseLog(new File(logFile));
            }
        }
        //if all else fails, write to std err
        return new EclipseLog(new PrintWriter(System.err));
    }
}
