/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.HelpPlugin;

import dwt.dwthelper.utils;

import java.io.File;
import java.io.InputStream;
import java.net.URL;

import org.osgi.framework.BundleContext;

import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Plugin;
import dwtx.core.runtime.Status;
import dwtx.help.internal.context.ContextManager;
import dwtx.help.internal.extension.ContentExtensionManager;
import dwtx.help.internal.index.IndexManager;
import dwtx.help.internal.toc.TocManager;
import dwtx.osgi.service.datalocation.Location;

/**
 * Help System Core plug-in
 */
public class HelpPlugin extends Plugin {

    public final static String PLUGIN_ID = "dwtx.help"; //$NON-NLS-1$
    // debug options
    public static bool DEBUG = false;
    public static bool DEBUG_CONTEXT = false;
    public static bool DEBUG_SEARCH = false;
    public static bool DEBUG_TOC = false;
    public static bool DEBUG_INDEX = false;
    
    public final static String HELP_DATA_KEY = "HELP_DATA"; //$NON-NLS-1$
    public final static String BASE_TOCS_KEY = "baseTOCS"; //$NON-NLS-1$
    public final static String IGNORED_TOCS_KEY = "ignoredTOCS"; //$NON-NLS-1$
    public final static String IGNORED_INDEXES_KEY = "ignoredIndexes"; //$NON-NLS-1$

    private static HelpPlugin plugin;
    private static Object tocManagerCreateLock = new Object();
    
    private TocManager tocManager;
    private ContextManager contextManager;
    private ContentExtensionManager contentExtensionManager;
    private IndexManager indexManager;
    private IHelpProvider helpProvider;
    private File configurationDirectory;

    public static void logWarning(String message) {
        Status errorStatus = new Status(IStatus.WARNING, PLUGIN_ID, IStatus.OK, message, null);
        logStatus(errorStatus);
    }

    public static void logError(String message) {
        Status errorStatus = new Status(IStatus.ERROR, PLUGIN_ID, IStatus.OK, message, null);
        logStatus(errorStatus);
    }

    public static void logError(String message, Throwable ex) {
        if (message is null) {
            message = ""; //$NON-NLS-1$
        }
        Status errorStatus = new Status(IStatus.ERROR, PLUGIN_ID, IStatus.OK, message, ex);
        logStatus(errorStatus);
    }

    private static synchronized void logStatus(IStatus errorStatus) {
        HelpPlugin.getDefault().getLog().log(errorStatus);
    }

    /**
     * @return the singleton instance of the plugin
     */
    public static HelpPlugin getDefault() {
        return plugin;
    }

    /**
     * Used to obtain Toc Naviagiont Manager
     * 
     * @return instance of TocManager
     */
    public static TocManager getTocManager() {
        if (getDefault().tocManager is null) {
            synchronized (tocManagerCreateLock) {
                if (getDefault().tocManager is null) {
                    getDefault().tocManager = new TocManager();
                }
            }
        }
        return getDefault().tocManager;
    }

    /**
     * Used to obtain Context Manager returns an instance of ContextManager
     */
    public static ContextManager getContextManager() {
        if (getDefault().contextManager is null)
            getDefault().contextManager = new ContextManager();
        return getDefault().contextManager;
    }

    /**
     * Used to obtain the ContentExtensionManager
     */
    public static ContentExtensionManager getContentExtensionManager() {
        if (getDefault().contentExtensionManager is null)
            getDefault().contentExtensionManager = new ContentExtensionManager();
        return getDefault().contentExtensionManager;
    }

    public static IndexManager getIndexManager() {
        if (getDefault().indexManager is null)
            getDefault().indexManager = new IndexManager();
        return getDefault().indexManager;
    }

    /*
     * Returns the provider responsible for serving help documents.
     */
    public IHelpProvider getHelpProvider() {
        return helpProvider;
    }
    
    /*
     * Sets the provider responsible for serving help documents. Called
     * on startup.
     */
    public void setHelpProvider(IHelpProvider helpProvider) {
        this.helpProvider = helpProvider;
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.Plugin#start(org.osgi.framework.BundleContext)
     */
    public void start(BundleContext context) throws Exception {
        super.start(context);
        plugin = this;
        
        // determine configuration location for this plug-in
        Location location = Platform.getConfigurationLocation();
        if (location !is null) {
            URL configURL = location.getURL();
            if (configURL !is null && configURL.getProtocol().startsWith("file")) { //$NON-NLS-1$
                configurationDirectory = new File(configURL.getFile(), PLUGIN_ID);
            }
        }
        if (configurationDirectory is null) {
            configurationDirectory = getStateLocation().toFile();
        }
        // Setup debugging options
        DEBUG = isDebugging();
        if (DEBUG) {
            DEBUG_CONTEXT = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/debug/context")); //$NON-NLS-1$ //$NON-NLS-2$
            DEBUG_SEARCH = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/debug/search")); //$NON-NLS-1$ //$NON-NLS-2$
            DEBUG_TOC = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/debug/toc")); //$NON-NLS-1$ //$NON-NLS-2$     
            DEBUG_INDEX = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/debug/index")); //$NON-NLS-1$ //$NON-NLS-2$         
        }
    }

    public static File getConfigurationDirectory() {
        return getDefault().configurationDirectory;
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
     */
    public void stop(BundleContext context) throws Exception {
        plugin = null;
        super.stop(context);
    }

    /*
     * An interface by which higher plug-ins can serve help content.
     */
    public static interface IHelpProvider {
        public InputStream getHelpContent(String href, String locale);
    }
}
