/*******************************************************************************
 * Copyright (c) 2005, 2007 Intel Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Intel Corporation - initial API and implementation
 *     IBM Corporation - 122967 [Help] Remote help system
 *                       163558 Dynamic content support for all UA
 *                       165168 [Help] Better control of how help content is arranged and ordered
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.index.IndexManager;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtensionRegistry;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Preferences;
import dwtx.help.AbstractIndexProvider;
import dwtx.help.IIndex;
import dwtx.help.IIndexContribution;
import dwtx.help.internal.HelpData;
import dwtx.help.internal.HelpPlugin;
import dwtx.help.internal.UAElementFactory;

public class IndexManager {

    private static final String EXTENSION_POINT_ID_INDEX = HelpPlugin.PLUGIN_ID + ".index"; //$NON-NLS-1$
    private static final String ELEMENT_NAME_INDEX_PROVIDER = "indexProvider"; //$NON-NLS-1$
    private static final String ATTRIBUTE_NAME_CLASS = "class"; //$NON-NLS-1$
    
    private Map indexContributionsByLocale = new HashMap();
    private Map indexesByLocale = new HashMap();
    private AbstractIndexProvider[] indexProviders;
    
    public synchronized IIndex getIndex(String locale) {
        Index index = (Index)indexesByLocale.get(locale);
        if (index is null) {
            HelpPlugin.getTocManager().getTocs(locale);  // Ensure Tocs and index not built simultaneously
            long start = System.currentTimeMillis();
            if (HelpPlugin.DEBUG_INDEX) {
                System.out.println("Start to update keyword index for locale " + locale); //$NON-NLS-1$
            }
            List contributions = new ArrayList(Arrays.asList(readIndexContributions(locale)));
            filterIndexContributions(contributions);
            IndexAssembler assembler = new IndexAssembler();
            index = assembler.assemble(contributions, locale);
            indexesByLocale.put(locale, index);
            long stop = System.currentTimeMillis();
            if (HelpPlugin.DEBUG_INDEX) {
                System.out.println("Milliseconds to update keyword index for locale " + locale +  " = " + (stop - start)); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
        return index;
    }
    
    /*
     * Returns all index contributions for the given locale, from all
     * providers.
     */
    public synchronized IndexContribution[] getIndexContributions(String locale) {
        IndexContribution[] contributions = (IndexContribution[])indexContributionsByLocale.get(locale);
        if (contributions is null) {
            contributions = readIndexContributions(locale);
            indexContributionsByLocale.put(locale, contributions);
        }
        return contributions;
    }

    private IndexContribution[] readIndexContributions(String locale) {
        IndexContribution[] cached;
        List contributions = new ArrayList();
        AbstractIndexProvider[] providers = getIndexProviders();
        for (int i=0;i<providers.length;++i) {
            IIndexContribution[] contrib;
            try {
                contrib = providers[i].getIndexContributions(locale);
            }
            catch (Throwable t) {
                // log, and skip the offending provider
                String msg = "Error getting help keyword index data from provider: " + providers[i].getClass().getName() + " (skipping provider)"; //$NON-NLS-1$ //$NON-NLS-2$
                HelpPlugin.logError(msg, t);
                continue;
            }
            
            // check for nulls and root element
            for (int j=0;j<contrib.length;++j) {
                if (contrib[j] is null) {
                    String msg = "Help keyword index provider \"" + providers[i].getClass().getName() + "\" returned a null contribution (skipping)"; //$NON-NLS-1$ //$NON-NLS-2$
                    HelpPlugin.logError(msg);
                }
                else if (contrib[j].getIndex() is null) {
                    String msg = "Help keyword index provider \"" + providers[i].getClass().getName() + "\" returned a contribution with a null root element (expected a \"" + Index.NAME + "\" element; skipping)"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                    HelpPlugin.logError(msg);
                }
                else {
                    IndexContribution contribution = new IndexContribution();
                    contribution.setId(contrib[j].getId());
                    contribution.setLocale(contrib[j].getLocale());
                    IIndex index = contrib[j].getIndex();
                    contribution.setIndex(index instanceof Index ? (Index)index : (Index)UAElementFactory.newElement(index));
                    contributions.add(contribution);
                }
            }
        }
        cached = (IndexContribution[])contributions.toArray(new IndexContribution[contributions.size()]);
        return cached;
    }
    
    /*
     * Clears all cached contributions, forcing the manager to query the
     * providers again next time a request is made.
     */
    public void clearCache() {
        indexContributionsByLocale.clear();
        indexesByLocale.clear();
    }

    /*
     * Internal hook for unit testing.
     */
    public AbstractIndexProvider[] getIndexProviders() {
        if (indexProviders is null) {
            List providers = new ArrayList();
            IExtensionRegistry registry = Platform.getExtensionRegistry();
            IConfigurationElement[] elements = registry.getConfigurationElementsFor(EXTENSION_POINT_ID_INDEX);
            for (int i=0;i<elements.length;++i) {
                IConfigurationElement elem = elements[i];
                if (elem.getName().equals(ELEMENT_NAME_INDEX_PROVIDER)) {
                    try {
                        AbstractIndexProvider provider = (AbstractIndexProvider)elem.createExecutableExtension(ATTRIBUTE_NAME_CLASS);
                        providers.add(provider);
                    }
                    catch (CoreException e) {
                        // log and skip
                        String msg = "Error instantiating help keyword index provider class \"" + elem.getAttribute(ATTRIBUTE_NAME_CLASS) + '"'; //$NON-NLS-1$
                        HelpPlugin.logError(msg, e);
                    }
                }
            }
            indexProviders = (AbstractIndexProvider[])providers.toArray(new AbstractIndexProvider[providers.size()]);
        }
        return indexProviders;
    }
    
    /*
     * Returns whether or not the index has been completely loaded for the
     * given locale yet or not.
     */
    public bool isIndexLoaded(String locale) {
        return indexesByLocale.get(locale) !is null;
    }

    /*
     * Internal hook for unit testing.
     */
    public void setIndexProviders(AbstractIndexProvider[] indexProviders) {
        this.indexProviders = indexProviders;
    }

    /*
     * Filters the given contributions according to product preferences. If
     * either the contribution's id or its category's id is listed in the
     * ignoredIndexes, filter the contribution.
     */
    private void filterIndexContributions(List unfiltered) {
        Set indexesToFilter = getIgnoredIndexContributions();
        ListIterator iter = unfiltered.listIterator();
        while (iter.hasNext()) {
            IIndexContribution contribution = (IIndexContribution)iter.next();
            if (indexesToFilter.contains(contribution.getId())) {
                iter.remove();
            }
        }
    }

    private Set getIgnoredIndexContributions() {
        HelpData helpData = HelpData.getProductHelpData();
        if (helpData !is null) {
            return helpData.getHiddenIndexes();
        }
        else {
            HashSet ignored = new HashSet();
            Preferences pref = HelpPlugin.getDefault().getPluginPreferences();
            String preferredIndexes = pref.getString(HelpPlugin.IGNORED_INDEXES_KEY);
            if (preferredIndexes.length() > 0) {
                StringTokenizer suggestdOrderedInfosets = new StringTokenizer(preferredIndexes, " ;,"); //$NON-NLS-1$
                while (suggestdOrderedInfosets.hasMoreTokens()) {
                    ignored.add(suggestdOrderedInfosets.nextToken());
                }
            }
            return ignored;
        }
    }
}
