/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.help.internal.util.ProductPreferences;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;

import org.osgi.framework.Bundle;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IProduct;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.Plugin;
import dwtx.core.runtime.Preferences;
import dwtx.help.internal.HelpData;
import dwtx.help.internal.HelpPlugin;

/*
 * Reads and processes product preferences by considering not only the active
 * product, but all installed products.
 * 
 * For example, help orders the books in the table of contents in such a way that
 * satisfies the currently running product's preferred order, and as many other product's
 * preferred orderings.
 */
public class ProductPreferences {

    private static final String TRUE = String.valueOf(true);
    private static Properties[] productPreferences;
    private static SequenceResolver orderResolver;
    private static Map preferencesToPluginIdMap;
    private static Map preferencesToProductIdMap;
    private static List primaryTocOrdering;
    private static List[] secondaryTocOrderings;    
    private static final String PLUGINS_ROOT_SLASH = "PLUGINS_ROOT/"; //$NON-NLS-1$
    
    /*
     * Returns the recommended order to display the given toc entries in. Each
     * toc entry is a String, either the id of the toc contribution or the
     * id of the category of tocs.
     */
    public static List getTocOrder(List itemsToOrder, Map nameIdMap) {
        List primaryOrdering = getPrimaryTocOrdering();
        List[] secondaryOrdering = new List[0];
        if (primaryOrdering is null || primaryOrdering.size() is 0) {
            secondaryOrdering = getSecondaryTocOrderings();
        }
        return getOrderedList(itemsToOrder, primaryOrdering, secondaryOrdering, nameIdMap);
    }
    
    /*
     * Returns the primary toc ordering. This is the preferred order for the active
     * product (either specified via help data xml file or deprecated comma-separated
     * list in plugin_customization.ini). Help data takes precedence.
     */
    public static List getPrimaryTocOrdering() {
        if (primaryTocOrdering is null) {
            IProduct product = Platform.getProduct();
            if (product !is null) {
                String pluginId = product.getDefiningBundle().getSymbolicName();
                Preferences prefs = HelpPlugin.getDefault().getPluginPreferences();
                String helpDataFile = prefs.getString(HelpPlugin.HELP_DATA_KEY);
                String baseTOCS = prefs.getString(HelpPlugin.BASE_TOCS_KEY);
                primaryTocOrdering = getTocOrdering(pluginId, helpDataFile, baseTOCS);
            }
            // active product has no preference for toc order
            if (primaryTocOrdering is null) {
                primaryTocOrdering = new ArrayList();
            }
        }
        return primaryTocOrdering;
    }
    
    /*
     * Returns all secondary toc ordering. These are the preferred toc orders of all
     * defined products except the active product.
     */
    public static List[] getSecondaryTocOrderings() {
        if (secondaryTocOrderings is null) {
            List list = new ArrayList();
            Properties[] productPreferences = getProductPreferences(false);
            for (int i=0;i<productPreferences.length;++i) {
                String pluginId = (String)preferencesToPluginIdMap.get(productPreferences[i]);
                String helpDataFile = (String)productPreferences[i].get(HelpPlugin.PLUGIN_ID + '/' + HelpPlugin.HELP_DATA_KEY);
                String baseTOCS = (String)productPreferences[i].get(HelpPlugin.PLUGIN_ID + '/' + HelpPlugin.BASE_TOCS_KEY);
                List ordering = getTocOrdering(pluginId, helpDataFile, baseTOCS);
                if (ordering !is null) {
                    list.add(ordering);
                }
            }
            secondaryTocOrderings = (List[])list.toArray(new List[list.size()]);
        }
        return secondaryTocOrderings;
    }

    /*
     * Returns the preferred toc ordering of the product defined by the given
     * plug-in that has the given helpDataFile and baseTOCS specified (these last
     * two may be null if not specified).
     */
    public static List getTocOrdering(String pluginId, String helpDataFile, String baseTOCS) {
        if (helpDataFile !is null && helpDataFile.length() > 0) {
            String helpDataPluginId = pluginId;
            String helpDataPath = helpDataFile;
            if (helpDataFile.startsWith(PLUGINS_ROOT_SLASH)) {
                int nextSlash = helpDataFile.indexOf('/', PLUGINS_ROOT_SLASH.length());
                if (nextSlash > 0) {
                    helpDataPluginId = helpDataFile.substring(PLUGINS_ROOT_SLASH.length(), nextSlash);
                    helpDataPath = helpDataFile.substring(nextSlash + 1);
                }
            }
            Bundle bundle = Platform.getBundle(helpDataPluginId);
            URL helpDataUrl = bundle.getEntry(helpDataPath);
            HelpData helpData = new HelpData(helpDataUrl);
            return helpData.getTocOrder();
        }
        else {
            if (baseTOCS !is null) {
                return tokenize(baseTOCS);
            }
        }
        return null;
    }
    
    /*
     * Returns the bool preference for the given key by consulting every
     * product's preferences. If any of the products want the preference to
     * be true (or use the default and the default is true), returns true.
     * Otherwise returns false (if no products want it true).
     */
    public static bool getBoolean(Plugin plugin, String key) {
        Properties[] properties = getProductPreferences(true);
        String defaultValue = plugin.getPluginPreferences().getDefaultString(key);
        String currentValue = plugin.getPluginPreferences().getString(key);
        String pluginId = plugin.getBundle().getSymbolicName();
        if (currentValue !is null && currentValue.equalsIgnoreCase(TRUE)) {
            return true;
        }
        for (int i=0;i<properties.length;++i) {
            String value = (String)properties[i].get(pluginId + '/' + key);
            if (value is null) {
                value = defaultValue;
            }
            if (value !is null && value.equalsIgnoreCase(TRUE)) {
                return true;
            }
        }
        return false;
    }

    /*
     * Returns the given items in the order specified. Items listed in the order
     * but not present are skipped, and items present but not ordered are added
     * at the end.
     */
    public static List getOrderedList(List items, List order) {
        return getOrderedList(items, order, null, null);
    }

    /*
     * Returns the given items in an order that best satisfies the given orderings.
     * The primary ordering must be satisfied in all cases. As many secondary orderings
     * as reasonably possible will be satisfied.
     */
    public static List getOrderedList(List items, List primary, List[] secondary, Map nameIdMap) {
        List result = new ArrayList();
        Set set = new HashSet(items);
        if (orderResolver is null) {
            orderResolver = new SequenceResolver();
        }
        List order = orderResolver.getSequence(primary, secondary);
        Iterator iter = order.iterator();
        while (iter.hasNext()) {
            Object obj = iter.next();
            if (set.contains(obj)) {
                result.add(obj);
                set.remove(obj);
            }
        }
        List remaining = new ArrayList();
        remaining.addAll(set);
        if (nameIdMap !is null) {
            sortByName(remaining, nameIdMap);
        }
        result.addAll(remaining);
        return result;
    }
    
    private static class NameComparator implements Comparator {

        private Map tocNames;
        public NameComparator(Map tocNames) {
            this.tocNames = tocNames;
        }
        
        public int compare(Object o1, Object o2) {
            Object name1 = tocNames.get(o1);
            Object name2 = tocNames.get(o2);
            if (!(name1 instanceof String)) {
                return (name2 instanceof String) ? -1 : 0;
            }
            if (!(name2 instanceof String)) {
                return 1;
            }
            return ((String)name1).compareToIgnoreCase((String)name2);
        }
        
    }

    private static void sortByName(List remaining, Map categorized) {
        Collections.sort(remaining, new NameComparator(categorized));
    }

    public static synchronized String getPluginId(Properties prefs) {
        return (String)preferencesToPluginIdMap.get(prefs);
    }

    public static synchronized String getProductId(Properties prefs) {
        return (String)preferencesToProductIdMap.get(prefs);
    }

    /*
     * Returns the preferences for all products in the runtime environment (even if
     * they are not active).
     */
    public static synchronized Properties[] getProductPreferences(bool includeActiveProduct) {
        if (productPreferences is null) {
            String activeProductId = null;
            IProduct activeProduct = Platform.getProduct();
            if (activeProduct !is null) {
                activeProductId = activeProduct.getId();
            }
            Collection collection = new ArrayList();
            IConfigurationElement[] elements = Platform.getExtensionRegistry().getConfigurationElementsFor("dwtx.core.runtime.products"); //$NON-NLS-1$
            for (int i=0;i<elements.length;++i) {
                if (elements[i].getName().equals("product")) { //$NON-NLS-1$
                    String productId = elements[i].getDeclaringExtension().getUniqueIdentifier();
                    if (includeActiveProduct || activeProductId is null || !activeProductId.equals(productId)) {
                        String contributor = elements[i].getContributor().getName();
                        IConfigurationElement[] propertyElements = elements[i].getChildren("property"); //$NON-NLS-1$
                        for (int j=0;j<propertyElements.length;++j) {
                            String name = propertyElements[j].getAttribute("name"); //$NON-NLS-1$
                            if (name !is null && name.equals("preferenceCustomization")) { //$NON-NLS-1$
                                String value = propertyElements[j].getAttribute("value"); //$NON-NLS-1$
                                if (value !is null) {
                                    Properties properties = loadPropertiesFile(contributor, value);
                                    if (properties !is null) {
                                        collection.add(properties);
                                    }
                                    if (preferencesToPluginIdMap is null) {
                                        preferencesToPluginIdMap = new HashMap();
                                    }
                                    preferencesToPluginIdMap.put(properties, contributor);
                                    if (preferencesToProductIdMap is null) {
                                        preferencesToProductIdMap = new HashMap();
                                    }
                                    preferencesToProductIdMap.put(properties, productId);
                                }
                            }
                        }
                    }
                }
            }
            productPreferences = (Properties[])collection.toArray(new Properties[collection.size()]);
        }
        return productPreferences;
    }
    
    /*
     * Returns all the unique values for the given key in all the properties
     * provided. For example, if two of them have "true" and one has "false", it
     * will return "true" and "false".
     */
    public static Set getUniqueValues(Plugin plugin, String key, Properties[] properties) {
        Set set = new HashSet();
        String defaultValue = plugin.getPluginPreferences().getDefaultString(key);
        String currentValue = plugin.getPluginPreferences().getString(key);
        String pluginId = plugin.getBundle().getSymbolicName();
        for (int i=0;i<properties.length;++i) {
            String value = (String)properties[i].get(pluginId + '/' + key);
            set.add(value !is null ? value : defaultValue);
        }
        set.add(currentValue !is null ? currentValue : defaultValue);
        return set;
    }
    
    /*
     * Returns the value for the given key by consulting the given properties, but giving
     * precedence to the primary properties. If the primary properties has the key, it is
     * returned. Otherwise, it will return the value of the first secondary properties that
     * has the key, or null if none of them has it.
     */
    public static String getValue(String key, Properties primary, Properties[] secondary) {
        String value = null;
        if (primary !is null) {
            value = primary.getProperty(key);
        }
        if (value is null) {
            for (int i=0;i<secondary.length;++i) {
                if (secondary[i] !is primary) {
                    value = secondary[i].getProperty(key);
                    if (value !is null) {
                        break;
                    }
                }
            }
        }
        return value;
    }

    /*
     * Loads and returns the properties in the given properties file. The path is
     * relative to the bundle with the given id.
     */
    public static Properties loadPropertiesFile(String bundleId, String path) {
        Bundle bundle = Platform.getBundle(bundleId);
        if (bundle !is null) {
            URL url = bundle.getEntry(path);
            if (url !is null) {
                InputStream in = null;
                try {
                    in = url.openStream();
                    Properties properties = new Properties();
                    properties.load(in);
                    return properties;
                }
                catch (IOException e) {
                    // log the fact that it couldn't load it
                    HelpPlugin.logError("Error opening product's plugin customization file: " + bundleId + "/" + path, e); //$NON-NLS-1$ //$NON-NLS-2$
                }
                finally {
                    if (in !is null) {
                        try {
                            in.close();
                        }
                        catch (IOException e) {
                            // nothing we can do here
                        }
                    }
                }
            }
        }
        return null;
    }
    
    /*
     * Tokenizes the given list of items, allowing them to be separated by whitespace, commas,
     * and/or semicolons.
     * 
     * e.g. "item1, item2, item3"
     * would return a list of strings containing "item1", "item2", and "item3".
     */
    public static List tokenize(String str) {
        if (str !is null) {
            StringTokenizer tok = new StringTokenizer(str, " \n\r\t;,"); //$NON-NLS-1$
            List list = new ArrayList();
            while (tok.hasMoreElements()) {
                list.add(tok.nextToken());
            }
            return list;
        }
        return new ArrayList();
    }

    public int compare(Object o1, Object o2) {
        // TODO Auto-generated method stub
        return 0;
    }
}
