/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.contentassist.AbstractControlContentAssistSubjectAdapter;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import dwt.DWT;
import dwt.custom.VerifyKeyListener;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.events.KeyEvent;
import dwt.events.KeyListener;
import dwt.events.VerifyEvent;
import dwt.graphics.Image;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.Platform;
import dwtx.jface.fieldassist.ControlDecoration;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.text.IEventConsumer;
import dwtx.jface.viewers.ILabelProvider;
import dwtx.jface.viewers.ILabelProviderListener;
import dwtx.jface.viewers.LabelProviderChangedEvent;


/**
 * An <code>AbstractControlContentAssistSubjectAdapter</code> delegates assistance requests from a
 * {@linkplain dwtx.jface.text.contentassist.ContentAssistant content assistant}
 * to a <code>Control</code>.
 *
 * A visual feedback can be configured via {@link #setContentAssistCueProvider(ILabelProvider)}.
 *
 * @since 3.0
 * @deprecated As of 3.2, replaced by Platform UI's field assist support
 */
public abstract class AbstractControlContentAssistSubjectAdapter implements IContentAssistSubjectControl {

    protected static final bool DEBUG= "true".equalsIgnoreCase(Platform.getDebugOption("dwtx.jface.text/debug/ContentAssistSubjectAdapters"));  //$NON-NLS-1$//$NON-NLS-2$

    /**
     * VerifyKeyListeners for the control.
     */
    private List fVerifyKeyListeners;
    /**
     * KeyListeners for the control.
     */
    private Set fKeyListeners;
    /**
     * The Listener installed on the control which passes events to
     * {@link #fVerifyKeyListeners fVerifyKeyListeners} and {@link #fKeyListeners}.
     */
    private Listener fControlListener;
    /**
     * The cue label provider, or <code>null</code> iff none.
     * @since 3.3
     */
    private ILabelProvider fCueLabelProvider;
    /**
     * The control decoration, or <code>null</code> iff fCueLabelProvider is null.
     * @since 3.3
     */
    private ControlDecoration fControlDecoration;
    /**
     * The default cue image, or <code>null</code> if not cached yet.
     * @since 3.3
     */
    private Image fCachedDefaultCueImage;
    
    /**
     * Creates a new {@link AbstractControlContentAssistSubjectAdapter}.
     */
    public AbstractControlContentAssistSubjectAdapter() {
        fVerifyKeyListeners= new ArrayList(1);
        fKeyListeners= new HashSet(1);
    }

    /*
     * @see dwtx.jface.text.contentassist.IContentAssistSubjectControl#getControl()
     */
    public abstract Control getControl();

    /*
     * @see dwtx.jface.text.contentassist.IContentAssistSubjectControl#addKeyListener(dwt.events.KeyListener)
     */
    public void addKeyListener(KeyListener keyListener) {
        fKeyListeners.add(keyListener);

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#addKeyListener()"); //$NON-NLS-1$

        installControlListener();
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#removeKeyListener(dwt.events.KeyListener)
     */
    public void removeKeyListener(KeyListener keyListener) {
        bool deleted= fKeyListeners.remove(keyListener);

        if (DEBUG) {
            if (!deleted)
                System.out.println("removeKeyListener -> wasn't here"); //$NON-NLS-1$
            System.out.println("AbstractControlContentAssistSubjectAdapter#removeKeyListener() -> " + fKeyListeners.size()); //$NON-NLS-1$
        }

        uninstallControlListener();
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#supportsVerifyKeyListener()
     */
    public bool supportsVerifyKeyListener() {
        return true;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#appendVerifyKeyListener(dwt.custom.VerifyKeyListener)
     */
    public bool appendVerifyKeyListener(final VerifyKeyListener verifyKeyListener) {
        fVerifyKeyListeners.add(verifyKeyListener);

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#appendVerifyKeyListener() -> " + fVerifyKeyListeners.size()); //$NON-NLS-1$

        installControlListener();
        return true;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#prependVerifyKeyListener(dwt.custom.VerifyKeyListener)
     */
    public bool prependVerifyKeyListener(final VerifyKeyListener verifyKeyListener) {
        fVerifyKeyListeners.add(0, verifyKeyListener);

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#prependVerifyKeyListener() -> " + fVerifyKeyListeners.size()); //$NON-NLS-1$

        installControlListener();
        return true;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#removeVerifyKeyListener(dwt.custom.VerifyKeyListener)
     */
    public void removeVerifyKeyListener(VerifyKeyListener verifyKeyListener) {
        fVerifyKeyListeners.remove(verifyKeyListener);

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#removeVerifyKeyListener() -> " + fVerifyKeyListeners.size()); //$NON-NLS-1$

        uninstallControlListener();
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#setEventConsumer(dwtx.jface.text.IEventConsumer)
     */
    public void setEventConsumer(IEventConsumer eventConsumer) {
        // this is not supported
        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#setEventConsumer()"); //$NON-NLS-1$
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getLineDelimiter()
     */
    public String getLineDelimiter() {
        return System.getProperty("line.separator"); //$NON-NLS-1$
    }

    /**
     * Installs <code>fControlListener</code>, which handles VerifyEvents and KeyEvents by
     * passing them to {@link #fVerifyKeyListeners} and {@link #fKeyListeners}.
     */
    private void installControlListener() {
        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#installControlListener() -> k: " + fKeyListeners.size() + ", v: " + fVerifyKeyListeners.size()); //$NON-NLS-1$ //$NON-NLS-2$

        if (fControlListener !is null)
            return;

        fControlListener= new Listener() {
            public void handleEvent(Event e) {
                if (! getControl().isFocusControl())
                    return; //DWT.TRAVERSE_MNEMONIC events can also come in to inactive widgets
                VerifyEvent verifyEvent= new VerifyEvent(e);
                KeyEvent keyEvent= new KeyEvent(e);
                switch (e.type) {
                    case DWT.Traverse :

                        if (DEBUG)
                            dump("before traverse", e, verifyEvent); //$NON-NLS-1$

                        verifyEvent.doit= true;
                        for (Iterator iter= fVerifyKeyListeners.iterator(); iter.hasNext(); ) {
                            ((VerifyKeyListener) iter.next()).verifyKey(verifyEvent);
                            if (! verifyEvent.doit) {
                                e.detail= DWT.TRAVERSE_NONE;
                                e.doit= true;
                                if (DEBUG)
                                    dump("traverse eaten by verify", e, verifyEvent); //$NON-NLS-1$
                                return;
                            }

                            if (DEBUG)
                                dump("traverse OK", e, verifyEvent); //$NON-NLS-1$
                        }
                        break;

                    case DWT.KeyDown:
                        for (Iterator iter= fVerifyKeyListeners.iterator(); iter.hasNext(); ) {
                            ((VerifyKeyListener) iter.next()).verifyKey(verifyEvent);
                            if (! verifyEvent.doit) {
                                e.doit= verifyEvent.doit;
                                if (DEBUG)
                                    dump("keyDown eaten by verify", e, verifyEvent); //$NON-NLS-1$
                                return;
                            }
                        }

                        if (DEBUG)
                            dump("keyDown OK", e, verifyEvent); //$NON-NLS-1$

                        for (Iterator iter= fKeyListeners.iterator(); iter.hasNext();) {
                            ((KeyListener) iter.next()).keyPressed(keyEvent);
                        }
                        break;

                    default :
                        Assert.isTrue(false);
                }
            }

            /**
             * Dump the given events to "standard" output.
             *
             * @param who who dump's
             * @param e the event
             * @param ve the verify event
             */
            private void dump(String who, Event e, VerifyEvent ve) {
                StringBuffer sb= new StringBuffer("--- [AbstractControlContentAssistSubjectAdapter]\n"); //$NON-NLS-1$
                sb.append(who);
                sb.append(" - e: keyCode="+e.keyCode+hex(e.keyCode)); //$NON-NLS-1$
                sb.append("; character="+e.character+hex(e.character)); //$NON-NLS-1$
                sb.append("; stateMask="+e.stateMask+hex(e.stateMask)); //$NON-NLS-1$
                sb.append("; doit="+e.doit); //$NON-NLS-1$
                sb.append("; detail="+e.detail+hex(e.detail)); //$NON-NLS-1$
                sb.append("; widget="+e.widget); //$NON-NLS-1$
                sb.append("\n"); //$NON-NLS-1$
                sb.append("  verifyEvent keyCode="+e.keyCode+hex(e.keyCode)); //$NON-NLS-1$
                sb.append("; character="+e.character+hex(e.character)); //$NON-NLS-1$
                sb.append("; stateMask="+e.stateMask+hex(e.stateMask)); //$NON-NLS-1$
                sb.append("; doit="+ve.doit); //$NON-NLS-1$
                sb.append("; widget="+e.widget); //$NON-NLS-1$
                System.out.println(sb);
            }

            private String hex(int i) {
                return "[0x" + Integer.toHexString(i) + ']'; //$NON-NLS-1$
            }
        };
        getControl().addListener(DWT.Traverse, fControlListener);
        getControl().addListener(DWT.KeyDown, fControlListener);

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#installControlListener() - installed"); //$NON-NLS-1$
    }

    /**
     * Uninstalls <code>fControlListener</code> iff there are no <code>KeyListener</code>s and no
     * <code>VerifyKeyListener</code>s registered.
     * Otherwise does nothing.
     */
    private void uninstallControlListener() {
        if (fControlListener is null || fKeyListeners.size() + fVerifyKeyListeners.size() !is 0) {

            if (DEBUG)
                System.out.println("AbstractControlContentAssistSubjectAdapter#uninstallControlListener() -> k: " + fKeyListeners.size() + ", v: " + fVerifyKeyListeners.size()); //$NON-NLS-1$ //$NON-NLS-2$

            return;
        }
        getControl().removeListener(DWT.Traverse, fControlListener);
        getControl().removeListener(DWT.KeyDown, fControlListener);
        fControlListener= null;

        if (DEBUG)
            System.out.println("AbstractControlContentAssistSubjectAdapter#uninstallControlListener() - done"); //$NON-NLS-1$
    }

    /**
     * Sets the visual feedback provider for content assist.
     * The given {@link ILabelProvider} methods are called with
     * {@link #getControl()} as argument.
     *
     * <ul>
     *   <li><code>getImage(Object)</code> provides the visual cue image.
     *     The image can maximally be 5 pixels wide and 8 pixels high.
     *     If <code>getImage(Object)</code> returns <code>null</code>, a default image is used.
     *   </li>
     *   <li><code>getText(Object)</code> provides the hover info text.
     *     It is shown when hovering over the cue image or the adapted {@link Control}.
     *     No info text is shown if <code>getText(Object)</code> returns <code>null</code>.
     *   </li>
     * </ul>
     * <p>
     * The given {@link ILabelProvider} becomes owned by the {@link AbstractControlContentAssistSubjectAdapter},
     * i.e. it gets disposed when the adapted {@link Control} is disposed
     * or when another {@link ILabelProvider} is set.
     * </p>
     *
     * @param labelProvider a {@link ILabelProvider}, or <code>null</code>
     *  if no visual feedback should be shown
     */
    public void setContentAssistCueProvider(final ILabelProvider labelProvider) {
        if (fCueLabelProvider !is null) {
            fCueLabelProvider.dispose();
        }
        
        fCueLabelProvider= labelProvider;
        
        if (labelProvider is null) {
            if (fControlDecoration !is null) {
                fControlDecoration.dispose();
                fControlDecoration= null;
            }
            
        } else {
            if (fControlDecoration is null) {
                fControlDecoration= new ControlDecoration(getControl(), (DWT.TOP | DWT.LEFT));
                getControl().addDisposeListener(new DisposeListener() {
                    public void widgetDisposed(DisposeEvent e) {
                        if (fCueLabelProvider !is null) {
                            fCueLabelProvider.dispose();
                            fCueLabelProvider= null;
                        }
                        if (fControlDecoration !is null) {
                            fControlDecoration.dispose();
                            fControlDecoration= null;
                        }
                        if (fCachedDefaultCueImage !is null) {
                            fCachedDefaultCueImage.dispose();
                            fCachedDefaultCueImage= null;
                        }
                    }
                });
                fControlDecoration.setShowHover(true);
                fControlDecoration.setShowOnlyOnFocus(true);
            }
            
            ILabelProviderListener listener= new ILabelProviderListener() {
                public void labelProviderChanged(LabelProviderChangedEvent event) {
                    fControlDecoration.setDescriptionText(labelProvider.getText(getControl()));
                    Image image= labelProvider.getImage(getControl());
                    if (image is null)
                        image= getDefaultCueImage();
                    fControlDecoration.setImage(image);
                }
            };
            labelProvider.addListener(listener);
            //initialize control decoration:
            listener.labelProviderChanged(new LabelProviderChangedEvent(labelProvider));
        }
    }
    
    /**
     * Returns the default cue image.
     * 
     * @return the default cue image
     * @since 3.3
     */
    private Image getDefaultCueImage() {
        if (fCachedDefaultCueImage is null) {
            ImageDescriptor cueID= ImageDescriptor.createFromFile(AbstractControlContentAssistSubjectAdapter.class, "images/content_assist_cue.gif"); //$NON-NLS-1$
            fCachedDefaultCueImage= cueID.createImage(getControl().getDisplay());
        }
        return fCachedDefaultCueImage;
    }
}
