/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.contentassist.TextContentAssistSubjectAdapter;

import dwt.dwthelper.utils;

import java.util.HashMap;

import dwt.DWT;
import dwt.events.ModifyEvent;
import dwt.events.ModifyListener;
import dwt.events.SelectionEvent;
import dwt.events.SelectionListener;
import dwt.graphics.Point;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.Text;
import dwtx.core.runtime.Assert;
import dwtx.jface.text.BadLocationException;
import dwtx.jface.text.Document;
import dwtx.jface.text.IDocument;


/**
 * Adapts a {@link dwt.widgets.Text} to an {@link dwtx.jface.contentassist.IContentAssistSubjectControl}.
 *
 * @see dwt.widgets.Text
 * @see dwtx.jface.contentassist.IContentAssistSubjectControl
 * @since 3.0
 * @deprecated As of 3.2, replaced by Platform UI's field assist support
 */
public class TextContentAssistSubjectAdapter extends AbstractControlContentAssistSubjectAdapter {

    /**
     * The document backing this adapter's text widget.
     */
    private class InternalDocument extends Document {

        /**
         * Updates this document with changes in this adapter's text widget.
         */
        private ModifyListener fModifyListener;

        private InternalDocument() {
            super(fText.getText());
            fModifyListener= new ModifyListener() {
                /*
                 * @see dwt.events.ModifyListener#modifyText(dwt.events.ModifyEvent)
                 */
                public void modifyText(ModifyEvent e) {
                    set(fText.getText());
                }
            };
            fText.addModifyListener(fModifyListener);
        }

        /*
         * @see dwtx.jface.text.AbstractDocument#replace(int, int, java.lang.String)
         */
        public void replace(int pos, int length, String text) throws BadLocationException {
            super.replace(pos, length, text);
            fText.removeModifyListener(fModifyListener);
            fText.setText(get());
            fText.addModifyListener(fModifyListener);
        }
    }

    /** The text. */
    private Text fText;
    /** The modify listeners. */
    private HashMap fModifyListeners= new HashMap();

    /**
     * Creates a content assist subject control adapter for the given text widget.
     *
     * @param text the text widget to adapt
     */
    public TextContentAssistSubjectAdapter(Text text) {
        Assert.isNotNull(text);
        fText= text;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getControl()
     */
    public Control getControl() {
        return fText;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getLineHeight()
     */
    public int getLineHeight() {
        return fText.getLineHeight();
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getCaretOffset()
     */
    public int getCaretOffset() {
        return fText.getCaretPosition();
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getLocationAtOffset(int)
     */
    public Point getLocationAtOffset(int offset) {
        Point caretLocation= fText.getCaretLocation();
        /*
         * XXX workaround for https://bugs.eclipse.org/bugs/show_bug.cgi?id=52520
         */
        caretLocation.y += 2;
        return caretLocation;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getWidgetSelectionRange()
     */
    public Point getWidgetSelectionRange() {
        return new Point(fText.getSelection().x, Math.abs(fText.getSelection().y - fText.getSelection().x));
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getSelectedRange()
     */
    public Point getSelectedRange() {
        return new Point(fText.getSelection().x, Math.abs(fText.getSelection().y - fText.getSelection().x));
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#getDocument()
     */
    public IDocument getDocument() {
        IDocument document= (IDocument)fText.getData("document"); //$NON-NLS-1$
        if (document is null) {
            document= new InternalDocument() ;
            fText.setData("document", document); //$NON-NLS-1$
        }
        return document;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#setSelectedRange(int, int)
     */
    public void setSelectedRange(int i, int j) {
        fText.setSelection(new Point(i, i+j));
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#revealRange(int, int)
     */
    public void revealRange(int i, int j) {
        // XXX: this should be improved
        fText.setSelection(new Point(i, i+j));
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#addSelectionListener(dwt.events.SelectionListener)
     */
    public bool addSelectionListener(final SelectionListener selectionListener) {
        fText.addSelectionListener(selectionListener);
        Listener listener= new Listener() {
            /*
             * @see dwt.widgets.Listener#handleEvent(dwt.widgets.Event)
             */
            public void handleEvent(Event e) {
                selectionListener.widgetSelected(new SelectionEvent(e));

            }
        };
        fText.addListener(DWT.Modify, listener);
        fModifyListeners.put(selectionListener, listener);
        return true;
    }

    /*
     * @see dwtx.jface.contentassist.IContentAssistSubjectControl#removeSelectionListener(dwt.events.SelectionListener)
     */
    public void removeSelectionListener(SelectionListener selectionListener) {
        fText.removeSelectionListener(selectionListener);
        Object listener= fModifyListeners.get(selectionListener);
        if (listener instanceof Listener)
            fText.removeListener(DWT.Modify, (Listener)listener);
    }
}
