/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Matt Carter - bug 170668
 *     Brad Reynolds - bug 170848
 *     Matthew Hall - bug 180746, bug 207844
 *     Michael Krauter, bug 180223
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.databinding.swt.SWTObservables;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Iterator;

import dwt.custom.CCombo;
import dwt.custom.CLabel;
import dwt.widgets.Button;
import dwt.widgets.Combo;
import dwt.widgets.Control;
import dwt.widgets.Display;
import dwt.widgets.Label;
import dwt.widgets.Link;
import dwt.widgets.List;
import dwt.widgets.Scale;
import dwt.widgets.Shell;
import dwt.widgets.Spinner;
import dwt.widgets.Table;
import dwt.widgets.Text;
import dwtx.core.databinding.observable.Realm;
import dwtx.core.databinding.observable.list.IObservableList;
import dwtx.jface.internal.databinding.internal.swt.LinkObservableValue;
import dwtx.jface.internal.databinding.swt.ButtonObservableValue;
import dwtx.jface.internal.databinding.swt.CComboObservableList;
import dwtx.jface.internal.databinding.swt.CComboObservableValue;
import dwtx.jface.internal.databinding.swt.CComboSingleSelectionObservableValue;
import dwtx.jface.internal.databinding.swt.CLabelObservableValue;
import dwtx.jface.internal.databinding.swt.ComboObservableList;
import dwtx.jface.internal.databinding.swt.ComboObservableValue;
import dwtx.jface.internal.databinding.swt.ComboSingleSelectionObservableValue;
import dwtx.jface.internal.databinding.swt.ControlObservableValue;
import dwtx.jface.internal.databinding.swt.DelayedObservableValue;
import dwtx.jface.internal.databinding.swt.LabelObservableValue;
import dwtx.jface.internal.databinding.swt.ListObservableList;
import dwtx.jface.internal.databinding.swt.ListObservableValue;
import dwtx.jface.internal.databinding.swt.ListSingleSelectionObservableValue;
import dwtx.jface.internal.databinding.swt.SWTProperties;
import dwtx.jface.internal.databinding.swt.ScaleObservableValue;
import dwtx.jface.internal.databinding.swt.ShellObservableValue;
import dwtx.jface.internal.databinding.swt.SpinnerObservableValue;
import dwtx.jface.internal.databinding.swt.TableSingleSelectionObservableValue;
import dwtx.jface.internal.databinding.swt.TextEditableObservableValue;
import dwtx.jface.internal.databinding.swt.TextObservableValue;

/**
 * A factory for creating observables for DWT widgets
 * 
 * @since 1.1
 * 
 */
public class SWTObservables {

    private static java.util.List realms = new ArrayList();

    /**
     * Returns the realm representing the UI thread for the given display.
     * 
     * @param display
     * @return the realm representing the UI thread for the given display
     */
    public static Realm getRealm(final Display display) {
        synchronized (realms) {
            for (Iterator it = realms.iterator(); it.hasNext();) {
                DisplayRealm displayRealm = (DisplayRealm) it.next();
                if (displayRealm.display is display) {
                    return displayRealm;
                }
            }
            DisplayRealm result = new DisplayRealm(display);
            realms.add(result);
            return result;
        }
    }

    /**
     * Returns an observable which delays notification of value change events
     * from <code>observable</code> until <code>delay</code> milliseconds
     * have passed since the last change event, or until a FocusOut event is
     * received from the underlying widget (whichever happens earlier). This
     * class helps to delay validation until the user stops typing. To notify
     * about pending changes, the returned observable value will fire a stale
     * event when the wrapped observable value fires a change event, but this
     * change is being delayed.
     * 
     * @param delay
     * @param observable
     * @return an observable which delays notification of value change events
     *         from <code>observable</code> until <code>delay</code>
     *         milliseconds have passed since the last change event.
     * 
     * @since 1.2
     */
    public static ISWTObservableValue observeDelayedValue(int delay, ISWTObservableValue observable) {
      return new DelayedObservableValue(delay, observable);
    }

    /**
     * @param control
     * @return an observable value tracking the enabled state of the given
     *         control
     */
    public static ISWTObservableValue observeEnabled(Control control) {
        return new ControlObservableValue(control, SWTProperties.ENABLED);
    }

    /**
     * @param control
     * @return an observable value tracking the visible state of the given
     *         control
     */
    public static ISWTObservableValue observeVisible(Control control) {
        return new ControlObservableValue(control, SWTProperties.VISIBLE);
    }

    /**
     * @param control
     * @return an observable value tracking the tooltip text of the given
     *         control
     */
    public static ISWTObservableValue observeTooltipText(Control control) {
        return new ControlObservableValue(control, SWTProperties.TOOLTIP_TEXT);
    }

    /**
     * Returns an observable observing the selection attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Spinner</li>
     * <li>dwt.widgets.Button</li>
     * <li>dwt.widgets.Combo</li>
     * <li>dwt.custom.CCombo</li>
     * <li>dwt.widgets.List</li>
     * <li>dwt.widgets.Scale</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeSelection(Control control) {
        if (control instanceof Spinner) {
            return new SpinnerObservableValue((Spinner) control,
                    SWTProperties.SELECTION);
        } else if (control instanceof Button) {
            return new ButtonObservableValue((Button) control);
        } else if (control instanceof Combo) {
            return new ComboObservableValue((Combo) control,
                    SWTProperties.SELECTION);
        } else if (control instanceof CCombo) {
            return new CComboObservableValue((CCombo) control,
                    SWTProperties.SELECTION);
        } else if (control instanceof List) {
            return new ListObservableValue((List) control);
        } else if (control instanceof Scale) {
            return new ScaleObservableValue((Scale) control,
                    SWTProperties.SELECTION);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the minimum attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Spinner</li>
     * <li>dwt.widgets.Scale</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeMin(Control control) {
        if (control instanceof Spinner) {
            return new SpinnerObservableValue((Spinner) control,
                    SWTProperties.MIN);
        } else if (control instanceof Scale) {
            return new ScaleObservableValue((Scale) control, SWTProperties.MIN);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the maximum attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Spinner</li>
     * <li>dwt.widgets.Scale</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeMax(Control control) {
        if (control instanceof Spinner) {
            return new SpinnerObservableValue((Spinner) control,
                    SWTProperties.MAX);
        } else if (control instanceof Scale) {
            return new ScaleObservableValue((Scale) control, SWTProperties.MAX);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the text attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Text</li>
     * </ul>
     * 
     * <li>dwt.widgets.Label</li>
     * @param control
     * @param event event type to register for change events
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeText(Control control, int event) {
        if (control instanceof Text) {
            return new TextObservableValue((Text) control, event);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the text attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Label</li>
     * <li>dwt.widgets.Link (as of 1.2)</li>
     * <li>dwt.custom.Label</li>
     * <li>dwt.widgets.Combo</li>
     * <li>dwt.custom.CCombo</li>
     * <li>dwt.widgets.Shell</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeText(Control control) {
        if (control instanceof Label) {
            return new LabelObservableValue((Label) control);
        } else if (control instanceof Link) {
            return new LinkObservableValue((Link) control);
        } else if (control instanceof CLabel) {
            return new CLabelObservableValue((CLabel) control);
        } else if (control instanceof Combo) {
            return new ComboObservableValue((Combo) control, SWTProperties.TEXT);
        } else if (control instanceof CCombo) {
            return new CComboObservableValue((CCombo) control,
                    SWTProperties.TEXT);
        } else if (control instanceof Shell) {
            return new ShellObservableValue((Shell) control);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the items attribute of the provided
     * <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Combo</li>
     * <li>dwt.custom.CCombo</li>
     * <li>dwt.widgets.List</li>
     * </ul>
     * 
     * @param control
     * @return observable list
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static IObservableList observeItems(Control control) {
        if (control instanceof Combo) {
            return new ComboObservableList((Combo) control);
        } else if (control instanceof CCombo) {
            return new CComboObservableList((CCombo) control);
        } else if (control instanceof List) {
            return new ListObservableList((List) control);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * Returns an observable observing the single selection index attribute of
     * the provided <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Table</li>
     * <li>dwt.widgets.Combo</li>
     * <li>dwt.custom.CCombo</li>
     * <li>dwt.widgets.List</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeSingleSelectionIndex(
            Control control) {
        if (control instanceof Table) {
            return new TableSingleSelectionObservableValue((Table) control);
        } else if (control instanceof Combo) {
            return new ComboSingleSelectionObservableValue((Combo) control);
        } else if (control instanceof CCombo) {
            return new CComboSingleSelectionObservableValue((CCombo) control);
        } else if (control instanceof List) {
            return new ListSingleSelectionObservableValue((List) control);
        }

        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    /**
     * @param control
     * @return an observable value tracking the foreground color of the given
     *         control
     */
    public static ISWTObservableValue observeForeground(Control control) {
        return new ControlObservableValue(control, SWTProperties.FOREGROUND);
    }

    /**
     * @param control
     * @return an observable value tracking the background color of the given
     *         control
     */
    public static ISWTObservableValue observeBackground(Control control) {
        return new ControlObservableValue(control, SWTProperties.BACKGROUND);
    }

    /**
     * @param control
     * @return an observable value tracking the font of the given control
     */
    public static ISWTObservableValue observeFont(Control control) {
        return new ControlObservableValue(control, SWTProperties.FONT);
    }
    
    /**
     * Returns an observable observing the editable attribute of
     * the provided <code>control</code>. The supported types are:
     * <ul>
     * <li>dwt.widgets.Text</li>
     * </ul>
     * 
     * @param control
     * @return observable value
     * @throws IllegalArgumentException
     *             if <code>control</code> type is unsupported
     */
    public static ISWTObservableValue observeEditable(Control control) {
        if (control instanceof Text) {
            return new TextEditableObservableValue((Text) control);
        }
        
        throw new IllegalArgumentException(
                "Widget [" + control.getClass().getName() + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
    }

    private static class DisplayRealm extends Realm {
        private Display display;

        /**
         * @param display
         */
        private DisplayRealm(Display display) {
            this.display = display;
        }

        public bool isCurrent() {
            return Display.getCurrent() is display;
        }

        public void asyncExec(final Runnable runnable) {
            Runnable safeRunnable = new Runnable() {
                public void run() {
                    safeRun(runnable);
                }
            };
            if (!display.isDisposed()) {
                display.asyncExec(safeRunnable);
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#hashCode()
         */
        public int hashCode() {
            return (display is null) ? 0 : display.hashCode();
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#equals(java.lang.Object)
         */
        public bool equals(Object obj) {
            if (this is obj)
                return true;
            if (obj is null)
                return false;
            if (getClass() !is obj.getClass())
                return false;
            final DisplayRealm other = (DisplayRealm) obj;
            if (display is null) {
                if (other.display !is null)
                    return false;
            } else if (!display.equals(other.display))
                return false;
            return true;
        }
    }
}
