/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Brad Reynolds (bug 135446)
 *     Brad Reynolds - bug 164653
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.jface.internal.databinding.swt.TextObservableValue;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.VerifyEvent;
import dwt.events.VerifyListener;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.Text;
import dwtx.core.databinding.observable.Diffs;
import dwtx.core.databinding.observable.IObservable;
import dwtx.core.databinding.observable.Realm;
import dwtx.jface.databinding.swt.SWTObservables;
import dwtx.jface.internal.databinding.provisional.swt.AbstractSWTVetoableValue;

/**
 * {@link IObservable} implementation that wraps a {@link Text} widget. The time
 * at which listeners should be notified about changes to the text is specified
 * on construction.
 * 
 * <dl>
 * <dt>Events:</dt>
 * <dd> If the update event type (specified on construction) is
 * <code>DWT.Modify</code> a value change event will be fired on every key
 * stroke. If the update event type is <code>DWT.FocusOut</code> a value
 * change event will be fired on focus out. When in either mode if the user is
 * entering text and presses [Escape] the value will be reverted back to the
 * last value set using doSetValue(). Regardless of the update event type a
 * value changing event will fire on verify to enable vetoing of changes.</dd>
 * </dl>
 * 
 * @since 1.0
 */
public class TextObservableValue extends AbstractSWTVetoableValue {

    /**
     * {@link Text} widget that this is being observed.
     */
    private final Text text;

    /**
     * Flag to track when the model is updating the widget. When
     * <code>true</code> the handlers for the DWT events should not process
     * the event as this would cause an infinite loop.
     */
    private bool updating = false;

    /**
     * DWT event that on firing this observable will fire change events to its
     * listeners.
     */
    private final int updateEventType;

    /**
     * Valid types for the {@link #updateEventType}.
     */
    private static final int[] validUpdateEventTypes = new int[] { DWT.Modify,
            DWT.FocusOut, DWT.None };

    /**
     * Previous value of the Text.
     */
    private String oldValue;

    private Listener updateListener = new Listener() {
        public void handleEvent(Event event) {
            if (!updating) {
                String newValue = text.getText();

                if (!newValue.equals(oldValue)) {
                    fireValueChange(Diffs.createValueDiff(oldValue, newValue));                 
                    oldValue = newValue;
                }
            }
        }
    };

    private VerifyListener verifyListener;

    /**
     * Constructs a new instance bound to the given <code>text</code> widget
     * and configured to fire change events to its listeners at the time of the
     * <code>updateEventType</code>.
     * 
     * @param text
     * @param updateEventType
     *            DWT event constant as to what DWT event to update the model in
     *            response to. Appropriate values are: <code>DWT.Modify</code>,
     *            <code>DWT.FocusOut</code>, <code>DWT.None</code>.
     * @throws IllegalArgumentException
     *             if <code>updateEventType</code> is an incorrect type.
     */
    public TextObservableValue(final Text text, int updateEventType) {
        this(SWTObservables.getRealm(text.getDisplay()), text, updateEventType);
    }
    
    /**
     * Constructs a new instance.
     * 
     * @param realm can not be <code>null</code>
     * @param text
     * @param updateEventType
     */
    public TextObservableValue(final Realm realm, Text text, int updateEventType) {
        super(realm, text);
        
        bool eventValid = false;
        for (int i = 0; !eventValid && i < validUpdateEventTypes.length; i++) {
            eventValid = (updateEventType is validUpdateEventTypes[i]);
        }
        if (!eventValid) {
            throw new IllegalArgumentException(
                    "UpdateEventType [" + updateEventType + "] is not supported."); //$NON-NLS-1$//$NON-NLS-2$
        }
        this.text = text;
        this.updateEventType = updateEventType;
        if (updateEventType !is DWT.None) {
            text.addListener(updateEventType, updateListener);
        }
        
        oldValue = text.getText();
        
        verifyListener = new VerifyListener() {
            public void verifyText(VerifyEvent e) {
                if (!updating) {
                    String currentText = TextObservableValue.this.text
                            .getText();
                    String newText = currentText.substring(0, e.start) + e.text
                            + currentText.substring(e.end);
                    if (!fireValueChanging(Diffs.createValueDiff(currentText,
                            newText))) {
                        e.doit = false;
                    }
                }
            }
        };
        text.addVerifyListener(verifyListener);
    }

    /**
     * Sets the bound {@link Text Text's} text to the passed <code>value</code>.
     * 
     * @param value
     *            new value, String expected
     * @see dwtx.core.databinding.observable.value.AbstractVetoableValue#doSetApprovedValue(java.lang.Object)
     * @throws ClassCastException
     *             if the value is anything other than a String
     */
    protected void doSetApprovedValue(final Object value) {
        try {
            updating = true;
            text.setText(value is null ? "" : value.toString()); //$NON-NLS-1$
            oldValue = text.getText();
        } finally {
            updating = false;
        }
    }

    /**
     * Returns the current value of the {@link Text}.
     * 
     * @see dwtx.core.databinding.observable.value.AbstractVetoableValue#doGetValue()
     */
    public Object doGetValue() {
        return oldValue = text.getText();
    }

    /**
     * Returns the type of the value from {@link #doGetValue()}, i.e.
     * String.class
     * 
     * @see dwtx.core.databinding.observable.value.IObservableValue#getValueType()
     */
    public Object getValueType() {
        return String.class;
    }

    public void dispose() {
        if (!text.isDisposed()) {
            if (updateEventType !is DWT.None) {
                text.removeListener(updateEventType, updateListener);
            }
            text.removeVerifyListener(verifyListener);
        }
        super.dispose();
    }
}
