/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.osgi.framework.internal.core.BundleLoaderProxy;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleException;
import org.osgi.framework.FrameworkEvent;
import org.osgi.framework.Version;
import org.osgi.service.packageadmin.RequiredBundle;

import dwtx.osgi.framework.util.KeyedHashSet;
import dwtx.osgi.service.resolver.BundleDescription;
import dwtx.osgi.service.resolver.ExportPackageDescription;

/*
 * The BundleLoaderProxy proxies a BundleLoader object for a Bundle.  This
 * allows for a Bundle's depedencies to be linked without forcing the 
 * creating of the BundleLoader or BundleClassLoader objects.  This class
 * keeps track of the depedencies between the bundles installed in the 
 * Framework.
 */
public class BundleLoaderProxy implements RequiredBundle {
    // The BundleLoader that this BundleLoaderProxy is managing
    private BundleLoader loader;
    // The Bundle that this BundleLoaderProxy is for
    final private BundleHost bundle;
    // the BundleDescription for the Bundle
    final private BundleDescription description;
    // Indicates if this BundleLoaderProxy is stale; 
    // this is true when the bundle is updated or uninstalled.
    private bool stale = false;
    // cached of package sources for the bundle
    final private KeyedHashSet pkgSources;

    public BundleLoaderProxy(BundleHost bundle, BundleDescription description) {
        this.bundle = bundle;
        this.description = description;
        this.pkgSources = new KeyedHashSet(false);
    }

    synchronized BundleLoader getBundleLoader() {
        if (loader !is null)
            return loader;
        if (bundle.isResolved()) {
            try {
                if (bundle.getBundleId() is 0) // this is the system bundle
                    loader = new SystemBundleLoader(bundle, this);
                else
                    loader = new BundleLoader(bundle, this);
            } catch (BundleException e) {
                bundle.framework.publishFrameworkEvent(FrameworkEvent.ERROR, bundle, e);
                return null;
            }
        }
        return loader;
    }

    BundleLoader getBasicBundleLoader() {
        return loader;
    }

    AbstractBundle getBundleHost() {
        return bundle;
    }

    void setStale() {
        stale = true;
    }

    bool isStale() {
        return stale;
    }

    public String toString() {
        String symbolicName = bundle.getSymbolicName();
        StringBuffer sb = new StringBuffer(symbolicName is null ? bundle.getBundleData().getLocation() : symbolicName);
        sb.append("; ").append(Constants.BUNDLE_VERSION_ATTRIBUTE); //$NON-NLS-1$
        sb.append("=\"").append(description.getVersion().toString()).append("\""); //$NON-NLS-1$//$NON-NLS-2$
        return sb.toString();
    }

    public org.osgi.framework.Bundle getBundle() {
        if (isStale())
            return null;

        return bundle;
    }

    public org.osgi.framework.Bundle[] getRequiringBundles() {
        if (isStale())
            return null;
        // This is VERY slow; but never gets called in regular execution.
        BundleDescription[] dependents = description.getDependents();
        if (dependents is null || dependents.length is 0)
            return new Bundle[0];
        ArrayList result = new ArrayList(dependents.length);
        for (int i = 0; i < dependents.length; i++)
            addRequirers(dependents[i], result);
        return (Bundle[]) result.toArray(new org.osgi.framework.Bundle[result.size()]);
    }

    void addRequirers(BundleDescription dependent, ArrayList result) {
        if (dependent.getHost() !is null) // don't look in fragments.
            return;
        BundleLoaderProxy dependentProxy = getBundleLoader().getLoaderProxy(dependent);
        if (dependentProxy is null)
            return; // bundle must have been uninstalled
        if (result.contains(dependentProxy.bundle))
            return; // prevent endless recusion
        BundleLoader dependentLoader = dependentProxy.getBundleLoader();
        BundleLoaderProxy[] requiredBundles = dependentLoader.requiredBundles;
        int[] reexportTable = dependentLoader.reexportTable;
        if (requiredBundles is null)
            return;
        int size = reexportTable is null ? 0 : reexportTable.length;
        int reexportIndex = 0;
        for (int i = 0; i < requiredBundles.length; i++) {
            if (requiredBundles[i] is this) {
                result.add(dependentProxy.bundle);
                if (reexportIndex < size && reexportTable[reexportIndex] is i) {
                    reexportIndex++;
                    BundleDescription[] dependents = dependent.getDependents();
                    if (dependents is null)
                        return;
                    for (int j = 0; j < dependents.length; j++)
                        dependentProxy.addRequirers(dependents[j], result);
                }
                return;
            }
        }
        return;
    }

    public String getSymbolicName() {
        return description.getSymbolicName();
    }

    public Version getVersion() {
        return description.getVersion();
    }

    public bool isRemovalPending() {
        return description.isRemovalPending();
    }

    BundleDescription getBundleDescription() {
        return description;
    }

    PackageSource getPackageSource(String pkgName) {
        // getByKey is called outside of a synch block because we really do not
        // care too much of duplicates getting created.  Only the first one will
        // successfully get stored into pkgSources
        PackageSource pkgSource = (PackageSource) pkgSources.getByKey(pkgName);
        if (pkgSource is null) {
            pkgSource = new SingleSourcePackage(pkgName, this);
            synchronized (pkgSources) {
                pkgSources.add(pkgSource);
            }
        }
        return pkgSource;
    }

    bool inUse() {
        return description.getDependents().length > 0;
    }

    bool forceSourceCreation(ExportPackageDescription export) {
        bool strict = Constants.STRICT_MODE.equals(bundle.framework.adaptor.getState().getPlatformProperties()[0].get(Constants.OSGI_RESOLVER_MODE));
        return (export.getDirective(Constants.INCLUDE_DIRECTIVE) !is null) || (export.getDirective(Constants.EXCLUDE_DIRECTIVE) !is null) || (strict && export.getDirective(Constants.FRIENDS_DIRECTIVE) !is null);
    }

    // creates a PackageSource from an ExportPackageDescription.  This is called when initializing
    // a BundleLoader to ensure that the proper PackageSource gets created and used for
    // filtered and reexport packages.  The storeSource flag is used by initialize to indicate
    // that the source for special case package sources (filtered or re-exported should be stored 
    // in the cache.  if this flag is set then a normal SinglePackageSource will not be created
    // (i.e. it will be created lazily)
    PackageSource createPackageSource(ExportPackageDescription export, bool storeSource) {
        PackageSource pkgSource = null;

        // check to see if it is a filtered export
        String includes = (String) export.getDirective(Constants.INCLUDE_DIRECTIVE);
        String excludes = (String) export.getDirective(Constants.EXCLUDE_DIRECTIVE);
        String[] friends = (String[]) export.getDirective(Constants.FRIENDS_DIRECTIVE);
        if (friends !is null) {
            bool strict = Constants.STRICT_MODE.equals(bundle.framework.adaptor.getState().getPlatformProperties()[0].get(Constants.OSGI_RESOLVER_MODE));
            if (!strict)
                friends = null; // do not pay attention to friends if not in strict mode
        }
        if (includes !is null || excludes !is null || friends !is null) {
            pkgSource = new FilteredSourcePackage(export.getName(), this, includes, excludes, friends);
        }

        if (storeSource) {
            // if the package source is not null then store the source only if it is not already present;
            // getByKey is called outside of a synch block because we really do not
            // care too much of duplicates getting created.  Only the first one will
            // successfully get stored into pkgSources
            if (pkgSource !is null && pkgSources.getByKey(export.getName()) is null)
                synchronized (pkgSources) {
                    pkgSources.add(pkgSource);
                }
        } else {
            // we are not storing the special case sources, but pkgSource is null this means this
            // is a normal package source; get it and return it.
            if (pkgSource is null) {
                pkgSource = getPackageSource(export.getName());
                // the first export cached may not be a simple single source like we need.
                if (pkgSource.getClass() !is SingleSourcePackage.class)
                    return new SingleSourcePackage(export.getName(), this);
            }
        }

        return pkgSource;
    }
}
