/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.framework.internal.core.BundleResourceHandler;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;

import org.osgi.framework.AdminPermission;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import dwtx.osgi.baseadaptor.BaseAdaptor;
import dwtx.osgi.baseadaptor.bundlefile.BundleEntry;
import dwtx.osgi.baseadaptor.loader.BaseClassLoader;
import dwtx.osgi.framework.adaptor.FrameworkAdaptor;
import dwtx.osgi.framework.internal.protocol.ProtocolActivator;
import dwtx.osgi.internal.baseadaptor.AdaptorMsg;
import dwtx.osgi.util.NLS;

/**
 * URLStreamHandler the bundleentry and bundleresource protocols.
 */

public abstract class BundleResourceHandler extends URLStreamHandler implements ProtocolActivator {
    public static final String SECURITY_CHECKED = "SECURITY_CHECKED"; //$NON-NLS-1$
    public static final String SECURITY_UNCHECKED = "SECURITY_UNCHECKED"; //$NON-NLS-1$
    private BaseAdaptor adaptor;
    protected BundleEntry bundleEntry;

    /**
     * Constructor for a bundle protocol resource URLStreamHandler.
     */
    public BundleResourceHandler() {
        this(null, null);
    }

    public BundleResourceHandler(BundleEntry bundleEntry, BaseAdaptor adaptor) {
        this.bundleEntry = bundleEntry;
        this.adaptor = adaptor;
    }

    public void start(BundleContext context, FrameworkAdaptor baseAdaptor) {
        this.adaptor = (BaseAdaptor) baseAdaptor;
    }

    /** 
     * Parse reference URL. 
     */
    protected void parseURL(URL url, String str, int start, int end) {
        if (end < start)
            return;
        if (url.getPath() !is null)
            // A call to a URL constructor has been made that uses an authorized URL as its context.
            // Null out bundleEntry because it will not be valid for the new path
            bundleEntry = null;
        String spec = ""; //$NON-NLS-1$
        if (start < end)
            spec = str.substring(start, end);
        end -= start;
        //Default is to use path and bundleId from context
        String path = url.getPath();
        String bundleId = url.getHost();
        int resIndex = 0; // must start at 0 index if using a context
        int pathIdx = 0;
        if (spec.startsWith("//")) { //$NON-NLS-1$
            int bundleIdIdx = 2;
            pathIdx = spec.indexOf('/', bundleIdIdx);
            if (pathIdx is -1) {
                pathIdx = end;
                // Use default
                path = ""; //$NON-NLS-1$
            }
            int bundleIdEnd = spec.indexOf(':', bundleIdIdx);
            if (bundleIdEnd > pathIdx || bundleIdEnd is -1)
                bundleIdEnd = pathIdx;
            if (bundleIdEnd < pathIdx - 1)
                try {
                    resIndex = Integer.parseInt(spec.substring(bundleIdEnd + 1, pathIdx));
                } catch (NumberFormatException e) {
                    // do nothing; results in resIndex is 0
                }
            bundleId = spec.substring(bundleIdIdx, bundleIdEnd);
        }
        if (pathIdx < end && spec.charAt(pathIdx) is '/')
            path = spec.substring(pathIdx, end);
        else if (end > pathIdx) {
            if (path is null || path.equals("")) //$NON-NLS-1$
                path = "/"; //$NON-NLS-1$
            int last = path.lastIndexOf('/') + 1;
            if (last is 0)
                path = spec.substring(pathIdx, end);
            else
                path = path.substring(0, last) + spec.substring(pathIdx, end);
        }
        if (path is null)
            path = ""; //$NON-NLS-1$
        //modify path if there's any relative references
        // see RFC2396 Section 5.2
        // Note: For ".." references above the root the approach taken is removing them from the resolved path
        if (path.endsWith("/.") || path.endsWith("/..")) //$NON-NLS-1$ //$NON-NLS-2$
            path = path + '/';
        int dotIndex;
        while ((dotIndex = path.indexOf("/./")) >= 0) //$NON-NLS-1$
            path = path.substring(0, dotIndex + 1) + path.substring(dotIndex + 3);
        while ((dotIndex = path.indexOf("/../")) >= 0) { //$NON-NLS-1$
            if (dotIndex !is 0)
                path = path.substring(0, path.lastIndexOf('/', dotIndex - 1)) + path.substring(dotIndex + 3);
            else
                path = path.substring(dotIndex + 3);
        }
        while ((dotIndex = path.indexOf("//")) >= 0) //$NON-NLS-1$
            path = path.substring(0, dotIndex + 1) + path.substring(dotIndex + 2);

        // Check the permission of the caller to see if they
        // are allowed access to the resource.
        String authorized = SECURITY_UNCHECKED;
        Bundle bundle = adaptor is null ? null : adaptor.getContext().getBundle(Long.parseLong(bundleId));
        if (checkAuthorization(bundle))
            authorized = SECURITY_CHECKED;

        // Setting the authority portion of the URL to SECURITY_ATHORIZED
        // ensures that this URL was created by using this parseURL
        // method.  The openConnection method will only open URLs
        // that have the authority set to this.
        setURL(url, url.getProtocol(), bundleId, resIndex, authorized, null, path, null, url.getRef());
    }

    /**
     * Establishes a connection to the resource specified by <code>URL</code>.
     * Since different protocols may have unique ways of connecting, it must be
     * overridden by the subclass.
     *
     * @return java.net.URLConnection
     * @param url java.net.URL
     *
     * @exception   IOException     thrown if an IO error occurs during connection establishment
     */
    protected URLConnection openConnection(URL url) throws IOException {
        if (bundleEntry !is null) // if the bundleEntry is not null then return quick
            return (new BundleURLConnection(url, bundleEntry));

        String bidString = url.getHost();
        if (bidString is null) {
            throw new IOException(NLS.bind(AdaptorMsg.URL_NO_BUNDLE_ID, url.toExternalForm()));
        }
        AbstractBundle bundle = null;
        long bundleID;
        try {
            bundleID = Long.parseLong(bidString);
        } catch (NumberFormatException nfe) {
            throw new MalformedURLException(NLS.bind(AdaptorMsg.URL_INVALID_BUNDLE_ID, bidString));
        }
        bundle = adaptor is null ? null : (AbstractBundle) adaptor.getContext().getBundle(bundleID);
        if (bundle is null)
            throw new IOException(NLS.bind(AdaptorMsg.URL_NO_BUNDLE_FOUND, url.toExternalForm()));
        // check to make sure that this URL was created using the
        // parseURL method.  This ensures the security check was done
        // at URL construction.
        if (!url.getAuthority().equals(SECURITY_CHECKED)) {
            // No admin security check was made better check now.
            checkAuthorization(bundle);
        }
        return (new BundleURLConnection(url, findBundleEntry(url, bundle)));
    }

    /**
     * Finds the bundle entry for this protocal.  This is handled
     * differently for Bundle.gerResource() and Bundle.getEntry()
     * because getResource uses the bundle classloader and getEntry
     * only used the base bundle file.
     * @param url The URL to find the BundleEntry for.
     * @return the bundle entry
     */
    abstract protected BundleEntry findBundleEntry(URL url, AbstractBundle bundle) throws IOException;

    /**
     * Converts a bundle URL to a String.
     *
     * @param   url   the URL.
     * @return  a string representation of the URL.
     */
    protected String toExternalForm(URL url) {
        StringBuffer result = new StringBuffer(url.getProtocol());
        result.append("://"); //$NON-NLS-1$

        String bundleId = url.getHost();
        if ((bundleId !is null) && (bundleId.length() > 0))
            result.append(bundleId);
        int index = url.getPort();
        if (index > 0)
            result.append(':').append(index);

        String path = url.getPath();
        if (path !is null) {
            if ((path.length() > 0) && (path.charAt(0) !is '/')) /* if name doesn't have a leading slash */
            {
                result.append("/"); //$NON-NLS-1$
            }

            result.append(path);
        }
        String ref = url.getRef();
        if (ref !is null && ref.length() > 0)
            result.append('#').append(ref);

        return (result.toString());
    }

    protected int hashCode(URL url) {
        int hash = 0;
        String protocol = url.getProtocol();
        if (protocol !is null)
            hash += protocol.hashCode();

        String host = url.getHost();
        if (host !is null)
            hash += host.hashCode();

        hash += url.getPort();

        String path = url.getPath();
        if (path !is null)
            hash += path.hashCode();

        if (adaptor !is null)
            hash += adaptor.hashCode();
        return hash;
    }

    protected bool equals(URL url1, URL url2) {
        return sameFile(url1, url2);
    }

    protected synchronized InetAddress getHostAddress(URL url) {
        return null;
    }

    protected bool hostsEqual(URL url1, URL url2) {
        String host1 = url1.getHost();
        String host2 = url2.getHost();
        if (host1 !is null && host2 !is null)
            return host1.equalsIgnoreCase(host2);
        return (host1 is null && host2 is null);
    }

    protected bool sameFile(URL url1, URL url2) {
        // do a hashcode test to allow each handler to check the adaptor first
        if (url1.hashCode() !is url2.hashCode())
            return false;
        String p1 = url1.getProtocol();
        String p2 = url2.getProtocol();
        if (!((p1 is p2) || (p1 !is null && p1.equalsIgnoreCase(p2))))
            return false;

        if (!hostsEqual(url1, url2))
            return false;

        if (url1.getPort() !is url2.getPort())
            return false;

        String path1 = url1.getPath();
        String path2 = url2.getPath();
        if (!((path1 is path2) || (path1 !is null && path1.equals(path2))))
            return false;

        return true;
        // note that the authority is not checked here because it can be different for two
        // URLs depending on how they were constructed.
    }

    protected bool checkAuthorization(Bundle bundle) {
        SecurityManager sm = System.getSecurityManager();
        if (sm is null)
            return true;
        if (bundle is null)
            return false;
        sm.checkPermission(new AdminPermission(bundle, AdminPermission.RESOURCE));
        return true;
    }

    protected static BaseClassLoader getBundleClassLoader(AbstractBundle bundle) {
        BundleLoader loader = bundle.getBundleLoader();
        if (loader is null)
            return null;
        return (BaseClassLoader) loader.createClassLoader();
    }
}
