/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.framework.internal.core.DependentPolicy;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import dwtx.osgi.service.resolver.BundleDescription;

/**
 * DependentPolicy is an implementation of a buddy policy. 
 * It is responsible for looking up a class in the dependents of the bundle
 * to which this policy is attached to.
 */
public class DependentPolicy implements IBuddyPolicy {
    BundleLoader buddyRequester;
    int lastDependentOfAdded = -1; //remember the index of the bundle for which we last added the dependent
    List allDependents = null; //the list of all dependents known so far

    public DependentPolicy(BundleLoader requester) {
        buddyRequester = requester;

        //Initialize with the first level of dependent the list
        allDependents = new ArrayList();
        basicAddImmediateDependents(buddyRequester.getBundle().getBundleDescription());
        //If there is no dependent, reset to null
        if (allDependents.size() is 0)
            allDependents = null;
    }

    public Class loadClass(String name) {
        if (allDependents is null)
            return null;

        Class result = null;
        //size may change, so we must check it every time
        for (int i = 0; i < allDependents.size() && result is null; i++) {
            BundleDescription searchedBundle = (BundleDescription) allDependents.get(i);
            try {
                BundleLoaderProxy proxy = buddyRequester.getLoaderProxy(searchedBundle);
                if (proxy is null)
                    continue;
                result = proxy.getBundleLoader().findClass(name, true);
            } catch (ClassNotFoundException e) {
                if (result is null)
                    addDependent(i, searchedBundle);
            }
        }
        return result;
    }

    private synchronized void addDependent(int i, BundleDescription searchedBundle) {
        if (i > lastDependentOfAdded) {
            lastDependentOfAdded = i;
            basicAddImmediateDependents(searchedBundle);
        }
    }

    public URL loadResource(String name) {
        if (allDependents is null)
            return null;

        URL result = null;
        //size may change, so we must check it every time
        for (int i = 0; i < allDependents.size() && result is null; i++) {
            BundleDescription searchedBundle = (BundleDescription) allDependents.get(i);
            BundleLoaderProxy proxy = buddyRequester.getLoaderProxy(searchedBundle);
            if (proxy is null)
                continue;
            result = proxy.getBundleLoader().findResource(name, true);
            if (result is null) {
                addDependent(i, searchedBundle);
            }
        }
        return result;
    }

    public Enumeration loadResources(String name) {
        if (allDependents is null)
            return null;

        Enumeration results = null;
        //size may change, so we must check it every time
        for (int i = 0; i < allDependents.size(); i++) {
            BundleDescription searchedBundle = (BundleDescription) allDependents.get(i);
            try {
                BundleLoaderProxy proxy = buddyRequester.getLoaderProxy(searchedBundle);
                if (proxy is null)
                    continue;
                results = BundleLoader.compoundEnumerations(results, proxy.getBundleLoader().findResources(name));
                addDependent(i, searchedBundle);
            } catch (IOException e) {
                //Ignore and keep looking
            }
        }
        return results;
    }

    private void basicAddImmediateDependents(BundleDescription root) {
        BundleDescription[] dependents = root.getDependents();
        for (int i = 0; i < dependents.length; i++) {
            BundleDescription toAdd = dependents[i];
            if (toAdd.getHost() is null && !allDependents.contains(toAdd)) {
                allDependents.add(toAdd);
            }
        }
    }
}
