/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.osgi.internal.baseadaptor.BaseClassLoadingHook;

import dwt.dwthelper.utils;

import java.io.File;
import java.security.ProtectionDomain;
import java.util.ArrayList;

import dwtx.osgi.baseadaptor.BaseData;
import dwtx.osgi.baseadaptor.bundlefile.BundleEntry;
import dwtx.osgi.baseadaptor.hooks.ClassLoadingHook;
import dwtx.osgi.baseadaptor.loader.BaseClassLoader;
import dwtx.osgi.baseadaptor.loader.ClasspathEntry;
import dwtx.osgi.baseadaptor.loader.ClasspathManager;
import dwtx.osgi.framework.adaptor.BundleProtectionDomain;
import dwtx.osgi.framework.adaptor.ClassLoaderDelegate;
import dwtx.osgi.framework.debug.Debug;
import dwtx.osgi.framework.internal.core.FrameworkProperties;
import dwtx.osgi.util.ManifestElement;

public class BaseClassLoadingHook implements ClassLoadingHook {
    private static final String[] LIB_EXTENSIONS = ManifestElement.getArrayFromList(FrameworkProperties.getProperty("osgi.framework.library.extensions"), ","); //$NON-NLS-1$ //$NON-NLS-2$
    private static final String[] EMPTY_STRINGS = new String[0];

    /*
     * Maps an already mapped library name to additional library file extensions.
     * This is needed on platforms like AIX where .a and .so can be used as library file
     * extensions, but System.mapLibraryName only returns a single string.
     */
    public static String[] mapLibraryNames(String mappedLibName) {
        int extIndex = mappedLibName.lastIndexOf('.');
        if (LIB_EXTENSIONS.length is 0 || extIndex < 0)
            return EMPTY_STRINGS;
        String libNameBase = mappedLibName.substring(0, extIndex);
        String[] results = new String[LIB_EXTENSIONS.length];
        for (int i = 0; i < results.length; i++)
            results[i] = libNameBase + LIB_EXTENSIONS[i];
        return results;
    }

    public String findLibrary(BaseData data, String libName) {
        String mappedName = System.mapLibraryName(libName);
        String path = null;
        if (Debug.DEBUG && Debug.DEBUG_LOADER)
            Debug.println("  mapped library name: " + mappedName); //$NON-NLS-1$
        path = findNativePath(data, mappedName);
        if (path is null) {
            String[] mappedNames = mapLibraryNames(mappedName);
            for (int i = 0; i < mappedNames.length && path is null; i++)
                path = findNativePath(data, mappedNames[i]);
        }
        if (path is null) {
            if (Debug.DEBUG && Debug.DEBUG_LOADER)
                Debug.println("  library does not exist: " + mappedName); //$NON-NLS-1$
            path = findNativePath(data, libName);
        }
        if (Debug.DEBUG && Debug.DEBUG_LOADER)
            Debug.println("  returning library: " + path); //$NON-NLS-1$
        return path;
    }

    private String findNativePath(BaseData bundledata, String libname) {
        int slash = libname.lastIndexOf('/');
        if (slash >= 0)
            libname = libname.substring(slash + 1);
        String[] nativepaths = getNativePaths(bundledata);
        if (nativepaths is null)
            return null;
        for (int i = 0; i < nativepaths.length; i++) {
            slash = nativepaths[i].lastIndexOf('/');
            String path = slash < 0 ? nativepaths[i] : nativepaths[i].substring(slash + 1);
            if (path.equals(libname)) {
                if (nativepaths[i].startsWith(BaseStorageHook.EXTERNAL_LIB_PREFIX)) {
                    // references an external library; do variable substitution
                    String externalPath = BaseStorageHook.substituteVars(nativepaths[i].substring(BaseStorageHook.EXTERNAL_LIB_PREFIX.length()));
                    File nativeFile = new File(externalPath);
                    return nativeFile.getAbsolutePath();
                }
                // this is a normal library contained within the bundle
                File nativeFile = bundledata.getBundleFile().getFile(nativepaths[i], true);
                if (nativeFile !is null)
                    return nativeFile.getAbsolutePath();
            }
        }
        return null;
    }

    private String[] getNativePaths(BaseData bundledata) {
        BaseStorageHook storageHook = (BaseStorageHook) bundledata.getStorageHook(BaseStorageHook.KEY);
        return storageHook !is null ? storageHook.getNativePaths() : null;
    }

    public bool addClassPathEntry(ArrayList cpEntries, String cp, ClasspathManager hostmanager, BaseData sourcedata, ProtectionDomain sourcedomain) {
        // do nothing
        return false;
    }

    public ClassLoader getBundleClassLoaderParent() {
        // do nothing
        return null;
    }

    public byte[] processClass(String name, byte[] classbytes, ClasspathEntry classpathEntry, BundleEntry entry, ClasspathManager manager) {
        // do nothing
        return null;
    }

    public BaseClassLoader createClassLoader(ClassLoader parent, ClassLoaderDelegate delegate, BundleProtectionDomain domain, BaseData data, String[] bundleclasspath) {
        // do nothing
        return null;
    }

    public void initializedClassLoader(BaseClassLoader baseClassLoader, BaseData data) {
        // do nothing
    }

}
