/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.actions.CompoundContributionItem;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.jface.action.ContributionItem;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.IContributionManager;
import dwtx.jface.action.IMenuListener;
import dwtx.jface.action.IMenuManager;

/**
 * A compound contribution is a contribution item consisting of a
 * dynamic list of contribution items.
 * 
 * @since 3.1
 */
public abstract class CompoundContributionItem extends ContributionItem {

    private bool dirty = true;

    private IMenuListener menuListener = new IMenuListener() {
        public void menuAboutToShow(IMenuManager manager) {
            manager.markDirty();
            dirty = true;
        }
    };
    
    private IContributionItem[] oldItems;
    
    /**
     * Creates a compound contribution item with a <code>null</code> id.
     */
    protected CompoundContributionItem() {
        super();
    }

    /**
     * Creates a compound contribution item with the given (optional) id.
     *
     * @param id the contribution item identifier, or <code>null</code>
     */
    protected CompoundContributionItem(String id) {
        super(id);
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.ContributionItem#fill(dwt.widgets.Menu, int)
     */
    public void fill(Menu menu, int index) {
        if (index is -1) {
            index = menu.getItemCount();
        }
        if (!dirty && menu.getParentItem() !is null) {
            // insert a dummy item so that the parent item is not disabled
            new MenuItem(menu, DWT.NONE, index);
            return;
        }
        
        IContributionItem[] items = getContributionItemsToFill();
        for (int i = 0; i < items.length; i++) {
            IContributionItem item = items[i];
            int oldItemCount = menu.getItemCount();
            if (item.isVisible()) {
                item.fill(menu, index);
            }
            int newItemCount = menu.getItemCount();
            int numAdded = newItemCount - oldItemCount;
            index += numAdded;
        }
        dirty = false;
    }
    
    /**
     * Return a list of contributions items that will replace this item in the
     * parent manager. The list must contain new contribution items every call
     * since the old ones will be disposed.
     * 
     * @return an array list of items to display. Must not be <code>null</code>.
     */
    protected abstract IContributionItem[] getContributionItems();
    
    private IContributionItem[] getContributionItemsToFill() {
        if (oldItems !is null) {
            for (int i = 0; i < oldItems.length; i++) {
                IContributionItem oldItem = oldItems[i];
                oldItem.dispose();
            }
            oldItems = null;
        }
        oldItems = getContributionItems();
        return oldItems;
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.ContributionItem#isDirty()
     */
    public bool isDirty() {
        return dirty;
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.ContributionItem#isDynamic()
     */
    public bool isDynamic() {
        return true;
    }
    
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.ContributionItem#setParent(dwtx.jface.action.IContributionManager)
     */
    public void setParent(IContributionManager parent) {
        if (getParent() instanceof IMenuManager) {
            IMenuManager menuMgr = (IMenuManager) getParent();
            menuMgr.removeMenuListener(menuListener);
        }
        if (parent instanceof IMenuManager) {
            IMenuManager menuMgr = (IMenuManager) parent;
            menuMgr.addMenuListener(menuListener);
        }
        super.setParent(parent);
    }
}
