/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.actions.ContributedAction;

import dwt.dwthelper.utils;

import java.util.Collections;

import dwt.widgets.Event;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.IHandler;
import dwtx.core.commands.IHandler2;
import dwtx.core.commands.NotEnabledException;
import dwtx.core.commands.NotHandledException;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.core.expressions.EvaluationContext;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.jface.viewers.StructuredSelection;
import dwtx.ui.IEditorSite;
import dwtx.ui.IPartListener;
import dwtx.ui.ISources;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.internal.actions.CommandAction;
import dwtx.ui.internal.handlers.ActionDelegateHandlerProxy;
import dwtx.ui.internal.handlers.HandlerService;
import dwtx.ui.internal.handlers.IActionCommandMappingService;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.part.MultiPageEditorSite;
import dwtx.ui.services.IServiceLocator;

/**
 * For a declarative editor action, see if we can link it to a command.
 * <p>
 * This is a legacy bridge class, and should not be used outside of the Eclipse
 * SDK. Please use menu contributions to display a command in a menu or toolbar.
 * </p>
 * <p>
 * <b>Note:</b> Clients may instantiate.
 * </p>
 * 
 * @since 3.3
 */
public final class ContributedAction extends CommandAction {
    private IEvaluationContext appContext;

    private IHandler partHandler;

    private bool localHandler = false;

    private IPartListener partListener;

    /**
     * Create an action that can call a command.
     * 
     * @param locator
     *            The appropriate service locator to use. If you use a part site
     *            as your locator, this action will be tied to your part.
     * @param element
     *            the contributed action element
     */
    public ContributedAction(IServiceLocator locator,
            IConfigurationElement element) throws CommandNotMappedException {

        String actionId = element
                .getAttribute(IWorkbenchRegistryConstants.ATT_ID);
        String commandId = element
                .getAttribute(IWorkbenchRegistryConstants.ATT_DEFINITION_ID);

        // TODO throw some more exceptions here :-)

        String contributionId = null;
        if (commandId is null) {

            Object obj = element.getParent();
            if (obj instanceof IConfigurationElement) {
                contributionId = ((IConfigurationElement) obj)
                        .getAttribute(IWorkbenchRegistryConstants.ATT_ID);
                if (contributionId is null) {
                    throw new CommandNotMappedException("Action " //$NON-NLS-1$
                            + actionId + " configuration element invalid"); //$NON-NLS-1$
                }
            }
            // legacy bridge part
            IActionCommandMappingService mapping = (IActionCommandMappingService) locator
                    .getService(IActionCommandMappingService.class);
            if (mapping is null) {
                throw new CommandNotMappedException(
                        "No action mapping service available"); //$NON-NLS-1$
            }

            commandId = mapping.getCommandId(mapping.getGeneratedCommandId(
                    contributionId, actionId));
        }
        // what, still no command?
        if (commandId is null) {
            throw new CommandNotMappedException("Action " + actionId //$NON-NLS-1$
                    + " in contribution " + contributionId //$NON-NLS-1$
                    + " not mapped to a command"); //$NON-NLS-1$
        }

        init(locator, commandId, null);

        if (locator instanceof IWorkbenchPartSite) {
            updateSiteAssociations((IWorkbenchPartSite) locator, commandId,
                    actionId, element);
        }

        setId(actionId);
    }

    private void updateSiteAssociations(IWorkbenchPartSite site,
            String commandId, String actionId, IConfigurationElement element) {
        IWorkbenchLocationService wls = (IWorkbenchLocationService) site
                .getService(IWorkbenchLocationService.class);
        IWorkbench workbench = wls.getWorkbench();
        IWorkbenchWindow window = wls.getWorkbenchWindow();
        IHandlerService serv = (IHandlerService) workbench
                .getService(IHandlerService.class);
        appContext = new EvaluationContext(serv.getCurrentState(),
                Collections.EMPTY_LIST);

        // set up the appContext as we would want it.
        appContext.addVariable(ISources.ACTIVE_CURRENT_SELECTION_NAME,
                StructuredSelection.EMPTY);
        appContext.addVariable(ISources.ACTIVE_PART_NAME, site.getPart());
        appContext.addVariable(ISources.ACTIVE_PART_ID_NAME, site.getId());
        appContext.addVariable(ISources.ACTIVE_SITE_NAME, site);
        if (site instanceof IEditorSite) {
            appContext.addVariable(ISources.ACTIVE_EDITOR_NAME, site.getPart());
            appContext
                    .addVariable(ISources.ACTIVE_EDITOR_ID_NAME, site.getId());
        }
        appContext.addVariable(ISources.ACTIVE_WORKBENCH_WINDOW_NAME, window);
        appContext.addVariable(ISources.ACTIVE_WORKBENCH_WINDOW_SHELL_NAME,
                window.getShell());

        HandlerService realService = (HandlerService) serv;
        partHandler = realService.findHandler(commandId, appContext);
        if (partHandler is null) {
            localHandler = true;
            // if we can't find the handler, then at least we can
            // call the action delegate run method
            partHandler = new ActionDelegateHandlerProxy(element,
                    IWorkbenchRegistryConstants.ATT_CLASS, actionId,
                    getParameterizedCommand(), site.getWorkbenchWindow(), null,
                    null, null);
        }
        if (site instanceof MultiPageEditorSite) {
            IHandlerService siteServ = (IHandlerService) site
                    .getService(IHandlerService.class);
            siteServ.activateHandler(commandId, partHandler);
        }

        if (getParameterizedCommand() !is null) {
            getParameterizedCommand().getCommand().removeCommandListener(
                    getCommandListener());
        }
        site.getPage().addPartListener(getPartListener());
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.actions.CommandAction#runWithEvent(dwt.widgets.Event)
     */
    public void runWithEvent(Event event) {
        if (partHandler !is null) {
            IHandler oldHandler = getParameterizedCommand().getCommand()
                    .getHandler();
            try {
                getParameterizedCommand().getCommand().setHandler(partHandler);
                getParameterizedCommand().executeWithChecks(event, appContext);
            } catch (ExecutionException e) {
                // TODO some logging, perhaps?
            } catch (NotDefinedException e) {
                // TODO some logging, perhaps?
            } catch (NotEnabledException e) {
                // TODO some logging, perhaps?
            } catch (NotHandledException e) {
                // TODO some logging, perhaps?
            } finally {
                getParameterizedCommand().getCommand().setHandler(oldHandler);
            }
        } else {
            super.runWithEvent(event);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.Action#isEnabled()
     */
    public bool isEnabled() {
        if (partHandler !is null) {
            if (partHandler instanceof IHandler2) {
                ((IHandler2) partHandler).setEnabled(appContext);
            }
            return partHandler.isEnabled();
        }
        return false;
    }

    private IPartListener getPartListener() {
        if (partListener is null) {
            final IWorkbenchPartSite site = (IWorkbenchPartSite) appContext
                    .getVariable(ISources.ACTIVE_SITE_NAME);

            final IWorkbenchPart currentPart;
            if (site instanceof MultiPageEditorSite) {
                currentPart = ((MultiPageEditorSite) site).getMultiPageEditor();
            } else {
                currentPart = site.getPart();
            }

            partListener = new IPartListener() {
                public void partActivated(IWorkbenchPart part) {
                }

                public void partBroughtToTop(IWorkbenchPart part) {
                }

                public void partClosed(IWorkbenchPart part) {
                    if (part is currentPart) {
                        ContributedAction.this.disposeAction();
                    }
                }

                public void partDeactivated(IWorkbenchPart part) {
                }

                public void partOpened(IWorkbenchPart part) {
                }
            };
        }
        return partListener;
    }

    // TODO make public in 3.4
    private void disposeAction() {
        if (appContext !is null) {
            if (localHandler) {
                partHandler.dispose();
            }
            if (partListener !is null) {
                IWorkbenchPartSite site = (IWorkbenchPartSite) appContext
                        .getVariable(ISources.ACTIVE_SITE_NAME);
                site.getPage().removePartListener(partListener);
                partListener = null;
            }
            appContext = null;
            partHandler = null;
        }
        dispose();
    }
}
