/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.actions.WorkingSetFilterActionGroup;

import dwt.dwthelper.utils;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import dwt.widgets.Shell;
import dwtx.core.runtime.Assert;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.IMenuListener;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.Separator;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.jface.util.PropertyChangeEvent;
import dwtx.ui.IActionBars;
import dwtx.ui.IMemento;
import dwtx.ui.IWorkbenchActionConstants;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.IWorkingSet;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.WorkingSetComparator;
import dwtx.ui.internal.WorkingSetMenuContributionItem;
import dwtx.ui.internal.actions.ClearWorkingSetAction;
import dwtx.ui.internal.actions.EditWorkingSetAction;
import dwtx.ui.internal.actions.SelectWorkingSetAction;
import dwtx.ui.internal.util.Util;

/**
 * Adds working set filter actions (set / clear / edit)
 * 
 * @since 2.1 
 */
public class WorkingSetFilterActionGroup extends ActionGroup {
    private static final String TAG_WORKING_SET_NAME = "workingSetName"; //$NON-NLS-1$

    private static final String TAG_IS_WINDOW_WORKING_SET = "isWindowWorkingSet"; //$NON-NLS-1$
    
    /**
     * Indicates if working set was changed
     */
    public static final String CHANGE_WORKING_SET = "changeWorkingSet"; //$NON-NLS-1$
    
    private static final String START_SEPARATOR_ID = "workingSetGroupStartSeparator"; //$NON-NLS-1$

    private static final String SEPARATOR_ID = "workingSetGroupSeparator"; //$NON-NLS-1$

    private static final String WORKING_SET_ACTION_GROUP = "workingSetActionGroup"; //$NON-NLS-1$

    private IWorkingSet workingSet = null;

    private ClearWorkingSetAction clearWorkingSetAction;

    private SelectWorkingSetAction selectWorkingSetAction;

    private EditWorkingSetAction editWorkingSetAction;

    private IPropertyChangeListener workingSetUpdater;

    private int mruMenuCount;

    private IMenuManager menuManager;

    private IMenuListener menuListener;

    private IWorkbenchWindow workbenchWindow;

    private IWorkbenchPage page;

    private bool allowWindowWorkingSetByDefault;

    /**
     * Creates a new instance of the receiver.
     * 
     * @param shell
     *            shell to open dialogs and wizards on
     * @param workingSetUpdater
     *            property change listener notified when a working set is set
     * @since 3.2 Please note that it is expected that clients treat any
     *        selected working sets whose
     *        {@link IWorkingSet#isAggregateWorkingSet()} method returns
     *        <code>true</code> somewhat differently from traditional working
     *        sets. Please see the documentation for
     *        {@link IWorkbenchPage#getAggregateWorkingSet()} for details.
     */
    public WorkingSetFilterActionGroup(Shell shell,
            IPropertyChangeListener workingSetUpdater) {
        Assert.isNotNull(shell);

        this.workingSetUpdater = workingSetUpdater;
        clearWorkingSetAction = new ClearWorkingSetAction(this);
        selectWorkingSetAction = new SelectWorkingSetAction(this, shell);
        editWorkingSetAction = new EditWorkingSetAction(this, shell);
        
        workbenchWindow = Util.getWorkbenchWindowForShell(shell);
        allowWindowWorkingSetByDefault = false;
        // set the default working set to be that of the window.
        page = workbenchWindow.getActivePage();
        if (page is null) {
            IWorkbenchPage[] pages = workbenchWindow.getPages();
            if (pages.length > 0) {
                page = pages[0];
            }
        }
    }

    /**
     * Adds actions for the most recently used working sets to the specified
     * menu manager.
     * 
     * @param menuManager
     *            menu manager to add actions to
     */
    private void addMruWorkingSetActions(IMenuManager menuManager) {
        IWorkingSet[] workingSets = PlatformUI.getWorkbench()
                .getWorkingSetManager().getRecentWorkingSets();
        List sortedWorkingSets = Arrays.asList(workingSets);
        Collections.sort(sortedWorkingSets, new WorkingSetComparator());

        Iterator iter = sortedWorkingSets.iterator();
        mruMenuCount = 0;
        while (iter.hasNext()) {
            IWorkingSet workingSet = (IWorkingSet) iter.next();
            if (workingSet !is null) {
                IContributionItem item = new WorkingSetMenuContributionItem(
                        ++mruMenuCount, this, workingSet);
                menuManager.insertBefore(SEPARATOR_ID, item);
            }
        }
    }

 
    /* (non-Javadoc)
     * @see dwtx.ui.actions.ActionGroup#dispose()
     */
    public void dispose() {
        if (menuManager !is null) {
            menuManager.removeMenuListener(menuListener);
        }
        super.dispose();
    }

    
    /* (non-Javadoc)
     * @see dwtx.ui.actions.ActionGroup#fillActionBars(dwtx.ui.IActionBars)
     */
    public void fillActionBars(IActionBars actionBars) {
        menuManager = actionBars.getMenuManager();
        
        if(menuManager.find(IWorkbenchActionConstants.MB_ADDITIONS) !is null) 
            menuManager.insertAfter(IWorkbenchActionConstants.MB_ADDITIONS, new Separator(WORKING_SET_ACTION_GROUP));
        else
            menuManager.add(new Separator(WORKING_SET_ACTION_GROUP));
        
        menuManager.appendToGroup(WORKING_SET_ACTION_GROUP, selectWorkingSetAction);
        menuManager.appendToGroup(WORKING_SET_ACTION_GROUP, clearWorkingSetAction);
        menuManager.appendToGroup(WORKING_SET_ACTION_GROUP, editWorkingSetAction);
        menuManager.appendToGroup(WORKING_SET_ACTION_GROUP, new Separator(START_SEPARATOR_ID));
        menuManager.appendToGroup(WORKING_SET_ACTION_GROUP, new Separator(SEPARATOR_ID));

        menuListener = new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                removePreviousMruWorkingSetActions(manager);
                addMruWorkingSetActions(manager);
            }
        };
        menuManager.addMenuListener(menuListener);
    }
    
    
    /* (non-Javadoc)
     * @see dwtx.ui.actions.ActionGroup#fillContextMenu(dwtx.jface.action.IMenuManager)
     */
    public void fillContextMenu(IMenuManager menuManager) {
        menuManager.add(selectWorkingSetAction);
        menuManager.add(clearWorkingSetAction);
        menuManager.add(editWorkingSetAction);
        menuManager.add(new Separator());
        menuManager.add(new Separator(SEPARATOR_ID));

        menuListener = new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                removePreviousMruWorkingSetActions(manager);
                addMruWorkingSetActions(manager);
            }
        };
        menuManager.addMenuListener(menuListener);
    }

    /**
     * Returns the working set which is currently selected.
     * 
     * @return the working set which is currently selected.
     */
    public IWorkingSet getWorkingSet() {
        return workingSet;
    }

    /**
     * Removes the most recently used working set actions that were
     * added to the specified menu.
     * 
     * @param menuManager menu manager to remove actions from
     */
    private void removePreviousMruWorkingSetActions(IMenuManager menuManager) {
        for (int i = 1; i <= mruMenuCount; i++) {
            menuManager.remove(WorkingSetMenuContributionItem.getId(i));
        }
    }

    /**
     * Sets the current working set.
     * 
     * @param newWorkingSet the new working set
     */
    public void setWorkingSet(IWorkingSet newWorkingSet) {
        IWorkingSet oldWorkingSet = workingSet;

        workingSet = newWorkingSet;
        // Update action
        clearWorkingSetAction.setEnabled(newWorkingSet !is null);
        editWorkingSetAction.setEnabled(newWorkingSet !is null && newWorkingSet.isEditable());

        firePropertyChange(newWorkingSet, oldWorkingSet);
    }

    /**
     * Fire the property change to the updater if there is one available.
     * 
     * @param newWorkingSet the new working set
     * @param oldWorkingSet the previous working set
     * @since 3.2
     */
    private void firePropertyChange(IWorkingSet newWorkingSet, IWorkingSet oldWorkingSet) {
        // Update viewer
        if (workingSetUpdater !is null) {
            workingSetUpdater.propertyChange(new PropertyChangeEvent(this,
                    WorkingSetFilterActionGroup.CHANGE_WORKING_SET,
                    oldWorkingSet, newWorkingSet));
        }
    }

    /**
     * Saves the state of the filter actions in a memento.
     * 
     * @param memento
     *            the memento
     * @since 3.3
     */
    public void saveState(IMemento memento) {
        String workingSetName = ""; //$NON-NLS-1$
        bool isWindowWorkingSet = false;
        if (workingSet !is null) {
            if (workingSet.isAggregateWorkingSet()) {
                isWindowWorkingSet = true;
            } else {
                workingSetName = workingSet.getName();
            }
        }
        memento.putString(TAG_IS_WINDOW_WORKING_SET,
                isWindowWorkingSet ? "true" : "false"); //$NON-NLS-1$ //$NON-NLS-2$
        memento.putString(TAG_WORKING_SET_NAME, workingSetName);
    }

    /**
     * Restores the state of the filter actions from a memento.
     * <p>
     * Note: This method does not refresh the viewer.
     * </p>
     * 
     * @param memento
     * @since 3.3
     */
    public void restoreState(IMemento memento) {
        bool isWindowWorkingSet;
        if (memento.getString(TAG_IS_WINDOW_WORKING_SET) !is null) {
            isWindowWorkingSet = Boolean.valueOf(
                    memento.getString(TAG_IS_WINDOW_WORKING_SET))
                    .booleanValue();
        } else {
            isWindowWorkingSet = useWindowWorkingSetByDefault();
        }
        String workingSetName = memento.getString(TAG_WORKING_SET_NAME);
        bool hasWorkingSetName = workingSetName !is null
                && workingSetName.length() > 0;

        IWorkingSet ws = null;
        // First handle name if present.
        if (hasWorkingSetName) {
            ws = PlatformUI.getWorkbench().getWorkingSetManager()
                    .getWorkingSet(workingSetName);
        } else if (isWindowWorkingSet && page !is null) {
            ws = page.getAggregateWorkingSet();
        }

        setWorkingSet(ws);
    }

    private bool useWindowWorkingSetByDefault() {
        return allowWindowWorkingSetByDefault
                && PlatformUI
                        .getPreferenceStore()
                        .getBoolean(
                                IWorkbenchPreferenceConstants.USE_WINDOW_WORKING_SET_BY_DEFAULT);
    }
}
