/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.activities.ActivitiesPreferencePage;

import dwt.dwthelper.utils;

import java.util.Hashtable;
import java.util.Properties;

import dwt.DWT;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExecutableExtension;
import dwtx.jface.dialogs.Dialog;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.preference.PreferencePage;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPreferencePage;
import dwtx.ui.internal.IPreferenceConstants;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.activities.ws.ActivityEnabler;
import dwtx.ui.internal.activities.ws.ActivityMessages;

/**
 * Preference page that allows configuration of the activity set. This page may
 * be used by product developers to provide basic ability to tweak the enabled
 * activity set. You may provide the certain strings to this class via method #2
 * of {@link dwtx.core.runtime.IExecutableExtension}.
 * 
 * @see #ACTIVITY_NAME
 * @see #ACTIVITY_PROMPT_BUTTON
 * @see #ACTIVITY_PROMPT_BUTTON_TOOLTIP
 * @since 3.1
 */
public final class ActivitiesPreferencePage extends PreferencePage implements
        IWorkbenchPreferencePage, IExecutableExtension {

    /**
     * The name to use for the activities.  Ie: "Capabilities".
     */
    public static final String ACTIVITY_NAME = "activityName"; //$NON-NLS-1$
    
    /**
     * The label to be used for the prompt button. Ie: "&Prompt when enabling capabilities".
     */    
    public static final String ACTIVITY_PROMPT_BUTTON = "activityPromptButton"; //$NON-NLS-1$
    
    /**
     * The tooltip to be used for the prompt button. Ie: "Prompt when a feature is first used that requires enablement of capabilities".
     */    
    public static final String ACTIVITY_PROMPT_BUTTON_TOOLTIP = "activityPromptButtonTooltip"; //$NON-NLS-1$
    
    private Button activityPromptButton;

    private IWorkbench workbench;

    private ActivityEnabler enabler;
    
    private Properties strings = new Properties();

    private IMutableActivityManager workingCopy;
    
    /**
     * Create the prompt for activity enablement.
     * 
     * @param composite the parent
     */
    protected void createActivityPromptPref(Composite composite) {
        activityPromptButton = new Button(composite, DWT.CHECK);
        activityPromptButton.setText(strings.getProperty(ACTIVITY_PROMPT_BUTTON, ActivityMessages.activityPromptButton)); 
        activityPromptButton.setToolTipText(strings.getProperty(ACTIVITY_PROMPT_BUTTON_TOOLTIP, ActivityMessages.activityPromptToolTip));

        setActivityButtonState();
    }

    /**
     * Sets the state of the activity prompt button from preferences.
     */
    private void setActivityButtonState() {
        activityPromptButton.setSelection(getPreferenceStore().getBoolean(
                IPreferenceConstants.SHOULD_PROMPT_FOR_ENABLEMENT));
    }

    /* (non-Javadoc)
     * @see dwtx.jface.preference.PreferencePage#createContents(dwt.widgets.Composite)
     */
    protected Control createContents(Composite parent) {
        initializeDialogUnits(parent);
        
        Composite composite = new Composite(parent, DWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
        layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
        composite.setLayout(layout);

        createActivityPromptPref(composite);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        activityPromptButton.setLayoutData(data);

        data = new GridData(GridData.FILL_BOTH);
        workingCopy = workbench.getActivitySupport().createWorkingCopy();
        enabler = new ActivityEnabler(workingCopy, strings);
        enabler.createControl(composite).setLayoutData(data);
        
        Dialog.applyDialogFont(composite);

        return composite;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPreferencePage#init(dwtx.ui.IWorkbench)
     */
    public void init(IWorkbench aWorkbench) {
        this.workbench = aWorkbench;
        setPreferenceStore(WorkbenchPlugin.getDefault().getPreferenceStore());
    }

    /* (non-Javadoc)
     * @see dwtx.jface.preference.IPreferencePage#performOk()
     */
    public bool performOk() {
        enabler.updateActivityStates();
        workbench.getActivitySupport().setEnabledActivityIds(workingCopy.getEnabledActivityIds());
        
        getPreferenceStore().setValue(
                IPreferenceConstants.SHOULD_PROMPT_FOR_ENABLEMENT,
                activityPromptButton.getSelection());

        return true;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        enabler.restoreDefaults();
        activityPromptButton.setSelection(getPreferenceStore()
                .getDefaultBoolean(
                        IPreferenceConstants.SHOULD_PROMPT_FOR_ENABLEMENT));
        super.performDefaults();
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.IExecutableExtension#setInitializationData(dwtx.core.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
     */
    public void setInitializationData(IConfigurationElement config, String propertyName, Object data) {
        if (data instanceof Hashtable) {
            strings.putAll((Hashtable)data);
        }       
    }
}
