/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.activities.CategoryEvent;

import dwt.dwthelper.utils;

/**
 * An instance of this class describes changes to an instance of 
 * <code>ICategory</code>.  This class does not give details as to the 
 * specifics of a change, only that the given property on the source object has 
 * changed.
 * 
 * <p>
 * This class is not intended to be extended by clients.
 * </p>
 * 
 * @since 3.0
 * @see ICategoryListener#categoryChanged(CategoryEvent)
 */
public final class CategoryEvent {
    private ICategory category;

    private bool categoryActivityBindingsChanged;

    private bool definedChanged;

    private bool nameChanged;

    private bool descriptionChanged;

    /**
     * Creates a new instance of this class.
     * 
     * @param category
     *        the instance of the interface that changed.
     * @param categoryActivityBindingsChanged
     *        <code>true</code>, iff the categoryActivityBindings property changed.
     * @param definedChanged
     *        <code>true</code>, iff the defined property changed.
     * @param descriptionChanged
     *        <code>true</code>, iff the description property changed. 
     * @param nameChanged
     *        <code>true</code>, iff the name property changed.
     */
    public CategoryEvent(ICategory category,
            bool categoryActivityBindingsChanged, bool definedChanged,
            bool descriptionChanged, bool nameChanged) {
        if (category is null) {
            throw new NullPointerException();
        }

        this.category = category;
        this.categoryActivityBindingsChanged = categoryActivityBindingsChanged;
        this.definedChanged = definedChanged;
        this.nameChanged = nameChanged;
        this.descriptionChanged = descriptionChanged;
    }

    /**
     * Returns the instance of the interface that changed.
     * 
     * @return the instance of the interface that changed. Guaranteed not to be
     *         <code>null</code>.
     */
    public ICategory getCategory() {
        return category;
    }

    /**
     * Returns whether or not the defined property changed.
     * 
     * @return <code>true</code>, iff the defined property changed.
     */
    public bool hasDefinedChanged() {
        return definedChanged;
    }

    /**
     * Returns whether or not the name property changed.
     * 
     * @return <code>true</code>, iff the name property changed.
     */
    public bool hasNameChanged() {
        return nameChanged;
    }

    /**
     * Returns whether or not the description property changed.
     * 
     * @return <code>true</code>, iff the description property changed.
     */
    public bool hasDescriptionChanged() {
        return descriptionChanged;
    }

    /**
     * Returns whether or not the categoryActivityBindings property changed.
     * 
     * @return <code>true</code>, iff the categoryActivityBindings property changed.
     */
    public bool haveCategoryActivityBindingsChanged() {
        return categoryActivityBindingsChanged;
    }
}
