/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.forms.editor.SharedHeaderFormEditor;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.layout.FillLayout;
import dwt.widgets.Composite;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwtx.ui.IEditorPart;
import dwtx.ui.forms.IFormPart;
import dwtx.ui.forms.IManagedForm;
import dwtx.ui.forms.ManagedForm;
import dwtx.ui.forms.widgets.ScrolledForm;
import dwtx.ui.internal.forms.widgets.FormUtil;

/**
 * A variation of {@link FormEditor}, this editor has a stable header that does
 * not change when pages are switched. Pages that are added to this editor
 * should not have the title or image set.
 * 
 * @since 3.3
 */
public abstract class SharedHeaderFormEditor extends FormEditor {
    private HeaderForm headerForm;
    
    private bool wasHeaderActive= true;
    private Listener activationListener= null;

    private static class HeaderForm extends ManagedForm {
        public HeaderForm(FormEditor editor, ScrolledForm form) {
            super(editor.getToolkit(), form);
            setContainer(editor);
            if (editor.getEditorInput() !is null)
                setInput(editor.getEditorInput());
        }

        private FormEditor getEditor() {
            return (FormEditor) getContainer();
        }

        public void dirtyStateChanged() {
            getEditor().editorDirtyStateChanged();
        }

        public void staleStateChanged() {
            refresh();
        }
    }

    /**
     * The default constructor.
     */

    public SharedHeaderFormEditor() {
    }

    /**
     * Overrides <code>super</code> to create a form in which to host the tab
     * folder. This form will be responsible for managing
     * 
     * @param parent
     *            the page container parent
     * 
     * @see dwtx.ui.part.MultiPageEditorPart#createPageContainer(dwt.widgets.Composite)
     */

    protected Composite createPageContainer(Composite parent) {
        parent = super.createPageContainer(parent);
        parent.setLayout(new FillLayout());
        ScrolledForm scform = getToolkit().createScrolledForm(parent);
        scform.getForm().setData(FormUtil.IGNORE_BODY, Boolean.TRUE);
        headerForm = new HeaderForm(this, scform);
        createHeaderContents(headerForm);
        return headerForm.getForm().getBody();
    }

    /**
     * Returns the form that owns the shared header.
     * 
     * @return the shared header
     */

    public IManagedForm getHeaderForm() {
        return headerForm;
    }
    
    protected void createPages() {
        super.createPages();
        
        // preempt MultiPageEditorPart#createPartControl(Composite)
        if (getActivePage() is -1) {
            // create page control and initialize page, keep focus on header by calling super implementation
            super.setActivePage(0);
        }
    }
    
    protected void setActivePage(int pageIndex) {
        // programmatic focus change
        wasHeaderActive= false;
        super.setActivePage(pageIndex);
    }
    
    public void setFocus() {
        installActivationListener();
        if (wasHeaderActive)
            ((ManagedForm) getHeaderForm()).setFocus();
        else {
            int index= getActivePage();
            if (index is -1)
                ((ManagedForm) getHeaderForm()).setFocus();
            else
                super.setFocus();
        }
    }
    
    private void installActivationListener() {
        if (activationListener is null) {
            activationListener = new Listener() {
                public void handleEvent(Event event) {
                    bool wasHeaderActive = event.widget !is getContainer();
                    
                    int activePage = getActivePage();
                    if (SharedHeaderFormEditor.this.wasHeaderActive !is wasHeaderActive && activePage !is -1 && pages.get(activePage) instanceof IEditorPart) {
                        
                        if (wasHeaderActive) {
                            deactivateSite(true, true);
                        } else {
                            activateSite();
                        }
                    }
                    
                    SharedHeaderFormEditor.this.wasHeaderActive = wasHeaderActive;
                }
            };
            getContainer().addListener(DWT.Activate, activationListener);
            getHeaderForm().getForm().getForm().getHead().addListener(DWT.Activate, activationListener);
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.forms.editor.FormEditor#dispose()
     */
    public void dispose() {
        if (headerForm !is null) {
            headerForm.dispose();
            headerForm = null;
        }
        super.dispose();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.forms.editor.FormEditor#isDirty()
     */
    public bool isDirty() {
        return headerForm.isDirty() || super.isDirty();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.forms.editor.FormEditor#commitPages(bool)
     */
    protected void commitPages(bool onSave) {
        if (headerForm !is null && headerForm.isDirty())
            headerForm.commit(onSave);
        super.commitPages(onSave);
    }

    /**
     * Subclasses should extend this method to configure the form that owns the
     * shared header. If the header form will contain controls that can change
     * the state of the editor, they should be wrapped in an IFormPart so that
     * they can participate in the life cycle event management.
     * 
     * @param headerForm
     *            the form that owns the shared header
     * @see IFormPart
     */
    protected void createHeaderContents(IManagedForm headerForm) {
    }
}
