/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Remy Chi Jian Suen <remy.suen@gmail.com> - Bug 218540 [Perspectives] Perspectives always open in same window, regardless of preference
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.handlers.ShowPerspectiveHandler;

import dwt.dwthelper.utils;

import java.util.Map;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.runtime.IAdaptable;
import dwtx.jface.dialogs.ErrorDialog;
import dwtx.jface.window.Window;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.WorkbenchException;
import dwtx.ui.internal.IPreferenceConstants;
import dwtx.ui.internal.Workbench;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.dialogs.SelectPerspectiveDialog;

/**
 * Shows the given perspective. If no perspective is specified in the
 * parameters, then this opens the perspective selection dialog.
 * 
 * @since 3.1
 */
public final class ShowPerspectiveHandler extends AbstractHandler {

    /**
     * The name of the parameter providing the perspective identifier.
     */
    private static final String PARAMETER_NAME_VIEW_ID = "dwtx.ui.perspectives.showPerspective.perspectiveId"; //$NON-NLS-1$

    /**
     * True/false value to open the perspective in a new window.
     */
    private static final String PARAMETER_NEW_WINDOW = "dwtx.ui.perspectives.showPerspective.newWindow"; //$NON-NLS-1$

    public final Object execute(final ExecutionEvent event)
            throws ExecutionException {
        IWorkbenchWindow window = HandlerUtil
                .getActiveWorkbenchWindowChecked(event);

        // Get the view identifier, if any.
        final Map parameters = event.getParameters();
        final Object value = parameters.get(PARAMETER_NAME_VIEW_ID);
        final String newWindow = (String) parameters.get(PARAMETER_NEW_WINDOW);

        if (value is null) {
            openOther(window);
        } else {

            if (newWindow is null || newWindow.equalsIgnoreCase("false")) { //$NON-NLS-1$
                openPerspective((String) value, window);
            } else {
                openNewWindowPerspective((String) value, window);
            }
        }
        return null;
    }

    /**
     * Opens the specified perspective in a new window.
     * 
     * @param perspectiveId
     *            The perspective to open; must not be <code>null</code>
     * @throws ExecutionException
     *             If the perspective could not be opened.
     */
    private void openNewWindowPerspective(String perspectiveId,
            IWorkbenchWindow activeWorkbenchWindow) throws ExecutionException {
        final IWorkbench workbench = PlatformUI.getWorkbench();
        try {
            IAdaptable input = ((Workbench) workbench).getDefaultPageInput();
            workbench.openWorkbenchWindow(perspectiveId, input);
        } catch (WorkbenchException e) {
            ErrorDialog.openError(activeWorkbenchWindow.getShell(),
                    WorkbenchMessages.ChangeToPerspectiveMenu_errorTitle, e
                            .getMessage(), e.getStatus());
        }
    }

    /**
     * Opens a view selection dialog, allowing the user to chose a view.
     * 
     * @throws ExecutionException
     *             If the perspective could not be opened.
     */
    private final void openOther(final IWorkbenchWindow activeWorkbenchWindow)
            throws ExecutionException {
        final SelectPerspectiveDialog dialog = new SelectPerspectiveDialog(
                activeWorkbenchWindow.getShell(), WorkbenchPlugin.getDefault()
                        .getPerspectiveRegistry());
        dialog.open();
        if (dialog.getReturnCode() is Window.CANCEL) {
            return;
        }

        final IPerspectiveDescriptor descriptor = dialog.getSelection();
        if (descriptor !is null) {
            int openPerspMode = WorkbenchPlugin.getDefault().getPreferenceStore()
                    .getInt(IPreferenceConstants.OPEN_PERSP_MODE);
            IWorkbenchPage page = activeWorkbenchWindow.getActivePage();
            IPerspectiveDescriptor persp = page is null ? null : page.getPerspective();
            String perspectiveId = descriptor.getId();
            // only open it in a new window if the preference is set and the
            // current workbench page doesn't have an active perspective
            if (IPreferenceConstants.OPM_NEW_WINDOW is openPerspMode && persp !is null) {
                openNewWindowPerspective(perspectiveId, activeWorkbenchWindow);
            } else {
                openPerspective(perspectiveId, activeWorkbenchWindow);
            }
        }
    }

    /**
     * Opens the perspective with the given identifier.
     * 
     * @param perspectiveId
     *            The perspective to open; must not be <code>null</code>
     * @throws ExecutionException
     *             If the perspective could not be opened.
     */
    private final void openPerspective(final String perspectiveId,
            final IWorkbenchWindow activeWorkbenchWindow)
            throws ExecutionException {
        final IWorkbench workbench = PlatformUI.getWorkbench();

        final IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
        IPerspectiveDescriptor desc = activeWorkbenchWindow.getWorkbench()
                .getPerspectiveRegistry().findPerspectiveWithId(perspectiveId);
        if (desc is null) {
            throw new ExecutionException("Perspective " + perspectiveId //$NON-NLS-1$
                    + " cannot be found."); //$NON-NLS-1$
        }

        try {
            if (activePage !is null) {
                activePage.setPerspective(desc);
            } else {
                IAdaptable input = ((Workbench) workbench)
                        .getDefaultPageInput();
                activeWorkbenchWindow.openPage(perspectiveId, input);
            }
        } catch (WorkbenchException e) {
            throw new ExecutionException("Perspective could not be opened.", e); //$NON-NLS-1$
        }
    }
}
