/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.EditorSite;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import dwtx.core.runtime.Assert;
import dwtx.jface.action.MenuManager;
import dwtx.jface.viewers.ISelectionProvider;
import dwtx.ui.IActionBars2;
import dwtx.ui.IEditorActionBarContributor;
import dwtx.ui.IEditorPart;
import dwtx.ui.IEditorReference;
import dwtx.ui.IEditorSite;
import dwtx.ui.SubActionBars;
import dwtx.ui.SubActionBars2;
import dwtx.ui.dnd.IDragAndDropService;
import dwtx.ui.internal.registry.EditorDescriptor;

/**
 * An editor container manages the services for an editor.
 */
public class EditorSite extends PartSite implements IEditorSite {
    /* package */ //static final int PROP_REUSE_EDITOR = -0x101;

    private EditorDescriptor desc;

    //private ListenerList propChangeListeners = new ListenerList(1);

    private SubActionBars ab = null;
    
    /**
     * Constructs an EditorSite for an editor.
     */
    public EditorSite(IEditorReference ref, IEditorPart editor,
            WorkbenchPage page, EditorDescriptor desc) {
        super(ref, editor, page);
        Assert.isNotNull(desc);
        this.desc = desc;
        
        if (desc.getConfigurationElement() !is null) {
            setConfigurationElement(desc.getConfigurationElement());
        } else {
            // system external and in-place editors do not have a corresponding configuration element
            setId(desc.getId());
            setRegisteredName(desc.getLabel());
        }

        // Initialize the services specific to this editor site.
        initializeDefaultServices();
    }

    /**
     * Initialize the local services.
     */
    private void initializeDefaultServices() {
        // Register an implementation of the service appropriate for the 
        // EditorSite.
        final IDragAndDropService editorDTService = new EditorSiteDragAndDropServiceImpl();
        serviceLocator.registerService(IDragAndDropService.class, editorDTService);
    }
    
    public void setActionBars(SubActionBars bars) {
        super.setActionBars(bars);
        
        if (bars instanceof IActionBars2) {
            ab = new SubActionBars2((IActionBars2)bars, this);
        } else {
            ab = new SubActionBars(bars, this);
        }
    }
    
    public void activateActionBars(bool forceVisibility) {
        if (ab !is null) {
            ab.activate(forceVisibility);
        }
        super.activateActionBars(forceVisibility);
    }

    public void deactivateActionBars(bool forceHide) {
        if (ab !is null) {
            ab.deactivate(forceHide);
        }
        super.deactivateActionBars(forceHide);
    }
    
    /**
     * Returns the editor action bar contributor for this editor.
     * <p>
     * An action contributor is responsable for the creation of actions.
     * By design, this contributor is used for one or more editors of the same type.
     * Thus, the contributor returned by this method is not owned completely
     * by the editor.  It is shared.
     * </p>
     *
     * @return the editor action bar contributor
     */
    public IEditorActionBarContributor getActionBarContributor() {
        EditorActionBars bars = (EditorActionBars) getActionBars();
        if (bars !is null) {
            return bars.getEditorContributor();
        }
        
        return null;
    }

    /**
     * Returns the extension editor action bar contributor for this editor.
     */
    public IEditorActionBarContributor getExtensionActionBarContributor() {
        EditorActionBars bars = (EditorActionBars) getActionBars();
        if (bars !is null) {
            return bars.getExtensionContributor();
        }
        
        return null;
    }

    /**
     * Returns the editor
     */
    public IEditorPart getEditorPart() {
        return (IEditorPart) getPart();
    }

    public EditorDescriptor getEditorDescriptor() {
        return desc;
    }

    protected String getInitialScopeId() {
        return "dwtx.ui.textEditorScope"; //$NON-NLS-1$
    }
    
    public void dispose() {
        super.dispose();
        
        if (ab !is null) {
            ab.dispose();
        }
    }
    
    public final void registerContextMenu(final MenuManager menuManager,
            final ISelectionProvider selectionProvider,
            final bool includeEditorInput) {
        registerContextMenu(getId(), menuManager, selectionProvider,
                includeEditorInput);
    }
    
    public final void registerContextMenu(final String menuId,
            final MenuManager menuManager,
            final ISelectionProvider selectionProvider,
            final bool includeEditorInput) {
        if (menuExtenders is null) {
            menuExtenders = new ArrayList(1);
        }
        
        PartSite.registerContextMenu(menuId, menuManager, selectionProvider,
                includeEditorInput, getPart(), menuExtenders);
    }
}
