/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.FastViewDnDHandler;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import dwt.DWT;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.graphics.Cursor;
import dwt.graphics.Point;
import dwt.graphics.Rectangle;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.ToolBar;
import dwt.widgets.ToolItem;
import dwtx.jface.action.ToolBarManager;
import dwtx.jface.util.Geometry;
import dwtx.ui.IViewReference;
import dwtx.ui.internal.dnd.AbstractDropTarget;
import dwtx.ui.internal.dnd.DragUtil;
import dwtx.ui.internal.dnd.IDragOverListener;
import dwtx.ui.internal.dnd.IDropTarget;
import dwtx.ui.presentations.PresentationUtil;

/**
 * @since 3.3
 *
 */
public class FastViewDnDHandler implements IDragOverListener {
    private String id;
    private ToolBarManager tbm;
    private WorkbenchWindow wbw;
    private ViewDropTarget dropTarget = null;

    private Listener dragListener = new Listener() {
        public void handleEvent(Event event) {
            Point position = DragUtil.getEventLoc(event);
            
            ToolBar toolbar = tbm.getControl();
            Point local = toolbar.toControl(position);
            ToolItem item = toolbar.getItem(local);
            IViewReference ref = (IViewReference) item
            .getData(ShowFastViewContribution.FAST_VIEW);

            if (ref !is null) {
                startDraggingFastView(ref, position, false);
            }
        }
    };

    class ViewDropTarget extends AbstractDropTarget {
        List panes;
        ToolItem curItem;

        /**
         * @param panesToDrop the list of ViewPanes to drop at the given position
         */
        public ViewDropTarget(List panesToDrop, ToolItem position) {
            setTarget(panesToDrop, position);
        }
        
        public void setTarget(List panesToDrop, ToolItem position) {
            panes = panesToDrop;
            this.curItem = position;            
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.dnd.IDropTarget#drop()
         */
        public void drop() {
            Perspective persp = wbw.getActiveWorkbenchPage().getActivePerspective();
            FastViewManager fvm = persp.getFastViewManager();

            int insertIndex = tbm.getControl().indexOf(curItem);
            Iterator iter = panes.iterator();
            while (iter.hasNext()) {
                ViewPane pane = (ViewPane) iter.next();
                IViewReference ref = pane.getViewReference();
                adoptRef(ref);
                fvm.addViewReference(id, insertIndex++, ref, !iter.hasNext());
            }
        }

        private void adoptRef(IViewReference ref) {
            Perspective persp = wbw.getActiveWorkbenchPage()
                    .getActivePerspective();
            PerspectiveHelper helper = persp.getPresentation();
            ContainerPlaceholder ourContainerPlaceholder = (ContainerPlaceholder) helper
                    .findPart(id, null);
            LayoutPart refPart = helper.findPart(ref.getId(), ref
                    .getSecondaryId());
            ILayoutContainer refContainer = refPart.container;
            if (refContainer !is ourContainerPlaceholder) {
                // remove the old part... if it's represented by a
                // placeholder then just remove it...
                if (refPart instanceof PartPlaceholder) {
                    if (refContainer instanceof ContainerPlaceholder) {
                        // just remove the placeholder
                        ViewStack realContainer = (ViewStack) ((ContainerPlaceholder) refContainer)
                                .getRealContainer();
                        realContainer.remove(refPart);
                    }
                    else if (refContainer instanceof ViewStack) {
                        refContainer.remove(refPart);
                    }
                }
                else {
                    // If its a real view ref then defref it...
                    helper.derefPart(refPart);
                }
                PartPlaceholder newPlaceholder = new PartPlaceholder(ref
                        .getId());
                ourContainerPlaceholder.add(newPlaceholder);
            }
        }
        
        /*
         * (non-Javadoc)
         * 
         * @see dwtx.ui.internal.dnd.IDropTarget#getCursor()
         */
        public Cursor getCursor() {
            return DragCursors.getCursor(DragCursors.FASTVIEW);
        }

        public Rectangle getSnapRectangle() {
            if (curItem is null) {
                // As long as the toolbar is not empty, highlight the place
                // where this view will appear (we
                // may have compressed it to save space when empty, so the actual
                // icon location may not be over the toolbar when it is empty)
                if (tbm.getControl().getItemCount() > 0) {
                    return getLocationOfNextIcon();
                }
                // If the toolbar is empty, highlight the entire toolbar 
                return DragUtil.getDisplayBounds(tbm.getControl());
            }

            return Geometry.toDisplay(tbm.getControl(), curItem.getBounds());
        }
    }
    
    /**
     * 
     */
    public FastViewDnDHandler(String id, final ToolBarManager tbm, WorkbenchWindow wbw) {
        this.id = id;
        this.tbm = tbm;
        this.wbw = wbw;
        
        // Hook the 'drop' listener to the control
        DragUtil.addDragTarget(tbm.getControl(), this);
        PresentationUtil.addDragListener(tbm.getControl(), dragListener);
        
        // Clean up on dispose
        tbm.getControl().addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                DragUtil.removeDragTarget((Control)(e.widget), FastViewDnDHandler.this);
                PresentationUtil.removeDragListener(tbm.getControl(), dragListener);
            }
        });
    }

    /**
     * Returns the toolbar item at the given position, in display coordinates
     * @param position
     */
    private ToolItem getToolItem(Point position) {
        ToolBar toolbar = tbm.getControl();
        Point local = toolbar.toControl(position);
        return toolbar.getItem(local);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.internal.dnd.IDragOverListener#drag(dwt.widgets.Control, java.lang.Object, dwt.graphics.Point, dwt.graphics.Rectangle)
     */
    public IDropTarget drag(Control currentControl, Object draggedObject,
            Point position, Rectangle dragRectangle) {
        // If we're trying to drop onto a 'standalone' stack, don't...
        if (isStandaloneStack())
            return null;
        
        ToolItem targetItem = getToolItem(position);
        if (draggedObject instanceof ViewPane) {
            ViewPane pane = (ViewPane) draggedObject;

            // Can't drag views between windows
            if (pane.getWorkbenchWindow() !is wbw) {
                return null;
            }

            List newList = new ArrayList(1);
            newList.add(draggedObject);

            return createDropTarget(newList, targetItem);
        }
        if (draggedObject instanceof ViewStack) {
            ViewStack folder = (ViewStack) draggedObject;

            if (folder.getWorkbenchWindow() !is wbw) {
                return null;
            }

            List viewList = new ArrayList(folder.getItemCount());
            LayoutPart[] children = folder.getChildren();

            for (int idx = 0; idx < children.length; idx++) {
                if (!(children[idx] instanceof PartPlaceholder)) {
                    viewList.add(children[idx]);
                }
            }

            return createDropTarget(viewList, targetItem);
        }

        return null;
    }
    /**
     * Tests the view references associated with the stack and
     * returns <code>true</code> if any view is a stand-alone view
     * 
     * @return <code>true</code> is any view is stand-alone
     */
    private bool isStandaloneStack() {
        Perspective persp = wbw.getActiveWorkbenchPage().getActivePerspective();
        List fvs = persp.getFastViewManager().getFastViews(id);
        for (Iterator iterator = fvs.iterator(); iterator.hasNext();) {
            IViewReference ref = (IViewReference) iterator.next();
            if (persp.isStandaloneView(ref))
                return true;
        }
        
        return false;
    }

    private IDropTarget createDropTarget(List viewList, ToolItem targetItem) {
        if (dropTarget is null) {
            dropTarget = new ViewDropTarget(viewList, targetItem);
        } else {
            dropTarget.setTarget(viewList, targetItem);
        }
        return dropTarget;
    }

    /**
     * Returns the approximate location where the next fastview icon
     * will be drawn (display coordinates)
     */
    public Rectangle getLocationOfNextIcon() {
        ToolBar control = tbm.getControl();

        Rectangle result = control.getBounds();
        Point size = control.computeSize(DWT.DEFAULT, DWT.DEFAULT, false);
        result.height = size.y;
        result.width = size.x;
        
        bool horizontal = (control.getStyle() & DWT.VERTICAL) is 0;
        if (control.getItemCount() is 0) {
            Geometry.setDimension(result, horizontal, 0);
        }
        
        int hoverSide = horizontal ? DWT.RIGHT : DWT.BOTTOM;

        result = Geometry.getExtrudedEdge(result, -Geometry.getDimension(
                result, !horizontal), hoverSide);

        return Geometry.toDisplay(control.getParent(), result);
    }

    /**
     * Returns the index of the ToolItem fronting the view ref
     * @param toFind the view reference to find the index of
     * @return the index or -1 if not found
     */
    private int getIndex(IViewReference toFind) {
        ToolItem[] items = tbm.getControl().getItems();
        for (int i = 0; i < items.length; i++) {
            if (items[i].getData(ShowFastViewContribution.FAST_VIEW) is toFind) {
                return i;
            }
        }

        return -1;
    }
    
    /**
     * Begins dragging a particular fast view
     * 
     * @param ref
     * @param position
     */
    protected void startDraggingFastView(IViewReference ref, Point position,
            bool usingKeyboard) {
        int index = getIndex(ref);
        if (index is -1)
            return;
        
        ToolItem item = tbm.getControl().getItem(index);
        Rectangle dragRect = Geometry.toDisplay(tbm.getControl(), item.getBounds());
        startDrag(((WorkbenchPartReference) ref).getPane(), dragRect, position,
                usingKeyboard);
    }

    private void startDrag(Object toDrag, Rectangle dragRect, Point position,
            bool usingKeyboard) {
        WorkbenchPage page = wbw.getActiveWorkbenchPage();
        Perspective persp = page.getActivePerspective();

        // Prevent dragging non-movable refs out of a minimized stack  
        if (toDrag instanceof ViewPane) {
            ViewPane pane = (ViewPane) toDrag;
            if (!persp.isMoveable(pane.getViewReference()))
                return;
        }
        
        IViewReference oldFastView = null;
        if (persp !is null) {
            oldFastView = persp.getActiveFastView();

            if (page !is null) {
                page.hideFastView();
            }
        }

        if (page.isPageZoomed()) {
            page.zoomOut();
        }

        bool success = DragUtil.performDrag(toDrag, dragRect, position,
                !usingKeyboard);

        // If the drag was cancelled, reopen the old fast view
        if (!success && oldFastView !is null && page !is null) {
            page.toggleFastView(oldFastView);
        }
    }
}
