/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.JFaceUtil;

import dwt.dwthelper.utils;

import dwtx.core.runtime.ISafeRunnable;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.preferences.IEclipsePreferences;
import dwtx.core.runtime.preferences.InstanceScope;
import dwtx.core.runtime.preferences.IEclipsePreferences.NodeChangeEvent;
import dwtx.jface.internal.InternalPolicy;
import dwtx.jface.preference.JFacePreferences;
import dwtx.jface.util.ILogger;
import dwtx.jface.util.ISafeRunnableRunner;
import dwtx.jface.util.Policy;
import dwtx.jface.util.SafeRunnable;
import dwtx.jface.util.StatusHandler;
import dwtx.ui.statushandlers.StatusAdapter;
import dwtx.ui.statushandlers.StatusManager;

/**
 * Utility class for setting up JFace for use by Eclipse.
 * 
 * @since 3.1
 */
final class JFaceUtil {

    private JFaceUtil() {
        // prevents intantiation
    }

    /**
     * Initializes JFace for use by Eclipse.
     */
    public static void initializeJFace() {
        // Set the Platform to run all SafeRunnables
        SafeRunnable.setRunner(new ISafeRunnableRunner() {
            public void run(ISafeRunnable code) {
                Platform.run(code);
            }
        });

        // Pass all errors and warnings to the status handling facility
        // and the rest to the main runtime log
        Policy.setLog(new ILogger() {
            public void log(IStatus status) {
                if (status.getSeverity() is IStatus.WARNING
                        || status.getSeverity() is IStatus.ERROR) {
                    StatusManager.getManager().handle(status);
                } else {
                    WorkbenchPlugin.log(status);
                }
            }
        });
        
        Policy.setStatusHandler(new StatusHandler() {
            public void show(IStatus status, String title) {
                StatusAdapter statusAdapter = new StatusAdapter(status);
                statusAdapter.setProperty(StatusAdapter.TITLE_PROPERTY, title);
                StatusManager.getManager().handle(statusAdapter, StatusManager.SHOW);
            }
        });

        // Get all debug options from Platform
        if ("true".equalsIgnoreCase(Platform.getDebugOption("/debug"))) { //$NON-NLS-1$ //$NON-NLS-2$
            Policy.DEBUG_DIALOG_NO_PARENT = "true".equalsIgnoreCase(Platform.getDebugOption(Policy.JFACE + "/debug/dialog/noparent")); //$NON-NLS-1$ //$NON-NLS-2$
            Policy.TRACE_ACTIONS = "true".equalsIgnoreCase(Platform.getDebugOption(Policy.JFACE + "/trace/actions")); //$NON-NLS-1$ //$NON-NLS-2$
            Policy.TRACE_TOOLBAR = "true".equalsIgnoreCase(Platform.getDebugOption(Policy.JFACE + "/trace/toolbarDisposal")); //$NON-NLS-1$ //$NON-NLS-2$
            InternalPolicy.DEBUG_LOG_REENTRANT_VIEWER_CALLS = "true".equalsIgnoreCase(Platform.getDebugOption(Policy.JFACE + "/debug/viewers/reentrantViewerCalls")); //$NON-NLS-1$ //$NON-NLS-2$
        }
    }

    /**
     * Adds a preference listener so that the JFace preference store is initialized
     * as soon as the workbench preference store becomes available.
     */
    public static void initializeJFacePreferences() {
        IEclipsePreferences rootNode = (IEclipsePreferences) Platform.getPreferencesService().getRootNode().node(InstanceScope.SCOPE);
        final String workbenchName = WorkbenchPlugin.getDefault().getBundle().getSymbolicName();
        
        rootNode.addNodeChangeListener(new IEclipsePreferences.INodeChangeListener() {
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.preferences.IEclipsePreferences.INodeChangeListener#added(dwtx.core.runtime.preferences.IEclipsePreferences.NodeChangeEvent)
             */
            public void added(NodeChangeEvent event) {
                if (!event.getChild().name().equals(workbenchName)) {
                    return;
                }
                ((IEclipsePreferences) event.getChild()).addPreferenceChangeListener(PlatformUIPreferenceListener.getSingleton());

            }
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.preferences.IEclipsePreferences.INodeChangeListener#removed(dwtx.core.runtime.preferences.IEclipsePreferences.NodeChangeEvent)
             */
            public void removed(NodeChangeEvent event) {
                // Nothing to do here

            }
        });
        
        JFacePreferences.setPreferenceStore(WorkbenchPlugin.getDefault().getPreferenceStore());
    }
}
