/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Randy Hudson <hudsonr@us.ibm.com>
 *     - Fix for bug 19524 - Resizing WorkbenchWindow resizes views
 *     Cagatay Kavukcuoglu <cagatayk@acm.org>
 *     - Fix for bug 10025 - Resizing views should not use height ratios
 *     Matthew Hatem Matthew_Hatem@notesdev.ibm.com Bug 189953
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.LayoutPartSash;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.events.SelectionListener;
import dwt.graphics.Rectangle;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Sash;
import dwtx.jface.util.Geometry;
import dwtx.ui.presentations.AbstractPresentationFactory;

class LayoutPartSash extends LayoutPart {

    private Sash sash;
    private bool enabled = false;

    private PartSashContainer rootContainer;

    private int style;

    private LayoutPartSash preLimit;

    private LayoutPartSash postLimit;

    SelectionListener selectionListener;

    private int left = 300, right = 300;

    private Rectangle bounds = new Rectangle(0,0,0,0);
    
    private AbstractPresentationFactory presFactory;
    
    /**
     * Stores whether or not the sash is visible. (This is expected to have a meaningful
     * value even if the underlying control doesn't exist).
     */
    private bool isVisible;
    
    LayoutPartSash(PartSashContainer rootContainer, int style) {
        super(null);
        this.style = style;
        this.rootContainer = rootContainer;

        selectionListener = new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) { 
                checkDragLimit(e);
                
                if (e.detail !is DWT.DRAG) {    
                    LayoutPartSash.this.widgetSelected(e.x, e.y, e.width,
                            e.height);
                }
            }
        };
    }

    // checkDragLimit contains changes by cagatayk@acm.org
    private void checkDragLimit(SelectionEvent event) {
        LayoutTree root = rootContainer.getLayoutTree();
        LayoutTreeNode node = root.findSash(this);
        Rectangle nodeBounds = node.getBounds();
        Rectangle eventRect = new Rectangle(event.x, event.y, event.width, event.height);
        
        bool vertical = (style is DWT.VERTICAL);
        
        // If a horizontal sash, flip the coordinate system so that we
        // can handle horizontal and vertical sashes without special cases
        if (!vertical) {
            Geometry.flipXY(nodeBounds);
            Geometry.flipXY(eventRect);
        }

        int eventX = eventRect.x;
        int left = Math.max(0, eventX - nodeBounds.x);
        left = Math.min(left, nodeBounds.width - getSashSize()); 
        int right = nodeBounds.width - left - getSashSize();
        
        LayoutTreeNode.ChildSizes sizes = node.computeChildSizes(nodeBounds.width, nodeBounds.height, left, right, nodeBounds.width);

        eventRect.x = nodeBounds.x + sizes.left;
        
        // If it's a horizontal sash, restore eventRect to its original coordinate system
        if (!vertical) {
            Geometry.flipXY(eventRect);
        }
        
        event.x = eventRect.x;
        event.y = eventRect.y;        
    }

    /**
     * Creates the control. As an optimization, creation of the control is deferred if
     * the control is invisible.
     */
    public void createControl(Composite parent) {
        // Defer creation of the control until it becomes visible
        if (isVisible) {
            doCreateControl();
        }
    }
    
    /**
     * Creates the underlying DWT control.
     * 
     * @since 3.1
     */
    private void doCreateControl() {         
        if (sash is null) {         
            // ask the presentation factory to create the sash
            AbstractPresentationFactory factory = getPresentationFactory();
            
            int sashStyle = AbstractPresentationFactory.SASHTYPE_NORMAL | style;
            sash = factory.createSash(this.rootContainer.getParent(), sashStyle);
            
            sash.addSelectionListener(selectionListener);
            sash.setEnabled(enabled);
            sash.setBounds(bounds);
        }
    }
    
    public void setBounds(Rectangle r) {
        super.setBounds(r);
        
        bounds = r;
    }

    /**
     * Makes the sash visible or invisible. Note: as an optimization, the actual widget is destroyed when the
     * sash is invisible.
     */
    public void setVisible(bool visible) {
        if (visible is isVisible) {
            return;
        }
        
        if (visible) {
            doCreateControl();
        } else {
            dispose();
        }
        
        super.setVisible(visible);
        
        isVisible = visible;
    }
    
    public bool isVisible() {
        return isVisible;
    }
    
    /**
     * See LayoutPart#dispose
     */
    public void dispose() {

        if (sash !is null) {
            bounds = sash.getBounds();
            sash.dispose();
        }
        sash = null;
    }

    /**
     * Gets the presentation bounds.
     */
    public Rectangle getBounds() {
        if (sash is null) {
            return bounds;
        }

        return sash.getBounds();
    }

    /**
     * Returns the part control.
     */
    public Control getControl() {
        return sash;
    }

    /**
     *  
     */
    public String getID() {
        return null;
    }

    LayoutPartSash getPostLimit() {
        return postLimit;
    }

    LayoutPartSash getPreLimit() {
        return preLimit;
    }

    int getLeft() {
        return left;
    }

    int getRight() {
        return right;
    }

    bool isHorizontal() {
        return ((style & DWT.HORIZONTAL) is DWT.HORIZONTAL);
    }

    bool isVertical() {
        return ((style & DWT.VERTICAL) is DWT.VERTICAL);
    }

    void setPostLimit(LayoutPartSash newPostLimit) {
        postLimit = newPostLimit;
    }

    void setPreLimit(LayoutPartSash newPreLimit) {
        preLimit = newPreLimit;
    }

    void setRatio(float newRatio) {
        int total = left + right;
        int newLeft = (int) (total * newRatio);
        setSizes(newLeft, total - newLeft);
    }

    void setSizes(int left, int right) {
        if (left < 0 || right < 0) {
            return;
        }
        
        if (left is this.left && right is this.right) {
            return;
        }
        
        this.left = left;
        this.right = right;
        
        flushCache();
    }
    
    private void flushCache() {
        LayoutTree root = rootContainer.getLayoutTree();

        if (root !is null) {
            LayoutTreeNode node = root.findSash(this);
            if (node !is null) {
                node.flushCache();
            }
        }
    }

    private void widgetSelected(int x, int y, int width, int height) {
        if (!enabled) {
            return;
        }
        
        LayoutTree root = rootContainer.getLayoutTree();
        LayoutTreeNode node = root.findSash(this);
        Rectangle nodeBounds = node.getBounds();
        //Recompute ratio
        x -= nodeBounds.x;
        y -= nodeBounds.y;
        if (style is DWT.VERTICAL) {
            setSizes(x, nodeBounds.width - x - getSashSize());
        } else {
            setSizes(y, nodeBounds.height - y - getSashSize());
        }

        node.setBounds(nodeBounds);        
    }

    /**
     * @param resizable
     * @since 3.1
     */
    public void setEnabled(bool resizable) {
        this.enabled = resizable;
        if (sash !is null) {
            sash.setEnabled(enabled);
        }
    }
    
    /* package */ int getSashSize() {
        AbstractPresentationFactory factory = getPresentationFactory();
        int sashStyle = AbstractPresentationFactory.SASHTYPE_NORMAL | style;
        int size = factory.getSashSize(sashStyle);
        return size;
    }
    
    private AbstractPresentationFactory getPresentationFactory() {
        if (presFactory is null) {
            WorkbenchWindow wbw = (WorkbenchWindow)rootContainer.getPage().getWorkbenchWindow();
            WorkbenchWindowConfigurer configurer = wbw.getWindowConfigurer();
            presFactory = configurer.getPresentationFactory();
        }
        return presFactory;
    }

}
