/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.ObjectActionContributorManager;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtension;
import dwtx.core.runtime.dynamichelpers.IExtensionTracker;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.viewers.ISelection;
import dwtx.jface.viewers.ISelectionProvider;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;

/**
 * This manager is used to populate a popup menu manager with actions
 * for a given type.
 */
public class ObjectActionContributorManager extends ObjectContributorManager {
    private static ObjectActionContributorManager sharedInstance;

    /**
     * PopupMenuManager constructor.
     */
    public ObjectActionContributorManager() {
        super();
        loadContributors();
    }

    /**
     * Contributes submenus and/or actions applicable to the selection in the
     * provided viewer into the provided popup menu.
     * 
     * @param part the part being contributed to
     * @param popupMenu the menu being contributed to
     * @param selProv the selection provider
     * @return whether anything was contributed
     */
    public bool contributeObjectActions(IWorkbenchPart part,
            IMenuManager popupMenu, ISelectionProvider selProv) {
        // Get a selection. 
        ISelection selection = selProv.getSelection();
        if (selection is null) {
            return false;
        }

        // Convert the selection into an element vector.
        // According to the dictionary, a selection is "one that
        // is selected", or "a collection of selected things".  
        // In reflection of this, we deal with one or a collection.
        List elements = null;
        if (selection instanceof IStructuredSelection) {
            elements = ((IStructuredSelection) selection).toList();
        } else {
            elements = new ArrayList(1);
            elements.add(selection);
        }

        List contributors = getContributors(elements);
       
        if (contributors.isEmpty()) {
            return false;
        }

        // First pass, add the menus and collect the overrides. Prune from the
        // list any non-applicable contributions.
        bool actualContributions = false;
        ArrayList overrides = new ArrayList(4);
        for (Iterator it = contributors.iterator(); it.hasNext();) {
            IObjectActionContributor contributor = (IObjectActionContributor) it.next();
            if (!isApplicableTo(elements, contributor)) {
                it.remove();            
                continue;
            }
            if (contributor.contributeObjectMenus(popupMenu, selProv)) {
                actualContributions = true;
            }
            contributor.contributeObjectActionIdOverrides(overrides);
        }
        
        // Second pass, add the contributions that are applicable to
        // the selection.
        for (Iterator it = contributors.iterator(); it.hasNext();) {
            IObjectActionContributor contributor = (IObjectActionContributor) it.next();        
            if (contributor.contributeObjectActions(part, popupMenu, selProv,
                    overrides)) {
                actualContributions = true;
            }
        }
        return actualContributions;
    }

    /**
     * Returns the shared instance of this manager.
     * @return the shared instance of this manager
     */
    public static ObjectActionContributorManager getManager() {
        if (sharedInstance is null) {
            sharedInstance = new ObjectActionContributorManager();
        }
        return sharedInstance;
    }

    /**
     * Loads the contributors from the workbench's registry.
     */
    private void loadContributors() {
        ObjectActionContributorReader reader = new ObjectActionContributorReader();
        reader.readPopupContributors(this);
    }
    
    /* (non-Javadoc)
     * @see dwtx.core.runtime.dynamicHelpers.IExtensionChangeHandler#addExtension(dwtx.core.runtime.dynamicHelpers.IExtensionTracker, dwtx.core.runtime.IExtension)
     */
    public void addExtension(IExtensionTracker tracker, IExtension addedExtension) {
        IConfigurationElement[] addedElements = addedExtension.getConfigurationElements();
        for (int i = 0; i < addedElements.length; i++) {
            ObjectActionContributorReader reader = new ObjectActionContributorReader();
            reader.setManager(this);
            reader.readElement(addedElements[i]);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.ObjectContributorManager#getExtensionPointFilter()
     */
    protected String getExtensionPointFilter() {
        return IWorkbenchRegistryConstants.PL_POPUP_MENU;
    }
}
