/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.PinEditorAction;

import dwt.dwthelper.utils;

import dwtx.ui.IEditorPart;
import dwtx.ui.IPropertyListener;
import dwtx.ui.IWorkbenchPartConstants;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.internal.tweaklets.TabBehaviour;
import dwtx.ui.internal.tweaklets.Tweaklets;

/**
 * Action to toggle the pin state of an editor. If an editor is
 * pinned, then it is not reused.
 */
public class PinEditorAction extends ActiveEditorAction {
    private IPropertyListener propListener = new IPropertyListener() {
        public void propertyChanged(Object source, int propId) {
            if (propId is WorkbenchPartReference.INTERNAL_PROPERTY_PINNED) {
                WorkbenchPartReference ref = (WorkbenchPartReference)source;
                setChecked(ref.isPinned());
            } else if (propId is IWorkbenchPartConstants.PROP_DIRTY) {
                if (((TabBehaviour)Tweaklets.get(TabBehaviour.KEY)).autoPinOnDirty()) {
                    WorkbenchPartReference ref = (WorkbenchPartReference) source;
                    if (ref.isDirty()) {
                        ref.setPinned(true);
                    }
                }
            }
        }
    };

    /**
     * Creates a PinEditorAction.
     */
    public PinEditorAction(IWorkbenchWindow window) {
        super(WorkbenchMessages.PinEditorAction_text, window); 
        setActionDefinitionId("dwtx.ui.window.pinEditor"); //$NON-NLS-1$
        setToolTipText(WorkbenchMessages.PinEditorAction_toolTip); 
        setId("dwtx.ui.internal.PinEditorAction"); //$NON-NLS-1$
        // @issue need help constant for this?
        //  WorkbenchHelp.setHelp(this, new Object[] {IHelpContextIds.SAVE_ACTION});
        setImageDescriptor(WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_ETOOL_PIN_EDITOR));
        setDisabledImageDescriptor(WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_ETOOL_PIN_EDITOR_DISABLED));
    }

    /* (non-Javadoc)
     * Method declared on IAction.
     */
    public void run() {
        if (getWorkbenchWindow() is null) {
            // action has been dispose
            return;
        }
        IEditorPart editor = getActiveEditor();
        if (editor !is null) {
            WorkbenchPartReference ref = getReference(editor);
            ref.setPinned(isChecked());
        }
    }

    private WorkbenchPartReference getReference(IEditorPart editor) {
        return (WorkbenchPartReference)((EditorSite)editor.getSite()).getPartReference();
    }
    
    /* (non-Javadoc)
     * Method declared on ActiveEditorAction.
     */
    protected void updateState() {
        if (getWorkbenchWindow() is null || getActivePage() is null) {
            setChecked(false);
            setEnabled(false);
            return;
        }

        IEditorPart editor = getActiveEditor();
        bool enabled = (editor !is null);
        setEnabled(enabled);
        if (enabled) {
            setChecked(getReference(editor).isPinned());
        } else {
            setChecked(false);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.ActiveEditorAction#editorActivated(dwtx.ui.IEditorPart)
     */
    protected void editorActivated(IEditorPart part) {
        super.editorActivated(part);
        if (part !is null) {
            getReference(part).addInternalPropertyListener(propListener);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.ActiveEditorAction#editorDeactivated(dwtx.ui.IEditorPart)
     */
    protected void editorDeactivated(IEditorPart part) {
        super.editorDeactivated(part);
        if (part !is null) {
            getReference(part).removeInternalPropertyListener(propListener);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.actions.ActionFactory.IWorkbenchAction#dispose()
     */
    public void dispose() {
        // deactivate current editor now before super dispose because active editor will be null after call
        editorDeactivated(getActiveEditor());
        super.dispose();
    }
}
