/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.PluginActionContributionItem;

import dwt.dwthelper.utils;

import dwtx.jface.action.ActionContributionItem;
import dwtx.jface.action.IContributionManager;
import dwtx.jface.viewers.ISelection;
import dwtx.ui.IPluginContribution;
import dwtx.ui.PlatformUI;
import dwtx.ui.activities.ActivityManagerEvent;
import dwtx.ui.activities.IActivityManagerListener;
import dwtx.ui.activities.IIdentifier;
import dwtx.ui.activities.IIdentifierListener;
import dwtx.ui.activities.IWorkbenchActivitySupport;
import dwtx.ui.activities.IdentifierEvent;
import dwtx.ui.activities.WorkbenchActivityHelper;

/**
 * Contribution item for actions provided by plugins via workbench action
 * extension points.
 */
public class PluginActionContributionItem extends ActionContributionItem
        implements IIdentifierListener, IActivityManagerListener {

    private IIdentifier identifier = null;

    /**
     * Creates a new contribution item from the given action. The id of the
     * action is used as the id of the item.
     * 
     * @param action
     *            the action
     */
    public PluginActionContributionItem(PluginAction action) {
        // dynamic UI (DDW) - this constructor has changed since 1113
        super(action);
    }

    /**
     * Hook the activity and identifier listener (if necessary);
     * 
     * @since 3.1
     */
    private void hookListeners() {
        PlatformUI.getWorkbench().getActivitySupport().getActivityManager()
                .addActivityManagerListener(this);
        // set up the identifier if necessary
        IIdentifier id = getIdentifier();
        if (id !is null) {
            id.addIdentifierListener(this);
        }
    }
    
    /**
     * Unhook the activity and identifier listener (if necessary);
     * 
     * @since 3.1
     */
    private void unhookListeners() {
        PlatformUI.getWorkbench().getActivitySupport().getActivityManager()
                .removeActivityManagerListener(this);

        IIdentifier id = getIdentifier();
        if (id !is null) {
            id.removeIdentifierListener(this);
        }
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.action.IContributionItem#setParent(dwtx.jface.action.IContributionManager)
     */
    public void setParent(IContributionManager parent) {
        IContributionManager oldParent = getParent();
        super.setParent(parent);
        if (oldParent is parent) {
            return;
        }
        
        if (parent is null) {
            unhookListeners();
        } else {
            hookListeners();
        }
    }
    
    /**
     * Create the IIdentifier reference for this item.
     *
     * @since 3.0
     */
    private IIdentifier getIdentifier() {
        if (!WorkbenchActivityHelper.isFiltering()) {
            return null;
        }
        
        if (identifier is null) {
            IWorkbenchActivitySupport workbenchActivitySupport = PlatformUI
                    .getWorkbench().getActivitySupport();
            IPluginContribution contribution = (IPluginContribution) getAction();
            // no need to check if contribution.getPluginId() is null - plugin
            // actions are always from plugins.
            identifier = workbenchActivitySupport.getActivityManager()
                    .getIdentifier(
                            WorkbenchActivityHelper
                                    .createUnifiedId(contribution));
        }
        return identifier;
    }

    /**
     * Dispose of the IIdentifier if necessary.
     * 
     * @since 3.0
     */
    private void disposeIdentifier() {
        identifier = null;
    }

    /**
     * The default implementation of this <code>IContributionItem</code>
     * method notifies the delegate if loaded and implements the <code>IActionDelegate2</code>
     * interface.
     */
    public void dispose() {
        unhookListeners();
        disposeIdentifier();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.ActionContributionItem#isVisible()
     */
    public bool isVisible() {
        if (identifier !is null && !identifier.isEnabled()) {
            return false;
        }
        return super.isVisible();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IIdentifierListener#identifierChanged(dwtx.ui.activities.IdentifierEvent)
     */
    public void identifierChanged(IdentifierEvent identifierEvent) {
        invalidateParent();
    }

    /**
     * Mark the parent dirty if we have a parent.
     * 
     * @since 3.1
     */
    private void invalidateParent() {
        IContributionManager parent = getParent();
        if (parent !is null) {
            parent.markDirty();
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IActivityManagerListener#activityManagerChanged(dwtx.ui.activities.ActivityManagerEvent)
     */
    public void activityManagerChanged(ActivityManagerEvent activityManagerEvent) {
        // ensure that if we're going from a non-filtering state that we get an identifier
        // and vice versa.
        if (WorkbenchActivityHelper.isFiltering() && identifier is null) {
            hookListeners();
            invalidateParent();
        } else if (!WorkbenchActivityHelper.isFiltering() && identifier !is null) {
            unhookListeners();
            disposeIdentifier();
            invalidateParent();
        }
    }
    
    /*
     * For testing purposes only
     */
    public ISelection getSelection() {
        return ((PluginAction)getAction()).getSelection();
    }
}
