/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Dan Rubel (dan_rubel@instantiations.com) - accessor to get menu id
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.PopupMenuExtender;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import dwt.widgets.Display;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtensionDelta;
import dwtx.core.runtime.IExtensionPoint;
import dwtx.core.runtime.IRegistryChangeEvent;
import dwtx.core.runtime.IRegistryChangeListener;
import dwtx.core.runtime.Platform;
import dwtx.jface.action.ContributionManager;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.IMenuListener2;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.MenuManager;
import dwtx.jface.action.Separator;
import dwtx.jface.action.SubMenuManager;
import dwtx.jface.viewers.ISelection;
import dwtx.jface.viewers.ISelectionChangedListener;
import dwtx.jface.viewers.ISelectionProvider;
import dwtx.jface.viewers.StructuredSelection;
import dwtx.ui.IEditorPart;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.internal.menus.InternalMenuService;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.menus.IMenuService;
import dwtx.ui.menus.MenuUtil;

/**
 * This class extends a single popup menu
 */
public class PopupMenuExtender implements IMenuListener2,
        IRegistryChangeListener {
    
    /**
     * The bit in <code>bitSet</code> that stores whether the static actions
     * have been read from the registry.
     */
    private static final int STATIC_ACTION_READ = 1;

    /**
     * The bit in <code>bitSet</code> that stores whether the editor input
     * should be included for the sake of object contributions.
     */
    private static final int INCLUDE_EDITOR_INPUT = 1 << 1;

    private final MenuManager menu;

    private SubMenuManager menuWrapper;

    private final ISelectionProvider selProvider;

    private final IWorkbenchPart part;

    private Map staticActionBuilders = null;

    /**
     * The bool properties maintained by this extender. A bit set is used to
     * save memory.
     */
    private int bitSet = 0;
    
    private ArrayList actionContributionCache = new ArrayList();
    private ArrayList managerContributionCache = new ArrayList();

    /**
     * Construct a new menu extender.
     * 
     * @param id
     *            the menu id
     * @param menu
     *            the menu to extend
     * @param prov
     *            the selection provider
     * @param part
     *            the part to extend
     */
    public PopupMenuExtender(String id, MenuManager menu,
            ISelectionProvider prov, IWorkbenchPart part) {
        this(id, menu, prov, part, true);
    }

    /**
     * Construct a new menu extender.
     * 
     * @param id
     *            the menu id
     * @param menu
     *            the menu to extend
     * @param prov
     *            the selection provider
     * @param part
     *            the part to extend
     * @param includeEditorInput
     *            Whether the editor input should be included when adding object
     *            contributions to this context menu.
     */
    public PopupMenuExtender(final String id, final MenuManager menu,
            final ISelectionProvider prov, final IWorkbenchPart part,
            final bool includeEditorInput) {
        super();
        this.menu = menu;
        this.selProvider = prov;
        this.part = part;
        if (includeEditorInput) {
            bitSet |= INCLUDE_EDITOR_INPUT;
        }
        menu.addMenuListener(this);
        if (!menu.getRemoveAllWhenShown()) {
            menuWrapper = new SubMenuManager(menu);
            menuWrapper.setVisible(true);
        }
        readStaticActionsFor(id);
                
        Platform.getExtensionRegistry().addRegistryChangeListener(this);
    }

    // getMenuId() added by Dan Rubel (dan_rubel@instantiations.com)
    /**
     * Return the menu identifiers for this extender.
     * 
     * @return The set of all identifiers that represent this extender.
     */
    public Set getMenuIds() {
        if (staticActionBuilders is null) {
            return Collections.EMPTY_SET;
        }
        
        return staticActionBuilders.keySet();
    }

    /**
     * <p>
     * Adds another menu identifier to this extender. An extender can represent
     * many menu identifiers. These identifiers should represent the same menu
     * manager, selection provider and part. Duplicate identifiers are
     * automatically ignored.
     * </p>
     * <p>
     * For example, it is necessary to filter out duplicate identifiers for
     * <code>CompilationUnitEditor</code> instances, as these define both
     * <code>"#CompilationUnitEditorContext"</code> and
     * <code>"dwtx.jdt.ui.CompilationUnitEditor.EditorContext"</code>
     * as menu identifier for the same pop-up menu. We don't want to contribute
     * duplicate items in this case.
     * </p>
     * 
     * @param menuId
     *            The menu identifier to add to this extender; should not be
     *            <code>null</code>.
     */
    public final void addMenuId(final String menuId) {
        bitSet &= ~STATIC_ACTION_READ;
        readStaticActionsFor(menuId);
    }

    /**
     * Determines whether this extender would be the same as another extender
     * created with the given values. Two extenders are equivalent if they have
     * the same menu manager, selection provider and part (i.e., if the menu
     * they represent is about to show, they would populate it with duplicate
     * values).
     * 
     * @param menuManager
     *            The menu manager with which to compare; may be
     *            <code>null</code>.
     * @param selectionProvider
     *            The selection provider with which to compare; may be
     *            <code>null</code>.
     * @param part
     *            The part with which to compare; may be <code>null</code>.
     * @return <code>true</code> if the menu manager, selection provider and
     *         part are all the same.
     */
    public final bool matches(final MenuManager menuManager,
            final ISelectionProvider selectionProvider,
            final IWorkbenchPart part) {
        return (this.menu is menuManager)
                && (this.selProvider is selectionProvider)
                && (this.part is part);
    }

    /**
     * Contributes items registered for the currently active editor.
     */
    private void addEditorActions(IMenuManager mgr) {
        ISelectionProvider activeEditor = new ISelectionProvider() {

            /* (non-Javadoc)
             * @see dwtx.jface.viewers.ISelectionProvider#addSelectionChangedListener(dwtx.jface.viewers.ISelectionChangedListener)
             */
            public void addSelectionChangedListener(
                    ISelectionChangedListener listener) {
                throw new UnsupportedOperationException(
                "This ISelectionProvider is static, and cannot be modified."); //$NON-NLS-1$
            }

            /* (non-Javadoc)
             * @see dwtx.jface.viewers.ISelectionProvider#getSelection()
             */
            public ISelection getSelection() {
                if (part instanceof IEditorPart) {
                    final IEditorPart editorPart = (IEditorPart) part;
                    return new StructuredSelection(new Object[] { editorPart
                            .getEditorInput() });
                }

                return new StructuredSelection(new Object[0]);
            }

            /* (non-Javadoc)
             * @see dwtx.jface.viewers.ISelectionProvider#removeSelectionChangedListener(dwtx.jface.viewers.ISelectionChangedListener)
             */
            public void removeSelectionChangedListener(
                    ISelectionChangedListener listener) {
                throw new UnsupportedOperationException(
                "This ISelectionProvider is static, and cannot be modified."); //$NON-NLS-1$
            }

            /* (non-Javadoc)
             * @see dwtx.jface.viewers.ISelectionProvider#setSelection(dwtx.jface.viewers.ISelection)
             */
            public void setSelection(ISelection selection) {
                throw new UnsupportedOperationException(
                        "This ISelectionProvider is static, and cannot be modified."); //$NON-NLS-1$
            }
        };
        
        if (ObjectActionContributorManager.getManager()
                .contributeObjectActions(part, mgr, activeEditor)) {
            mgr.add(new Separator());
        }
    }

    /**
     * Contributes items registered for the object type(s) in
     * the current selection.
     */
    private void addObjectActions(IMenuManager mgr) {
        if (selProvider !is null) {
            if (ObjectActionContributorManager.getManager()
                    .contributeObjectActions(part, mgr, selProvider)) {
                mgr.add(new Separator());
            }
        }
    }
    
    /**
     * Disposes all of the static actions.
     */
    private final void clearStaticActions() {
        bitSet &= ~STATIC_ACTION_READ;
        if (staticActionBuilders !is null) {
            final Iterator staticActionBuilderItr = staticActionBuilders
                    .values().iterator();
            while (staticActionBuilderItr.hasNext()) {
                final Object staticActionBuilder = staticActionBuilderItr
                        .next();
                if (staticActionBuilder instanceof ViewerActionBuilder) {
                    ((ViewerActionBuilder) staticActionBuilder).dispose();
                }
            }
        }
    }

    /**
     * Adds static items to the context menu.
     */
    private void addStaticActions(IMenuManager mgr) {
        if (staticActionBuilders !is null) {
            final Iterator staticActionBuilderItr = staticActionBuilders
                    .values().iterator();
            while (staticActionBuilderItr.hasNext()) {
                final ViewerActionBuilder staticActionBuilder = (ViewerActionBuilder) staticActionBuilderItr
                        .next();
                staticActionBuilder.contribute(mgr, null, true);
            }
        }
    }

    /**
     * Notifies the listener that the menu is about to be shown.
     */
    public void menuAboutToShow(IMenuManager mgr) {
        IMenuManager originalManager = mgr;
        
        // Add this menu as a visible menu.
        final IWorkbenchPartSite site = part.getSite();
        if (site !is null) {
            final IWorkbench workbench = site.getWorkbenchWindow()
                    .getWorkbench();
            if (workbench instanceof Workbench) {
                final Workbench realWorkbench = (Workbench) workbench;
                ISelection input = null;
                if ((bitSet & INCLUDE_EDITOR_INPUT) !is 0) {
                    if (part instanceof IEditorPart) {
                        final IEditorPart editorPart = (IEditorPart) part;
                        input = new StructuredSelection(
                                new Object[] { editorPart.getEditorInput() });
                    }
                }
                ISelection s = (selProvider is null ? null : selProvider
                        .getSelection());
                realWorkbench.addShowingMenus(getMenuIds(), s, input);
            }
        }
        
        readStaticActions();
        // test for additions removed to comply with menu contributions
        if (menuWrapper !is null) {
            mgr = menuWrapper;
            menuWrapper.removeAll();
        }
        addMenuContributions(originalManager);
        if ((bitSet & INCLUDE_EDITOR_INPUT) !is 0) {
            addEditorActions(mgr);
        }
        addObjectActions(mgr);
        addStaticActions(mgr);
        cleanUpContributionCache();
    }
    
    private bool contributionsPopulated = false;
    
    private void addMenuContributions(IMenuManager mgr) {
        final IMenuService menuService = (IMenuService) part.getSite()
                .getService(IMenuService.class);
        if (menuService is null) {
            return;
        }
        if ((mgr.getRemoveAllWhenShown() || !contributionsPopulated)
                && mgr instanceof ContributionManager) {
            ContributionManager manager = (ContributionManager) mgr;
            contributionsPopulated = true;
            menuService
                    .populateContributionManager(manager, MenuUtil.ANY_POPUP);
            Iterator i = getMenuIds().iterator();
            InternalMenuService realService = (InternalMenuService) menuService;
            while (i.hasNext()) {
                String id = "popup:" + i.next(); //$NON-NLS-1$
                realService.populateContributionManager(manager, id, false);
            }
        }
    }

    /**
     * Notifies the listener that the menu is about to be hidden.
     */
    public final void menuAboutToHide(final IMenuManager mgr) {
        gatherContributions(mgr);
        // Remove this menu as a visible menu.
        final IWorkbenchPartSite site = part.getSite();
        if (site !is null) {
            final IWorkbench workbench = site.getWorkbenchWindow().getWorkbench();
            if (workbench instanceof Workbench) {
                // try delaying this until after the selection event
                // has been fired.
                // This is less threatening if the popup: menu
                // contributions aren't tied to the evaluation service
                workbench.getDisplay().asyncExec(new Runnable() {
                    public void run() {
                        final Workbench realWorkbench = (Workbench) workbench;
                        realWorkbench.removeShowingMenus(getMenuIds(), null, null);
                        cleanUpContributionCache();
                    }
                });
            }
        }
    }

    private void gatherContributions(final IMenuManager mgr) {
        final IContributionItem[] items = mgr.getItems();
        for (int i = 0; i < items.length; i++) {
            if (items[i] instanceof PluginActionContributionItem) {
                actionContributionCache.add(items[i]);
            } else if (items[i] instanceof IMenuManager) {
                if (items[i] instanceof ContributionManager) {
                    managerContributionCache.add(items[i]);
                }
                gatherContributions(((IMenuManager)items[i]));
            }
        }
    }
    
    private void cleanUpContributionCache() {
        if (!actionContributionCache.isEmpty()) {
            PluginActionContributionItem[] items = (PluginActionContributionItem[]) actionContributionCache
                    .toArray(new PluginActionContributionItem[actionContributionCache
                            .size()]);
            actionContributionCache.clear();
            for (int i = 0; i < items.length; i++) {
                items[i].dispose();
            }
        }
        if (!managerContributionCache.isEmpty() && menu.getRemoveAllWhenShown()) {
            ContributionManager[] items = (ContributionManager[]) managerContributionCache
                    .toArray(new ContributionManager[managerContributionCache
                            .size()]);
            managerContributionCache.clear();
            final IMenuService menuService = (IMenuService) part.getSite()
                    .getService(IMenuService.class);
            if (menuService instanceof InternalMenuService) {
                InternalMenuService realService = (InternalMenuService) menuService;
                for (int i = 0; i < items.length; i++) {
                    realService.releaseContributions(items[i]);
                    items[i].removeAll();
                }
            }
        } else {
            managerContributionCache.clear();
        }
    }

    /**
     * Read all of the static items for the content menu.
     */
    private final void readStaticActions() {
        if (staticActionBuilders !is null) {
            final Iterator menuIdItr = staticActionBuilders.keySet().iterator();
            while (menuIdItr.hasNext()) {
                final String menuId = (String) menuIdItr.next();
                readStaticActionsFor(menuId);
            }
        }
    }

    /**
     * Read static items for a particular menu id, into the context menu.
     */
    private void readStaticActionsFor(final String menuId) {
        if ((bitSet & STATIC_ACTION_READ) !is 0) {
            return;
        }

        bitSet |= STATIC_ACTION_READ;

        // If no menu id provided, then there is no contributions
        // to add. Fix for bug #33140.
        if ((menuId is null) || (menuId.length() < 1)) {
            return;
        }

        if (staticActionBuilders is null) {
            staticActionBuilders = new HashMap();
        }

        Object object = staticActionBuilders.get(menuId);
        if (!(object instanceof ViewerActionBuilder)) {
            object = new ViewerActionBuilder();
            staticActionBuilders.put(menuId, object);
        }
        final ViewerActionBuilder staticActionBuilder = (ViewerActionBuilder) object;
        staticActionBuilder.readViewerContributions(menuId, selProvider, part);
    }

    /**
     * Dispose of the menu extender. Should only be called when the part
     * is disposed.
     */
    public void dispose() {
        clearStaticActions();
        final IMenuService menuService = (IMenuService) part.getSite()
                .getService(IMenuService.class);
        if (menuService !is null) {
            menuService.releaseContributions(menu);
        }
        Platform.getExtensionRegistry().removeRegistryChangeListener(this);
        menu.removeMenuListener(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.runtime.IRegistryChangeListener#registryChanged(dwtx.core.runtime.IRegistryChangeEvent)
     */
    public void registryChanged(final IRegistryChangeEvent event) {
        Display display = Display.getDefault();
        if (part !is null) {
            display = part.getSite().getPage().getWorkbenchWindow().getWorkbench().getDisplay();
        }
        //check the delta to see if there are any viewer contribution changes.  if so, null our builder to cause reparsing on the next menu show
        IExtensionDelta [] deltas = event.getExtensionDeltas();
        for (int i = 0; i < deltas.length; i++) {
            IExtensionDelta delta = deltas[i];
            IExtensionPoint extensionPoint = delta.getExtensionPoint();
            if (extensionPoint.getNamespace().equals(
                    WorkbenchPlugin.PI_WORKBENCH)
                    && extensionPoint.getSimpleIdentifier().equals(
                            IWorkbenchRegistryConstants.PL_POPUP_MENU)) {

                bool clearPopups = false;
                IConfigurationElement [] elements = delta.getExtension().getConfigurationElements();
                for (int j = 0; j < elements.length; j++) {
                    IConfigurationElement element = elements[j];
                    if (element.getName().equals(IWorkbenchRegistryConstants.TAG_VIEWER_CONTRIBUTION)) {
                        clearPopups = true;
                        break;
                    }                   
                }
                                        
                if (clearPopups) {
                    display.syncExec(new Runnable() {
                        public void run() {
                            clearStaticActions();
                        }
                    });
                }
            }
        }
    }
    
    public MenuManager getManager() {
        return menu;
    }
}
