/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.SaveablesList;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import dwt.DWT;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.AssertionFailedException;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.ListenerList;
import dwtx.core.runtime.SubProgressMonitor;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.jface.dialogs.MessageDialogWithToggle;
import dwtx.jface.operation.IRunnableContext;
import dwtx.jface.operation.IRunnableWithProgress;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.jface.viewers.ArrayContentProvider;
import dwtx.jface.viewers.ILabelProvider;
import dwtx.jface.viewers.IStructuredContentProvider;
import dwtx.jface.window.IShellProvider;
import dwtx.osgi.util.NLS;
import dwtx.ui.ISaveablePart;
import dwtx.ui.ISaveablePart2;
import dwtx.ui.ISaveablesLifecycleListener;
import dwtx.ui.ISaveablesSource;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.Saveable;
import dwtx.ui.SaveablesLifecycleEvent;
import dwtx.ui.dialogs.ListSelectionDialog;
import dwtx.ui.internal.dialogs.EventLoopProgressMonitor;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.internal.util.PrefUtil;
import dwtx.ui.model.WorkbenchPartLabelProvider;

/**
 * The model manager maintains a list of open saveable models.
 * 
 * @see Saveable
 * @see ISaveablesSource
 * 
 * @since 3.2
 */
public class SaveablesList implements ISaveablesLifecycleListener {

    private ListenerList listeners = new ListenerList();

    // event source (mostly ISaveablesSource) -> Set of Saveable
    private Map modelMap = new HashMap();

    // reference counting map, Saveable -> Integer
    private Map modelRefCounts = new HashMap();

    private Set nonPartSources = new HashSet();

    /**
     * Returns the list of open models managed by this model manager.
     * 
     * @return a list of models
     */
    public Saveable[] getOpenModels() {
        Set allDistinctModels = new HashSet();
        Iterator saveables = modelMap.values().iterator();
        while (saveables.hasNext())
            allDistinctModels.addAll((Set)saveables.next());
        
        return (Saveable[]) allDistinctModels.toArray(
                new Saveable[allDistinctModels.size()]);
    }

    // returns true if this model has not yet been in getModels()
    private bool addModel(Object source, Saveable model) {
        if (model is null) {
            logWarning(
                    "Ignored attempt to add invalid saveable", source, model); //$NON-NLS-1$
            return false;
        }
        bool result = false;
        Set modelsForSource = (Set) modelMap.get(source);
        if (modelsForSource is null) {
            modelsForSource = new HashSet();
            modelMap.put(source, modelsForSource);
        }
        if (modelsForSource.add(model)) {
            result = incrementRefCount(modelRefCounts, model);
        } else {
            logWarning(
                    "Ignored attempt to add saveable that was already registered", source, model); //$NON-NLS-1$
        }
        return result;
    }

    /**
     * returns true if the given key was added for the first time
     * 
     * @param referenceMap
     * @param key
     * @return true if the ref count of the given key is now 1
     */
    private bool incrementRefCount(Map referenceMap, Object key) {
        bool result = false;
        Integer refCount = (Integer) referenceMap.get(key);
        if (refCount is null) {
            result = true;
            refCount = new Integer(0);
        }
        referenceMap.put(key, new Integer(refCount.intValue() + 1));
        return result;
    }

    /**
     * returns true if the given key has been removed
     * 
     * @param referenceMap
     * @param key
     * @return true if the ref count of the given key was 1
     */
    private bool decrementRefCount(Map referenceMap, Object key) {
        bool result = false;
        Integer refCount = (Integer) referenceMap.get(key);
        Assert.isTrue(refCount !is null);
        if (refCount.intValue() is 1) {
            referenceMap.remove(key);
            result = true;
        } else {
            referenceMap.put(key, new Integer(refCount.intValue() - 1));
        }
        return result;
    }

    // returns true if this model was removed from getModels();
    private bool removeModel(Object source, Saveable model) {
        bool result = false;
        Set modelsForSource = (Set) modelMap.get(source);
        if (modelsForSource is null) {
            logWarning(
                    "Ignored attempt to remove a saveable when no saveables were known", source, model); //$NON-NLS-1$
        } else {
            if (modelsForSource.remove(model)) {
                result = decrementRefCount(modelRefCounts, model);
                if (modelsForSource.isEmpty()) {
                    modelMap.remove(source);
                }
            } else {
                logWarning(
                        "Ignored attempt to remove a saveable that was not registered", source, model); //$NON-NLS-1$
            }
        }
        return result;
    }

    private void logWarning(String message, Object source, Saveable model) {
        // create a new exception 
        AssertionFailedException assertionFailedException = new AssertionFailedException("unknown saveable: " + model //$NON-NLS-1$
                + " from part: " + source); //$NON-NLS-1$
        // record the current stack trace to help with debugging
        assertionFailedException.fillInStackTrace();
        WorkbenchPlugin.log(StatusUtil.newStatus(IStatus.WARNING, message,
                assertionFailedException));
    }

    /**
     * This implementation of handleModelLifecycleEvent must be called by
     * implementers of ISaveablesSource whenever the list of models of the model
     * source changes, or when the dirty state of models changes. The
     * ISaveablesSource instance must be passed as the source of the event
     * object.
     * <p>
     * This method may also be called by objects that hold on to models but are
     * not workbench parts. In this case, the event source must be set to an
     * object that is not an instanceof IWorkbenchPart.
     * </p>
     * <p>
     * Corresponding open and close events must originate from the same
     * (identical) event source.
     * </p>
     * <p>
     * This method must be called on the UI thread.
     * </p>
     */
    public void handleLifecycleEvent(SaveablesLifecycleEvent event) {
        if (!(event.getSource() instanceof IWorkbenchPart)) {
            // just update the set of non-part sources. No prompting necessary.
            // See bug 139004.
            updateNonPartSource((ISaveablesSource) event.getSource());
            return;
        }
        Saveable[] modelArray = event.getSaveables();
        switch (event.getEventType()) {
        case SaveablesLifecycleEvent.POST_OPEN:
            addModels(event.getSource(), modelArray);
            break;
        case SaveablesLifecycleEvent.PRE_CLOSE:
            Saveable[] models = event.getSaveables();
            Map modelsDecrementing = new HashMap();
            Set modelsClosing = new HashSet();
            for (int i = 0; i < models.length; i++) {
                incrementRefCount(modelsDecrementing, models[i]);
            }

            fillModelsClosing(modelsClosing, modelsDecrementing);
            bool canceled = promptForSavingIfNecessary(PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow(), modelsClosing, modelsDecrementing,
                    !event.isForce());
            if (canceled) {
                event.setVeto(true);
            }
            break;
        case SaveablesLifecycleEvent.POST_CLOSE:
            removeModels(event.getSource(), modelArray);
            break;
        case SaveablesLifecycleEvent.DIRTY_CHANGED:
            fireModelLifecycleEvent(new SaveablesLifecycleEvent(this, event
                    .getEventType(), event.getSaveables(), false));
            break;
        }
    }

    /**
     * Updates the set of non-part saveables sources.
     * @param source
     */
    private void updateNonPartSource(ISaveablesSource source) {
        Saveable[] saveables = source.getSaveables();
        if (saveables.length is 0) {
            nonPartSources.remove(source);
        } else {
            nonPartSources.add(source);
        }
    }

    /**
     * @param source
     * @param modelArray
     */
    private void removeModels(Object source, Saveable[] modelArray) {
        List removed = new ArrayList();
        for (int i = 0; i < modelArray.length; i++) {
            Saveable model = modelArray[i];
            if (removeModel(source, model)) {
                removed.add(model);
            }
        }
        if (removed.size() > 0) {
            fireModelLifecycleEvent(new SaveablesLifecycleEvent(this,
                    SaveablesLifecycleEvent.POST_OPEN, (Saveable[]) removed
                            .toArray(new Saveable[removed.size()]), false));
        }
    }

    /**
     * @param source
     * @param modelArray
     */
    private void addModels(Object source, Saveable[] modelArray) {
        List added = new ArrayList();
        for (int i = 0; i < modelArray.length; i++) {
            Saveable model = modelArray[i];
            if (addModel(source, model)) {
                added.add(model);
            }
        }
        if (added.size() > 0) {
            fireModelLifecycleEvent(new SaveablesLifecycleEvent(this,
                    SaveablesLifecycleEvent.POST_OPEN, (Saveable[]) added
                            .toArray(new Saveable[added.size()]), false));
        }
    }

    /**
     * @param event
     */
    private void fireModelLifecycleEvent(SaveablesLifecycleEvent event) {
        Object[] listenerArray = listeners.getListeners();
        for (int i = 0; i < listenerArray.length; i++) {
            ((ISaveablesLifecycleListener) listenerArray[i])
                    .handleLifecycleEvent(event);
        }
    }

    /**
     * Adds the given listener to the list of listeners. Has no effect if the
     * same (identical) listener has already been added. The listener will be
     * notified about changes to the models managed by this model manager. Event
     * types include: <br>
     * POST_OPEN when models were added to the list of models <br>
     * POST_CLOSE when models were removed from the list of models <br>
     * DIRTY_CHANGED when the dirty state of models changed
     * <p>
     * Listeners should ignore all other event types, including PRE_CLOSE. There
     * is no guarantee that listeners are notified before models are closed.
     * 
     * @param listener
     */
    public void addModelLifecycleListener(ISaveablesLifecycleListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes the given listener from the list of listeners. Has no effect if
     * the given listener is not contained in the list.
     * 
     * @param listener
     */
    public void removeModelLifecycleListener(ISaveablesLifecycleListener listener) {
        listeners.remove(listener);
    }

    /**
     * @param partsToClose
     * @param save
     * @param window
     * @return the post close info to be passed to postClose
     */
    public Object preCloseParts(List partsToClose, bool save,
            final IWorkbenchWindow window) {
        // reference count (how many occurrences of a model will go away?)
        PostCloseInfo postCloseInfo = new PostCloseInfo();
        for (Iterator it = partsToClose.iterator(); it.hasNext();) {
            IWorkbenchPart part = (IWorkbenchPart) it.next();
            postCloseInfo.partsClosing.add(part);
            if (part instanceof ISaveablePart) {
                ISaveablePart saveablePart = (ISaveablePart) part;
                if (save && !saveablePart.isSaveOnCloseNeeded()) {
                    // pretend for now that this part is not closing
                    continue;
                }
            }
            if (save && part instanceof ISaveablePart2) {
                ISaveablePart2 saveablePart2 = (ISaveablePart2) part;
                // TODO show saveablePart2 before prompting, see
                // EditorManager.saveAll
                int response = SaveableHelper.savePart(saveablePart2, window,
                        true);
                if (response is ISaveablePart2.CANCEL) {
                    // user canceled
                    return null;
                } else if (response !is ISaveablePart2.DEFAULT) {
                    // only include this part in the following logic if it returned
                    // DEFAULT
                    continue;
                }
            }
            Saveable[] modelsFromSource = getSaveables(part);
            for (int i = 0; i < modelsFromSource.length; i++) {
                incrementRefCount(postCloseInfo.modelsDecrementing,
                        modelsFromSource[i]);
            }
        }
        fillModelsClosing(postCloseInfo.modelsClosing,
                postCloseInfo.modelsDecrementing);
        if (save) {
            bool canceled = promptForSavingIfNecessary(window,
                    postCloseInfo.modelsClosing, postCloseInfo.modelsDecrementing, true);
            if (canceled) {
                return null;
            }
        }
        return postCloseInfo;
    }

    /**
     * @param window
     * @param modelsClosing
     * @param canCancel
     * @return true if the user canceled
     */
    private bool promptForSavingIfNecessary(final IWorkbenchWindow window,
            Set modelsClosing, Map modelsDecrementing, bool canCancel) {
        List modelsToOptionallySave = new ArrayList();
        for (Iterator it = modelsDecrementing.keySet().iterator(); it.hasNext();) {
            Saveable modelDecrementing = (Saveable) it.next();
            if (modelDecrementing.isDirty() && !modelsClosing.contains(modelDecrementing)) {
                modelsToOptionallySave.add(modelDecrementing);
            }
        }
        
        bool shouldCancel = modelsToOptionallySave.isEmpty() ? false : promptForSaving(modelsToOptionallySave,
                window, window, canCancel, true);
        
        if (shouldCancel) {
            return true;
        }

        List modelsToSave = new ArrayList();
        for (Iterator it = modelsClosing.iterator(); it.hasNext();) {
            Saveable modelClosing = (Saveable) it.next();
            if (modelClosing.isDirty()) {
                modelsToSave.add(modelClosing);
            }
        }
        return modelsToSave.isEmpty() ? false : promptForSaving(modelsToSave,
                window, window, canCancel, false);
    }

    /**
     * @param modelsClosing
     * @param modelsDecrementing
     */
    private void fillModelsClosing(Set modelsClosing, Map modelsDecrementing) {
        for (Iterator it = modelsDecrementing.keySet().iterator(); it.hasNext();) {
            Saveable model = (Saveable) it.next();
            if (modelsDecrementing.get(model).equals(modelRefCounts.get(model))) {
                modelsClosing.add(model);
            }
        }
    }

    /**
     * Prompt the user to save the given saveables.
     * @param modelsToSave the saveables to be saved
     * @param shellProvider the provider used to obtain a shell in prompting is
     *            required. Clients can use a workbench window for this.
     * @param runnableContext a runnable context that will be used to provide a
     *            progress monitor while the save is taking place. Clients can
     *            use a workbench window for this.
     * @param canCancel whether the operation can be canceled
     * @param stillOpenElsewhere whether the models are referenced by open parts
     * @return true if the user canceled
     */
    public bool promptForSaving(List modelsToSave,
            final IShellProvider shellProvider, IRunnableContext runnableContext, final bool canCancel, bool stillOpenElsewhere) {
        // Save parts, exit the method if cancel is pressed.
        if (modelsToSave.size() > 0) {
            bool canceled = SaveableHelper.waitForBackgroundSaveJobs(modelsToSave);
            if (canceled) {
                return true;
            }

            IPreferenceStore apiPreferenceStore = PrefUtil.getAPIPreferenceStore();
            bool dontPrompt = stillOpenElsewhere && !apiPreferenceStore.getBoolean(IWorkbenchPreferenceConstants.PROMPT_WHEN_SAVEABLE_STILL_OPEN);

            if (dontPrompt) {
                modelsToSave.clear();
                return false;
            } else if (modelsToSave.size() is 1) {
                Saveable model = (Saveable) modelsToSave.get(0);
                // Show a dialog.
                String[] buttons;
                if(canCancel) {
                    buttons = new String[] { IDialogConstants.YES_LABEL,
                            IDialogConstants.NO_LABEL,
                            IDialogConstants.CANCEL_LABEL };
                } else {
                    buttons = new String[] { IDialogConstants.YES_LABEL,
                            IDialogConstants.NO_LABEL};
                }

                // don't save if we don't prompt
                int choice = ISaveablePart2.NO;
                
                MessageDialog dialog;
                if (stillOpenElsewhere) {
                    String message = NLS
                            .bind(
                                    WorkbenchMessages.EditorManager_saveChangesOptionallyQuestion,
                                    model.getName());
                    MessageDialogWithToggle dialogWithToggle = new MessageDialogWithToggle(shellProvider.getShell(),
                            WorkbenchMessages.Save_Resource, null, message,
                            MessageDialog.QUESTION, buttons, 0, WorkbenchMessages.EditorManager_closeWithoutPromptingOption, false) {
                        protected int getShellStyle() {
                            return (canCancel ? DWT.CLOSE : DWT.NONE)
                                    | DWT.TITLE | DWT.BORDER
                                    | DWT.APPLICATION_MODAL
                                    | getDefaultOrientation();
                        }
                    };
                    dialog = dialogWithToggle;
                } else {
                    String message = NLS
                            .bind(
                                    WorkbenchMessages.EditorManager_saveChangesQuestion,
                                    model.getName());
                    dialog = new MessageDialog(shellProvider.getShell(),
                            WorkbenchMessages.Save_Resource, null, message,
                            MessageDialog.QUESTION, buttons, 0) {
                        protected int getShellStyle() {
                            return (canCancel ? DWT.CLOSE : DWT.NONE)
                                    | DWT.TITLE | DWT.BORDER
                                    | DWT.APPLICATION_MODAL
                                    | getDefaultOrientation();
                        }
                    };
                }

                choice = SaveableHelper.testGetAutomatedResponse();
                if (SaveableHelper.testGetAutomatedResponse() is SaveableHelper.USER_RESPONSE) {
                    choice = dialog.open();
                    
                    if(stillOpenElsewhere) {
                        // map value of choice back to ISaveablePart2 values
                        switch (choice) {
                        case IDialogConstants.YES_ID:
                            choice = ISaveablePart2.YES;
                            break;
                        case IDialogConstants.NO_ID:
                            choice = ISaveablePart2.NO;
                            break;
                        case IDialogConstants.CANCEL_ID:
                            choice = ISaveablePart2.CANCEL;
                            break;
                        default:
                            break;
                        }
                        MessageDialogWithToggle dialogWithToggle = (MessageDialogWithToggle) dialog;
                        if (choice !is ISaveablePart2.CANCEL && dialogWithToggle.getToggleState()) {
                            apiPreferenceStore.setValue(IWorkbenchPreferenceConstants.PROMPT_WHEN_SAVEABLE_STILL_OPEN, false);
                        }
                    }
                }

                // Branch on the user choice.
                // The choice id is based on the order of button labels
                // above.
                switch (choice) {
                case ISaveablePart2.YES: // yes
                    break;
                case ISaveablePart2.NO: // no
                    modelsToSave.clear();
                    break;
                default:
                case ISaveablePart2.CANCEL: // cancel
                    return true;
                }
            } else {
                MyListSelectionDialog dlg = new MyListSelectionDialog(
                        shellProvider.getShell(),
                        modelsToSave,
                        new ArrayContentProvider(),
                        new WorkbenchPartLabelProvider(),
                        stillOpenElsewhere ? WorkbenchMessages.EditorManager_saveResourcesOptionallyMessage
                                : WorkbenchMessages.EditorManager_saveResourcesMessage,
                        canCancel, stillOpenElsewhere);
                dlg.setInitialSelections(modelsToSave.toArray());
                dlg.setTitle(EditorManager.SAVE_RESOURCES_TITLE);

                // this "if" statement aids in testing.
                if (SaveableHelper.testGetAutomatedResponse() is SaveableHelper.USER_RESPONSE) {
                    int result = dlg.open();
                    // Just return null to prevent the operation continuing
                    if (result is IDialogConstants.CANCEL_ID)
                        return true;

                    if (dlg.getDontPromptSelection()) {
                        apiPreferenceStore.setValue(IWorkbenchPreferenceConstants.PROMPT_WHEN_SAVEABLE_STILL_OPEN, false);
                    }
                    
                    modelsToSave = Arrays.asList(dlg.getResult());
                }
            }
        }
        // Create save block.
        return saveModels(modelsToSave, shellProvider, runnableContext);
    }

    /**
     * Save the given models.
     * @param finalModels the list of models to be saved
     * @param shellProvider the provider used to obtain a shell in prompting is
     *            required. Clients can use a workbench window for this.
     * @param runnableContext a runnable context that will be used to provide a
     *            progress monitor while the save is taking place. Clients can
     *            use a workbench window for this.
     * @return <code>true</code> if the operation was canceled
     */
    public bool saveModels(final List finalModels, final IShellProvider shellProvider, IRunnableContext runnableContext) {
        IRunnableWithProgress progressOp = new IRunnableWithProgress() {
            public void run(IProgressMonitor monitor) {
                IProgressMonitor monitorWrap = new EventLoopProgressMonitor(
                        monitor);
                monitorWrap.beginTask("", finalModels.size()); //$NON-NLS-1$
                for (Iterator i = finalModels.iterator(); i.hasNext();) {
                    Saveable model = (Saveable) i.next();
                    // handle case where this model got saved as a result of
                    // saving another
                    if (!model.isDirty()) {
                        monitor.worked(1);
                        continue;
                    }
                    SaveableHelper.doSaveModel(model, new SubProgressMonitor(monitorWrap, 1), shellProvider, true);
                    if (monitorWrap.isCanceled())
                        break;
                }
                monitorWrap.done();
            }
        };

        // Do the save.
        return !SaveableHelper.runProgressMonitorOperation(
                WorkbenchMessages.Save_All, progressOp, runnableContext,
                shellProvider);
    }

    private static class PostCloseInfo {
        private List partsClosing = new ArrayList();

        private Map modelsDecrementing = new HashMap();

        private Set modelsClosing = new HashSet();
    }

    /**
     * @param postCloseInfoObject
     */
    public void postClose(Object postCloseInfoObject) {
        PostCloseInfo postCloseInfo = (PostCloseInfo) postCloseInfoObject;
        List removed = new ArrayList();
        for (Iterator it = postCloseInfo.partsClosing.iterator(); it.hasNext();) {
            IWorkbenchPart part = (IWorkbenchPart) it.next();
            Set saveables = (Set) modelMap.get(part);
            if (saveables !is null) {
                // make a copy to avoid a ConcurrentModificationException - we
                // will remove from the original set as we iterate
                saveables = new HashSet(saveables);
                for (Iterator it2 = saveables.iterator(); it2.hasNext();) {
                    Saveable saveable = (Saveable) it2.next();
                    if (removeModel(part, saveable)) {
                        removed.add(saveable);
                    }
                }
            }
        }
        if (removed.size() > 0) {
            fireModelLifecycleEvent(new SaveablesLifecycleEvent(this,
                    SaveablesLifecycleEvent.POST_CLOSE, (Saveable[]) removed
                            .toArray(new Saveable[removed.size()]), false));
        }
    }

    /**
     * Returns the saveable models provided by the given part. If the part does
     * not provide any models, a default model is returned representing the
     * part.
     * 
     * @param part
     *            the workbench part
     * @return the saveable models
     */
    private Saveable[] getSaveables(IWorkbenchPart part) {
        if (part instanceof ISaveablesSource) {
            ISaveablesSource source = (ISaveablesSource) part;
            return source.getSaveables();
        } else if (part instanceof ISaveablePart) {
            return new Saveable[] { new DefaultSaveable(part) };
        } else {
            return new Saveable[0];
        }
    }

    /**
     * @param actualPart
     */
    public void postOpen(IWorkbenchPart part) {
        addModels(part, getSaveables(part));
    }

    /**
     * @param actualPart
     */
    public void dirtyChanged(IWorkbenchPart part) {
        Saveable[] saveables = getSaveables(part);
        if (saveables.length > 0) {
            fireModelLifecycleEvent(new SaveablesLifecycleEvent(this,
                    SaveablesLifecycleEvent.DIRTY_CHANGED, saveables, false));
        }
    }

    /**
     * For testing purposes. Not to be called by clients.
     * 
     * @param model
     * @return
     */
    public Object[] testGetSourcesForModel(Saveable model) {
        List result = new ArrayList();
        for (Iterator it = modelMap.entrySet().iterator(); it.hasNext();) {
            Map.Entry entry = (Map.Entry) it.next();
            Set values = (Set) entry.getValue();
            if (values.contains(model)) {
                result.add(entry.getKey());
            }
        }
        return result.toArray();
    }

    private static final class MyListSelectionDialog extends
            ListSelectionDialog {
        private final bool canCancel;
        private Button checkbox;
        private bool dontPromptSelection;
        private bool stillOpenElsewhere;

        private MyListSelectionDialog(Shell shell, Object input,
                IStructuredContentProvider contentprovider,
                ILabelProvider labelProvider, String message, bool canCancel, bool stillOpenElsewhere) {
            super(shell, input, contentprovider, labelProvider, message);
            this.canCancel = canCancel;
            this.stillOpenElsewhere = stillOpenElsewhere;
            if (!canCancel) {
                int shellStyle = getShellStyle();
                shellStyle &= ~DWT.CLOSE;
                setShellStyle(shellStyle);
            }
        }

        /**
         * @return
         */
        public bool getDontPromptSelection() {
            return dontPromptSelection;
        }

        protected void createButtonsForButtonBar(Composite parent) {
            createButton(parent, IDialogConstants.OK_ID,
                    IDialogConstants.OK_LABEL, true);
            if (canCancel) {
                createButton(parent, IDialogConstants.CANCEL_ID,
                        IDialogConstants.CANCEL_LABEL, false);
            }
        }
        
        protected Control createDialogArea(Composite parent) {
             Composite dialogAreaComposite = (Composite) super.createDialogArea(parent);
             
             if (stillOpenElsewhere) {
                 Composite checkboxComposite = new Composite(dialogAreaComposite, DWT.NONE);
                 checkboxComposite.setLayout(new GridLayout(2, false));
                 
                 checkbox = new Button(checkboxComposite, DWT.CHECK);
                 checkbox.addSelectionListener(new SelectionAdapter() {
                    public void widgetSelected(SelectionEvent e) {
                        dontPromptSelection = checkbox.getSelection();
                    }
                 });
                 GridData gd = new GridData();
                 gd.horizontalAlignment = DWT.BEGINNING;
                 checkbox.setLayoutData(gd);
                 
                 Label label = new Label(checkboxComposite, DWT.NONE);
                 label.setText(WorkbenchMessages.EditorManager_closeWithoutPromptingOption);
                 gd = new GridData();
                 gd.grabExcessHorizontalSpace = true;
                 gd.horizontalAlignment = DWT.BEGINNING;
             }
             
             return dialogAreaComposite;
        }
    }

    /**
     * @return a list of ISaveablesSource objects registered with this saveables
     *         list which are not workbench parts.
     */
    public ISaveablesSource[] getNonPartSources() {
        return (ISaveablesSource[]) nonPartSources
                .toArray(new ISaveablesSource[nonPartSources.size()]);
    }

    /**
     * @param model
     */
    public IWorkbenchPart[] getPartsForSaveable(Saveable model) {
        List result = new ArrayList();
        for (Iterator it = modelMap.entrySet().iterator(); it.hasNext();) {
            Map.Entry entry = (Map.Entry) it.next();
            Set values = (Set) entry.getValue();
            if (values.contains(model) && entry.getKey() instanceof IWorkbenchPart) {
                result.add(entry.getKey());
            }
        }
        return (IWorkbenchPart[]) result.toArray(new IWorkbenchPart[result.size()]);
    }

}
