/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.ShowInHandler;

import dwt.dwthelper.utils;

import java.util.Map;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.jface.viewers.ISelection;
import dwtx.ui.IViewPart;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.PartInitException;
import dwtx.ui.commands.IElementUpdater;
import dwtx.ui.handlers.HandlerUtil;
import dwtx.ui.internal.util.Util;
import dwtx.ui.menus.UIElement;
import dwtx.ui.part.IShowInTarget;
import dwtx.ui.part.ShowInContext;
import dwtx.ui.views.IViewDescriptor;
import dwtx.ui.views.IViewRegistry;

/**
 * The show in command, which only needs a target id.
 * 
 * @since 3.4
 */
public class ShowInHandler extends AbstractHandler implements IElementUpdater {
    static final String SHOW_IN_ID = "dwtx.ui.navigate.showIn"; //$NON-NLS-1$
    static final String TARGET_ID = "dwtx.ui.navigate.showIn.targetId"; //$NON-NLS-1$

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public Object execute(ExecutionEvent event) throws ExecutionException {
        String targetId = event.getParameter(TARGET_ID);
        if (targetId is null) {
            throw new ExecutionException("No targetId specified"); //$NON-NLS-1$
        }
        ShowInContext context = getContext(HandlerUtil
                .getShowInSelection(event), HandlerUtil.getShowInInput(event));
        if (context is null) {
            return null;
        }

        IWorkbenchPage page = HandlerUtil
                .getActiveWorkbenchWindowChecked(event).getActivePage();
        try {
            IViewPart view = page.showView(targetId);
            IShowInTarget target = getShowInTarget(view);
            if (target !is null) {
                target.show(context);
            }
            ((WorkbenchPage) page).performedShowIn(targetId); // TODO: move
            // back up
        } catch (PartInitException e) {
            throw new ExecutionException("Failed to show in", e); //$NON-NLS-1$
        }

        return null;
    }

    /**
     * Returns the <code>ShowInContext</code> to show in the selected target,
     * or <code>null</code> if there is no valid context to show.
     * <p>
     * This implementation obtains the context from global variables provide.
     * showInSelection and showInInput should be available.
     * <p>
     * 
     * @return the <code>ShowInContext</code> to show or <code>null</code>
     */
    private ShowInContext getContext(ISelection showInSelection, Object input) {
        if (input is null && showInSelection is null) {
            return null;
        }
        return new ShowInContext(input, showInSelection);
    }

    /**
     * Returns the <code>IShowInTarget</code> for the given part, or
     * <code>null</code> if it does not provide one.
     * 
     * @param targetPart
     *            the target part
     * @return the <code>IShowInTarget</code> or <code>null</code>
     */
    private IShowInTarget getShowInTarget(IWorkbenchPart targetPart) {
        return (IShowInTarget) Util.getAdapter(targetPart, IShowInTarget.class);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.IElementUpdater#updateElement(dwtx.ui.menus.UIElement,
     *      java.util.Map)
     */
    public void updateElement(UIElement element, Map parameters) {
        String targetId = (String) parameters.get(TARGET_ID);
        if (targetId is null || targetId.length() is 0) {
            return;
        }
        IViewRegistry reg = WorkbenchPlugin.getDefault().getViewRegistry();
        IViewDescriptor desc = reg.find(targetId);
        if (desc !is null) {
            element.setIcon(desc.getImageDescriptor());
            element.setText(desc.getLabel());
        }
    }
}
