/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.ShowViewMenu;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ibm.icu.text.Collator;

import dwt.DWT;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.core.commands.Command;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.IParameter;
import dwtx.core.commands.NotEnabledException;
import dwtx.core.commands.NotHandledException;
import dwtx.core.commands.Parameterization;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.jface.action.Action;
import dwtx.jface.action.ContributionItem;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.IMenuListener;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.MenuManager;
import dwtx.jface.action.Separator;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.activities.WorkbenchActivityHelper;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.internal.intro.IIntroConstants;
import dwtx.ui.menus.CommandContributionItem;
import dwtx.ui.menus.CommandContributionItemParameter;
import dwtx.ui.views.IViewDescriptor;
import dwtx.ui.views.IViewRegistry;

/**
 * A <code>ShowViewMenu</code> is used to populate a menu manager with Show
 * View actions. The visible views are determined by user preference from the
 * Perspective Customize dialog.
 */
public class ShowViewMenu extends ContributionItem {
    public static final String SHOW_VIEW_ID = "dwtx.ui.views.showView"; //$NON-NLS-1$
    public static final String VIEW_ID_PARM = "dwtx.ui.views.showView.viewId"; //$NON-NLS-1$
    private static final String PARAMETER_MAKE_FAST = "dwtx.ui.views.showView.makeFast"; //$NON-NLS-1$

    private IWorkbenchWindow window;

    private static final String NO_TARGETS_MSG = WorkbenchMessages.Workbench_showInNoTargets;

    private Comparator actionComparator = new Comparator() {
        public int compare(Object o1, Object o2) {
            if (collator is null) {
                collator = Collator.getInstance();
            }
            CommandContributionItemParameter a1 = (CommandContributionItemParameter) o1;
            CommandContributionItemParameter a2 = (CommandContributionItemParameter) o2;
            return collator.compare(a1.label, a2.label);
        }
    };

    private Action showDlgAction;

    private Map actions = new HashMap(21);

    // Maps pages to a list of opened views
    private Map openedViews = new HashMap();

    protected bool dirty = true;

    private IMenuListener menuListener = new IMenuListener() {
        public void menuAboutToShow(IMenuManager manager) {
            manager.markDirty();
            dirty = true;
        }
    };
    private bool makeFast;

    private static Collator collator;


    /**
     * Creates a Show View menu.
     * 
     * @param window
     *            the window containing the menu
     * @param id
     *            the id
     */
    public ShowViewMenu(IWorkbenchWindow window, String id) {
        this(window, id, false);
    }

    /**
     * Creates a Show View menu.
     * 
     * @param window
     *            the window containing the menu
     * @param id
     *            the id
     * @param makeFast use the fact view variant of the command
     */
    public ShowViewMenu(IWorkbenchWindow window, String id,
            final bool makeFast) {
        super(id);
        this.window = window;
        this.makeFast = makeFast;
        final IHandlerService handlerService = (IHandlerService) window
                .getService(IHandlerService.class);
        final ICommandService commandService = (ICommandService) window
                .getService(ICommandService.class);
        final ParameterizedCommand cmd = getCommand(commandService, makeFast);

        showDlgAction = new Action(WorkbenchMessages.ShowView_title) {
            public void run() {
                try {
                    handlerService.executeCommand(cmd, null);
                } catch (final ExecutionException e) {
                    // Do nothing.
                } catch (NotDefinedException e) {
                    // Do nothing.
                } catch (NotEnabledException e) {
                    // Do nothing.
                } catch (NotHandledException e) {
                    // Do nothing.
                }
            }
        };

        window.getWorkbench().getHelpSystem().setHelp(showDlgAction,
                IWorkbenchHelpContextIds.SHOW_VIEW_OTHER_ACTION);
        // indicate that a show views submenu has been created
        ((WorkbenchWindow) window)
                .addSubmenu(WorkbenchWindow.SHOW_VIEW_SUBMENU);

        showDlgAction.setActionDefinitionId(SHOW_VIEW_ID);
        
    }

    public bool isDirty() {
        return dirty;
    }

    /**
     * Overridden to always return true and force dynamic menu building.
     */
    public bool isDynamic() {
        return true;
    }

    /**
     * Fills the menu with Show View actions.
     */
    private void fillMenu(IMenuManager innerMgr) {
        // Remove all.
        innerMgr.removeAll();

        // If no page disable all.
        IWorkbenchPage page = window.getActivePage();
        if (page is null) {
            return;
        }

        // If no active perspective disable all
        if (page.getPerspective() is null) {
            return;
        }

        // Get visible actions.
        List viewIds = Arrays.asList(page.getShowViewShortcuts());

        // add all open views
        viewIds = addOpenedViews(page, viewIds);

        List actions = new ArrayList(viewIds.size());
        for (Iterator i = viewIds.iterator(); i.hasNext();) {
            String id = (String) i.next();
            if (id.equals(IIntroConstants.INTRO_VIEW_ID)) {
                continue;
            }
            CommandContributionItemParameter item = getItem(id);
            if (item !is null) {
                actions.add(item);
            }
        }
        Collections.sort(actions, actionComparator);
        for (Iterator i = actions.iterator(); i.hasNext();) {
            CommandContributionItem item = new CommandContributionItem((CommandContributionItemParameter) i.next());
            if (WorkbenchActivityHelper.filterItem(item)) {
                item.dispose();
                continue;
            }
            innerMgr.add(item);
        }

        // Add Other ..
        innerMgr.add(new Separator());
        innerMgr.add(showDlgAction);
    }

    private CommandContributionItemParameter getItem(String viewId) {
        IViewRegistry reg = WorkbenchPlugin.getDefault().getViewRegistry();
        IViewDescriptor desc = reg.find(viewId);
        if (desc is null) {
            return null;
        }
        String label = desc.getLabel();
        
        CommandContributionItemParameter parms = new CommandContributionItemParameter(
                window, viewId, SHOW_VIEW_ID,
                CommandContributionItem.STYLE_PUSH);
        parms.label = label;
        parms.icon = desc.getImageDescriptor();
        parms.parameters = new HashMap();

        parms.parameters.put(VIEW_ID_PARM, viewId);
        if (makeFast) {
            parms.parameters.put(PARAMETER_MAKE_FAST, "true"); //$NON-NLS-1$
        }
        return parms;
    }

    private List addOpenedViews(IWorkbenchPage page, List actions) {
        ArrayList views = getParts(page);
        ArrayList result = new ArrayList(views.size() + actions.size());

        for (int i = 0; i < actions.size(); i++) {
            Object element = actions.get(i);
            if (result.indexOf(element) < 0) {
                result.add(element);
            }
        }
        for (int i = 0; i < views.size(); i++) {
            Object element = views.get(i);
            if (result.indexOf(element) < 0) {
                result.add(element);
            }
        }
        return result;
    }

    private ArrayList getParts(IWorkbenchPage page) {
        ArrayList parts = (ArrayList) openedViews.get(page);
        if (parts is null) {
            parts = new ArrayList();
            openedViews.put(page, parts);
        }
        return parts;
    }

    public void fill(Menu menu, int index) {
        if (getParent() instanceof MenuManager) {
            ((MenuManager) getParent()).addMenuListener(menuListener);
        }

        if (!dirty) {
            return;
        }

        MenuManager manager = new MenuManager();
        fillMenu(manager);
        IContributionItem items[] = manager.getItems();
        if (items.length is 0) {
            MenuItem item = new MenuItem(menu, DWT.NONE, index++);
            item.setText(NO_TARGETS_MSG);
            item.setEnabled(false);
        } else {
            for (int i = 0; i < items.length; i++) {
                items[i].fill(menu, index++);
            }
        }
        dirty = false;
    }

    // for dynamic UI
    protected void removeAction(String viewId) {
        actions.remove(viewId);
    }

    /**
     * @param commandService
     * @param makeFast
     */
    private ParameterizedCommand getCommand(ICommandService commandService,
            final bool makeFast) {
        Command c = commandService.getCommand(SHOW_VIEW_ID);
        Parameterization[] parms = null;
        if (makeFast) {
            try {
                IParameter parmDef = c.getParameter(PARAMETER_MAKE_FAST);
                parms = new Parameterization[] { new Parameterization(parmDef,
                        "true") //$NON-NLS-1$
                };
            } catch (NotDefinedException e) {
                // this should never happen
            }
        }
        return new ParameterizedCommand(c, parms);
    }
}
