/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.ViewIntroAdapterPart;

import dwt.dwthelper.utils;


import dwt.custom.BusyIndicator;
import dwt.graphics.Image;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.ui.IMemento;
import dwtx.ui.IPropertyListener;
import dwtx.ui.IViewSite;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.PartInitException;
import dwtx.ui.internal.intro.IntroMessages;
import dwtx.ui.intro.IIntroPart;
import dwtx.ui.intro.IIntroSite;
import dwtx.ui.part.ViewPart;

/**
 * Simple view that will wrap an <code>IIntroPart</code>.
 * 
 * @since 3.0
 */
public final class ViewIntroAdapterPart extends ViewPart {

    private IIntroPart introPart;

    private IIntroSite introSite;

    private bool handleZoomEvents = true;

    /**
     * Adds a listener that toggles standby state if the view pane is zoomed. 
     */
    private void addPaneListener() {
        IWorkbenchPartSite site = getSite();
        if (site instanceof PartSite) {        
            final WorkbenchPartReference ref = ((WorkbenchPartReference)((PartSite) site).getPartReference()); 
            ref.addInternalPropertyListener(
                    new IPropertyListener() {
                        public void propertyChanged(Object source, int propId) {
                            if (handleZoomEvents) {
                                if (propId is WorkbenchPartReference.INTERNAL_PROPERTY_ZOOMED) {
                                    setStandby(!ref.getPane().isZoomed());
                                }
                            }
                        }
                    });
        }
    }

    /**
     * Forces the standby state of the intro part.
     * 
     * @param standby update the standby state
     */
    public void setStandby(final bool standby) {
        final Control control = ((PartSite) getSite()).getPane().getControl();
        BusyIndicator.showWhile(control.getDisplay(), new Runnable() {
            public void run() {
                try {
                    control.setRedraw(false);
                    introPart.standbyStateChanged(standby);
                } finally {
                    control.setRedraw(true);
                }

                setBarVisibility(standby);
            }
        });
    }

    /**
     * Toggles handling of zoom events.
     * 
     * @param handle whether to handle zoom events
     */
    public void setHandleZoomEvents(bool handle) {
        handleZoomEvents = handle;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPart#createPartControl(dwt.widgets.Composite)
     */
    public void createPartControl(Composite parent) {
        addPaneListener();
        introPart.createPartControl(parent);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPart#dispose()
     */
    public void dispose() {
        setBarVisibility(true);
        super.dispose();
        getSite().getWorkbenchWindow().getWorkbench().getIntroManager()
                .closeIntro(introPart);
        introPart.dispose();
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter) {
        return introPart.getAdapter(adapter);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPart#getTitleImage()
     */
    public Image getTitleImage() {
        return introPart.getTitleImage();
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.part.WorkbenchPart#getTitle()
     */
    public String getTitle() {
        // this method is called eagerly before our init method is called (and
        // therefore before our intropart is created).  By default return 
        // the view title from the view declaration.  We will fire a property
        // change to set the title to the proper value in the init method.
        return introPart is null ? super.getTitle() : introPart.getTitle();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IViewPart#init(dwtx.ui.IViewSite, dwtx.ui.IMemento)
     */
    public void init(IViewSite site, IMemento memento) throws PartInitException {
        super.init(site);
        Workbench workbench = (Workbench) site.getWorkbenchWindow()
                .getWorkbench();
        try {
            introPart = workbench.getWorkbenchIntroManager()
                    .createNewIntroPart();
            // reset the part name of this view to be that of the intro title
            setPartName(introPart.getTitle());
            introPart.addPropertyListener(new IPropertyListener() {
                public void propertyChanged(Object source, int propId) {
                    firePropertyChange(propId);
                }
            });
            introSite = new ViewIntroAdapterSite(site, workbench
                    .getIntroDescriptor());
            introPart.init(introSite, memento);
            
        } catch (CoreException e) {
            WorkbenchPlugin
                    .log(
                            IntroMessages.Intro_could_not_create_proxy, new Status(IStatus.ERROR, WorkbenchPlugin.PI_WORKBENCH, IStatus.ERROR, IntroMessages.Intro_could_not_create_proxy, e)); 
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {
        introPart.setFocus();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IViewPart#saveState(dwtx.ui.IMemento)
     */
    public void saveState(IMemento memento) {
        introPart.saveState(memento);
    }

    /**
     * Sets whether the CoolBar/PerspectiveBar should be visible.
     * 
     * @param visible whether the CoolBar/PerspectiveBar should be visible
     * @since 3.1
     */
    private void setBarVisibility(final bool visible) {
        WorkbenchWindow window = (WorkbenchWindow) getSite()
                .getWorkbenchWindow();
        
        final bool layout = (visible !is window.getCoolBarVisible())
                || (visible !is window.getPerspectiveBarVisible()); // don't layout unless things have actually changed
        if (visible) {
            window.setCoolBarVisible(true);
            window.setPerspectiveBarVisible(true);
        } else {
            window.setCoolBarVisible(false);
            window.setPerspectiveBarVisible(false);
        }

        if (layout) {
            window.getShell().layout();
        }
    }
}
