/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.WindowTrimProxy;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.widgets.Control;
import dwtx.ui.internal.layout.IWindowTrim;

/**
 * A transition class for any control that wants to participate in the workbench
 * window trim. It must supply a unique ID so that it's position can be
 * saved/restored.
 * 
 * @since 3.2
 */
public class WindowTrimProxy implements IWindowTrim {

    private Control fTrimControl;

    private String fId;

    private String fDisplayName;

    private int fValidSides;

    private bool fIsResizeable = false;
    
    private int fWidthHint = DWT.DEFAULT;
    
    private int fHeightHint = DWT.DEFAULT;

    /**
     * Create the trim proxy for a control.
     * 
     * @param c
     *            the trim control
     * @param id
     *            an ID that it's known by
     * @param displayName
     *            the NLS name, for use in created menus
     * @param validSides
     *            bitwise or of valid sides
     * @see #getValidSides()
     */
    public WindowTrimProxy(Control c, String id, String displayName,
            int validSides) {
        fTrimControl = c;
        fId = id;
        fDisplayName = displayName;
        fValidSides = validSides;
    }

    /**
     * Create a trim proxy for a control.
     * 
     * @param c
     * @param id
     * @param displayName
     * @param validSides
     * @param resizeable
     */
    public WindowTrimProxy(Control c, String id, String displayName,
            int validSides, bool resizeable) {
        this(c, id, displayName, validSides);
        fIsResizeable = resizeable;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#getControl()
     */
    public Control getControl() {
        return fTrimControl;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#getValidSides()
     */
    public int getValidSides() {
        return fValidSides;
    }

    /**
     * The default for a proxied window trim is to do nothing, as it can't be
     * moved around.
     * 
     * @see dwtx.ui.internal.layout.IWindowTrim#dock(int)
     */
    public void dock(int dropSide) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#getId()
     */
    public String getId() {
        return fId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#getDisplayName()
     */
    public String getDisplayName() {
        return fDisplayName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#isCloseable()
     */
    public bool isCloseable() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.IWindowTrim#handleClose()
     */
    public void handleClose() {
        // nothing to do...
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWindowTrim#getWidthHint()
     */
    public int getWidthHint() {
        return fWidthHint;
    }
    
    /**
     * Update the width hint for this control.
     * @param w pixels, or DWT.DEFAULT
     */
    public void setWidthHint(int w) {
        fWidthHint = w;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWindowTrim#getHeightHint()
     */
    public int getHeightHint() {
        return fHeightHint;
    }
    
    /**
     * Update the height hint for this control.
     * @param h pixels, or DWT.DEFAULT
     */
    public void setHeightHint(int h) {
        fHeightHint = h;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWindowTrim#isResizeable()
     */
    public bool isResizeable() {
        return fIsResizeable;
    }
}
