/*******************************************************************************
 * Copyright (c) 2003, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.WorkbenchConfigurer;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.core.runtime.IStatus;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.window.WindowManager;
import dwtx.ui.IMemento;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.WorkbenchException;
import dwtx.ui.application.IWorkbenchConfigurer;
import dwtx.ui.application.IWorkbenchWindowConfigurer;
import dwtx.ui.application.WorkbenchAdvisor;

/**
 * Internal class providing special access for configuring the workbench.
 * <p>
 * Note that these objects are only available to the main application
 * (the plug-in that creates and owns the workbench).
 * </p>
 * <p>
 * This class is not intended to be instantiated or subclassed by clients.
 * </p>
 * 
 * @since 3.0
 */
public final class WorkbenchConfigurer implements IWorkbenchConfigurer {

    /**
     * Table to hold arbitrary key-data settings (key type: <code>String</code>,
     * value type: <code>Object</code>).
     * @see #setData
     */
    private Map extraData = new HashMap();

    /**
     * Indicates whether workbench state should be saved on close and 
     * restored on subsequent open.
     */
    private bool saveAndRestore = false;

    /**
     * Indicates whether the workbench is being force to close. During
     * an emergency close, no interaction with the user should be done.
     */
    private bool isEmergencyClosing = false;

    /**
     * Indicates the behaviour when the last window is closed.
     * If <code>true</code>, the workbench will exit (saving the last window's state, 
     * if configured to do so).
     * If <code>false</code> the window will be closed, leaving the workbench running.
     * 
     * @since 3.1
     */
    private bool exitOnLastWindowClose = true;

    /**
     * Creates a new workbench configurer.
     * <p>
     * This method is declared package-private. Clients are passed an instance
     * only via {@link WorkbenchAdvisor#initialize WorkbenchAdvisor.initialize}
     * </p>
     */
    /* package */WorkbenchConfigurer() {
        super();
    }

    /* (non-javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getWorkbench
     */
    public IWorkbench getWorkbench() {
        return PlatformUI.getWorkbench();
    }

    /* (non-javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getWorkbenchWindowManager
     */
    public WindowManager getWorkbenchWindowManager() {
        // return the global workbench window manager
        return ((Workbench) getWorkbench()).getWindowManager();
    }

    /* (non-javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#declareImage
     */
    public void declareImage(String symbolicName, ImageDescriptor descriptor,
            bool shared) {
        if (symbolicName is null || descriptor is null) {
            throw new IllegalArgumentException();
        }
        WorkbenchImages.declareImage(symbolicName, descriptor, shared);
    }

    /* (non-javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getWindowConfigurer
     */
    public IWorkbenchWindowConfigurer getWindowConfigurer(
            IWorkbenchWindow window) {
        if (window is null) {
            throw new IllegalArgumentException();
        }
        return ((WorkbenchWindow) window).getWindowConfigurer();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getSaveAndRestore()
     */
    public bool getSaveAndRestore() {
        return saveAndRestore;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#setSaveAndRestore(bool)
     */
    public void setSaveAndRestore(bool enabled) {
        saveAndRestore = enabled;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getData
     */
    public Object getData(String key) {
        if (key is null) {
            throw new IllegalArgumentException();
        }
        return extraData.get(key);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#setData(String, Object)
     */
    public void setData(String key, Object data) {
        if (key is null) {
            throw new IllegalArgumentException();
        }
        if (data !is null) {
            extraData.put(key, data);
        } else {
            extraData.remove(key);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#emergencyClose()
     */
    public void emergencyClose() {
        if (!isEmergencyClosing) {
            isEmergencyClosing = true;
            if (Workbench.getInstance() !is null
                    && !Workbench.getInstance().isClosing()) {
                Workbench.getInstance().close(
                        PlatformUI.RETURN_EMERGENCY_CLOSE, true);
            }
        }

    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#emergencyClosing()
     */
    public bool emergencyClosing() {
        return isEmergencyClosing;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#restoreState()
     */
    public IStatus restoreState() {
        return ((Workbench) getWorkbench()).restoreState();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#openFirstTimeWindow()
     */
    public void openFirstTimeWindow() {
        ((Workbench) getWorkbench()).openFirstTimeWindow();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#restoreWorkbenchWindow(dwtx.ui.IMemento)
     */
    public IWorkbenchWindowConfigurer restoreWorkbenchWindow(IMemento memento) throws WorkbenchException {
        return getWindowConfigurer(((Workbench) getWorkbench()).restoreWorkbenchWindow(memento));
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#getExitOnLastWindowClose()
     */
    public bool getExitOnLastWindowClose() {
        return exitOnLastWindowClose;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.application.IWorkbenchConfigurer#setExitOnLastWindowClose(bool)
     */
    public void setExitOnLastWindowClose(bool enabled) {
        exitOnLastWindowClose = enabled;
    }
}
