/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.WorkbenchIntroManager;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IExtension;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.core.runtime.dynamichelpers.IExtensionChangeHandler;
import dwtx.core.runtime.dynamichelpers.IExtensionTracker;
import dwtx.ui.IPerspectiveDescriptor;
import dwtx.ui.IViewPart;
import dwtx.ui.IViewReference;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PartInitException;
import dwtx.ui.internal.intro.IIntroConstants;
import dwtx.ui.internal.intro.IntroDescriptor;
import dwtx.ui.internal.intro.IntroMessages;
import dwtx.ui.intro.IIntroManager;
import dwtx.ui.intro.IIntroPart;
import dwtx.ui.intro.IntroContentDetector;

/**
 * Workbench implementation of the IIntroManager interface.
 * 
 * @since 3.0
 */
public class WorkbenchIntroManager implements IIntroManager {   
    
    private final Workbench workbench;

    /**
     * Create a new instance of the receiver.
     * 
     * @param workbench the workbench instance
     */
    WorkbenchIntroManager(Workbench workbench) {
        this.workbench = workbench;
        workbench.getExtensionTracker().registerHandler(new IExtensionChangeHandler(){
            
            /* (non-Javadoc)
             * @see dwtx.core.runtime.dynamicHelpers.IExtensionChangeHandler#addExtension(dwtx.core.runtime.dynamicHelpers.IExtensionTracker, dwtx.core.runtime.IExtension)
             */
            public void addExtension(IExtensionTracker tracker,IExtension extension) {
                //Do nothing
            }
            
            /* (non-Javadoc)
             * @see dwtx.core.runtime.dynamicHelpers.IExtensionChangeHandler#removeExtension(dwtx.core.runtime.IExtension, java.lang.Object[])
             */
            public void removeExtension(IExtension source, Object[] objects) {
                for (int i = 0; i < objects.length; i++) {
                    if (objects[i] instanceof IIntroPart) {
                        closeIntro((IIntroPart) objects[i]);
                    }
                }
                
            }}, null);
        
    }

    /**
     * The currently active introPart in this workspace, <code>null</code> if none.
     */
    private IIntroPart introPart;

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbench#closeIntro(dwtx.ui.intro.IIntroPart)
     */
    public bool closeIntro(IIntroPart part) {
        if (introPart is null || !introPart.equals(part)) {
            return false;
        }

        IViewPart introView = getViewIntroAdapterPart();
        if (introView !is null) {
            //assumption is that there is only ever one intro per workbench
            //if we ever support one per window then this will need revisiting
            IWorkbenchPage page = introView.getSite().getPage();
            IViewReference reference = page
                    .findViewReference(IIntroConstants.INTRO_VIEW_ID);
            page.hideView(introView);
            if (reference is null || reference.getPart(false) is null) {
                introPart = null;                
                return true;
            }
            return false;
        }
        
        // if there is no part then null our reference
        introPart = null;
        
        return true;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbench#showIntro(dwtx.ui.IWorkbenchWindow)
     */
    public IIntroPart showIntro(IWorkbenchWindow preferredWindow,
            bool standby) {
        if (preferredWindow is null) {
            preferredWindow = this.workbench.getActiveWorkbenchWindow();
        }

        if (preferredWindow is null) {
            return null;
        }

        ViewIntroAdapterPart viewPart = getViewIntroAdapterPart();
        if (viewPart is null) {
            createIntro(preferredWindow);
        } else {
            try {
                IWorkbenchPage page = viewPart.getSite().getPage();
                IWorkbenchWindow window = page.getWorkbenchWindow();
                if (!window.equals(preferredWindow)) {
                    window.getShell().setActive();
                }

                page.showView(IIntroConstants.INTRO_VIEW_ID);
            } catch (PartInitException e) {
                WorkbenchPlugin
                        .log(
                                "Could not open intro", new Status(IStatus.ERROR, WorkbenchPlugin.PI_WORKBENCH, IStatus.ERROR, "Could not open intro", e)); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }
        setIntroStandby(introPart, standby);
        return introPart;
    }

    /**  
     * @param testWindow the window to test
     * @return whether the intro exists in the given window
     */
    /*package*/bool isIntroInWindow(IWorkbenchWindow testWindow) {
        ViewIntroAdapterPart viewPart = getViewIntroAdapterPart();
        if (viewPart is null) {
            return false;
        }

        IWorkbenchWindow window = viewPart.getSite().getWorkbenchWindow();
        if (window.equals(testWindow)) {
            return true;
        }
        return false;
    }

    /**
     * Create a new Intro area (a view, currently) in the provided window.  If there is no intro
     * descriptor for this workbench then no work is done.
     *
     * @param preferredWindow the window to create the intro in.
     */
    private void createIntro(IWorkbenchWindow preferredWindow) {
        if (this.workbench.getIntroDescriptor() is null) {
            return;
        }

        IWorkbenchPage workbenchPage = preferredWindow.getActivePage();
        if (workbenchPage is null) {
            return;
        }
        try {
            workbenchPage.showView(IIntroConstants.INTRO_VIEW_ID);
        } catch (PartInitException e) {
            WorkbenchPlugin
                    .log(
                            IntroMessages.Intro_could_not_create_part, new Status(IStatus.ERROR, WorkbenchPlugin.PI_WORKBENCH, IStatus.ERROR, IntroMessages.Intro_could_not_create_part, e));
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbench#setIntroStandby(dwtx.ui.intro.IIntroPart, bool)
     */
    public void setIntroStandby(IIntroPart part, bool standby) {
        if (introPart is null || !introPart.equals(part)) {
            return;
        }

        ViewIntroAdapterPart viewIntroAdapterPart = getViewIntroAdapterPart();
        if (viewIntroAdapterPart is null) {
            return;
        }

        PartPane pane = ((PartSite) viewIntroAdapterPart.getSite()).getPane();
        if (standby is !pane.isZoomed()) {
            // the zoom state is already correct - just update the part's state.
            viewIntroAdapterPart.setStandby(standby);
            return;
        }

        viewIntroAdapterPart.getSite().getPage().toggleZoom(
                pane.getPartReference());
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWorkbench#isIntroStandby(dwtx.ui.intro.IIntroPart)
     */
    public bool isIntroStandby(IIntroPart part) {
        if (introPart is null || !introPart.equals(part)) {
            return false;
        }

        ViewIntroAdapterPart viewIntroAdapterPart = getViewIntroAdapterPart();
        if (viewIntroAdapterPart is null) {
            return false;
        }

        return !((PartSite) viewIntroAdapterPart.getSite()).getPane()
                .isZoomed();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbench#findIntro()
     */
    public IIntroPart getIntro() {
        return introPart;
    }

    /** 
     * @return the <code>ViewIntroAdapterPart</code> for this workbench, <code>null</code> if it 
     * cannot be found.
     */
    /*package*/ViewIntroAdapterPart getViewIntroAdapterPart() {
        IWorkbenchWindow[] windows = this.workbench.getWorkbenchWindows();
        for (int i = 0; i < windows.length; i++) {
            IWorkbenchWindow window = windows[i];
            WorkbenchPage page = (WorkbenchPage) window.getActivePage();
            if (page is null) {
                continue;
            }
            IPerspectiveDescriptor[] perspDescs = page.getOpenPerspectives();
            for (int j = 0; j < perspDescs.length; j++) {
                IPerspectiveDescriptor descriptor = perspDescs[j];
                IViewReference reference = page.findPerspective(descriptor)
                        .findView(IIntroConstants.INTRO_VIEW_ID);
                if (reference !is null) {
                    IViewPart part = reference.getView(false);
                    if (part !is null && part instanceof ViewIntroAdapterPart) {
                        return (ViewIntroAdapterPart) part;
                    }
                }
            }
        }
        return null;
    }

    /**
     * @return a new IIntroPart.  This has the side effect of setting the introPart field to the new
     * value.
     */
    /*package*/IIntroPart createNewIntroPart() throws CoreException {
        IntroDescriptor introDescriptor = workbench.getIntroDescriptor();
        introPart = introDescriptor is null ? null
                : introDescriptor.createIntro();
        if (introPart !is null) {
            workbench.getExtensionTracker().registerObject(
                    introDescriptor.getConfigurationElement()
                            .getDeclaringExtension(), introPart,
                    IExtensionTracker.REF_WEAK);
        }
        return introPart;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbench#hasIntro()
     */
    public bool hasIntro() {
        return workbench.getIntroDescriptor() !is null;
    }
    
    public bool isNewContentAvailable() {
        IntroDescriptor introDescriptor = workbench.getIntroDescriptor();
        if (introDescriptor is null) {
            return false;
        }
        try {
            IntroContentDetector contentDetector = introDescriptor
                    .getIntroContentDetector();
            if (contentDetector !is null) {
                return contentDetector.isNewContentAvailable();
            }
        } catch (CoreException ex) {
            WorkbenchPlugin.log(new Status(IStatus.WARNING,
                    WorkbenchPlugin.PI_WORKBENCH, IStatus.WARNING,
                    "Could not load intro content detector", ex)); //$NON-NLS-1$
        }
        return false;
    }
}
