/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.activities.ws.WorkbenchActivitySupport;

import dwt.dwthelper.utils;

import java.lang.reflect.InvocationTargetException;
import java.util.HashSet;
import java.util.Set;

import dwt.custom.BusyIndicator;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IExtension;
import dwtx.core.runtime.IExtensionPoint;
import dwtx.core.runtime.IProduct;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.dynamichelpers.ExtensionTracker;
import dwtx.core.runtime.dynamichelpers.IExtensionChangeHandler;
import dwtx.core.runtime.dynamichelpers.IExtensionTracker;
import dwtx.jface.action.IContributionManager;
import dwtx.jface.dialogs.ProgressMonitorDialog;
import dwtx.jface.operation.IRunnableWithProgress;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IActionBars;
import dwtx.ui.IViewPart;
import dwtx.ui.IViewReference;
import dwtx.ui.IViewSite;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPage;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.PlatformUI;
import dwtx.ui.activities.ActivityManagerEvent;
import dwtx.ui.activities.IActivity;
import dwtx.ui.activities.IActivityManager;
import dwtx.ui.activities.IActivityManagerListener;
import dwtx.ui.activities.ICategory;
import dwtx.ui.activities.IMutableActivityManager;
import dwtx.ui.activities.ITriggerPointAdvisor;
import dwtx.ui.activities.ITriggerPointManager;
import dwtx.ui.activities.IWorkbenchActivitySupport;
import dwtx.ui.activities.WorkbenchTriggerPointAdvisor;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.WorkbenchWindow;
import dwtx.ui.internal.activities.MutableActivityManager;
import dwtx.ui.internal.activities.ProxyActivityManager;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;

/**
 * Implementation of {@link dwtx.ui.activities.IWorkbenchActivitySupport}.
 * @since 3.0
 */
public class WorkbenchActivitySupport implements IWorkbenchActivitySupport, IExtensionChangeHandler {
    private MutableActivityManager mutableActivityManager;

    private ProxyActivityManager proxyActivityManager;

    private ImageBindingRegistry activityImageBindingRegistry;

    private ImageBindingRegistry categoryImageBindingRegistry;
    
    private ITriggerPointManager triggerPointManager;

    private ITriggerPointAdvisor advisor;

    /**
     * Create a new instance of this class.
     */
    public WorkbenchActivitySupport() {
        triggerPointManager = new TriggerPointManager();
        IExtensionTracker tracker = PlatformUI.getWorkbench().getExtensionTracker();
        tracker.registerHandler(this, ExtensionTracker.createExtensionPointFilter(getActivitySupportExtensionPoint()));
        mutableActivityManager = new MutableActivityManager(getTriggerPointAdvisor());
        proxyActivityManager = new ProxyActivityManager(mutableActivityManager);
        mutableActivityManager
                .addActivityManagerListener(new IActivityManagerListener() {

                    private Set lastEnabled = new HashSet(
                            mutableActivityManager.getEnabledActivityIds());

                    /* (non-Javadoc)
                     * @see dwtx.ui.activities.IActivityManagerListener#activityManagerChanged(dwtx.ui.activities.ActivityManagerEvent)
                     */
                    public void activityManagerChanged(
                            ActivityManagerEvent activityManagerEvent) {
                        Set activityIds = mutableActivityManager
                                .getEnabledActivityIds();
                        // only update the windows if we've not processed this new enablement state already.
                        if (!activityIds.equals(lastEnabled)) {
                            lastEnabled = new HashSet(activityIds);

                            // abort if the workbench isn't running
                            if (!PlatformUI.isWorkbenchRunning()) {
                                return;
                            }

                            // refresh the managers on all windows.
                            final IWorkbench workbench = PlatformUI
                                    .getWorkbench();
                            IWorkbenchWindow[] windows = workbench
                                    .getWorkbenchWindows();
                            for (int i = 0; i < windows.length; i++) {
                                if (windows[i] instanceof WorkbenchWindow) {
                                    final WorkbenchWindow window = (WorkbenchWindow) windows[i];

                                    final ProgressMonitorDialog dialog = new ProgressMonitorDialog(
                                            window.getShell());

                                    final IRunnableWithProgress runnable = new IRunnableWithProgress() {

                                        /**
                                         * When this operation should open a dialog
                                         */
                                        private long openTime;

                                        /**
                                         * Whether the dialog has been opened yet.
                                         */
                                        private bool dialogOpened = false;

                                        /* (non-Javadoc)
                                         * @see dwtx.jface.operation.IRunnableWithProgress#run(dwtx.core.runtime.IProgressMonitor)
                                         */
                                        public void run(IProgressMonitor monitor) {

                                            openTime = System
                                                    .currentTimeMillis()
                                                    + workbench
                                                            .getProgressService()
                                                            .getLongOperationTime();

                                            //two work units - updating the window bars, and updating view bars
                                            monitor
                                                    .beginTask(
                                                            ActivityMessages.ManagerTask, 2);

                                            monitor
                                                    .subTask(ActivityMessages.ManagerWindowSubTask); 

                                            //update window managers...
                                            updateWindowBars(window);
                                            monitor.worked(1);

                                            monitor
                                                    .subTask(ActivityMessages.ManagerViewsSubTask); 
                                            // update all of the (realized) views in all of the pages
                                            IWorkbenchPage[] pages = window
                                                    .getPages();
                                            for (int j = 0; j < pages.length; j++) {
                                                IWorkbenchPage page = pages[j];
                                                IViewReference[] refs = page
                                                        .getViewReferences();
                                                for (int k = 0; k < refs.length; k++) {
                                                    IViewPart part = refs[k]
                                                            .getView(false);
                                                    if (part !is null) {
                                                        updateViewBars(part);
                                                    }
                                                }
                                            }
                                            monitor.worked(1);

                                            monitor.done();
                                        }

                                        /**
                                         * Update the managers on the given given view.
                                         * 
                                         * @param part the view to update
                                         */
                                        private void updateViewBars(
                                                IViewPart part) {
                                            IViewSite viewSite = part
                                                    .getViewSite();
                                            // check for badly behaving or badly initialized views
                                            if (viewSite is null) {
                                                return;
                                            }
                                            IActionBars bars = viewSite
                                                    .getActionBars();
                                            IContributionManager manager = bars
                                                    .getMenuManager();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                            manager = bars.getToolBarManager();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                            manager = bars
                                                    .getStatusLineManager();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                        }

                                        /**
                                         * Update the managers on the given window.
                                         * 
                                         * @param window the window to update
                                         */
                                        private void updateWindowBars(
                                                final WorkbenchWindow window) {
                                            IContributionManager manager = window
                                                    .getMenuBarManager();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                            manager = window
                                                    .getCoolBarManager2();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                            manager = window
                                                    .getToolBarManager2();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                            manager = window
                                                    .getStatusLineManager();
                                            if (manager !is null) {
                                                updateManager(manager);
                                            }
                                        }

                                        /**
                                         * Update the given manager in the UI thread.
                                         * This may also open the progress dialog if 
                                         * the operation is taking too long.
                                         * 
                                         * @param manager the manager to update
                                         */
                                        private void updateManager(
                                                final IContributionManager manager) {
                                            if (!dialogOpened
                                                    && System
                                                            .currentTimeMillis() > openTime) {
                                                dialog.open();
                                                dialogOpened = true;
                                            }

                                            manager.update(true);
                                        }
                                    };

                                    // don't open the dialog by default - that'll be
                                    // handled by the runnable if we take too long
                                    dialog.setOpenOnRun(false);
                                    // run this in the UI thread
                                    workbench.getDisplay().asyncExec(
                                            new Runnable() {

                                                /* (non-Javadoc)
                                                 * @see java.lang.Runnable#run()
                                                 */
                                                public void run() {
                                                    BusyIndicator
                                                            .showWhile(
                                                                    workbench
                                                                            .getDisplay(),
                                                                    new Runnable() {

                                                                        /* (non-Javadoc)
                                                                         * @see java.lang.Runnable#run()
                                                                         */
                                                                        public void run() {
                                                                            try {
                                                                                dialog
                                                                                        .run(
                                                                                                false,
                                                                                                false,
                                                                                                runnable);
                                                                            } catch (InvocationTargetException e) {
                                                                                log(e);
                                                                            } catch (InterruptedException e) {
                                                                                log(e);
                                                                            }
                                                                        }
                                                                    });
                                                }
                                            });
                                }
                            }
                        }
                    }

                    /**
                     * Logs an error message to the workbench log.
                     * 
                     * @param e the exception to log
                     */
                    private void log(Exception e) {
                        StatusUtil.newStatus(IStatus.ERROR,
                                "Could not update contribution managers", e); //$NON-NLS-1$ 
                    }
                });
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#getActivityManager()
     */
    public IActivityManager getActivityManager() {
        return proxyActivityManager;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#setEnabledActivityIds(java.util.Set)
     */
    public void setEnabledActivityIds(Set enabledActivityIds) {
        mutableActivityManager.setEnabledActivityIds(enabledActivityIds);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#getImageDescriptor(dwtx.ui.activities.IActivity)
     */
    public ImageDescriptor getImageDescriptor(IActivity activity) {
        if (activity.isDefined()) {
            ImageDescriptor descriptor = getActivityImageBindingRegistry()
                    .getImageDescriptor(activity.getId());
            if (descriptor !is null) {
                return descriptor;
            }
        }
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_OBJ_ACTIVITY);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#getImageDescriptor(dwtx.ui.activities.ICategory)
     */
    public ImageDescriptor getImageDescriptor(ICategory category) {
        if (category.isDefined()) {
            ImageDescriptor descriptor = getCategoryImageBindingRegistry()
                    .getImageDescriptor(category.getId());
            if (descriptor !is null) {
                return descriptor;
            }
        }
        return WorkbenchImages
                .getImageDescriptor(IWorkbenchGraphicConstants.IMG_OBJ_ACTIVITY_CATEGORY);
    }
    
    
    /**
     * Return the activity image registry.
     * 
     * @return the activity image registry
     * @since 3.1
     */
    private ImageBindingRegistry getActivityImageBindingRegistry() {
        if (activityImageBindingRegistry is null) {
            activityImageBindingRegistry = new ImageBindingRegistry(IWorkbenchRegistryConstants.TAG_ACTIVITY_IMAGE_BINDING);
            PlatformUI
                    .getWorkbench()
                    .getExtensionTracker()
                    .registerHandler(
                            activityImageBindingRegistry,
                            ExtensionTracker
                                    .createExtensionPointFilter(getActivitySupportExtensionPoint()));
        }
        return activityImageBindingRegistry;
    }
    
    /**
     * Return the category image registry.
     * 
     * @return the category image registry
     * @since 3.1
     */
    private ImageBindingRegistry getCategoryImageBindingRegistry() {
        if (categoryImageBindingRegistry is null) {
            categoryImageBindingRegistry = new ImageBindingRegistry(IWorkbenchRegistryConstants.TAG_CATEGORY_IMAGE_BINDING); 
            PlatformUI
            .getWorkbench()
            .getExtensionTracker()
            .registerHandler(
                    categoryImageBindingRegistry,
                    ExtensionTracker
                            .createExtensionPointFilter(getActivitySupportExtensionPoint()));
        }
        return categoryImageBindingRegistry;
    }

    /**
     * Dispose of the image registries.
     * 
     * @since 3.1
     */
    public void dispose() {
        if (activityImageBindingRegistry !is null) {
            activityImageBindingRegistry.dispose();
            PlatformUI.getWorkbench().getExtensionTracker().unregisterHandler(activityImageBindingRegistry);
        }
        if (categoryImageBindingRegistry !is null) {
            categoryImageBindingRegistry.dispose();
            PlatformUI.getWorkbench().getExtensionTracker().unregisterHandler(categoryImageBindingRegistry);
        }
        
        PlatformUI.getWorkbench().getExtensionTracker().unregisterHandler(this);
    }
    
    /**
     * Return the trigger point advisor.
     * 
     * TODO: should this be part of the interface?
     * 
     * @return the trigger point advisor
     * @since 3.1
     */
    public ITriggerPointAdvisor getTriggerPointAdvisor() {
        if (advisor !is null) {
            return advisor;
        }
        
        IProduct product = Platform.getProduct();
        if (product !is null) {
            TriggerPointAdvisorDescriptor descriptor = TriggerPointAdvisorRegistry
                    .getInstance().getAdvisorForProduct(product.getId());
            if (descriptor !is null) {
                try {
                    advisor = descriptor.createAdvisor();                   
                } catch (CoreException e) {
                    WorkbenchPlugin.log("could not create trigger point advisor", e); //$NON-NLS-1$
                }
            }
        }
        
        if (advisor is null) {
            advisor = new WorkbenchTriggerPointAdvisor();
        }
        
        return advisor;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#getTriggerPointManager()
     */
    public ITriggerPointManager getTriggerPointManager() {
        return triggerPointManager;
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.dynamicHelpers.IExtensionChangeHandler#addExtension(dwtx.core.runtime.dynamicHelpers.IExtensionTracker, dwtx.core.runtime.IExtension)
     */
    public void addExtension(IExtensionTracker tracker, IExtension extension) {
        // reset the advisor if it's the "default" advisor.
        // this will give getAdvisor the chance to find a proper trigger/binding if
        // it exists.
        if (advisor !is null && advisor.getClass().equals(WorkbenchTriggerPointAdvisor.class)) {
            advisor = null;
        }
    }

    /**
     * Return the activity support extension point.
     * 
     * @return the activity support extension point.
     * @since 3.1
     */
    private IExtensionPoint getActivitySupportExtensionPoint() {
        return Platform.getExtensionRegistry().getExtensionPoint(
                PlatformUI.PLUGIN_ID, IWorkbenchRegistryConstants.PL_ACTIVITYSUPPORT);
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.dynamicHelpers.IExtensionChangeHandler#removeExtension(dwtx.core.runtime.IExtension, java.lang.Object[])
     */
    public void removeExtension(IExtension extension, Object[] objects) {
        for (int i = 0; i < objects.length; i++) {
            if (objects[i] is advisor) {
                advisor = null;
                break;
            }
        }
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.activities.IWorkbenchActivitySupport#createWorkingCopy()
     */
    public IMutableActivityManager createWorkingCopy() {
        MutableActivityManager clone = (MutableActivityManager) mutableActivityManager.clone();
        clone.unhookRegistryListeners();
        return clone;
    }
}
