/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.commands.CommandLegacyWrapper;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dwtx.core.commands.Command;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.jface.bindings.BindingManager;
import dwtx.jface.bindings.TriggerSequence;
import dwtx.ui.commands.ExecutionException;
import dwtx.ui.commands.ICommand;
import dwtx.ui.commands.ICommandListener;
import dwtx.ui.commands.NotDefinedException;
import dwtx.ui.commands.NotHandledException;
import dwtx.ui.internal.keys.KeySequenceBinding;
import dwtx.ui.keys.KeySequence;

/**
 * A wrapper around a core command so that it satisfies the deprecated
 * <code>ICommand</code> interface.
 * 
 * @since 3.1
 */
final class CommandLegacyWrapper implements ICommand {

    /**
     * The supporting binding manager; never <code>null</code>.
     */
    private final BindingManager bindingManager;

    /**
     * The wrapped command; never <code>null</code>.
     */
    private final Command command;

    /**
     * A parameterized representation of the command. This is created lazily. If
     * it has not yet been created, it is <code>null</code>.
     */
    private ParameterizedCommand parameterizedCommand;

    /**
     * Constructs a new <code>CommandWrapper</code>
     * 
     * @param command
     *            The command to be wrapped; must not be <code>null</code>.
     * @param bindingManager
     *            The binding manager to support this wrapper; must not be
     *            <code>null</code>.
     */
    CommandLegacyWrapper(final Command command,
            final BindingManager bindingManager) {
        if (command is null) {
            throw new NullPointerException(
                    "The wrapped command cannot be <code>null</code>."); //$NON-NLS-1$
        }

        if (bindingManager is null) {
            throw new NullPointerException(
                    "A binding manager is required to wrap a command"); //$NON-NLS-1$
        }

        this.command = command;
        this.bindingManager = bindingManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#addCommandListener(dwtx.ui.commands.ICommandListener)
     */

    public final void addCommandListener(final ICommandListener commandListener) {
        command.addCommandListener(new LegacyCommandListenerWrapper(
                commandListener, bindingManager));
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#execute(java.util.Map)
     */
    public final Object execute(Map parameterValuesByName)
            throws ExecutionException, NotHandledException {
        try {
            return command.execute(new ExecutionEvent(command,
                    (parameterValuesByName is null) ? Collections.EMPTY_MAP
                            : parameterValuesByName, null, null));
        } catch (final dwtx.core.commands.ExecutionException e) {
            throw new ExecutionException(e);
        } catch (final dwtx.core.commands.NotHandledException e) {
            throw new NotHandledException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getAttributeValuesByName()
     */
    public final Map getAttributeValuesByName() {
        final Map attributeValues = new HashMap();
        // avoid using Boolean.valueOf to allow compilation against JCL
        // Foundation (bug 80053)
        attributeValues.put(ILegacyAttributeNames.ENABLED,
                command.isEnabled() ? Boolean.TRUE : Boolean.FALSE);
        attributeValues.put(ILegacyAttributeNames.HANDLED,
                command.isHandled() ? Boolean.TRUE : Boolean.FALSE);
        return attributeValues;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getCategoryId()
     */
    public final String getCategoryId() throws NotDefinedException {
        try {
            return command.getCategory().getId();
        } catch (final dwtx.core.commands.common.NotDefinedException e) {
            throw new NotDefinedException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getDescription()
     */
    public final String getDescription() throws NotDefinedException {
        try {
            return command.getDescription();
        } catch (final dwtx.core.commands.common.NotDefinedException e) {
            throw new NotDefinedException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getId()
     */
    public final String getId() {
        return command.getId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getKeySequenceBindings()
     */
    public final List getKeySequenceBindings() {
        final List legacyBindings = new ArrayList();
        if (parameterizedCommand is null) {
            parameterizedCommand = new ParameterizedCommand(command, null);
        }
        final TriggerSequence[] activeBindings = bindingManager
                .getActiveBindingsFor(parameterizedCommand);
        final int activeBindingsCount = activeBindings.length;
        for (int i = 0; i < activeBindingsCount; i++) {
            final TriggerSequence triggerSequence = activeBindings[i];
            if (triggerSequence instanceof dwtx.jface.bindings.keys.KeySequence) {
                legacyBindings
                        .add(new KeySequenceBinding(
                                KeySequence
                                        .getInstance((dwtx.jface.bindings.keys.KeySequence) triggerSequence),
                                0));
            }
        }

        return legacyBindings;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#getName()
     */
    public final String getName() throws NotDefinedException {
        try {
            return command.getName();
        } catch (final dwtx.core.commands.common.NotDefinedException e) {
            throw new NotDefinedException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#isDefined()
     */
    public final bool isDefined() {
        return command.isDefined();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#isHandled()
     */
    public final bool isHandled() {
        return command.isHandled();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.ICommand#removeCommandListener(dwtx.ui.commands.ICommandListener)
     */
    public final void removeCommandListener(
            final ICommandListener commandListener) {
        command.removeCommandListener(new LegacyCommandListenerWrapper(
                commandListener, bindingManager));
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public final int compareTo(final Object o) {
        return command.compareTo(o);
    }

}
