/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.contexts.ContextLegacyWrapper;

import dwt.dwthelper.utils;

import dwtx.core.commands.contexts.Context;
import dwtx.core.commands.contexts.ContextManager;
import dwtx.ui.contexts.IContext;
import dwtx.ui.contexts.IContextListener;
import dwtx.ui.contexts.NotDefinedException;
import dwtx.ui.internal.util.Util;

/**
 * This implements the old <code>IContext</code> interface based on the new
 * context implementation in <code>dwtx.ui.contexts</code>. This is a
 * wrapper.
 * 
 * @since 3.1
 */
public class ContextLegacyWrapper implements IContext {

    /**
     * The context manager that maintains the set of active contexts; must not
     * be <code>null</code>.
     */
    private final ContextManager contextManager;

    /**
     * The wrapped instance of context. This value will never be
     * <code>null</code>.
     */
    private final Context wrappedContext;

    /**
     * Constructs a new instance of <code>ContextWrapper</code>.
     * 
     * @param context
     *            The context to wrapper; must not be <code>null</code>.
     * @param contextManager
     *            The context manager that maintains the set of active contexts;
     *            must not be <code>null</code>.
     */
    public ContextLegacyWrapper(final Context context,
            final ContextManager contextManager) {
        if (context is null) {
            throw new NullPointerException(
                    "A wrapper cannot be created on a null context"); //$NON-NLS-1$
        }

        if (contextManager is null) {
            throw new NullPointerException(
                    "A wrapper cannot be created with a null manager"); //$NON-NLS-1$
        }

        wrappedContext = context;
        this.contextManager = contextManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#addContextListener(dwtx.ui.contexts.IContextListener)
     */
    public void addContextListener(IContextListener contextListener) {
        final LegacyContextListenerWrapper wrapper = new LegacyContextListenerWrapper(
                contextListener, contextManager, this);
        wrappedContext.addContextListener(wrapper);

        /*
         * We need to add the listener to the context manager as well, as only
         * the manager advertises changes to the enabled state.
         */
        contextManager.addContextManagerListener(wrapper);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(T)
     */
    public int compareTo(Object o) {
        return Util
                .compare(wrappedContext, ((ContextLegacyWrapper) o).wrappedContext);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#getId()
     */
    public String getId() {
        return wrappedContext.getId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#getName()
     */
    public String getName() throws NotDefinedException {
        try {
            return wrappedContext.getName();
        } catch (final dwtx.core.commands.common.NotDefinedException e) {
            throw new NotDefinedException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#getParentId()
     */
    public String getParentId() throws NotDefinedException {
        try {
            return wrappedContext.getParentId();
        } catch (final dwtx.core.commands.common.NotDefinedException e) {
            throw new NotDefinedException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#isDefined()
     */
    public bool isDefined() {
        return wrappedContext.isDefined();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#isEnabled()
     */
    public bool isEnabled() {
        return contextManager.getActiveContextIds().contains(
                wrappedContext.getId());
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.contexts.IContext#removeContextListener(dwtx.ui.contexts.IContextListener)
     */
    public void removeContextListener(IContextListener contextListener) {
        final LegacyContextListenerWrapper wrapper = new LegacyContextListenerWrapper(
                contextListener, contextManager, this);
        wrappedContext.removeContextListener(wrapper);
        contextManager.removeContextManagerListener(wrapper);
    }

}
