/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.contexts.LegacyContextListenerWrapper;

import dwt.dwthelper.utils;

import java.util.Set;

import dwtx.core.commands.contexts.ContextEvent;
import dwtx.core.commands.contexts.ContextManager;
import dwtx.core.commands.contexts.ContextManagerEvent;
import dwtx.core.commands.contexts.IContextListener;
import dwtx.core.commands.contexts.IContextManagerListener;
import dwtx.ui.contexts.IContext;

/**
 * <p>
 * This wraps an old context listener so it supports the new API. This is used
 * to support attaching old-style listens to the new context objects.
 * </p>
 * 
 * @since 3.1
 */
public class LegacyContextListenerWrapper implements IContextListener,
        IContextManagerListener {

    /**
     * The legacy context that this listener would previously have been attached
     * to. This value is never <code>null</code>.
     */
    private final IContext context;

    /**
     * The context manager used for constructing the context wrapper when an
     * event occurs; must not be <code>null</code>.
     */
    private final ContextManager contextManager;

    /**
     * The listener to be wrapped. This value is never <code>null</code>.
     */
    private final dwtx.ui.contexts.IContextListener wrappedListener;

    /**
     * Constructs a new instance of <code>ContextListenerWrapper</code>.
     * 
     * @param listener
     *            The listener to be wrapped. Must not be <code>null</code>.
     * @param contextManager
     *            The context manager used for constructing the context wrapper
     *            when an event occurs; must not be <code>null</code>.
     * @param context
     *            The legacy context this listener will be listening to; must
     *            not be <code>null</code>.
     */
    public LegacyContextListenerWrapper(
            final dwtx.ui.contexts.IContextListener listener,
            final ContextManager contextManager, final IContext context) {
        if (listener is null) {
            throw new NullPointerException(
                    "Cannot create a listener wrapper on a null listener"); //$NON-NLS-1$
        }

        if (contextManager is null) {
            throw new NullPointerException(
                    "Cannot create a listener wrapper with a null context manager"); //$NON-NLS-1$
        }

        if (context is null) {
            throw new NullPointerException(
                    "Cannot create a listener wrapper with a null context"); //$NON-NLS-1$
        }

        wrappedListener = listener;
        this.contextManager = contextManager;
        this.context = context;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.contexts.IContextListener#contextChanged(dwtx.core.commands.contexts.ContextEvent)
     */
    public final void contextChanged(final ContextEvent contextEvent) {
        wrappedListener
                .contextChanged(new dwtx.ui.contexts.ContextEvent(
                        new ContextLegacyWrapper(contextEvent.getContext(),
                                contextManager), contextEvent
                                .isDefinedChanged(), false, contextEvent
                                .isNameChanged(), contextEvent
                                .isParentIdChanged()));
    }

    public final void contextManagerChanged(final ContextManagerEvent event) {
        final String contextId = context.getId();
        final bool enabledChanged;
        if (event.isActiveContextsChanged()) {
            final Set previousContexts = event.getPreviouslyActiveContextIds();
            final Set currentContexts = contextManager.getActiveContextIds();
            if ((previousContexts !is null)
                    && (previousContexts.contains(contextId))
                    && ((currentContexts is null) || (currentContexts
                            .contains(contextId)))) {
                enabledChanged = true;
            } else if ((currentContexts !is null)
                    && (currentContexts.contains(contextId))
                    && ((previousContexts is null) || (previousContexts
                            .contains(contextId)))) {
                enabledChanged = true;
            } else {
                enabledChanged = false;
            }
        } else {
            enabledChanged = false;
        }

        wrappedListener
                .contextChanged(new dwtx.ui.contexts.ContextEvent(
                        context, false, enabledChanged, false, false));
    }

    public final bool equals(final Object object) {
        if (object instanceof LegacyContextListenerWrapper) {
            final LegacyContextListenerWrapper other = (LegacyContextListenerWrapper) object;
            return wrappedListener.equals(other.wrappedListener);
        }

        if (object instanceof dwtx.ui.contexts.IContextListener) {
            final dwtx.ui.contexts.IContextListener other = (dwtx.ui.contexts.IContextListener) object;
            return wrappedListener.equals(other);
        }

        return false;
    }

    public final int hashCode() {
        return wrappedListener.hashCode();
    }
}
