/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.decorators.FullDecoratorDefinition;

import dwt.dwthelper.utils;


import dwt.graphics.Image;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.Platform;
import dwtx.jface.util.SafeRunnable;
import dwtx.jface.viewers.IBaseLabelProvider;
import dwtx.jface.viewers.ILabelDecorator;
import dwtx.osgi.util.NLS;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;

/**
 * The RunnableDecoratorDefinition is the definition for 
 * decorators that have an ILabelDecorator class to instantiate.
 */

class FullDecoratorDefinition extends DecoratorDefinition {

    ILabelDecorator decorator;

    /**
     * Create a new instance of the receiver with the
     * supplied values.
     */

    FullDecoratorDefinition(String identifier, IConfigurationElement element) {
        super(identifier, element);
    }

    /**
     * Gets the decorator and creates it if it does
     * not exist yet. Throws a CoreException if there is a problem
     * creating the decorator.
     * This method should not be called unless a check for
     * enabled to be true is done first.
     * @return Returns a ILabelDecorator
     */
    protected ILabelDecorator internalGetDecorator() throws CoreException {
        if (labelProviderCreationFailed) {
            return null;
        }

        final CoreException[] exceptions = new CoreException[1];

        if (decorator is null) {
            Platform
                    .run(new SafeRunnable(
                            NLS.bind(WorkbenchMessages.DecoratorManager_ErrorActivatingDecorator, getName() )) { 
                        public void run() {
                            try {
                                decorator = (ILabelDecorator) WorkbenchPlugin
                                        .createExtension(
                                                definingElement,
                                                DecoratorDefinition.ATT_CLASS);
                                decorator.addListener(WorkbenchPlugin
                                        .getDefault().getDecoratorManager());
                            } catch (CoreException exception) {
                                exceptions[0] = exception;
                            }
                        }
                    });
        } else {
            return decorator;
        }

        if (decorator is null) {
            this.labelProviderCreationFailed = true;
            setEnabled(false);
        }

        if (exceptions[0] !is null) {
            throw exceptions[0];
        }

        return decorator;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.decorators.DecoratorDefinition#refreshDecorator()
     */
    protected void refreshDecorator() {
        //Only do something if disabled so as to prevent
        //gratutitous activation
        if (!this.enabled && decorator !is null) {
            IBaseLabelProvider cached = decorator;
            decorator = null;
            disposeCachedDecorator(cached);
        }
    }

    /**
     * Decorate the image provided for the element type.
     * This method should not be called unless a check for
     * isEnabled() has been done first.
     * Return null if there is no image or if an error occurs.
     */
    Image decorateImage(Image image, Object element) {
        try {
            //Internal decorator might be null so be prepared
            ILabelDecorator currentDecorator = internalGetDecorator();
            if (currentDecorator !is null) {
                return currentDecorator.decorateImage(image, element);
            }

        } catch (CoreException exception) {
            handleCoreException(exception);
        }
        return null;
    }

    /**
     * Decorate the text provided for the element type.
     * This method should not be called unless a check for
     * isEnabled() has been done first.
     * Return null if there is no text or if there is an exception.
     */
    String decorateText(String text, Object element) {
        try {
            //Internal decorator might be null so be prepared
            ILabelDecorator currentDecorator = internalGetDecorator();
            if (currentDecorator !is null) {
                return currentDecorator.decorateText(text, element);
            }
        } catch (CoreException exception) {
            handleCoreException(exception);
        }
        return null;
    }

    /**
     * Returns the decorator, or <code>null</code> if not enabled.
     * 
     * @return the decorator, or <code>null</code> if not enabled
     */
    public ILabelDecorator getDecorator() {
        return decorator;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.decorators.DecoratorDefinition#internalGetLabelProvider()
     */
    protected IBaseLabelProvider internalGetLabelProvider()
            throws CoreException {
        return internalGetDecorator();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.decorators.DecoratorDefinition#isFull()
     */
    public bool isFull() {
        return true;
    }

}
