/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.AboutDialog;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.LinkedList;

import dwt.DWT;
import dwt.accessibility.AccessibleAdapter;
import dwt.accessibility.AccessibleEvent;
import dwt.custom.BusyIndicator;
import dwt.custom.ScrolledComposite;
import dwt.custom.StyledText;
import dwt.events.ControlAdapter;
import dwt.events.ControlEvent;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.graphics.Color;
import dwt.graphics.Cursor;
import dwt.graphics.GC;
import dwt.graphics.Image;
import dwt.graphics.Point;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.layout.RowLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwtx.core.runtime.IBundleGroup;
import dwtx.core.runtime.IBundleGroupProvider;
import dwtx.core.runtime.IProduct;
import dwtx.core.runtime.Platform;
import dwtx.jface.action.MenuManager;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.resource.JFaceColors;
import dwtx.osgi.util.NLS;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.ProductProperties;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.about.AboutBundleGroupData;
import dwtx.ui.internal.about.AboutFeaturesButtonManager;
import dwtx.ui.menus.CommandContributionItem;
import dwtx.ui.menus.CommandContributionItemParameter;

/**
 * Displays information about the product.
 */
public class AboutDialog extends ProductInfoDialog {
    private final static int MAX_IMAGE_WIDTH_FOR_TEXT = 250;

    private final static int FEATURES_ID = IDialogConstants.CLIENT_ID + 1;

    private final static int PLUGINS_ID = IDialogConstants.CLIENT_ID + 2;

    private final static int INFO_ID = IDialogConstants.CLIENT_ID + 3;

    private String productName;

    private IProduct product;

    private AboutBundleGroupData[] bundleGroupInfos;

    private ArrayList images = new ArrayList();

    private AboutFeaturesButtonManager buttonManager = new AboutFeaturesButtonManager();

    // TODO should the styled text be disposed? if not then it likely
    //      doesn't need to be a member
    private StyledText text;

    /**
     * Create an instance of the AboutDialog for the given window.
     * @param parentShell The parent of the dialog.
     */
    public AboutDialog(Shell parentShell) {
        super(parentShell);

        product = Platform.getProduct();
        if (product !is null) {
            productName = product.getName();
        }
        if (productName is null) {
            productName = WorkbenchMessages.AboutDialog_defaultProductName;
        }

        // create a descriptive object for each BundleGroup
        IBundleGroupProvider[] providers = Platform.getBundleGroupProviders();
        LinkedList groups = new LinkedList();
        if (providers !is null) {
            for (int i = 0; i < providers.length; ++i) {
                IBundleGroup[] bundleGroups = providers[i].getBundleGroups();
                for (int j = 0; j < bundleGroups.length; ++j) {
                    groups.add(new AboutBundleGroupData(bundleGroups[j]));
                }
            }
        }
        bundleGroupInfos = (AboutBundleGroupData[]) groups
                .toArray(new AboutBundleGroupData[0]);
    }

    /*
     * (non-Javadoc) Method declared on Dialog.
     */
    protected void buttonPressed(int buttonId) {
        switch (buttonId) {
        case FEATURES_ID:
            BusyIndicator.showWhile(getShell().getDisplay(), new Runnable() {
                public void run() {
                    new AboutFeaturesDialog(getShell(), productName, bundleGroupInfos)
                            .open();
                }
            });
            break;
        case PLUGINS_ID:
            BusyIndicator.showWhile(getShell().getDisplay(), new Runnable() {
                public void run() {
                    new AboutPluginsDialog(getShell(), productName).open();
                }
            });
            break;
        case INFO_ID:
            BusyIndicator.showWhile(getShell().getDisplay(), new Runnable() {
                public void run() {
                    new AboutSystemDialog(getShell()).open();
                }
            });
            break;
        default:
            super.buttonPressed(buttonId);
            break;
        }
    }

    public bool close() {
        // dispose all images
        for (int i = 0; i < images.size(); ++i) {
            Image image = (Image) images.get(i);
            image.dispose();
        }

        return super.close();
    }

    /*
     * (non-Javadoc) Method declared on Window.
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(NLS.bind(WorkbenchMessages.AboutDialog_shellTitle,productName ));
        PlatformUI.getWorkbench().getHelpSystem().setHelp(newShell,
                IWorkbenchHelpContextIds.ABOUT_DIALOG);
    }

    /**
     * Add buttons to the dialog's button bar.
     * 
     * Subclasses should override.
     * 
     * @param parent
     *            the button bar composite
     */
    protected void createButtonsForButtonBar(Composite parent) {
        parent.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        // bug 64232: the feature details button should only be created if there
        // are features to show
        if (bundleGroupInfos !is null && bundleGroupInfos.length > 0) {
            createButton(parent, FEATURES_ID, WorkbenchMessages.AboutDialog_featureInfo, false);
        } 

        createButton(parent, PLUGINS_ID, WorkbenchMessages.AboutDialog_pluginInfo, false);
        createButton(parent, INFO_ID, WorkbenchMessages.AboutDialog_systemInfo, false); 

        Label l = new Label(parent, DWT.NONE);
        l.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        GridLayout layout = (GridLayout) parent.getLayout();
        layout.numColumns++;
        layout.makeColumnsEqualWidth = false;

        Button b = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        b.setFocus();
    }

    /**
     * Creates and returns the contents of the upper part 
     * of the dialog (above the button bar).
     *
     * Subclasses should overide.
     *
     * @param parent  the parent composite to contain the dialog area
     * @return the dialog area control
     */
    protected Control createDialogArea(Composite parent) {
        final Cursor hand = new Cursor(parent.getDisplay(), DWT.CURSOR_HAND);
        final Cursor busy = new Cursor(parent.getDisplay(), DWT.CURSOR_WAIT);
        setHandCursor(hand);
        setBusyCursor(busy);
        getShell().addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                setHandCursor(null);
                hand.dispose();
                setBusyCursor(null);
                busy.dispose();
            }
        });

        // brand the about box if there is product info
        Image aboutImage = null;
        if (product !is null) {
            ImageDescriptor imageDescriptor = ProductProperties
                    .getAboutImage(product);
            if (imageDescriptor !is null) {
                aboutImage = imageDescriptor.createImage();
            }

            // if the about image is small enough, then show the text
            if (aboutImage is null
                    || aboutImage.getBounds().width <= MAX_IMAGE_WIDTH_FOR_TEXT) {
                String aboutText = ProductProperties.getAboutText(product);
                if (aboutText !is null) {
                    setItem(scan(aboutText));
                }
            }

            if (aboutImage !is null) {
                images.add(aboutImage);
            }
        }

        // create a composite which is the parent of the top area and the bottom
        // button bar, this allows there to be a second child of this composite with 
        // a banner background on top but not have on the bottom
        Composite workArea = new Composite(parent, DWT.NONE);
        GridLayout workLayout = new GridLayout();
        workLayout.marginHeight = 0;
        workLayout.marginWidth = 0;
        workLayout.verticalSpacing = 0;
        workLayout.horizontalSpacing = 0;
        workArea.setLayout(workLayout);
        workArea.setLayoutData(new GridData(GridData.FILL_BOTH));

        // page group
        Color background = JFaceColors.getBannerBackground(parent.getDisplay());
        Color foreground = JFaceColors.getBannerForeground(parent.getDisplay());
        Composite top = (Composite) super.createDialogArea(workArea);

        // override any layout inherited from createDialogArea 
        GridLayout layout = new GridLayout();
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        layout.verticalSpacing = 0;
        layout.horizontalSpacing = 0;
        top.setLayout(layout);
        top.setLayoutData(new GridData(GridData.FILL_BOTH));
        top.setBackground(background);
        top.setForeground(foreground);

        // the image & text 
        final Composite topContainer = new Composite(top, DWT.NONE);
        topContainer.setBackground(background);
        topContainer.setForeground(foreground);

        layout = new GridLayout();
        layout.numColumns = (aboutImage is null || getItem() is null ? 1 : 2);
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        layout.verticalSpacing = 0;
        layout.horizontalSpacing = 0;
        topContainer.setLayout(layout);
        

        GC gc = new GC(parent);
        // arbitrary default
        int topContainerHeightHint = 100;
        try {
            // default height enough for 6 lines of text
            topContainerHeightHint = Math.max(topContainerHeightHint, gc.getFontMetrics().getHeight() * 6);         
        }
        finally {
            gc.dispose();
        }
        
        //image on left side of dialog
        if (aboutImage !is null) {
            Label imageLabel = new Label(topContainer, DWT.NONE);
            imageLabel.setBackground(background);
            imageLabel.setForeground(foreground);

            GridData data = new GridData();
            data.horizontalAlignment = GridData.FILL;
            data.verticalAlignment = GridData.BEGINNING;
            data.grabExcessHorizontalSpace = false;
            imageLabel.setLayoutData(data);
            imageLabel.setImage(aboutImage);
            topContainerHeightHint = Math.max(topContainerHeightHint, aboutImage.getBounds().height);
        }
        
        GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.verticalAlignment = GridData.FILL;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        data.heightHint = topContainerHeightHint;
        topContainer.setLayoutData(data);
        
        if (getItem() !is null) {
            final int minWidth = 400; // This value should really be calculated
            // from the computeSize(DWT.DEFAULT,
            // DWT.DEFAULT) of all the
            // children in infoArea excluding the
            // wrapped styled text
            // There is no easy way to do this.
            final ScrolledComposite scroller = new ScrolledComposite(topContainer,
                    DWT.V_SCROLL | DWT.H_SCROLL);
            data = new GridData(GridData.FILL_BOTH);
            data.widthHint = minWidth;
            scroller.setLayoutData(data);

            final Composite textComposite = new Composite(scroller, DWT.NONE);
            textComposite.setBackground(background);
            
            layout = new GridLayout();
            textComposite.setLayout(layout);

            text = new StyledText(textComposite, DWT.MULTI | DWT.WRAP | DWT.READ_ONLY);
            text.setCaret(null);
            text.setFont(parent.getFont());
            text.setText(getItem().getText());
            text.setCursor(null);
            text.setBackground(background);
            text.setForeground(foreground);
            setLinkRanges(text, getItem().getLinkRanges());
            addListeners(text);
            
            createTextMenu();
            
            GridData gd = new GridData();
            gd.verticalAlignment = GridData.BEGINNING;
            gd.horizontalAlignment = GridData.FILL;
            gd.grabExcessHorizontalSpace = true;
            text.setLayoutData(gd);

            // Adjust the scrollbar increments
            scroller.getHorizontalBar().setIncrement(20);
            scroller.getVerticalBar().setIncrement(20);

            final bool[] inresize = new bool[1]; // flag to stop unneccesary
            // recursion
            textComposite.addControlListener(new ControlAdapter() {
                public void controlResized(ControlEvent e) {
                    if (inresize[0])
                        return;
                    inresize[0] = true;
                    // required because of bugzilla report 4579
                    textComposite.layout(true);
                    // required because you want to change the height that the
                    // scrollbar will scroll over when the width changes.
                    int width = textComposite.getClientArea().width;
                    Point p = textComposite.computeSize(width, DWT.DEFAULT);
                    scroller.setMinSize(minWidth, p.y);
                    inresize[0] = false;
                }
            });

            scroller.setExpandHorizontal(true);
            scroller.setExpandVertical(true);
            Point p = textComposite.computeSize(minWidth, DWT.DEFAULT);
            textComposite.setSize(p.x, p.y);
            scroller.setMinWidth(minWidth);
            scroller.setMinHeight(p.y);

            scroller.setContent(textComposite);
        }

        // horizontal bar
        Label bar = new Label(workArea, DWT.HORIZONTAL | DWT.SEPARATOR);
        data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        bar.setLayoutData(data);

        // add image buttons for bundle groups that have them
        Composite bottom = (Composite) super.createDialogArea(workArea);
        // override any layout inherited from createDialogArea 
        layout = new GridLayout();
        bottom.setLayout(layout);
        data = new GridData();
        data.horizontalAlignment = DWT.FILL;
        data.verticalAlignment = DWT.FILL;
        data.grabExcessHorizontalSpace = true;
        
        bottom.setLayoutData(data);

        createFeatureImageButtonRow(bottom);

        // spacer
        bar = new Label(bottom, DWT.NONE);
        data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        bar.setLayoutData(data);

        return workArea;
    }

    /**
     * Create the context menu for the text widget.
     * 
     * @since 3.4
     */
    private void createTextMenu() {
        final MenuManager textManager = new MenuManager();
        textManager.add(new CommandContributionItem(
                new CommandContributionItemParameter(PlatformUI
                        .getWorkbench(), null, "dwtx.ui.edit.copy", //$NON-NLS-1$
                        CommandContributionItem.STYLE_PUSH)));
        textManager.add(new CommandContributionItem(
                new CommandContributionItemParameter(PlatformUI
                        .getWorkbench(), null, "dwtx.ui.edit.selectAll", //$NON-NLS-1$
                        CommandContributionItem.STYLE_PUSH)));
        text.setMenu(textManager.createContextMenu(text));
        text.addDisposeListener(new DisposeListener() {

            public void widgetDisposed(DisposeEvent e) {
                textManager.dispose();
            }
        });
        
    }

    private void createFeatureImageButtonRow(Composite parent) {
        Composite featureContainer = new Composite(parent, DWT.NONE);
        RowLayout rowLayout = new RowLayout();
        rowLayout.wrap = true;
        featureContainer.setLayout(rowLayout);
        GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        featureContainer.setLayoutData(data);

        for (int i = 0; i < bundleGroupInfos.length; i++) {
            createFeatureButton(featureContainer, bundleGroupInfos[i]);
        }
    }

    private Button createFeatureButton(Composite parent,
            final AboutBundleGroupData info) {
        if (!buttonManager.add(info)) {
            return null;
        }

        ImageDescriptor desc = info.getFeatureImage();
        Image featureImage = null;

        Button button = new Button(parent, DWT.FLAT | DWT.PUSH);
        button.setData(info);
        featureImage = desc.createImage();
        images.add(featureImage);
        button.setImage(featureImage);
        button
                .setToolTipText(NLS
                        .bind(
                                WorkbenchMessages.AboutDialog_concatenationOfProviderNameAndFeatureName,
                                new Object[] { info.getProviderName(),
                                        info.getName() }));
        
        button.getAccessible().addAccessibleListener(new AccessibleAdapter(){
            /* (non-Javadoc)
             * @see dwt.accessibility.AccessibleAdapter#getName(dwt.accessibility.AccessibleEvent)
             */
            public void getName(AccessibleEvent e) {
                e.result = info.getProviderName();
            }
        });
        button.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
                AboutBundleGroupData[] groupInfos = buttonManager
                        .getRelatedInfos(info);
                AboutBundleGroupData selection = (AboutBundleGroupData) event.widget
                        .getData();

                AboutFeaturesDialog d = new AboutFeaturesDialog(getShell(),
                        productName, groupInfos);
                d.setInitialSelection(selection);
                d.open();
            }
        });

        return button;
    }
}
