/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.AboutSystemDialog;

import dwt.dwthelper.utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import dwt.DWT;
import dwt.dnd.Clipboard;
import dwt.dnd.TextTransfer;
import dwt.dnd.Transfer;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwt.widgets.Text;
import dwtx.core.runtime.IPath;
import dwtx.core.runtime.Platform;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.jface.resource.JFaceResources;
import dwtx.osgi.util.NLS;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.ConfigurationInfo;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;


/**
 * Displays system information about the eclipse application.  The content of
 * what is displayed is selectable through the
 * <code>dwtx.ui.systemSummaryExtensions</code> extension point.
 */
public final class AboutSystemDialog extends ProductInfoDialog {

    private Text text;
    
    private final static int BROWSE_ERROR_LOG_BUTTON = IDialogConstants.CLIENT_ID;

    private final static int COPY_TO_CLIPBOARD_BUTTON = IDialogConstants.CLIENT_ID + 1;
    
    private final static String ERROR_LOG_COPY_FILENAME = "log"; //$NON-NLS-1$

    /**
     * Create an instance of this dialog.
     * 
     * @param parentShell the parent shell
     */
    public AboutSystemDialog(Shell parentShell) {
        super(parentShell);
    }

    /* (non-Javadoc)
     * Method declared on Window.
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(WorkbenchMessages.SystemSummary_title);
        PlatformUI.getWorkbench().getHelpSystem().setHelp(newShell,
                IWorkbenchHelpContextIds.SYSTEM_SUMMARY_DIALOG);
    }

    /* (non-Javadoc)
     * Method declared on Dialog.
     */
    protected void createButtonsForButtonBar(Composite parent) {
        parent.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Button button = createButton(parent, BROWSE_ERROR_LOG_BUTTON, WorkbenchMessages.AboutSystemDialog_browseErrorLogName, false);
        String filename = Platform.getLogFileLocation().toOSString();
        button.setEnabled(new File(filename).exists());

        createButton(parent, COPY_TO_CLIPBOARD_BUTTON, WorkbenchMessages.AboutSystemDialog_copyToClipboardName, false);

        new Label(parent, DWT.NONE).setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        GridLayout layout = (GridLayout) parent.getLayout();
        layout.numColumns++;
        layout.makeColumnsEqualWidth = false;

        createButton(parent, IDialogConstants.CLOSE_ID,
                IDialogConstants.CLOSE_LABEL, true);
    }

    /* (non-Javadoc)
     * Method declared on Dialog.
     */
    protected Control createDialogArea(Composite parent) {
        Composite outer = (Composite) super.createDialogArea(parent);

        text = new Text(outer, DWT.MULTI | DWT.BORDER | DWT.READ_ONLY
                | DWT.V_SCROLL | DWT.NO_FOCUS | DWT.H_SCROLL);
        text.setBackground(parent.getDisplay().getSystemColor(
                DWT.COLOR_LIST_BACKGROUND));
        GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_FILL);
        gridData.grabExcessVerticalSpace = true;
        gridData.grabExcessHorizontalSpace = true;
        gridData.heightHint = convertVerticalDLUsToPixels(300);
        gridData.widthHint = convertHorizontalDLUsToPixels(400);
        text.setLayoutData(gridData);
        text.setText(ConfigurationInfo.getSystemSummary());
        text.setFont(JFaceResources.getTextFont());
        return outer;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#buttonPressed(int)
     */
    protected void buttonPressed(int buttonId) {
        switch (buttonId) {
        case IDialogConstants.CLOSE_ID:
            close();
            break;
        case BROWSE_ERROR_LOG_BUTTON:
            openErrorLogBrowser();
            break;
        case COPY_TO_CLIPBOARD_BUTTON:
            runCopyToClipboard();
            break;
        }
        super.buttonPressed(buttonId);
    }

    private void openErrorLogBrowser() {
        String filename = Platform.getLogFileLocation().toOSString();

        File log = new File(filename);
        if (log.exists()) {
            // Make a copy of the file with a temporary name.
            // Working around an issue with windows file associations/browser malfunction
            // whereby the browser doesn't open on ".log" and we aren't returned an error.
            // See https://bugs.eclipse.org/bugs/show_bug.cgi?id=97783
            File logCopy = makeDisplayCopy(log);
            if (logCopy !is null) {
                openLink("file:///" + logCopy.getAbsolutePath()); //$NON-NLS-1$
                return;
            }
            // Couldn't make copy, try to open the original log.
            // We try the original in this case rather than putting up an error,
            // because the copy could fail due to an I/O or out of space problem.
            // In that case we may still be able to show the original log,
            // depending on the platform.  The risk is that users with configurations
            // that have bug #97783 will still get nothing (vs. an error) but we'd rather
            // try again than put up an error dialog on platforms where the ability to
            // view the original log works just fine.
            openLink("file:///" + filename); //$NON-NLS-1$
            return;
        }
        MessageDialog.openInformation(getShell(), WorkbenchMessages.AboutSystemDialog_noLogTitle, 
                NLS.bind(WorkbenchMessages.AboutSystemDialog_noLogMessage,  filename ));
    }
    
    /**
     * Returns a copy of the given file to be used for display in
     * a browser.
     * 
     * @return the file, or <code>null</code>
     */
    private File makeDisplayCopy(File file) {
        IPath path = WorkbenchPlugin.getDefault().getDataLocation();
        if(path is null) {
            return null;
        }
        path = path.append(ERROR_LOG_COPY_FILENAME);
        File copy = path.toFile();
        FileReader in = null;
        FileWriter out = null;
        try {
            in = new FileReader(file);
            // don't append data, overwrite what was there
            out = new FileWriter(copy);
            char buffer[] = new char[4096];
            int count;
            while ((count = in.read(buffer, 0, buffer.length)) > 0) {
                out.write(buffer, 0, count);
            }
        } catch (FileNotFoundException e) {
            return null;
        } catch (IOException e) {
            return null;
        } finally {
            try {
                if (in !is null) {
                    in.close();
                }
                if (out !is null) {
                    out.close();
                }
            } catch (IOException e) {
                return null;
            }
        }
        return copy;

    }

    private void runCopyToClipboard() {
        if (text is null) {
            return;
        }

        Clipboard clipboard = null;
        try {
            clipboard = new Clipboard(getShell().getDisplay());
            clipboard.setContents(new Object[] { text.getText() },
                    new Transfer[] { TextTransfer.getInstance() });
        } finally {
            if (clipboard !is null) {
                clipboard.dispose();
            }
        }
    }
}
