/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Benjamin Muskalla -  Bug 29633 [EditorMgmt] "Open" menu should
 *                          have Open With-->Other
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.FileEditorsPreferencePage;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import dwt.DWT;
import dwt.graphics.Image;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Label;
import dwt.widgets.Listener;
import dwt.widgets.Table;
import dwt.widgets.TableItem;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.content.IContentType;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.jface.preference.IPreferenceStore;
import dwtx.jface.preference.PreferencePage;
import dwtx.jface.window.Window;
import dwtx.osgi.util.NLS;
import dwtx.ui.IEditorDescriptor;
import dwtx.ui.IFileEditorMapping;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPreferencePage;
import dwtx.ui.dialogs.EditorSelectionDialog;
import dwtx.ui.dialogs.PreferenceLinkArea;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.EditorDescriptor;
import dwtx.ui.internal.registry.EditorRegistry;
import dwtx.ui.internal.registry.FileEditorMapping;
import dwtx.ui.internal.util.PrefUtil;
import dwtx.ui.preferences.IWorkbenchPreferenceContainer;

/**
 * The file editors page presents the collection of file names and extensions
 * for which the user has registered editors. It also lets the user add new
 * internal or external (program) editors for a given file name and extension.
 *
 * The user can add an editor for either a specific file name and extension
 * (e.g. report.doc), or for all file names of a given extension (e.g. *.doc)
 *
 * The set of registered editors is tracked by the EditorRegistery
 * available from the workbench plugin.
 */
public class FileEditorsPreferencePage extends PreferencePage implements
        IWorkbenchPreferencePage, Listener {
    
    private static final String DATA_EDITOR = "editor"; //$NON-NLS-1$

    private static final String DATA_FROM_CONTENT_TYPE = "type"; //$NON-NLS-1$
    
    protected Table resourceTypeTable;

    protected Button addResourceTypeButton;

    protected Button removeResourceTypeButton;

    protected Table editorTable;

    protected Button addEditorButton;

    protected Button removeEditorButton;

    protected Button defaultEditorButton;

    protected Label editorLabel;

    protected IWorkbench workbench;

    protected List imagesToDispose;

    protected Map editorsToImages;

    /**
     * Add a new resource type to the collection shown in the top of the page.
     * This is typically called after the extension dialog is shown to the user.
     * 
     * @param newName the new name
     * @param newExtension the new extension
     */
    public void addResourceType(String newName, String newExtension) {
        // Either a file name or extension must be provided
        Assert.isTrue((newName !is null && newName.length() !is 0)
                || (newExtension !is null && newExtension.length() !is 0));

        // Wild card only valid by itself (i.e. rep* is not valid)
        // And must have an extension
        int index = newName.indexOf('*');
        if (index > -1) {
            Assert.isTrue(index is 0 && newName.length() is 1);
            Assert.isTrue(newExtension !is null && newExtension.length() !is 0);
        }

        // Find the index at which to insert the new entry.
        String newFilename = (newName + (newExtension is null
                || newExtension.length() is 0 ? "" : "." + newExtension)).toUpperCase();//$NON-NLS-1$ //$NON-NLS-2$
        IFileEditorMapping resourceType;
        TableItem[] items = resourceTypeTable.getItems();
        bool found = false;
        int i = 0;

        while (i < items.length && !found) {
            resourceType = (IFileEditorMapping) items[i].getData();
            int result = newFilename.compareToIgnoreCase(resourceType
                    .getLabel());
            if (result is 0) {
                // Same resource type not allowed!
                MessageDialog
                        .openInformation(
                                getControl().getShell(),
                                WorkbenchMessages.FileEditorPreference_existsTitle,
                                WorkbenchMessages.FileEditorPreference_existsMessage);
                return;
            }

            if (result < 0) {
                found = true;
            } else {
                i++;
            }
        }

        // Create the new type and insert it
        resourceType = new FileEditorMapping(newName, newExtension);
        TableItem item = newResourceTableItem(resourceType, i, true);
        resourceTypeTable.setFocus();
        resourceTypeTable.showItem(item);
        fillEditorTable();
    }

    /**
     * Creates the page's UI content.
     */
    protected Control createContents(Composite parent) {
        imagesToDispose = new ArrayList();
        editorsToImages = new HashMap(50);

        // define container & its gridding
        Composite pageComponent = new Composite(parent, DWT.NULL);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        pageComponent.setLayout(layout);
        GridData data = new GridData();
        data.verticalAlignment = GridData.FILL;
        data.horizontalAlignment = GridData.FILL;
        pageComponent.setLayoutData(data);

        //layout the contents

        PreferenceLinkArea contentTypeArea = new PreferenceLinkArea(pageComponent, DWT.NONE,
                "dwtx.ui.preferencePages.ContentTypes", WorkbenchMessages.FileEditorPreference_contentTypesRelatedLink,//$NON-NLS-1$
                (IWorkbenchPreferenceContainer) getContainer(),null);
        
        data = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
        contentTypeArea.getControl().setLayoutData(data);

        //layout the top table & its buttons
        Label label = new Label(pageComponent, DWT.LEFT);
        label.setText(WorkbenchMessages.FileEditorPreference_fileTypes); 
        data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.horizontalSpan = 2;
        label.setLayoutData(data);

        resourceTypeTable = new Table(pageComponent, DWT.SINGLE | DWT.BORDER
                | DWT.FULL_SELECTION);
        resourceTypeTable.addListener(DWT.Selection, this);
        resourceTypeTable.addListener(DWT.DefaultSelection, this);
        data = new GridData(GridData.FILL_HORIZONTAL);

        int availableRows = DialogUtil.availableRows(pageComponent);

        data.heightHint = resourceTypeTable.getItemHeight()
                * (availableRows / 8);
        resourceTypeTable.setLayoutData(data);

        Composite groupComponent = new Composite(pageComponent, DWT.NULL);
        GridLayout groupLayout = new GridLayout();
        groupLayout.marginWidth = 0;
        groupLayout.marginHeight = 0;
        groupComponent.setLayout(groupLayout);
        data = new GridData();
        data.verticalAlignment = GridData.FILL;
        data.horizontalAlignment = GridData.FILL;
        groupComponent.setLayoutData(data);

        addResourceTypeButton = new Button(groupComponent, DWT.PUSH);
        addResourceTypeButton.setText(WorkbenchMessages.FileEditorPreference_add); 
        addResourceTypeButton.addListener(DWT.Selection, this);
        addResourceTypeButton.setLayoutData(data);
        setButtonLayoutData(addResourceTypeButton);

        removeResourceTypeButton = new Button(groupComponent, DWT.PUSH);
        removeResourceTypeButton.setText(WorkbenchMessages.FileEditorPreference_remove);
        removeResourceTypeButton.addListener(DWT.Selection, this);
        setButtonLayoutData(removeResourceTypeButton);

        //Spacer
        label = new Label(pageComponent, DWT.LEFT);
        data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.horizontalSpan = 2;
        label.setLayoutData(data);

        // layout the bottom table & its buttons
        editorLabel = new Label(pageComponent, DWT.LEFT);
        editorLabel.setText(WorkbenchMessages.FileEditorPreference_associatedEditors);
        data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.horizontalSpan = 2;
        editorLabel.setLayoutData(data);

        editorTable = new Table(pageComponent, DWT.SINGLE | DWT.BORDER);
        editorTable.addListener(DWT.Selection, this);
        editorTable.addListener(DWT.DefaultSelection, this);
        data = new GridData(GridData.FILL_BOTH);
        data.heightHint = editorTable.getItemHeight() * 7;
        editorTable.setLayoutData(data);

        groupComponent = new Composite(pageComponent, DWT.NULL);
        groupLayout = new GridLayout();
        groupLayout.marginWidth = 0;
        groupLayout.marginHeight = 0;
        groupComponent.setLayout(groupLayout);
        data = new GridData();
        data.verticalAlignment = GridData.FILL;
        data.horizontalAlignment = GridData.FILL;
        groupComponent.setLayoutData(data);

        addEditorButton = new Button(groupComponent, DWT.PUSH);
        addEditorButton.setText(WorkbenchMessages.FileEditorPreference_addEditor);
        addEditorButton.addListener(DWT.Selection, this);
        addEditorButton.setLayoutData(data);
        setButtonLayoutData(addEditorButton);

        removeEditorButton = new Button(groupComponent, DWT.PUSH);
        removeEditorButton.setText(WorkbenchMessages.FileEditorPreference_removeEditor); 
        removeEditorButton.addListener(DWT.Selection, this);
        setButtonLayoutData(removeEditorButton);

        defaultEditorButton = new Button(groupComponent, DWT.PUSH);
        defaultEditorButton.setText(WorkbenchMessages.FileEditorPreference_default);
        defaultEditorButton.addListener(DWT.Selection, this);
        setButtonLayoutData(defaultEditorButton);

        fillResourceTypeTable();
        if (resourceTypeTable.getItemCount() > 0) {
            resourceTypeTable.setSelection(0);
        }
        fillEditorTable();
        updateEnabledState();

        workbench.getHelpSystem().setHelp(parent,
                IWorkbenchHelpContextIds.FILE_EDITORS_PREFERENCE_PAGE);
        applyDialogFont(pageComponent);

        return pageComponent;
    }

    /**
     * The preference page is going to be disposed. So deallocate all allocated
     * DWT resources that aren't disposed automatically by disposing the page
     * (i.e fonts, cursors, etc). Subclasses should reimplement this method to 
     * release their own allocated DWT resources.
     */
    public void dispose() {
        super.dispose();
        if (imagesToDispose !is null) {
            for (Iterator e = imagesToDispose.iterator(); e.hasNext();) {
                ((Image) e.next()).dispose();
            }
            imagesToDispose = null;
        }
        if (editorsToImages !is null) {
            for (Iterator e = editorsToImages.values().iterator(); e.hasNext();) {
                ((Image) e.next()).dispose();
            }
            editorsToImages = null;
        }
    }

    /**
     * Hook method to get a page specific preference store. Reimplement this
     * method if a page don't want to use its parent's preference store.
     */
    protected IPreferenceStore doGetPreferenceStore() {
        return WorkbenchPlugin.getDefault().getPreferenceStore();
    }

    protected void fillEditorTable() {
        editorTable.removeAll();
        FileEditorMapping resourceType = getSelectedResourceType();
        if (resourceType !is null) {
            IEditorDescriptor[] array = resourceType.getEditors();
            for (int i = 0; i < array.length; i++) {
                IEditorDescriptor editor = array[i];
                TableItem item = new TableItem(editorTable, DWT.NULL);
                item.setData(DATA_EDITOR, editor);
                // Check if it is the default editor
                String defaultString = null;
                if (resourceType !is null) {
                    if (resourceType.getDefaultEditor() is editor && resourceType.isDeclaredDefaultEditor(editor)) {
                        defaultString = WorkbenchMessages.FileEditorPreference_defaultLabel;
                    }
                }

                if (defaultString !is null) {
                    item.setText(editor.getLabel() + " " + defaultString); //$NON-NLS-1$
                } else {
                    item.setText(editor.getLabel());
                }
                item.setImage(getImage(editor));
            }
            
            // now add any content type editors
            EditorRegistry registry = (EditorRegistry) WorkbenchPlugin
                    .getDefault().getEditorRegistry();
            IContentType[] contentTypes = Platform.getContentTypeManager()
                    .findContentTypesFor(resourceType.getLabel());
            for (int i = 0; i < contentTypes.length; i++) {
                array = registry.getEditorsForContentType(contentTypes[i]);
                for (int j = 0; j < array.length; j++) {
                    IEditorDescriptor editor = array[j];
                    // don't add duplicates
                    TableItem[] items = editorTable.getItems();
                    TableItem foundItem = null;
                    for (int k = 0; k < items.length; k++) {
                        if (items[k].getData(DATA_EDITOR).equals(editor)) {
                            foundItem = items[k];
                            break;
                        }
                    }
                    if (foundItem is null) {
                        TableItem item = new TableItem(editorTable, DWT.NULL);
                        item.setData(DATA_EDITOR, editor);
                        item.setData(DATA_FROM_CONTENT_TYPE, contentTypes[i]);
                        setLockedItemText(item, editor.getLabel());
                        item.setImage(getImage(editor));
                    } else { // update the item to reflect its origin
                        foundItem.setData(DATA_FROM_CONTENT_TYPE, contentTypes[i]);
                        setLockedItemText(foundItem, foundItem.getText());
                    }
                }
            }
            
        }
    }

    /**
     * Set the locked message on the item. Assumes the item has an instance of
     * IContentType in the data map.
     * 
     * @param item the item to set
     * @param baseLabel the base label
     */
    private void setLockedItemText(TableItem item, String baseLabel) {
        item.setText(NLS
                .bind(WorkbenchMessages.FileEditorPreference_isLocked,
                        baseLabel, ((IContentType) item
                                .getData(DATA_FROM_CONTENT_TYPE)).getName()));
    }
    
    /**
     * Place the existing resource types in the table
     */
    protected void fillResourceTypeTable() {
        // Populate the table with the items
        IFileEditorMapping[] array = WorkbenchPlugin.getDefault()
                .getEditorRegistry().getFileEditorMappings();
        for (int i = 0; i < array.length; i++) {
            FileEditorMapping mapping = (FileEditorMapping) array[i];
            mapping = (FileEditorMapping) mapping.clone(); // want a copy
            newResourceTableItem(mapping, i, false);
        }
    }

    /**
     * Returns the image associated with the given editor.
     */
    protected Image getImage(IEditorDescriptor editor) {
        Image image = (Image) editorsToImages.get(editor);
        if (image is null) {
            image = editor.getImageDescriptor().createImage();
            editorsToImages.put(editor, image);
        }
        return image;
    }

    protected FileEditorMapping getSelectedResourceType() {
        TableItem[] items = resourceTypeTable.getSelection();
        if (items.length > 0) {
            return (FileEditorMapping) items[0].getData(); //Table is single select
        }
        return null;        
    }

    protected IEditorDescriptor[] getAssociatedEditors() {
        if (getSelectedResourceType() is null) {
            return null;
        }
        if (editorTable.getItemCount() > 0) {
            ArrayList editorList = new ArrayList();
            for (int i = 0; i < editorTable.getItemCount(); i++) {
                editorList.add(editorTable.getItem(i).getData(DATA_EDITOR));
            }

            return (IEditorDescriptor[]) editorList
                    .toArray(new IEditorDescriptor[editorList.size()]);
        } 
        return null;
    }

    public void handleEvent(Event event) {
        if (event.widget is addResourceTypeButton) {
            promptForResourceType();
        } else if (event.widget is removeResourceTypeButton) {
            removeSelectedResourceType();
        } else if (event.widget is addEditorButton) {
            promptForEditor();
        } else if (event.widget is removeEditorButton) {
            removeSelectedEditor();
        } else if (event.widget is defaultEditorButton) {
            setSelectedEditorAsDefault();
        } else if (event.widget is resourceTypeTable) {
            fillEditorTable();
        }

        updateEnabledState();

    }

    /**
     * @see IWorkbenchPreferencePage
     */
    public void init(IWorkbench aWorkbench) {
        this.workbench = aWorkbench;
        noDefaultAndApplyButton();
    }

    /*
     * Create a new <code>TableItem</code> to represent the resource
     * type editor description supplied.
     */
    protected TableItem newResourceTableItem(IFileEditorMapping mapping,
            int index, bool selected) {
        Image image = mapping.getImageDescriptor().createImage(false);
        if (image !is null) {
            imagesToDispose.add(image);
        }

        TableItem item = new TableItem(resourceTypeTable, DWT.NULL, index);
        if (image !is null) {
            item.setImage(image);
        }
        item.setText(mapping.getLabel());
        item.setData(mapping);
        if (selected) {
            resourceTypeTable.setSelection(index);
        }

        return item;
    }

    /**
     * This is a hook for sublcasses to do special things when the ok
     * button is pressed.
     * For example reimplement this method if you want to save the 
     * page's data into the preference bundle.
     */
    public bool performOk() {
        TableItem[] items = resourceTypeTable.getItems();
        FileEditorMapping[] resourceTypes = new FileEditorMapping[items.length];
        for (int i = 0; i < items.length; i++) {
            resourceTypes[i] = (FileEditorMapping) (items[i].getData());
        }
        EditorRegistry registry = (EditorRegistry) WorkbenchPlugin.getDefault()
                .getEditorRegistry(); // cast to allow save to be called
        registry.setFileEditorMappings(resourceTypes);
        registry.saveAssociations();
        
        PrefUtil.savePrefs();
        return true;
    }

    /**
     * Prompt for editor.
     */
    public void promptForEditor() {
        EditorSelectionDialog dialog = new EditorSelectionDialog(getControl()
                .getShell());
        dialog.setEditorsToFilter(getAssociatedEditors());
        dialog
                .setMessage(NLS.bind(WorkbenchMessages.Choose_the_editor_for_file,getSelectedResourceType().getLabel() ));
        if (dialog.open() is Window.OK) {
            EditorDescriptor editor = (EditorDescriptor) dialog
                    .getSelectedEditor();
            if (editor !is null) {
                int i = editorTable.getItemCount();
                bool isEmpty = i < 1;
                TableItem item = new TableItem(editorTable, DWT.NULL, i);
                item.setData(DATA_EDITOR, editor);
                if (isEmpty) {
                    item
                            .setText(editor.getLabel()
                                    + " " + WorkbenchMessages.FileEditorPreference_defaultLabel); //$NON-NLS-1$
                } else {
                    item.setText(editor.getLabel());
                }
                item.setImage(getImage(editor));
                editorTable.setSelection(i);
                editorTable.setFocus();
                getSelectedResourceType().addEditor(editor);
                if (isEmpty) {
                    getSelectedResourceType().setDefaultEditor(editor);
                }
                updateSelectedResourceType(); //in case of new default
            }
        }
    }

    /**
     * Prompt for resource type.
     */
    public void promptForResourceType() {
        FileExtensionDialog dialog = new FileExtensionDialog(getControl()
                .getShell(), WorkbenchMessages.FileExtension_shellTitle,
                IWorkbenchHelpContextIds.FILE_EXTENSION_DIALOG,
                WorkbenchMessages.FileExtension_dialogTitle,
                WorkbenchMessages.FileExtension_fileTypeMessage,
                WorkbenchMessages.FileExtension_fileTypeLabel);
        if (dialog.open() is Window.OK) {
            String name = dialog.getName();
            String extension = dialog.getExtension();
            addResourceType(name, extension);
        }
    }

    /**
     * Remove the editor from the table
     */
    public void removeSelectedEditor() {
        TableItem[] items = editorTable.getSelection();
        bool defaultEditor = editorTable.getSelectionIndex() is 0;
        if (items.length > 0) {
            getSelectedResourceType().removeEditor(
                    (EditorDescriptor) items[0].getData(DATA_EDITOR));
            items[0].dispose(); //Table is single selection
        }
        if (defaultEditor && editorTable.getItemCount() > 0) {
            TableItem item = editorTable.getItem(0);
            // explicitly set the new editor first editor to default
            getSelectedResourceType().setDefaultEditor(
                    (EditorDescriptor) item.getData(DATA_EDITOR));
            if (item !is null) {
                item
                        .setText(((EditorDescriptor) (item.getData(DATA_EDITOR)))
                                .getLabel()
                                + " " + WorkbenchMessages.FileEditorPreference_defaultLabel); //$NON-NLS-1$
            }
            if (!isEditorRemovable(item)) {
                setLockedItemText(item, item.getText());
            }
        }

    }

    /**
     * Remove the type from the table
     */
    public void removeSelectedResourceType() {
        TableItem[] items = resourceTypeTable.getSelection();
        if (items.length > 0) {
            items[0].dispose(); //Table is single selection
        }
        //Clear out the editors too
        editorTable.removeAll();
    }

    /**
     * Add the selected editor to the default list.
     */
    public void setSelectedEditorAsDefault() {
        TableItem[] items = editorTable.getSelection();
        if (items.length > 0) {
            // First change the label of the old default
            TableItem oldDefaultItem = editorTable.getItem(0);
            oldDefaultItem
                    .setText(((EditorDescriptor) oldDefaultItem.getData(DATA_EDITOR))
                            .getLabel());
            // update the label to reflect the locked state
            if (!isEditorRemovable(oldDefaultItem)) {
                setLockedItemText(oldDefaultItem, oldDefaultItem.getText());
            }
            // Now set the new default
            EditorDescriptor editor = (EditorDescriptor) items[0].getData(DATA_EDITOR);
            getSelectedResourceType().setDefaultEditor(editor);
            IContentType fromContentType = (IContentType) items[0].getData(DATA_FROM_CONTENT_TYPE);
            items[0].dispose(); //Table is single selection
            TableItem item = new TableItem(editorTable, DWT.NULL, 0);
            item.setData(DATA_EDITOR, editor);
            if (fromContentType !is null) {
                item.setData(DATA_FROM_CONTENT_TYPE, fromContentType);
            }
            item
                    .setText(editor.getLabel()
                            + " " + WorkbenchMessages.FileEditorPreference_defaultLabel); //$NON-NLS-1$
            item.setImage(getImage(editor));
            if (!isEditorRemovable(item)) {
                setLockedItemText(item, item.getText());
            }
            editorTable.setSelection(new TableItem[] { item });
        }
    }

    /**
     * Update the enabled state.
     */
    public void updateEnabledState() {
        //Update enabled state
        bool resourceTypeSelected = resourceTypeTable.getSelectionIndex() !is -1;
        bool editorSelected = editorTable.getSelectionIndex() !is -1;

        removeResourceTypeButton.setEnabled(resourceTypeSelected);
        editorLabel.setEnabled(resourceTypeSelected);
        addEditorButton.setEnabled(resourceTypeSelected);
        removeEditorButton.setEnabled(editorSelected && isEditorRemovable());
        defaultEditorButton.setEnabled(editorSelected);
    }
    
    /**
     * Return whether the selected editor is removable. An editor is removable
     * if it is not submitted via a content-type binding.
     * 
     * @return whether the selected editor is removable
     * @since 3.1
     */
    private bool isEditorRemovable() {
        TableItem[] items = editorTable.getSelection();
        if (items.length > 0) {
            return isEditorRemovable(items[0]);
        }
        return false;
    }
    
    /**
     * Return whether the given editor is removable. An editor is removable
     * if it is not submitted via a content-type binding.
     * 
     * @param item the item to test
     * @return whether the selected editor is removable
     * @since 3.1
     */
    private bool isEditorRemovable(TableItem item) {
        IContentType fromContentType = (IContentType) item.getData(DATA_FROM_CONTENT_TYPE);
        return fromContentType is null;
    }

    /**
     * Update the selected type.
     */
    public void updateSelectedResourceType() {
        // TableItem item = resourceTypeTable.getSelection()[0]; //Single select
        //  Image image = ((IFileEditorMapping)item.getData()).getImageDescriptor().getImage();
        //  imagesToDispose.addElement(image);
        //  item.setImage(image);
    }
}
