/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.FileExtensionDialog;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.events.ModifyEvent;
import dwt.events.ModifyListener;
import dwt.graphics.Point;
import dwt.layout.GridData;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwt.widgets.Text;
import dwtx.jface.dialogs.Dialog;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.dialogs.IDialogSettings;
import dwtx.jface.dialogs.TitleAreaDialog;
import dwtx.jface.layout.GridLayoutFactory;
import dwtx.jface.layout.LayoutConstants;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;

/**
 * This class is used to prompt the user for a file name & extension.
 */
public class FileExtensionDialog extends TitleAreaDialog {
    
    private static final String DIALOG_SETTINGS_SECTION = "FileExtensionDialogSettings"; //$NON-NLS-1$
    
    private String filename = ""; //$NON-NLS-1$
    
    private String initialValue;

    private Text filenameField;

    private Button okButton;

    private String title;

    private String helpContextId;

    private final String headerTitle;

    private final String message2;

    private final String label;

    /**
     * Constructs a new file extension dialog.
     * @param parentShell the parent shell
     */
    public FileExtensionDialog(Shell parentShell) {
        this(parentShell, WorkbenchMessages.FileExtension_shellTitle,
                IWorkbenchHelpContextIds.FILE_EXTENSION_DIALOG,
                WorkbenchMessages.FileExtension_dialogTitle,
                WorkbenchMessages.FileExtension_fileTypeMessage,
                WorkbenchMessages.FileExtension_fileTypeLabel);
    }
    
    /**
     * Constructs a new file extension dialog.
     * 
     * @param parentShell the parent shell
     * @param title the dialog title
     * @param helpContextId the help context for this dialog
     * @param headerTitle the dialog header
     * @param message the dialog message
     * @param label the label for the "file type" field
     * @since 3.4
     */
    public FileExtensionDialog(Shell parentShell, String title, String helpContextId, String headerTitle, String message, String label) {
        super(parentShell);
        this.title = title;
        this.helpContextId = helpContextId;
        this.headerTitle = headerTitle;
        message2 = message;
        this.label = label;
        
    }
    
    /* (non-Javadoc)
     * @see dwtx.jface.window.Window#configureShell(dwt.widgets.Shell)
     */
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        shell.setText(title);
        PlatformUI.getWorkbench().getHelpSystem().setHelp(shell, helpContextId);
    }

   
    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.TitleAreaDialog#createDialogArea(dwt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite parentComposite = (Composite) super.createDialogArea(parent);

        Composite contents = new Composite(parentComposite, DWT.NONE);
        contents.setLayoutData(new GridData(DWT.FILL, DWT.FILL, true, true));

        setTitle(headerTitle);
        setMessage(message2);

        new Label(contents, DWT.LEFT)
                .setText(label);

        filenameField = new Text(contents, DWT.SINGLE | DWT.BORDER);
        if (initialValue !is null) {
            filenameField.setText(initialValue);
        }
        filenameField.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent event) {
                if (event.widget is filenameField) {
                    filename = filenameField.getText().trim();
                    okButton.setEnabled(validateFileType());
                }
            }
        });
        filenameField.setFocus();

        Dialog.applyDialogFont(parentComposite);

        Point defaultMargins = LayoutConstants.getMargins();
        GridLayoutFactory.fillDefaults().numColumns(2).margins(
                defaultMargins.x, defaultMargins.y).generateLayout(contents);
        
        return contents;
    }

   
    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#createButtonsForButtonBar(dwt.widgets.Composite)
     */
    protected void createButtonsForButtonBar(Composite parent) {
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        okButton.setEnabled(false);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }

    /**
     * Validate the user input for a file type
     */
    private bool validateFileType() {
        // We need kernel api to validate the extension or a filename

        // check for empty name and extension
        if (filename.length() is 0) {
            setErrorMessage(null);
            return false;
        }

        // check for empty extension if there is no name
        int index = filename.lastIndexOf('.');
        if (index is filename.length() - 1) {
            if (index is 0 || (index is 1 && filename.charAt(0) is '*')) {
                setErrorMessage(WorkbenchMessages.FileExtension_extensionEmptyMessage); 
                return false;
            }
        }

        // check for characters before * 
        // or no other characters
        // or next chatacter not '.'
        // or another *
        index = filename.indexOf('*');
        if (index > -1) {
            if (filename.length() is 1) {
                setErrorMessage(WorkbenchMessages.FileExtension_extensionEmptyMessage); 
                return false;
            }
            if (index !is 0 || filename.charAt(1) !is '.') {
                setErrorMessage(WorkbenchMessages.FileExtension_fileNameInvalidMessage);
                return false;
            }
            if (filename.length() > index && filename.indexOf('*', index + 1) !is -1) {
                setErrorMessage(WorkbenchMessages.FileExtension_fileNameInvalidMessage); 
                return false;
            }
        }

        setErrorMessage(null);
        return true;
    }

    /**
     * Get the extension.
     * 
     * @return the extension
     */
    public String getExtension() {
        // We need kernel api to validate the extension or a filename

        int index = filename.lastIndexOf('.');
        if (index is -1) {
            return ""; //$NON-NLS-1$
        }
        if (index is filename.length()) {
            return ""; //$NON-NLS-1$
        }
        return filename.substring(index + 1, filename.length());
    }

    /**
     * Get the name.
     * 
     * @return the name
     */
    public String getName() {
        // We need kernel api to validate the extension or a filename

        int index = filename.lastIndexOf('.');
        if (index is -1) {
            return filename;
        }
        if (index is 0) {
            return "*"; //$NON-NLS-1$
        }
        return filename.substring(0, index);
    }
    
    /**
     * Sets the initial value that should be prepopulated in this dialog.
     * 
     * @param initialValue
     *            the value to be displayed to the user
     * @since 3.4
     */
    public void setInitialValue(String initialValue) {
        this.initialValue = initialValue;
    }
   
    /* (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#getDialogBoundsSettings()
     */
    protected IDialogSettings getDialogBoundsSettings() {
        IDialogSettings settings = WorkbenchPlugin.getDefault().getDialogSettings();
        IDialogSettings section = settings.getSection(DIALOG_SETTINGS_SECTION);
        if (section is null) section = settings.addNewSection(DIALOG_SETTINGS_SECTION);
        return section;
    }
    
    /*
     * (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#isResizable()
     */
    protected bool isResizable() {
        return true;
    }
}
