/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.NewWizardSelectionPage;

import dwt.dwthelper.utils;


import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwtx.jface.dialogs.IDialogSettings;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.jface.wizard.WizardDialog;
import dwtx.ui.IWorkbench;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.activities.ws.WorkbenchTriggerPoints;
import dwtx.ui.wizards.IWizardCategory;
import dwtx.ui.wizards.IWizardDescriptor;

/**
 *  New wizard selection tab that allows the user to either select a
 *  registered 'New' wizard to be launched, or to select a solution or
 *  projects to be retrieved from an available server.  This page
 *  contains two visual tabs that allow the user to perform these tasks.
 *
 *  Temporarily has two inner pages.  The new format page is used if the system 
 *  is currently aware of activity categories.
 */
class NewWizardSelectionPage extends WorkbenchWizardSelectionPage {
    private IWizardCategory wizardCategories;

    // widgets
    private NewWizardNewPage newResourcePage;

    private IWizardDescriptor [] primaryWizards;

    private bool projectsOnly;
    
    private bool canFinishEarly = false, hasPages = true;
    
    /**
     * Create an instance of this class.
     *
     * @param workbench the workbench
     * @param selection the current selection
     * @param root the wizard root element
     * @param primary the primary wizard elements
     * @param projectsOnly if only projects should be shown
     */
    public NewWizardSelectionPage(IWorkbench workbench,
            IStructuredSelection selection, IWizardCategory root,
            IWizardDescriptor[] primary, bool projectsOnly) {
        super("newWizardSelectionPage", workbench, selection, null, WorkbenchTriggerPoints.NEW_WIZARDS);//$NON-NLS-1$
        
        setTitle(WorkbenchMessages.NewWizardSelectionPage_description); 
        wizardCategories = root;
        primaryWizards = primary;
        this.projectsOnly = projectsOnly;
    }

    /**
     * Makes the next page visible.
     */
    public void advanceToNextPageOrFinish() {
            if (canFlipToNextPage()) {
                getContainer().showPage(getNextPage());
            } else if (canFinishEarly()) {
                if (getWizard().performFinish()) {
                    ((WizardDialog)getContainer()).close();
                }
            }
    }

    /** (non-Javadoc)
     * Method declared on IDialogPage.
     */
    public void createControl(Composite parent) {
        IDialogSettings settings = getDialogSettings();
        newResourcePage = new NewWizardNewPage(this, wizardCategories,
                primaryWizards, projectsOnly);
        newResourcePage.setDialogSettings(settings);

        Control control = newResourcePage.createControl(parent);
        getWorkbench().getHelpSystem().setHelp(control,
                IWorkbenchHelpContextIds.NEW_WIZARD_SELECTION_WIZARD_PAGE);
        setControl(control);
    }

    /**
     * Since Finish was pressed, write widget values to the dialog store so that they
     *will persist into the next invocation of this wizard page
     */
    protected void saveWidgetValues() {
        newResourcePage.saveWidgetValues();
    }
        
    /* (non-Javadoc)
     * @see dwtx.jface.wizard.IWizardPage#canFlipToNextPage()
     */
    public bool canFlipToNextPage() {
        // if the current page advertises that it does have pages then ask it via the super call
            if (hasPages) {
                return super.canFlipToNextPage();
            }
            return false;
    }

    /**
     * Sets whether the selected wizard advertises that it does provide pages.
     * 
     * @param newValue whether the selected wizard has pages
     * @since 3.1
     */
    public void setHasPages(bool newValue) {
        hasPages = newValue;
    }

    /**
     * Sets whether the selected wizard advertises that it can finish early.
     * 
     * @param newValue whether the selected wizard can finish early
     * @since 3.1
     */
    public void setCanFinishEarly(bool newValue) {
        canFinishEarly = newValue;
    }

    /**
     * Answers whether the currently selected page, if any, advertises that it may finish early.
     * 
     * @return whether the page can finish early
     * @since 3.1
     */
    public bool canFinishEarly() {
        return canFinishEarly;
    }
}
