/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Jan-Hendrik Diederich, Bredex GmbH - bug 201052
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.PreferencePatternFilter;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import dwtx.jface.preference.IPreferenceNode;
import dwtx.jface.viewers.ITreeContentProvider;
import dwtx.jface.viewers.TreeViewer;
import dwtx.jface.viewers.Viewer;
import dwtx.ui.activities.WorkbenchActivityHelper;
import dwtx.ui.dialogs.PatternFilter;
import dwtx.ui.internal.preferences.WorkbenchPreferenceExtensionNode;

/**
 * A class that handles filtering preference node items based on a supplied
 * matching string.
 * 
 * @since 3.2
 * 
 */
public class PreferencePatternFilter extends PatternFilter {

    /**
     * this cache is needed because
     * WorkbenchPreferenceExtensionNode.getKeywordLabels() is expensive. When it
     * tracks keyword changes effectivly than this cache can be removed.
     */
    private Map keywordCache = new HashMap();

    /**
     * Create a new instance of a PreferencePatternFilter
     * 
     * @param isMatchItem
     */
    public PreferencePatternFilter() {
        super();
    }

    /*
     * Return true if the given Object matches with any possible keywords that
     * have been provided. Currently this is only applicable for preference and
     * property pages.
     */
    private String[] getKeywords(Object element) {
        List keywordList = new ArrayList();
        if (element instanceof WorkbenchPreferenceExtensionNode) {
            WorkbenchPreferenceExtensionNode workbenchNode = (WorkbenchPreferenceExtensionNode) element;

            Collection keywordCollection = (Collection) keywordCache
                    .get(element);
            if (keywordCollection is null) {
                keywordCollection = workbenchNode.getKeywordLabels();
                keywordCache.put(element, keywordCollection);
            }
            if (!keywordCollection.isEmpty()){
                Iterator keywords = keywordCollection.iterator();
                while (keywords.hasNext()) {
                    keywordList.add(keywords.next());
                }
            }
        }
        return (String[]) keywordList.toArray(new String[keywordList.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.dialogs.PatternFilter#isElementSelectable(java.lang.Object)
     */
    public bool isElementSelectable(Object element) {
        return element instanceof WorkbenchPreferenceExtensionNode;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.dialogs.PatternFilter#isElementVisible(dwtx.jface.viewers.Viewer, java.lang.Object)
     */
    public bool isElementVisible(Viewer viewer, Object element) {
        if (WorkbenchActivityHelper.restrictUseOf(
                element))
            return false;
        
        // Preference nodes are not differentiated based on category since 
        // categories are selectable nodes.
        if (isLeafMatch(viewer, element)) {
            return true;
        }

        ITreeContentProvider contentProvider = (ITreeContentProvider) ((TreeViewer) viewer)
                .getContentProvider();
        IPreferenceNode node = (IPreferenceNode) element;
        Object[] children = contentProvider.getChildren(node);
        // Will return true if any subnode of the element matches the search
        if (filter(viewer, element, children).length > 0) {
            return true;
        }       
        return false;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.dialogs.PatternFilter#isLeafMatch(dwtx.jface.viewers.Viewer, java.lang.Object)
     * 
     */
    protected bool isLeafMatch(Viewer viewer, Object element) {
        IPreferenceNode node = (IPreferenceNode) element;
        String text = node.getLabelText();

        if (wordMatches(text)) {
            return true;
        }

        // Also need to check the keywords
        String[] keywords = getKeywords(node);
        for (int i = 0; i < keywords.length; i++){
            if (wordMatches(keywords[i])) {
                return true;
            }
        }
        return false;
    }

}
