/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.WorkbenchWizardElement;

import dwt.dwthelper.utils;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IAdaptable;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.Platform;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.jface.viewers.StructuredSelection;
import dwtx.ui.IPluginContribution;
import dwtx.ui.IWorkbenchWizard;
import dwtx.ui.PlatformUI;
import dwtx.ui.SelectionEnabler;
import dwtx.ui.internal.ISelectionConversionService;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.registry.RegistryReader;
import dwtx.ui.model.IWorkbenchAdapter;
import dwtx.ui.model.IWorkbenchAdapter2;
import dwtx.ui.model.WorkbenchAdapter;
import dwtx.ui.plugin.AbstractUIPlugin;
import dwtx.ui.wizards.IWizardCategory;
import dwtx.ui.wizards.IWizardDescriptor;

/**
 * Instances represent registered wizards.
 */
public class WorkbenchWizardElement extends WorkbenchAdapter implements
        IAdaptable, IPluginContribution, IWizardDescriptor {
    private String id;
    
    private ImageDescriptor imageDescriptor;

    private SelectionEnabler selectionEnabler;

    private IConfigurationElement configurationElement;

    private ImageDescriptor descriptionImage;
    
    private WizardCollectionElement parentCategory;
    
    /**
     * TODO: DO we need to  make this API?
     */
    public static final String TAG_PROJECT = "project"; //$NON-NLS-1$

    private static final String [] EMPTY_TAGS = new String[0];

    private static final String [] PROJECT_TAGS = new String[] {TAG_PROJECT};

    
    /**
     * Create a new instance of this class
     * 
     * @param configurationElement
     * @since 3.1
     */
    public WorkbenchWizardElement(IConfigurationElement configurationElement) {
        this.configurationElement = configurationElement;
        id = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
    }

    /**
     * Answer a bool indicating whether the receiver is able to handle the
     * passed selection
     * 
     * @return bool
     * @param selection
     *            IStructuredSelection
     */
    public bool canHandleSelection(IStructuredSelection selection) {
        return getSelectionEnabler().isEnabledForSelection(selection);
    }

    /**
     * Answer the selection for the reciever based on whether the it can handle
     * the selection. If it can return the selection. If it can handle the
     * adapted to IResource value of the selection. If it satisfies neither of
     * these conditions return an empty IStructuredSelection.
     * 
     * @return IStructuredSelection
     * @param selection
     *            IStructuredSelection
     */
    public IStructuredSelection adaptedSelection(IStructuredSelection selection) {
        if (canHandleSelection(selection)) {
            return selection;
        }

        IStructuredSelection adaptedSelection = convertToResources(selection);
        if (canHandleSelection(adaptedSelection)) {
            return adaptedSelection;
        }

        //Couldn't find one that works so just return
        return StructuredSelection.EMPTY;
    }

    /**
     * Create an the instance of the object described by the configuration
     * element. That is, create the instance of the class the isv supplied in
     * the extension point.
     * @return the new object
     * @throws CoreException 
     */
    public Object createExecutableExtension() throws CoreException {
        return WorkbenchPlugin.createExtension(configurationElement,
                IWorkbenchRegistryConstants.ATT_CLASS);
    }

    /**
     * Returns an object which is an instance of the given class associated
     * with this object. Returns <code>null</code> if no such object can be
     * found.
     */
    public Object getAdapter(Class adapter) {
        if (adapter is IWorkbenchAdapter.class
                || adapter is IWorkbenchAdapter2.class) {
            return this;
        }
        else if (adapter is IPluginContribution.class) {
            return this;
        }
        else if (adapter is IConfigurationElement.class) {
            return configurationElement;
        }
        return Platform.getAdapterManager().getAdapter(this, adapter);
    }

    /**
     * @return IConfigurationElement
     */
    public IConfigurationElement getConfigurationElement() {
        return configurationElement;
    }

    /**
     * Answer the description parameter of this element
     * 
     * @return java.lang.String
     */
    public String getDescription() {
        return RegistryReader.getDescription(configurationElement);
    }

    /**
     * Answer the icon of this element.
     */
    public ImageDescriptor getImageDescriptor() {
        if (imageDescriptor is null) {
            String iconName = configurationElement
                    .getAttribute(IWorkbenchRegistryConstants.ATT_ICON);
            if (iconName is null) {
                return null;
            }
            imageDescriptor = AbstractUIPlugin.imageDescriptorFromPlugin(
                    configurationElement.getNamespaceIdentifier(), iconName);    
        }
        return imageDescriptor;
    }

    /**
     * Returns the name of this wizard element.
     */
    public ImageDescriptor getImageDescriptor(Object element) {
        return getImageDescriptor();
    }
    
    /**
     * Returns the name of this wizard element.
     */
    public String getLabel(Object element) {
        return configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_NAME);
    }

    /**
     * Answer self's action enabler, creating it first iff necessary
     */
    protected SelectionEnabler getSelectionEnabler() {
        if (selectionEnabler is null) {
            selectionEnabler = new SelectionEnabler(configurationElement);
        }

        return selectionEnabler;
    }

    /**
     * Attempt to convert the elements in the passed selection into resources
     * by asking each for its IResource property (iff it isn't already a
     * resource). If all elements in the initial selection can be converted to
     * resources then answer a new selection containing these resources;
     * otherwise answer an empty selection.
     * 
     * @param originalSelection the original selection
     * @return the converted selection or an empty selection
     */
    private IStructuredSelection convertToResources(
            IStructuredSelection originalSelection) {
        Object selectionService = PlatformUI.getWorkbench().getService(
                ISelectionConversionService.class);
        if (selectionService is null || originalSelection is null) {
            return StructuredSelection.EMPTY;
        }
        return ((ISelectionConversionService) selectionService)
                .convertToResources(originalSelection);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IPluginContribution#getLocalId()
     */
    public String getLocalId() {
        return getId();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IPluginContribution#getPluginId()
     */
    public String getPluginId() {
        return (configurationElement !is null) ? configurationElement
                .getNamespaceIdentifier() : null;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.wizards.INewWizardDescriptor#getDescriptionImage()
     */
    public ImageDescriptor getDescriptionImage() {
        if (descriptionImage is null) {
            String descImage = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_DESCRIPTION_IMAGE);
            if (descImage is null) {
                return null;
            }
            descriptionImage = AbstractUIPlugin.imageDescriptorFromPlugin(
                    configurationElement.getNamespaceIdentifier(), descImage);
        }
        return descriptionImage;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.wizards.INewWizardDescriptor#getHelpHref()
     */
    public String getHelpHref() {
        return configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_HELP_HREF);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.wizards.INewWizardDescriptor#createWizard()
     */
    public IWorkbenchWizard createWizard() throws CoreException {
        return (IWorkbenchWizard) createExecutableExtension();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartDescriptor#getId()
     */
    public String getId() {
        return id;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartDescriptor#getLabel()
     */
    public String getLabel() {      
        return getLabel(this);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.wizards.INewWizardDescriptor#getCategory()
     */
    public IWizardCategory getCategory() {
        return (IWizardCategory) getParent(this);
    }
    
    /**
     * Return the collection.
     * 
     * @return the collection
     * @since 3.1
     */
    public WizardCollectionElement getCollectionElement() {
        return (WizardCollectionElement) getParent(this);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.wizards.IWizardDescriptor#getTags()
     */
    public String [] getTags() {
 
        String flag = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_PROJECT);
        if (Boolean.valueOf(flag).booleanValue()) {
            return PROJECT_TAGS;
        }
        
        return EMPTY_TAGS;
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.model.IWorkbenchAdapter#getParent(java.lang.Object)
     */
    public Object getParent(Object object) {
        return parentCategory;
    }

    /**
     * Set the parent category.
     * 
     * @param parent the parent category
     * @since 3.1
     */
    public void setParent(WizardCollectionElement parent) {
        parentCategory = parent;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.wizards.IWizardDescriptor#canFinishEarly()
     */
    public bool canFinishEarly() {
        return Boolean.valueOf(configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_CAN_FINISH_EARLY)).booleanValue();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.wizards.IWizardDescriptor#hasPages()
     */
    public bool hasPages() {
        String hasPagesString = configurationElement.getAttribute(IWorkbenchRegistryConstants.ATT_HAS_PAGES);
        // default value is true
        if (hasPagesString is null) {
            return true;
        }
        return Boolean.valueOf(hasPagesString).booleanValue();
    }
}
