/*******************************************************************************
 * Copyright (c) 2000, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *        IBM Corporation - initial API and implementation 
 *        Sebastian Davids <sdavids@gmx.de> - Fix for bug 19346 - Dialog font should be
 *        activated and used by other components.
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.dialogs.WorkingSetTypePage;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.graphics.Font;
import dwt.graphics.Image;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Composite;
import dwt.widgets.Label;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.resource.JFaceResources;
import dwtx.jface.resource.LocalResourceManager;
import dwtx.jface.resource.ResourceManager;
import dwtx.jface.viewers.ArrayContentProvider;
import dwtx.jface.viewers.DoubleClickEvent;
import dwtx.jface.viewers.IDoubleClickListener;
import dwtx.jface.viewers.ISelection;
import dwtx.jface.viewers.ISelectionChangedListener;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.jface.viewers.LabelProvider;
import dwtx.jface.viewers.SelectionChangedEvent;
import dwtx.jface.viewers.TableViewer;
import dwtx.jface.wizard.WizardPage;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchGraphicConstants;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.WorkingSetDescriptor;

/**
 * The working set type page is used in the new working set 
 * wizard to select from a list of plugin defined working set 
 * types.
 * 
 * @since 2.0
 */
public class WorkingSetTypePage extends WizardPage {
    private final static int SIZING_SELECTION_WIDGET_WIDTH = 50;

    private final static int SIZING_SELECTION_WIDGET_HEIGHT = 200;

    private TableViewer typesListViewer;

    private WorkingSetDescriptor[] descriptors;

    /**
     * Creates a new instance of the receiver
     */
    public WorkingSetTypePage() {
        this(WorkbenchPlugin.getDefault().getWorkingSetRegistry().getNewPageWorkingSetDescriptors());
    }

    /**
     * @param descriptors a set of working set descriptors which can be selected on the page
     */
    public WorkingSetTypePage(WorkingSetDescriptor[] descriptors) {
        super(
                "workingSetTypeSelectionPage", WorkbenchMessages.WorkingSetTypePage_description, WorkbenchImages.getImageDescriptor(IWorkbenchGraphicConstants.IMG_WIZBAN_WORKINGSET_WIZ)); //$NON-NLS-1$ 
        this.descriptors = descriptors;
    }

    /**
     * Overrides method in WizardPage
     * 
     * @see dwtx.jface.wizard.IWizardPage#canFlipToNextPage()
     */
    public bool canFlipToNextPage() {
        return isPageComplete();
    }

    /** 
     * Implements IDialogPage
     * 
     * @see dwtx.jface.dialogs.IDialogPage#createControl(Composite)
     */
    public void createControl(Composite parent) {
        Font font = parent.getFont();
        Composite composite = new Composite(parent, DWT.NULL);
        composite.setLayout(new GridLayout());
        composite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
        setControl(composite);

        PlatformUI.getWorkbench().getHelpSystem().setHelp(composite,
                IWorkbenchHelpContextIds.WORKING_SET_TYPE_PAGE);
        Label typesLabel = new Label(composite, DWT.NONE);
        typesLabel.setText(WorkbenchMessages.WorkingSetTypePage_typesLabel);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        typesLabel.setLayoutData(data);
        typesLabel.setFont(font);

        typesListViewer = new TableViewer(composite, DWT.BORDER | DWT.SINGLE);
        data = new GridData(GridData.FILL_BOTH);
        data.heightHint = SIZING_SELECTION_WIDGET_HEIGHT;
        data.widthHint = SIZING_SELECTION_WIDGET_WIDTH;
        typesListViewer.getTable().setLayoutData(data);
        typesListViewer.getTable().setFont(font);
        typesListViewer
                .addSelectionChangedListener(new ISelectionChangedListener() {
                    public void selectionChanged(SelectionChangedEvent event) {
                        handleSelectionChanged();
                    }
                });
        typesListViewer.addDoubleClickListener(new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                handleDoubleClick();
            }
        });
        typesListViewer.setContentProvider(new ArrayContentProvider());
        typesListViewer.setLabelProvider(new LabelProvider() {
            private ResourceManager images = new LocalResourceManager(
                    JFaceResources.getResources());
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.jface.viewers.LabelProvider#getText(java.lang.Object)
             */
            public String getText(Object element) {
                return ((WorkingSetDescriptor)element).getName();
            }
            
            /* (non-Javadoc)
             * @see dwtx.jface.viewers.BaseLabelProvider#dispose()
             */
            public void dispose() {
                images.dispose();
                super.dispose();
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.jface.viewers.LabelProvider#getImage(java.lang.Object)
             */
            public Image getImage(Object element) {
                ImageDescriptor imageDescriptor = ((WorkingSetDescriptor) element)
                        .getIcon();
                return imageDescriptor is null ? null : (Image) images
                        .get(imageDescriptor);
            }
        });
        typesListViewer.setInput(descriptors);
        setPageComplete(false);
    }

    /** 
     * Overrides method in DialogPage
     * 
     * @see dwtx.jface.dialogs.IDialogPage#dispose()
     */
    public void dispose() {
        super.dispose();
    }

    /**
     * Returns the page id of the selected working set type.
     * 
     * @return the page id of the selected working set type.
     */
    public String getSelection() {
        WorkingSetDescriptor descriptor = getSelectedWorkingSet();
        if (descriptor !is null)
            return descriptor.getId();
        
        return null;
    }

    /**
     * Return the selected working set.
     *
     * @return the selected working set or <code>null</code>
     * @since 3.4
     */
    private WorkingSetDescriptor getSelectedWorkingSet() {
        ISelection selection = typesListViewer.getSelection();
        bool hasSelection = selection !is null
                && selection.isEmpty() is false;

        WorkingSetDescriptor descriptor = null;
        if (hasSelection && selection instanceof IStructuredSelection) {
            descriptor = (WorkingSetDescriptor) ((IStructuredSelection) selection)
                    .getFirstElement();
        }
        return descriptor;
    }

    /**
     * Called when a working set type is double clicked.
     */
    private void handleDoubleClick() {
        handleSelectionChanged();
        getContainer().showPage(getNextPage());
    }

    /**
     * Called when the selection has changed.
     */
    private void handleSelectionChanged() {
        ISelection selection = typesListViewer.getSelection();
        bool hasSelection = selection !is null
                && selection.isEmpty() is false;

        WorkingSetDescriptor descriptor = getSelectedWorkingSet();
        setDescription(descriptor is null ? "" : descriptor.getDescription()); //$NON-NLS-1$
        
        setPageComplete(hasSelection);
    }
}
