/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.expressions.WorkbenchWindowExpression;

import dwt.dwthelper.utils;

import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.ExpressionInfo;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.ui.ISources;
import dwtx.ui.IWorkbenchWindow;

/**
 * <p>
 * An expression that evaluates to {@link EvaluationResult#TRUE} when the active
 * workbench window matches the window held by this expression.
 * </p>
 * 
 * @since 3.2
 */
public class WorkbenchWindowExpression extends Expression {

    /**
     * The seed for the hash code for all schemes.
     */
    private static final int HASH_INITIAL = WorkbenchWindowExpression.class
            .getName().hashCode();

    /**
     * The workbench window that must be active for this expression to evaluate
     * to <code>true</code>. If this value is <code>null</code>, then any
     * workbench window may be active.
     */
    private final IWorkbenchWindow window;

    /**
     * Constructs a new instance.
     * 
     * @param window
     *            The workbench window which must be active for this expression
     *            to evaluate to <code>true</code>; may be <code>null</code>
     *            if this expression is always <code>true</code>.
     */
    public WorkbenchWindowExpression(final IWorkbenchWindow window) {
        this.window = window;
    }

    public void collectExpressionInfo(final ExpressionInfo info) {
        if (window !is null) {
            info.addVariableNameAccess(ISources.ACTIVE_WORKBENCH_WINDOW_NAME);
        }
    }

    protected int computeHashCode() {
        return HASH_INITIAL * HASH_FACTOR + hashCode(window);
    }

    public bool equals(final Object object) {
        if (object instanceof WorkbenchWindowExpression) {
            final WorkbenchWindowExpression that = (WorkbenchWindowExpression) object;
            return equals(this.window, that.window);
        }

        return false;
    }

    public EvaluationResult evaluate(final IEvaluationContext context)
            throws CoreException {
        if (window !is null) {
            Object value = context
                    .getVariable(ISources.ACTIVE_WORKBENCH_WINDOW_NAME);
            if (window.equals(value)) {
                return EvaluationResult.TRUE;
            }
        }

        return EvaluationResult.FALSE;
    }

    /**
     * Returns the workbench window to which this expression applies.
     * 
     * @return The workbench window to which this expression applies; may be
     *         <code>null</code>.
     */
    protected final IWorkbenchWindow getWindow() {
        return window;
    }

    public String toString() {
        final StringBuffer buffer = new StringBuffer();
        buffer.append("WorkbenchWindowExpression("); //$NON-NLS-1$
        buffer.append(window);
        buffer.append(')');
        return buffer.toString();
    }
}

