/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.handlers.HandlerProxy;

import dwt.dwthelper.utils;

import java.util.HashMap;
import java.util.Map;

import dwtx.core.commands.AbstractHandler;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.HandlerEvent;
import dwtx.core.commands.IHandler;
import dwtx.core.commands.IHandler2;
import dwtx.core.commands.IHandlerListener;
import dwtx.core.expressions.EvaluationResult;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.jface.util.PropertyChangeEvent;
import dwtx.ui.PlatformUI;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.commands.IElementUpdater;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.util.BundleUtility;
import dwtx.ui.internal.util.Util;
import dwtx.ui.menus.UIElement;
import dwtx.ui.services.IEvaluationReference;
import dwtx.ui.services.IEvaluationService;

/**
 * <p>
 * A proxy for a handler that has been defined in XML. This delays the class
 * loading until the handler is really asked for information (besides the
 * priority or the command identifier). Asking a proxy for anything but the
 * attributes defined publicly in this class will cause the proxy to instantiate
 * the proxied handler.
 * </p>
 * 
 * @since 3.0
 */
public final class HandlerProxy extends AbstractHandler implements
        IElementUpdater {

    private static Map CEToProxyMap = new HashMap();

    /**
     * 
     */
    private static final String PROP_ENABLED = "enabled"; //$NON-NLS-1$

    /**
     * The configuration element from which the handler can be created. This
     * value will exist until the element is converted into a real class -- at
     * which point this value will be set to <code>null</code>.
     */
    private IConfigurationElement configurationElement;

    /**
     * The <code>enabledWhen</code> expression for the handler. Only if this
     * expression evaluates to <code>true</code> (or the value is
     * <code>null</code>) should we consult the handler.
     */
    private final Expression enabledWhenExpression;

    /**
     * The real handler. This value is <code>null</code> until the proxy is
     * forced to load the real handler. At this point, the configuration element
     * is converted, nulled out, and this handler gains a reference.
     */
    private IHandler handler = null;

    /**
     * The name of the configuration element attribute which contains the
     * information necessary to instantiate the real handler.
     */
    private final String handlerAttributeName;

    private IHandlerListener handlerListener;

    /**
     * The evaluation service to use when evaluating
     * <code>enabledWhenExpression</code>. This value may be
     * <code>null</code> only if the <code>enabledWhenExpression</code> is
     * <code>null</code>.
     */
    private IEvaluationService evaluationService;

    private IPropertyChangeListener enablementListener;

    private IEvaluationReference enablementRef;

    private bool proxyEnabled;
    
    private String commandId;

    /**
     * Constructs a new instance of <code>HandlerProxy</code> with all the
     * information it needs to try to avoid loading until it is needed.
     * 
     * @param commandId the id for this handler
     * @param configurationElement
     *            The configuration element from which the real class can be
     *            loaded at run-time; must not be <code>null</code>.
     * @param handlerAttributeName
     *            The name of the attibute or element containing the handler
     *            executable extension; must not be <code>null</code>.
     */
    public HandlerProxy(final String commandId, final IConfigurationElement configurationElement,
            final String handlerAttributeName) {
        this(commandId, configurationElement, handlerAttributeName, null, null);
    }

    /**
     * Constructs a new instance of <code>HandlerProxy</code> with all the
     * information it needs to try to avoid loading until it is needed.
     * 
     * @param commandId the id for this handler
     * @param configurationElement
     *            The configuration element from which the real class can be
     *            loaded at run-time; must not be <code>null</code>.
     * @param handlerAttributeName
     *            The name of the attribute or element containing the handler
     *            executable extension; must not be <code>null</code>.
     * @param enabledWhenExpression
     *            The name of the element containing the enabledWhen expression.
     *            This should be a child of the
     *            <code>configurationElement</code>. If this value is
     *            <code>null</code>, then there is no enablement expression
     *            (i.e., enablement will be delegated to the handler when
     *            possible).
     * @param evaluationService
     *            The evaluation service to manage enabledWhen expressions
     *            trying to evaluate the <code>enabledWhenExpression</code>.
     *            This value may be <code>null</code> only if the
     *            <code>enabledWhenExpression</code> is <code>null</code>.
     */
    public HandlerProxy(final String commandId, 
            final IConfigurationElement configurationElement,
            final String handlerAttributeName,
            final Expression enabledWhenExpression,
            final IEvaluationService evaluationService) {
        if (configurationElement is null) {
            throw new NullPointerException(
                    "The configuration element backing a handler proxy cannot be null"); //$NON-NLS-1$
        }

        if (handlerAttributeName is null) {
            throw new NullPointerException(
                    "The attribute containing the handler class must be known"); //$NON-NLS-1$
        }

        if ((enabledWhenExpression !is null) && (evaluationService is null)) {
            throw new NullPointerException(
                    "We must have a handler service and evaluation service to support the enabledWhen expression"); //$NON-NLS-1$
        }

        this.commandId = commandId;
        this.configurationElement = configurationElement;
        this.handlerAttributeName = handlerAttributeName;
        this.enabledWhenExpression = enabledWhenExpression;
        this.evaluationService = evaluationService;
        if (enabledWhenExpression !is null) {
            setProxyEnabled(false);
            registerEnablement();
        } else {
            setProxyEnabled(true);
        }

        CEToProxyMap.put(configurationElement, this);
    }

    public static void updateStaleCEs(IConfigurationElement[] replacements) {
        for (int i = 0; i < replacements.length; i++) {
            HandlerProxy proxy = (HandlerProxy) CEToProxyMap
                    .get(replacements[i]);
            if (proxy !is null)
                proxy.configurationElement = replacements[i];
        }
    }

    /**
     * 
     */
    private void registerEnablement() {
        enablementRef = evaluationService.addEvaluationListener(
                enabledWhenExpression, getEnablementListener(), PROP_ENABLED);
    }

    public void setEnabled(Object evaluationContext) {
        if (!(evaluationContext instanceof IEvaluationContext)) {
            return;
        }
        IEvaluationContext context = (IEvaluationContext) evaluationContext;
        if (enabledWhenExpression !is null) {
            try {
                setProxyEnabled(enabledWhenExpression.evaluate(context) is EvaluationResult.TRUE);
            } catch (CoreException e) {
                // TODO should we log this exception, or just treat it as
                // a failure
            }
        }
        if (isOkToLoad() && loadHandler()) {
            if (handler instanceof IHandler2) {
                ((IHandler2) handler).setEnabled(evaluationContext);
            }
        }
    }

    void setProxyEnabled(bool enabled) {
        proxyEnabled = enabled;
    }

    bool getProxyEnabled() {
        return proxyEnabled;
    }

    /**
     * @return
     */
    private IPropertyChangeListener getEnablementListener() {
        if (enablementListener is null) {
            enablementListener = new IPropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent event) {
                    if (event.getProperty() is PROP_ENABLED) {
                        setProxyEnabled(event.getNewValue() is null ? false
                                : ((Boolean) event.getNewValue())
                                        .booleanValue());
                        fireHandlerChanged(new HandlerEvent(HandlerProxy.this,
                                true, false));
                    }
                }
            };
        }
        return enablementListener;
    }

    /**
     * Passes the dipose on to the proxied handler, if it has been loaded.
     */
    public final void dispose() {
        if (handler !is null) {
            if (handlerListener !is null) {
                handler.removeHandlerListener(handlerListener);
                handlerListener = null;
            }
            handler.dispose();
            handler = null;
        }
        if (enablementListener !is null) {
            evaluationService.removeEvaluationListener(enablementRef);
            enablementRef = null;
            enablementListener = null;
        }
    }

    public final Object execute(final ExecutionEvent event)
            throws ExecutionException {
        if (loadHandler()) {
            if (!isEnabled()) {
                MessageDialog.openInformation(Util.getShellToParentOn(),
                        WorkbenchMessages.Information,
                        WorkbenchMessages.PluginAction_disabledMessage);
                return null;
            }
            return handler.execute(event);
        }

        return null;
    }

    public final bool isEnabled() {
        if (enabledWhenExpression !is null) {
            // proxyEnabled reflects the enabledWhen clause
            if (!getProxyEnabled()) {
                return false;
            }
            if (isOkToLoad() && loadHandler()) {
                return handler.isEnabled();
            }

            return true;
        }

        /*
         * There is no enabled when expression, so we just need to consult the
         * handler.
         */
        if (isOkToLoad() && loadHandler()) {
            return handler.isEnabled();
        }
        return true;
    }

    public final bool isHandled() {
        if (configurationElement !is null && handler is null) {
            return true;
        }

        if (isOkToLoad() && loadHandler()) {
            return handler.isHandled();
        }

        return false;
    }

    /**
     * Loads the handler, if possible. If the handler is loaded, then the member
     * variables are updated accordingly.
     * 
     * @return <code>true</code> if the handler is now non-null;
     *         <code>false</code> otherwise.
     */
    private final bool loadHandler() {
        if (handler is null) {
            // Load the handler.
            try {
                if (configurationElement !is null) {
                    handler = (IHandler) configurationElement
                            .createExecutableExtension(handlerAttributeName);
                    handler.addHandlerListener(getHandlerListener());
                    setEnabled(evaluationService is null ? null
                            : evaluationService.getCurrentState());
                    refreshElements();
                    return true;
                }

            } catch (final ClassCastException e) {
                final String message = "The proxied handler was the wrong class"; //$NON-NLS-1$
                final IStatus status = new Status(IStatus.ERROR,
                        WorkbenchPlugin.PI_WORKBENCH, 0, message, e);
                WorkbenchPlugin.log(message, status);
                configurationElement = null;

            } catch (final CoreException e) {
                final String message = "The proxied handler for '" + configurationElement.getAttribute(handlerAttributeName) //$NON-NLS-1$
                        + "' could not be loaded"; //$NON-NLS-1$
                IStatus status = new Status(IStatus.ERROR,
                        WorkbenchPlugin.PI_WORKBENCH, 0, message, e);
                WorkbenchPlugin.log(message, status);
                configurationElement = null;
            }
            return false;
        }

        return true;
    }

    private IHandlerListener getHandlerListener() {
        if (handlerListener is null) {
            handlerListener = new IHandlerListener() {
                public void handlerChanged(HandlerEvent handlerEvent) {
                    fireHandlerChanged(new HandlerEvent(HandlerProxy.this,
                            handlerEvent.isEnabledChanged(), handlerEvent
                                    .isHandledChanged()));
                }
            };
        }
        return handlerListener;
    }

    public final String toString() {
        if (handler is null) {
            if (configurationElement !is null) {
                String configurationElementAttribute = getConfigurationElementAttribute();
                if (configurationElementAttribute !is null) {
                    return configurationElementAttribute;
                }
            }
            return "HandlerProxy()"; //$NON-NLS-1$
        }

        return handler.toString();
    }

    /**
     * Retrives the ConfigurationElement attribute according to the
     * <code>handlerAttributeName</code>.
     * 
     * @return the handlerAttributeName value, may be <code>null</code>.
     */
    private String getConfigurationElementAttribute() {
        String attribute = configurationElement
                .getAttribute(handlerAttributeName);
        if (attribute is null) {
            IConfigurationElement[] children = configurationElement
                    .getChildren(handlerAttributeName);
            for (int i = 0; i < children.length; i++) {
                String childAttribute = children[i]
                        .getAttribute(IWorkbenchRegistryConstants.ATT_CLASS);
                if (childAttribute !is null) {
                    return childAttribute;
                }
            }
        }
        return attribute;
    }

    private bool isOkToLoad() {
        if (configurationElement !is null && handler is null) {
            final String bundleId = configurationElement.getContributor()
                    .getName();
            return BundleUtility.isActive(bundleId);
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.commands.IElementUpdater#updateElement(dwtx.ui.menus.UIElement,
     *      java.util.Map)
     */
    public void updateElement(UIElement element, Map parameters) {
        if (handler !is null && handler instanceof IElementUpdater) {
            ((IElementUpdater) handler).updateElement(element, parameters);
        }
    }
    
    private void refreshElements() {
        if (commandId is null || !(handler instanceof IElementUpdater)) {
            return;
        }
        ICommandService cs = (ICommandService) PlatformUI.getWorkbench().getService(ICommandService.class);
        cs.refreshElements(commandId, null);
    }
}
