/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.handlers.LegacyHandlerWrapper;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.IHandler;
import dwtx.core.commands.IHandlerListener;
import dwtx.core.commands.util.Tracing;
import dwtx.ui.internal.commands.ILegacyAttributeNames;
import dwtx.ui.internal.misc.Policy;

/**
 * A handler that wraps a legacy handler. This provide backward compatibility
 * with the handlers release in Eclipse 3.0.
 * 
 * @since 3.1
 */
public final class LegacyHandlerWrapper implements IHandler {

    /**
     * This flag can be set to <code>true</code> if commands should print
     * information to <code>System.out</code> when changing handlers.
     */
    private static final bool DEBUG_HANDLERS = Policy.DEBUG_HANDLERS
            && Policy.DEBUG_HANDLERS_VERBOSE;

    /**
     * The wrapped handler; never <code>null</code>.
     */
    private final dwtx.ui.commands.IHandler handler;

    /**
     * Constructs a new instance of <code>HandlerWrapper</code>.
     * 
     * @param handler
     *            The handler that should be wrapped; must not be
     *            <code>null</code>.
     */
    public LegacyHandlerWrapper(final dwtx.ui.commands.IHandler handler) {
        if (handler is null) {
            throw new NullPointerException(
                    "A handler wrapper cannot be constructed on a null handler"); //$NON-NLS-1$
        }

        this.handler = handler;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#addHandlerListener(dwtx.core.commands.IHandlerListener)
     */
    public final void addHandlerListener(final IHandlerListener handlerListener) {
        handler.addHandlerListener(new LegacyHandlerListenerWrapper(this,
                handlerListener));
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#dispose()
     */
    public final void dispose() {
        handler.dispose();
    }

    public final bool equals(final Object object) {
        if (object instanceof dwtx.ui.commands.IHandler) {
            return this.handler is object;
        }

        if (object instanceof LegacyHandlerWrapper) {
            return this.handler is ((LegacyHandlerWrapper) object).handler;
        }

        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.IHandler#execute(dwtx.core.commands.ExecutionEvent)
     */
    public final Object execute(final ExecutionEvent event)
            throws ExecutionException {
        // Debugging output
        if (DEBUG_HANDLERS) {
            final StringBuffer buffer = new StringBuffer("Executing LegacyHandlerWrapper for "); //$NON-NLS-1$
            if (handler is null) {
                buffer.append("no handler"); //$NON-NLS-1$
            } else {
                buffer.append('\'');
                buffer.append(handler.getClass().getName());
                buffer.append('\'');
            }
            Tracing.printTrace("HANDLERS", buffer.toString()); //$NON-NLS-1$
        }

        try {
            return handler.execute(event.getParameters());
        } catch (final dwtx.ui.commands.ExecutionException e) {
            throw new ExecutionException(e.getMessage(), e.getCause());
        }
    }

    public final int hashCode() {
        return this.handler.hashCode();
    }

    public final bool isEnabled() {
        final Object enabled = handler.getAttributeValuesByName().get(
                ILegacyAttributeNames.ENABLED);
        if (enabled instanceof Boolean) {
            return ((Boolean) enabled).booleanValue();
        }

        return true;
    }

    public final bool isHandled() {
        final Object handled = handler.getAttributeValuesByName().get(
                ILegacyAttributeNames.HANDLED);
        if (handled instanceof Boolean) {
            return ((Boolean) handled).booleanValue();
        }

        return true;
    }

    public final void removeHandlerListener(
            final IHandlerListener handlerListener) {
        handler.removeHandlerListener(new LegacyHandlerListenerWrapper(this,
                handlerListener));
    }

    public final String toString() {
        final StringBuffer buffer = new StringBuffer();

        buffer.append("LegacyHandlerWrapper("); //$NON-NLS-1$
        buffer.append(handler);
        buffer.append(')');

        return buffer.toString();
    }
}
