/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.handlers.SlaveHandlerService;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import dwt.widgets.Event;
import dwtx.core.commands.Command;
import dwtx.core.commands.ExecutionEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.IHandler;
import dwtx.core.commands.NotEnabledException;
import dwtx.core.commands.NotHandledException;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.core.expressions.Expression;
import dwtx.core.expressions.IEvaluationContext;
import dwtx.ui.ISourceProvider;
import dwtx.ui.handlers.IHandlerActivation;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.internal.expressions.AndExpression;

/**
 * A handler service which delegates almost all responsibility to the parent
 * service. It is only responsible for disposing of locally activated handlers
 * when it is disposed.
 * <p>
 * This class is not intended for use outside of the
 * <code>dwtx.ui.workbench</code> plug-in.
 * </p>
 * 
 * @since 3.2
 */
public class SlaveHandlerService implements IHandlerService {

    /**
     * The default expression to use when
     * {@link #activateHandler(String, IHandler)} is called. Handlers
     * contributed using that method will only be active when this service is
     * active. However, this expression will be used for conflict resolution.
     */
    protected final Expression defaultExpression;

    /**
     * A collection of source providers. The listeners are not
     * activated/deactivated, but they will be removed when this service is
     * disposed.
     */
    private Collection fSourceProviders = new ArrayList();

    /**
     * A map of the local activation to the parent activations. If this service
     * is inactive, then all parent activations are <code>null</code>.
     * Otherwise, they point to the corresponding activation in the parent
     * service.
     */
    protected final Map localActivationsToParentActivations = new HashMap();

    /**
     * The parent handler service to which all requests are ultimately routed.
     * This value is never <code>null</code>.
     */
    protected final IHandlerService parent;

    /**
     * The activations registered with the parent handler service. This value is
     * never <code>null</code>.
     */
    protected final Set parentActivations = new HashSet();

    /**
     * Constructs a new instance.
     * 
     * @param parentHandlerService
     *            The parent handler service for this slave; must not be
     *            <code>null</code>.
     * @param defaultExpression
     *            The default expression to use if none is provided. This is
     *            primarily used for conflict resolution. This value may be
     *            <code>null</code>.
     */
    public SlaveHandlerService(final IHandlerService parentHandlerService,
            final Expression defaultExpression) {
        if (parentHandlerService is null) {
            throw new NullPointerException(
                    "The parent handler service cannot be null"); //$NON-NLS-1$
        }

        this.defaultExpression = defaultExpression;
        this.parent = parentHandlerService;
    }

    public final IHandlerActivation activateHandler(
            final IHandlerActivation childActivation) {
        final String commandId = childActivation.getCommandId();
        final IHandler handler = childActivation.getHandler();
        final Expression childExpression = childActivation.getExpression();
        final AndExpression expression;
        if (childExpression instanceof AndExpression) {
            expression = (AndExpression) childExpression;
        } else {
            expression = new AndExpression();
            if (childExpression !is null) {
                expression.add(childExpression);
            }
        }
        if (defaultExpression !is null) {
            expression.add(defaultExpression);
        }
        final int depth = childActivation.getDepth() + 1;
        final IHandlerActivation localActivation = new HandlerActivation(
                commandId, handler, expression, depth, this);

        return doActivation(localActivation);
    }

    public final IHandlerActivation activateHandler(final String commandId,
            final IHandler handler) {
        final IHandlerActivation localActivation = new HandlerActivation(
                commandId, handler, defaultExpression,
                IHandlerActivation.ROOT_DEPTH, this);
        return doActivation(localActivation);
    }

    public final IHandlerActivation activateHandler(final String commandId,
            final IHandler handler, final Expression expression) {
        return activateHandler(commandId, handler, expression, false);
    }

    public final IHandlerActivation activateHandler(final String commandId,
            final IHandler handler, final Expression expression,
            final bool global) {
        if (global) {
            final IHandlerActivation activation = parent.activateHandler(
                    commandId, handler, expression, global);
            parentActivations.add(activation);
            return activation;
        }

        final AndExpression andExpression;
        if (expression instanceof AndExpression) {
            andExpression = (AndExpression) expression;
        } else {
            andExpression = new AndExpression();
            if (expression !is null) {
                andExpression.add(expression);
            }
        }
        if (defaultExpression !is null) {
            andExpression.add(defaultExpression);
        }
        final IHandlerActivation localActivation = new HandlerActivation(
                commandId, handler, andExpression,
                IHandlerActivation.ROOT_DEPTH, this);
        return doActivation(localActivation);
    }

    public final IHandlerActivation activateHandler(final String commandId,
            final IHandler handler, final Expression expression,
            final int sourcePriorities) {
        return activateHandler(commandId, handler, expression);
    }

    public final void addSourceProvider(final ISourceProvider provider) {
        if (!fSourceProviders.contains(provider)) {
            fSourceProviders.add(provider);
        }
        parent.addSourceProvider(provider);
    }

    public final ExecutionEvent createExecutionEvent(final Command command,
            final Event event) {
        return parent.createExecutionEvent(command, event);
    }

    public final ExecutionEvent createExecutionEvent(
            final ParameterizedCommand command, final Event event) {
        return parent.createExecutionEvent(command, event);
    }

    public final void deactivateHandler(final IHandlerActivation activation) {
        final IHandlerActivation parentActivation;
        if (localActivationsToParentActivations.containsKey(activation)) {
            parentActivation = (IHandlerActivation) localActivationsToParentActivations
                    .remove(activation);
        } else {
            parentActivation = activation;
        }

        if (parentActivation !is null) {
            parent.deactivateHandler(parentActivation);
            parentActivations.remove(parentActivation);
        }
    }

    public final void deactivateHandlers(final Collection activations) {
        Object[] array = activations.toArray();
        for (int i = 0; i < array.length; i++) {
            deactivateHandler((IHandlerActivation) array[i]);
            array[i] = null;
        }
    }

    public final void dispose() {
        parent.deactivateHandlers(parentActivations);
        parentActivations.clear();
        localActivationsToParentActivations.clear();

        // Remove any "resource", like listeners, that were associated
        // with this service.
        if (!fSourceProviders.isEmpty()) {
            Object[] array = fSourceProviders.toArray();
            for (int i = 0; i < array.length; i++) {
                removeSourceProvider((ISourceProvider) array[i]);
            }
            fSourceProviders.clear();
        }
    }

    protected IHandlerActivation doActivation(
            final IHandlerActivation localActivation) {
        final IHandlerActivation parentActivation;
        parentActivation = parent.activateHandler(localActivation);
        parentActivations.add(parentActivation);
        localActivationsToParentActivations.put(localActivation,
                parentActivation);
        return localActivation;
    }

    public final Object executeCommand(final ParameterizedCommand command,
            final Event event) throws ExecutionException, NotDefinedException,
            NotEnabledException, NotHandledException {
        return parent.executeCommand(command, event);
    }

    public final Object executeCommand(final String commandId, final Event event)
            throws ExecutionException, NotDefinedException,
            NotEnabledException, NotHandledException {
        return parent.executeCommand(commandId, event);
    }

    public final IEvaluationContext getCurrentState() {
        return parent.getCurrentState();
    }

    public final void readRegistry() {
        parent.readRegistry();
    }

    public final void removeSourceProvider(final ISourceProvider provider) {
        fSourceProviders.remove(provider);
        parent.removeSourceProvider(provider);
    }

    public final void setHelpContextId(final IHandler handler,
            final String helpContextId) {
        parent.setHelpContextId(handler, helpContextId);
    }

    Expression getDefaultExpression() {
        return defaultExpression;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.handlers.IHandlerService#createContextSnapshot(bool)
     */
    public IEvaluationContext createContextSnapshot(bool includeSelection) {
        return parent.createContextSnapshot(includeSelection);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.handlers.IHandlerService#executeCommandInContext(dwtx.core.commands.ParameterizedCommand, dwt.widgets.Event, dwtx.core.expressions.IEvaluationContext)
     */
    public Object executeCommandInContext(ParameterizedCommand command,
            Event event, IEvaluationContext context) throws ExecutionException,
            NotDefinedException, NotEnabledException, NotHandledException {
        return parent.executeCommandInContext(command, event, context);
    }
}

