/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.keys.BindingService;

import dwt.dwthelper.utils;

import java.io.IOException;
import java.util.Map;

import dwt.DWT;
import dwt.widgets.Display;
import dwt.widgets.Listener;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.jface.bindings.Binding;
import dwtx.jface.bindings.BindingManager;
import dwtx.jface.bindings.Scheme;
import dwtx.jface.bindings.TriggerSequence;
import dwtx.jface.bindings.keys.SWTKeySupport;
import dwtx.jface.bindings.keys.formatting.KeyFormatterFactory;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.internal.Workbench;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.keys.IBindingService;

/**
 * <p>
 * Provides services related to the binding architecture (e.g., keyboard
 * shortcuts) within the workbench. This service can be used to access the
 * currently active bindings, as well as the current state of the binding
 * architecture.
 * </p>
 * 
 * @since 3.1
 */
public final class BindingService implements IBindingService {

    /**
     * The binding manager that supports this service. This value is never
     * <code>null</code>.
     */
    private final BindingManager bindingManager;

    /**
     * The persistence class responsible for bindings.
     */
    private final BindingPersistence bindingPersistence;

    /**
     * The key binding support for the contexts. In the workbench, key bindings
     * are intimately tied to the context mechanism.
     */
    private WorkbenchKeyboard keyboard;

    /**
     * Constructs a new instance of <code>BindingService</code> using a JFace
     * binding manager.
     * 
     * @param bindingManager
     *            The bind ing manager to use; must not be <code>null</code>.
     * @param commandService
     *            The command service providing support for this service; must
     *            not be <code>null</code>;
     * @param workbench
     *            The workbench on which this context service will act; must not
     *            be <code>null</code>.
     */
    public BindingService(final BindingManager bindingManager,
            final ICommandService commandService, final Workbench workbench) {
        if (bindingManager is null) {
            throw new NullPointerException(
                    "Cannot create a binding service with a null manager"); //$NON-NLS-1$
        }
        if (commandService is null) {
            throw new NullPointerException(
                    "Cannot create a binding service with a null command service"); //$NON-NLS-1$
        }
        this.bindingManager = bindingManager;
        this.bindingPersistence = new BindingPersistence(bindingManager,
                commandService);

        // Hook up the key binding support.
        keyboard = new WorkbenchKeyboard(workbench);
        final Display display = workbench.getDisplay();
        final Listener listener = keyboard.getKeyDownFilter();
        display.addFilter(DWT.KeyDown, listener);
        display.addFilter(DWT.Traverse, listener);

        // Initialize the key formatter.
        KeyFormatterFactory.setDefault(SWTKeySupport
                .getKeyFormatterForPlatform());
    }

    /**
     * TODO Promote this method to API.
     * <p>
     * Adds a single new binding to the existing array of bindings. If the array
     * is currently <code>null</code>, then a new array is created and this
     * binding is added to it. This method does not detect duplicates.
     * </p>
     * <p>
     * This method completes in amortized <code>O(1)</code>.
     * </p>
     * 
     * @param binding
     *            The binding to be added; must not be <code>null</code>.
     */
    public final void addBinding(final Binding binding) {
        bindingManager.addBinding(binding);
    }
    
    public final void dispose() {
        bindingPersistence.dispose();
    }

    public final TriggerSequence[] getActiveBindingsFor(
            final ParameterizedCommand parameterizedCommand) {
        return bindingManager.getActiveBindingsFor(parameterizedCommand);
    }

    public final TriggerSequence[] getActiveBindingsFor(final String commandId) {
        return bindingManager.getActiveBindingsFor(commandId);
    }

    public final Scheme getActiveScheme() {
        return bindingManager.getActiveScheme();
    }

    public final TriggerSequence getBestActiveBindingFor(final String commandId) {
        return bindingManager.getBestActiveBindingFor(commandId);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.keys.IBindingService#getBestActiveBindingFor(dwtx.core.commands.ParameterizedCommand)
     */
    public TriggerSequence getBestActiveBindingFor(ParameterizedCommand command) {
        return bindingManager.getBestActiveBindingFor(command);
    }

    public final String getBestActiveBindingFormattedFor(final String commandId) {
        return bindingManager.getBestActiveBindingFormattedFor(commandId);
    }

    public final Binding[] getBindings() {
        return bindingManager.getBindings();
    }

    public final TriggerSequence getBuffer() {
        return keyboard.getBuffer();
    }

    public final String getDefaultSchemeId() {
        return BindingPersistence.getDefaultSchemeId();
    }

    public final Scheme[] getDefinedSchemes() {
        return bindingManager.getDefinedSchemes();
    }

    /**
     * Returns the key binding architecture for the workbench. This method is
     * internal, and is only intended for testing. This must not be used by
     * clients.
     * 
     * @return The key binding support; never <code>null</code>.
     */
    public final WorkbenchKeyboard getKeyboard() {
        return keyboard;
    }

    public final String getLocale() {
        return bindingManager.getLocale();
    }

    public final Map getPartialMatches(final TriggerSequence trigger) {
        return bindingManager.getPartialMatches(trigger);
    }

    public final Binding getPerfectMatch(final TriggerSequence trigger) {
        return bindingManager.getPerfectMatch(trigger);
    }

    public final String getPlatform() {
        return bindingManager.getPlatform();
    }

    public final Scheme getScheme(final String schemeId) {
        return bindingManager.getScheme(schemeId);
    }

    public final bool isKeyFilterEnabled() {
        return keyboard.getKeyDownFilter().isEnabled();
    }

    public final bool isPartialMatch(final TriggerSequence sequence) {
        return bindingManager.isPartialMatch(sequence);
    }

    public final bool isPerfectMatch(final TriggerSequence sequence) {
        return bindingManager.isPerfectMatch(sequence);
    }

    public final void openKeyAssistDialog() {
        keyboard.openMultiKeyAssistShell();
    }

    public final void readRegistryAndPreferences(
            final ICommandService commandService) {
        bindingPersistence.read();
    }

    /**
     * Remove the specific binding by identity. Does nothing if the binding is
     * not in the manager.
     * 
     * @param binding
     *            The binding to be removed; must not be <code>null</code>.
     */
    public final void removeBinding(final Binding binding) {
        bindingManager.removeBinding(binding);
    }

    public final void savePreferences(final Scheme activeScheme,
            final Binding[] bindings) throws IOException {
        BindingPersistence.write(activeScheme, bindings);
        try {
            bindingManager.setActiveScheme(activeScheme);
        } catch (final NotDefinedException e) {
            WorkbenchPlugin.log("The active scheme is not currently defined.",  //$NON-NLS-1$
                    WorkbenchPlugin.getStatus(e));
        }
        bindingManager.setBindings(bindings);
    }

    public final void setKeyFilterEnabled(final bool enabled) {
        keyboard.getKeyDownFilter().setEnabled(enabled);
    }

    /**
     * @return Returns the bindingPersistence.
     */
    public BindingPersistence getBindingPersistence() {
        return bindingPersistence;
    }
    
}
