/*******************************************************************************
 * Copyright (c) 2007, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.menus.TrimContributionManager;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import dwt.DWT;
import dwt.widgets.Control;
import dwt.widgets.ToolBar;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IStatus;
import dwtx.jface.action.ContributionManager;
import dwtx.jface.action.IContributionItem;
import dwtx.jface.action.ToolBarManager;
import dwtx.jface.internal.provisional.action.IToolBarContributionItem;
import dwtx.ui.internal.WorkbenchWindow;
import dwtx.ui.internal.layout.IWindowTrim;
import dwtx.ui.internal.layout.TrimLayout;
import dwtx.ui.internal.misc.StatusUtil;
import dwtx.ui.menus.AbstractContributionFactory;
import dwtx.ui.menus.IMenuService;
import dwtx.ui.menus.MenuUtil;
import dwtx.ui.statushandlers.StatusManager;

/**
 * Manage trim contributions added through the 'dwtx.ui.menus'
 * extension point.
 * 
 * @since 3.3
 *
 */
public class TrimContributionManager extends ContributionManager {
    private class ToolBarTrimProxy implements IWindowTrim {
        private String id;
        private String uriSpec;
        private WorkbenchMenuService menuService;
        private WorkbenchWindow wbw;
        private ToolBar tb = null;
        private ToolBarManager tbm = null;

        ToolBarTrimProxy(String id, WorkbenchWindow wbw) {
            this.id = id;
            uriSpec = "toolbar:" + id; //$NON-NLS-1$
            this.wbw = wbw;
            
            this.menuService = (WorkbenchMenuService) wbw.getWorkbench().getService(
                    IMenuService.class);
        }
        
        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#dock(int)
         */
        public void dock(int dropSide) {
            dispose();
            
            int orientation = DWT.HORIZONTAL;
            if (dropSide is DWT.LEFT || dropSide is DWT.RIGHT)
                orientation = DWT.VERTICAL;
            
            // Create the new control, manager...
            tbm = new ToolBarManager(DWT.FLAT | orientation);
//          menuService.populateContributionManager(tbm, uriSpec);
            menuService.populateContributionManager(wbw, restrictionExpression, tbm, uriSpec, true);
            
            // Set the state for any Control entries
            IContributionItem[] items = tbm.getItems();
            for (int i = 0; i < items.length; i++) {
                if (items[i] instanceof InternalControlContribution) {
                    InternalControlContribution wbwcc = (InternalControlContribution) items[i];
                    wbwcc.setWorkbenchWindow(wbw);
                    wbwcc.setCurSide(dropSide);
                }
            }
            
            // OK, create the ToolBar (causes an 'update(true)'
            tb = tbm.createControl(wbw.getShell());
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getControl()
         */
        public Control getControl() {
            return tb;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getDisplayName()
         */
        public String getDisplayName() {
            return getId();
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getHeightHint()
         */
        public int getHeightHint() {
            return DWT.DEFAULT;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getId()
         */
        public String getId() {
            return id;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getValidSides()
         */
        public int getValidSides() {
            return DWT.TOP | DWT.BOTTOM | DWT.LEFT | DWT.RIGHT;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#getWidthHint()
         */
        public int getWidthHint() {
            return DWT.DEFAULT;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#handleClose()
         */
        public void handleClose() {
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#isCloseable()
         */
        public bool isCloseable() {
            return false;
        }

        /* (non-Javadoc)
         * @see dwtx.ui.internal.layout.IWindowTrim#isResizeable()
         */
        public bool isResizeable() {
            return false;
        }
        
        /**
         * Dispose any trim element resources
         */
        public void dispose() {
            if (tbm !is null) {
                tbm.removeAll();
                tbm.dispose();
            }
        }
    }
    
    /**
     * A List of the URI's representing the trim areas
     */
    private String[] trimAreaURIs = {
            MenuUtil.TRIM_COMMAND1,
            MenuUtil.TRIM_COMMAND2,
            MenuUtil.TRIM_VERTICAL1,
            MenuUtil.TRIM_VERTICAL2,
            MenuUtil.TRIM_STATUS
    };

    /**
     * The DWT 'side' corresponding to a URI
     */
    private int[] swtSides = { DWT.TOP, DWT.TOP, DWT.LEFT, DWT.RIGHT, DWT.BOTTOM }; 

    private WorkbenchWindow wbWindow;
    TrimLayout layout;
    private InternalMenuService menuService;
    
    List contributedTrim = new ArrayList();

    List contributedLists = new ArrayList();

    private Set restrictionExpression;

    /**
     * Construct a contribution manager for the given window 
     */
    public TrimContributionManager(WorkbenchWindow window) {
        wbWindow = window;
        layout = (TrimLayout) wbWindow.getShell().getLayout();
        menuService = (InternalMenuService) window.getService(
                IMenuService.class);
        restrictionExpression = wbWindow.getMenuRestrictions();
    }

    /* (non-Javadoc)
     * @see dwtx.jface.action.IContributionManager#update(bool)
     */
    public void update(bool force) {
        update(force, false);
    }
    
    public void update(bool force, bool hideTopTrim) {
        // Remove any contributed trim
        teardown();
        
        // Process the additions for each 'area'
        for (int i = 0; i < trimAreaURIs.length; i++) {
            // IntroBar want to hide the top trim
            if (hideTopTrim && swtSides[i] is DWT.TOP)
                continue;
            
            List contribs = menuService.getAdditionsForURI(new MenuLocationURI(trimAreaURIs[i]));
            
            for (Iterator cacheIter = contribs.iterator(); cacheIter.hasNext();) {
                AbstractContributionFactory cache = (AbstractContributionFactory) cacheIter.next();
                ContributionRoot ciList = new ContributionRoot(menuService,
                        restrictionExpression, this, cache);
                cache.createContributionItems(wbWindow, ciList);
                // save the list for later cleanup of any visibility expressions that were added.
                contributedLists.add(ciList);
                for (Iterator ciIter = ciList.getItems().iterator(); ciIter.hasNext();) {
                    IContributionItem ci = (IContributionItem) ciIter.next();
                    if (ci instanceof IToolBarContributionItem) {
                        // HACK!! Fake this
                        ToolBarTrimProxy tbProxy = new ToolBarTrimProxy(ci.getId(), wbWindow);
                        tbProxy.dock(swtSides[i]);
                        
                        // If we're adding to the 'command1' area then we're -before- the CoolBar
                        IWindowTrim insertBefore = null;
                        if (i is 0) {
                            insertBefore = layout.getTrim("dwtx.ui.internal.WorkbenchWindow.topBar"); //$NON-NLS-1$
                        }
                        layout.addTrim(swtSides[i], tbProxy, insertBefore);                     
                        contributedTrim.add(tbProxy);
                    }
                }
            }
        }
    }

    private void teardown() {
        // First, remove all trim
        for (Iterator iter = contributedTrim.iterator(); iter.hasNext();) {
            ToolBarTrimProxy proxy = (ToolBarTrimProxy) iter.next();
            layout.removeTrim(proxy);

            try {
                proxy.dispose();
            } catch (Throwable e) {
                IStatus status = null;
                if (e instanceof CoreException) {
                    status = ((CoreException) e).getStatus();
                } else {
                    status = StatusUtil
                            .newStatus(
                                    IStatus.ERROR,
                                    "Internal plug-in widget delegate error on dispose.", e); //$NON-NLS-1$
                }
                StatusUtil
                        .handleStatus(
                                status,
                                "widget delegate failed on dispose: id = " + proxy.getId(), StatusManager.LOG); //$NON-NLS-1$
            }
        }

        // Clear out the old list
        contributedTrim.clear();
        
        // clean up the list of ContributionLists
        for (Iterator iter = contributedLists.iterator(); iter.hasNext();) {
            ContributionRoot list = (ContributionRoot) iter.next();
            list.release();
        }
        
        contributedLists.clear();
    }
    
    /**
     * 
     */
    public void dispose() {
        teardown();
    }

    /**
     * @param knownIds
     */
    public void updateLocations(List knownIds) {
        // TODO Auto-generated method stub
        
    }
}
