/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Jan-Hendrik Diederich, Bredex GmbH - bug 201052
 *     Oakland Software (Francis Upton) <francisu@ieee.org> - bug 219273 
 *     
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.preferences.WorkbenchPreferenceExtensionNode;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;

import dwt.graphics.Image;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.internal.registry.IWorkbenchRegistryConstants;
import dwtx.ui.internal.registry.KeywordRegistry;
import dwtx.ui.model.IComparableContribution;
import dwtx.ui.plugin.AbstractUIPlugin;

/**
 * The WorkbenchPreferenceExtensionNode is the abstract class for all property
 * and page nodes in the workbench.
 * 
 * @since 3.1
 */
public abstract class WorkbenchPreferenceExtensionNode extends WorkbenchPreferenceExpressionNode 
    implements IComparableContribution {
    
    private static final String TAG_KEYWORD_REFERENCE = "keywordReference"; //$NON-NLS-1$

    private Collection keywordReferences;
    
    private IConfigurationElement configurationElement;

    private ImageDescriptor imageDescriptor;

    private Image image;

    private Collection keywordLabelCache;
    
    private int priority;

    private String pluginId;

    /**
     * Create a new instance of the reciever.
     * 
     * @param id
     * @param configurationElement 
     */
    public WorkbenchPreferenceExtensionNode(String id, IConfigurationElement configurationElement) {
        super(id);
        this.configurationElement = configurationElement;
        this.pluginId = configurationElement.getNamespaceIdentifier();
    }

    /**
     * Get the ids of the keywords the receiver is bound to.
     * 
     * @return Collection of <code>String</code>.  Never <code>null</code>.
     */
    public Collection getKeywordReferences() {
        if (keywordReferences is null) {
            IConfigurationElement[] references = getConfigurationElement()
                    .getChildren(TAG_KEYWORD_REFERENCE);
            HashSet list = new HashSet(references.length);
            for (int i = 0; i < references.length; i++) {
                IConfigurationElement page = references[i];
                String id = page.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
                if (id !is null) {
                    list.add(id);
                }
            }

            if (!list.isEmpty()) {
                keywordReferences = list;
            } else {
                keywordReferences = Collections.EMPTY_SET;
            }
            
        }
        return keywordReferences;
    }

    /**
     * Get the labels of all of the keywords of the receiver.
     * 
     * @return Collection of <code>String</code>.  Never <code>null</code>.
     */
    public Collection getKeywordLabels() {
        if (keywordLabelCache !is null) {
            return keywordLabelCache;
        }
        
        Collection refs = getKeywordReferences();
        
        if(refs is Collections.EMPTY_SET) {
            keywordLabelCache = Collections.EMPTY_SET; 
            return keywordLabelCache;
        }
        
        keywordLabelCache = new ArrayList(refs.size());
        Iterator referenceIterator = refs.iterator();
        while(referenceIterator.hasNext()){
            Object label = KeywordRegistry.getInstance().getKeywordLabel(
                    (String) referenceIterator.next());
            if(label !is null) {
                keywordLabelCache.add(label);
            }
        }
        
        return keywordLabelCache;
    }
    
    /**
     * Clear the keyword cache, if any.
     */
    public void clearKeywords() {
        keywordLabelCache = null;
    }

    /* (non-Javadoc)
     * @see dwtx.jface.preference.IPreferenceNode#disposeResources()
     */
    public void disposeResources() {
        if (image !is null) {
            image.dispose();
            image = null;
        }
        super.disposeResources();
    }

    /* (non-Javadoc)
     * @see dwtx.jface.preference.IPreferenceNode#getLabelImage()
     */
    public Image getLabelImage() {      
        if (image is null) {
            ImageDescriptor desc = getImageDescriptor();
            if (desc !is null) {
                image = imageDescriptor.createImage();
            }
        }
        return image;
    }


    /* (non-Javadoc)
     * @see dwtx.jface.preference.IPreferenceNode#getLabelText()
     */
    public String getLabelText() {
        return getConfigurationElement().getAttribute(IWorkbenchRegistryConstants.ATT_NAME);
    }

    /**
     * Returns the image descriptor for this node.
     * 
     * @return the image descriptor
     */
    public ImageDescriptor getImageDescriptor() {
        if (imageDescriptor !is null) {
            return imageDescriptor;
        }
        
        String imageName = getConfigurationElement().getAttribute(IWorkbenchRegistryConstants.ATT_ICON);
        if (imageName !is null) {
            String contributingPluginId = pluginId;
            imageDescriptor = AbstractUIPlugin.imageDescriptorFromPlugin(contributingPluginId, imageName);
        }
        return imageDescriptor;
    }
    
    /**
     * Return the configuration element.
     * 
     * @return the configuration element
     */
    public IConfigurationElement getConfigurationElement() {
        return configurationElement;
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.activities.support.IPluginContribution#getLocalId()
     */
    public String getLocalId() {
        return getId();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.activities.support.IPluginContribution#getPluginId()
     */
    public String getPluginId() {
        return pluginId;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.model.IComparableContribution#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter)
    {
        if (adapter is IConfigurationElement.class)
            return getConfigurationElement();
        return null;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.model.IComparableContribution#getLabel()
     */
    public String getLabel()
    {
        return getLabelText();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.model.IComparableContribution#getPriority()
     */
    public int getPriority()
    {
        return priority;
    }

    public void setPriority(int pri)
    {
        priority = pri;
    }


}
