/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.NativeStackPresentation;

import dwt.dwthelper.utils;

import java.util.ArrayList;

import dwt.DWT;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.events.MouseAdapter;
import dwt.events.MouseEvent;
import dwt.events.MouseListener;
import dwt.graphics.Image;
import dwt.graphics.Point;
import dwt.graphics.Rectangle;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.Menu;
import dwt.widgets.TabFolder;
import dwt.widgets.TabItem;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.MenuManager;
import dwtx.jface.util.Geometry;
import dwtx.ui.IPropertyListener;
import dwtx.ui.IWorkbenchPreferenceConstants;
import dwtx.ui.PlatformUI;
import dwtx.ui.presentations.IPresentablePart;
import dwtx.ui.presentations.IStackPresentationSite;
import dwtx.ui.presentations.PresentationUtil;
import dwtx.ui.presentations.StackDropResult;
import dwtx.ui.presentations.StackPresentation;

/**
 * A stack presentation using native widgets.
 * <p>
 * EXPERIMENTAL
 * </p> 
 * 
 * @since 3.0
 */
public class NativeStackPresentation extends StackPresentation {

    private TabFolder tabFolder;

    private Listener dragListener;

    private IPresentablePart current;

    private MenuManager systemMenuManager = new MenuManager();

    // don't reset this dynamically, so just keep the information static.
    // see bug:
    //   75422 [Presentations] Switching presentation to R21 switches immediately, but only partially
    private static int tabPos = PlatformUI.getPreferenceStore().getInt(IWorkbenchPreferenceConstants.VIEW_TAB_POSITION);
  
    private final static String TAB_DATA = NativeStackPresentation.class
            .getName()
            + ".partId"; //$NON-NLS-1$

    private MouseListener mouseListener = new MouseAdapter() {
        public void mouseDown(MouseEvent e) {
            //          // PR#1GDEZ25 - If selection will change in mouse up ignore mouse down.
            //          // Else, set focus.
            //          TabItem newItem = tabFolder.getItem(new Point(e.x, e.y));
            //          if (newItem !is null) {
            //              TabItem oldItem = tabFolder.getSelection();
            //              if (newItem !is oldItem)
            //                  return;
            //          }
            if (current !is null) {
                current.setFocus();
            }
        }
    };

    private Listener menuListener = new Listener() {
        /* (non-Javadoc)
         * @see dwt.widgets.Listener#handleEvent(dwt.widgets.Event)
         */
        public void handleEvent(Event event) {
            Point pos = new Point(event.x, event.y);
            //          TabItem item = tabFolder.getItem(pos);
            TabItem item = null;
            IPresentablePart part = null;
            if (item !is null) {
                part = getPartForTab(item);
            }
            showPaneMenu(part, pos);
        }
    };

    private Listener selectionListener = new Listener() {
        public void handleEvent(Event e) {
            IPresentablePart item = getPartForTab((TabItem) e.item);
            if (item !is null) {
                getSite().selectPart(item);
                //              item.setFocus();
            }
        }
    };

    private Listener resizeListener = new Listener() {
        public void handleEvent(Event e) {
            setControlSize();
        }
    };

    private IPropertyListener childPropertyChangeListener = new IPropertyListener() {
        public void propertyChanged(Object source, int property) {

            if (isDisposed()) {
                return;
            }

            if (source instanceof IPresentablePart) {
                IPresentablePart part = (IPresentablePart) source;
                childPropertyChanged(part, property);
            }
        }
    };

    private DisposeListener tabDisposeListener = new DisposeListener() {
        public void widgetDisposed(DisposeEvent e) {
            if (e.widget instanceof TabItem) {
                TabItem item = (TabItem) e.widget;
                IPresentablePart part = getPartForTab(item);
                part.removePropertyListener(childPropertyChangeListener);
            }
        }
    };

    public NativeStackPresentation(Composite parent,
            IStackPresentationSite stackSite) {
        super(stackSite);

        tabFolder = new TabFolder(parent, tabPos);

        // listener to switch between visible tabItems
        tabFolder.addListener(DWT.Selection, selectionListener);

        // listener to resize visible components
        tabFolder.addListener(DWT.Resize, resizeListener);

        // listen for mouse down on tab to set focus.
        tabFolder.addMouseListener(mouseListener);

        tabFolder.addListener(DWT.MenuDetect, menuListener);

        dragListener = new Listener() {
            public void handleEvent(Event event) {
                Point localPos = new Point(event.x, event.y);
                //              TabItem tabUnderPointer = tabFolder.getItem(localPos);
                TabItem tabUnderPointer = null;

                if (tabUnderPointer is null) {
                    return;
                }

                IPresentablePart part = getPartForTab(tabUnderPointer);

                if (getSite().isPartMoveable(part)) {
                    getSite().dragStart(part, tabFolder.toDisplay(localPos),
                            false);
                }
            }
        };

        PresentationUtil.addDragListener(tabFolder, dragListener);

    }

    /**
     * Returns the index of the tab for the given part, or returns tabFolder.getItemCount()
     * if there is no such tab.
     * 
     * @param part part being searched for
     * @return the index of the tab for the given part, or the number of tabs
     * if there is no such tab
     */
    private final int indexOf(IPresentablePart part) {
        if (part is null) {
            return tabFolder.getItemCount();
        }

        TabItem[] items = tabFolder.getItems();

        for (int idx = 0; idx < items.length; idx++) {
            IPresentablePart tabPart = getPartForTab(items[idx]);

            if (part is tabPart) {
                return idx;
            }
        }

        return items.length;
    }

    /**
     * Returns the tab for the given part, or null if there is no such tab
     * 
     * @param part the part being searched for
     * @return the tab for the given part, or null if there is no such tab
     */
    protected final TabItem getTab(IPresentablePart part) {
        TabItem[] items = tabFolder.getItems();

        int idx = indexOf(part);

        if (idx < items.length) {
            return items[idx];
        }

        return null;
    }

    /**
     * @param part
     * @param property
     */
    protected void childPropertyChanged(IPresentablePart part, int property) {
        TabItem tab = getTab(part);
        initTab(tab, part);
    }

    protected final IPresentablePart getPartForTab(TabItem item) {
        IPresentablePart part = (IPresentablePart) item.getData(TAB_DATA);
        return part;
    }

    protected TabFolder getTabFolder() {
        return tabFolder;
    }

    public bool isDisposed() {
        return tabFolder is null || tabFolder.isDisposed();
    }

    /**
     * Set the size of a page in the folder.
     */
    private void setControlSize() {
        if (current is null || tabFolder is null) {
            return;
        }
        //      Rectangle bounds;
        // @issue as above, the mere presence of a theme should not change the behaviour
        //      if ((mapTabToPart.size() > 1)
        //          || ((tabThemeDescriptor !is null) && (mapTabToPart.size() >= 1)))
        //          bounds = calculatePageBounds(tabFolder);
        //      else
        //          bounds = tabFolder.getBounds();
        current.setBounds(calculatePageBounds(tabFolder));
        //current.moveAbove(tabFolder);
    }

    public static Rectangle calculatePageBounds(TabFolder folder) {
        if (folder is null) {
            return new Rectangle(0, 0, 0, 0);
        }
        Rectangle bounds = folder.getBounds();
        Rectangle offset = folder.getClientArea();
        bounds.x += offset.x;
        bounds.y += offset.y;
        bounds.width = offset.width;
        bounds.height = offset.height;
        return bounds;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#dispose()
     */
    public void dispose() {
        if (isDisposed()) {
            return;
        }
        PresentationUtil.removeDragListener(tabFolder, dragListener);

        //systemMenuManager.dispose();

        tabFolder.dispose();
        tabFolder = null;
    }

    private TabItem createPartTab(IPresentablePart part, int tabIndex) {
        TabItem tabItem = new TabItem(tabFolder, DWT.NONE, tabIndex);
        tabItem.setData(TAB_DATA, part);
        part.addPropertyListener(childPropertyChangeListener);
        tabItem.addDisposeListener(tabDisposeListener);
        initTab(tabItem, part);
        return tabItem;
    }

    /**
     * Initializes a tab for the given part. Sets the text, icon, tool tip,
     * etc. This will also be called whenever a relevant property changes
     * in the part to reflect those changes in the tab. Subclasses may override
     * to change the appearance of tabs for a particular part.
     * 
     * @param tabItem tab for the part
     * @param part the part being displayed
     */
    protected void initTab(TabItem tabItem, IPresentablePart part) {
        tabItem.setText(part.getName());
        tabItem.setToolTipText(part.getTitleToolTip());

        Image tabImage = part.getTitleImage();
        if (tabImage !is tabItem.getImage()) {
            tabItem.setImage(tabImage);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.StackPresentation#addPart(dwtx.ui.internal.skins.IPresentablePart, dwtx.ui.internal.skins.IPresentablePart)
     */
    public void addPart(IPresentablePart newPart, Object cookie) {
        createPartTab(newPart, tabFolder.getItemCount());
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.StackPresentation#removePart(dwtx.ui.internal.skins.IPresentablePart)
     */
    public void removePart(IPresentablePart oldPart) {
        TabItem item = getTab(oldPart);
        if (item is null) {
            return;
        }
        oldPart.setVisible(false);

        item.dispose();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.StackPresentation#selectPart(dwtx.ui.internal.skins.IPresentablePart)
     */
    public void selectPart(IPresentablePart toSelect) {
        if (toSelect is current) {
            return;
        }

        if (current !is null) {
            current.setVisible(false);
        }

        current = toSelect;

        if (current !is null) {
            tabFolder.setSelection(indexOf(current));
            current.setVisible(true);
            setControlSize();
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#setBounds(dwt.graphics.Rectangle)
     */
    public void setBounds(Rectangle bounds) {
        tabFolder.setBounds(bounds);
        setControlSize();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#computeMinimumSize()
     */
    public Point computeMinimumSize() {
        return Geometry.getSize(tabFolder.computeTrim(0, 0, 0, 0));
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#setVisible(bool)
     */
    public void setVisible(bool isVisible) {
        if (current !is null) {
            current.setVisible(isVisible);
        }
        tabFolder.setVisible(isVisible);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#setState(int)
     */
    public void setState(int state) {
        //      tabFolder.setMinimized(state is IPresentationSite.STATE_MINIMIZED);
        //      tabFolder.setMaximized(state is IPresentationSite.STATE_MAXIMIZED);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#getSystemMenuManager()
     */
    public IMenuManager getSystemMenuManager() {
        return systemMenuManager;
    }

    /**
     * @param part
     * @param point
     */
    protected void showPaneMenu(IPresentablePart part, Point point) {
        systemMenuManager.update(false);
        Menu aMenu = systemMenuManager.createContextMenu(tabFolder.getParent());
        aMenu.setLocation(point.x, point.y);
        aMenu.setVisible(true);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.Presentation#getControl()
     */
    public Control getControl() {
        return tabFolder;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.skins.StackPresentation#dragOver(dwt.widgets.Control, dwt.graphics.Point)
     */
    public StackDropResult dragOver(Control currentControl, Point location) {

        // Determine which tab we're currently dragging over
        //      Point localPos = tabFolder.toControl(location);
        //      final TabItem tabUnderPointer = tabFolder.getItem(localPos);
        final TabItem tabUnderPointer = null;

        // This drop target only deals with tabs... if we're not dragging over
        // a tab, exit.
        if (tabUnderPointer is null) {
            return null;
        }

        //      return new StackDropResult(Geometry.toDisplay(tabFolder, tabUnderPointer.getBounds()),
        //          tabFolder.indexOf(tabUnderPointer));
        return null;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.StackPresentation#showSystemMenu()
     */
    public void showSystemMenu() {
        // TODO Auto-generated method stub

    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.StackPresentation#showPaneMenu()
     */
    public void showPaneMenu() {
        // TODO Auto-generated method stub

    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.StackPresentation#getTabList(IPresentablePart)
     */
    public Control[] getTabList(IPresentablePart part) {
        ArrayList list = new ArrayList();
        if (getControl() !is null) {
            list.add(getControl());
        }
        if (part.getToolBar() !is null) {
            list.add(part.getToolBar());
        }
        if (part.getControl() !is null) {
            list.add(part.getControl());
        }
        return (Control[]) list.toArray(new Control[list.size()]);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.StackPresentation#getCurrentPart()
     */
    public IPresentablePart getCurrentPart() {
        return current;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.StackPresentation#setActive(int)
     */
    public void setActive(int newState) {

    }

}
