/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Stefan Xenos, IBM; Chris Torrence, ITT Visual Information Solutions - bug 51580
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.presentations.PresentablePart;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.List;

import dwt.graphics.Image;
import dwt.graphics.Point;
import dwt.graphics.Rectangle;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwtx.core.runtime.ListenerList;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.jface.util.PropertyChangeEvent;
import dwtx.ui.IPropertyListener;
import dwtx.ui.ISharedImages;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.PartPane;
import dwtx.ui.internal.WorkbenchPartReference;
import dwtx.ui.internal.dnd.SwtUtil;
import dwtx.ui.presentations.IPartMenu;
import dwtx.ui.presentations.IPresentablePart;

/**
 * This is a lightweight adapter that allows PartPanes to be used by a StackPresentation. All methods
 * either redirect directly to PartPane or do trivial type conversions. All listeners registered by
 * the presentation are kept here rather than registering them directly on the PartPane. This allows
 * us to remove all listeners registered by a presentation that has been disposed, offering some
 * protection against memory leaks.
 */
public class PresentablePart implements IPresentablePart {

    private PartPane part;

    /**
     * Local listener list -- we use this rather than registering listeners directly on the part
     * in order to protect against memory leaks in badly behaved presentations.
     */
    private List listeners = new ArrayList();

    // Lazily initialized. Use getPropertyListenerProxy() to access.
    private IPropertyListener lazyPropertyListenerProxy;
    
    private ListenerList partPropertyChangeListeners = new ListenerList();
    
    private IPropertyChangeListener lazyPartPropertyChangeListener;

    // Lazily initialized. Use getMenu() to access 
    private IPartMenu viewMenu;
    
    // True iff the "set" methods on this object are talking to the real part (disabled
    // if the part is currently being managed by another presentation stack)
    private bool enableInputs = true;
    
    // True iff the "get" methods are returning up-to-date info from the real part (disabled
    // for efficency if the presentation is invisible)
    private bool enableOutputs = true;
    private Rectangle savedBounds = new Rectangle(0,0,0,0);
    private bool isVisible = false;
    
    // Saved state (only used when the part is inactive)
    private String name = ""; //$NON-NLS-1$
    private String titleStatus = ""; //$NON-NLS-1$
    private bool isDirty = false;
    private bool isBusy = false;
    private bool hasViewMenu = false;
   
    /**
     * Constructor
     * 
     * @param part
     */
    public PresentablePart(PartPane part, Composite parent) {
        this.part = part;
        getPane().addPropertyListener(getPropertyListenerProxy());
        getPane().addPartPropertyListener(getPartPropertyListenerProxy());
    }

    public PartPane getPane() {
        return part;
    }
    
    private IPropertyListener getPropertyListenerProxy() {
        if (lazyPropertyListenerProxy is null) {
            lazyPropertyListenerProxy = new IPropertyListener() {
                public void propertyChanged(Object source, int propId) {
                    firePropertyChange(propId);
                }
            };
        }

        return lazyPropertyListenerProxy;
    }
    
    private IPropertyChangeListener getPartPropertyListenerProxy() {
        if (lazyPartPropertyChangeListener is null) {
            lazyPartPropertyChangeListener = new IPropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent event) {
                    PropertyChangeEvent e = new PropertyChangeEvent(this,
                            event.getProperty(), event.getOldValue(), event.getNewValue());
                    firePartPropertyChange(e);
                }
            };
        }
        return lazyPartPropertyChangeListener;
    }

    /**
     * Detach this PresentablePart from the real part. No further methods should
     * be invoked on this object.
     */
    public void dispose() {
        // Ensure that the property listener is detached (necessary to prevent leaks)
        getPane().removePropertyListener(getPropertyListenerProxy());
        getPane().removePartPropertyListener(getPartPropertyListenerProxy());

        // Null out the various fields to ease garbage collection (optional)
        part = null;
        listeners.clear();
        listeners = null;
        partPropertyChangeListeners.clear();
        partPropertyChangeListeners = null;
    }

    public void firePropertyChange(int propertyId) {
        for (int i = 0; i < listeners.size(); i++) {
            ((IPropertyListener) listeners.get(i)).propertyChanged(this, propertyId);
        }
    }

    public void addPropertyListener(final IPropertyListener listener) {
        listeners.add(listener);
    }

    public void removePropertyListener(final IPropertyListener listener) {
        listeners.remove(listener);
    }

    protected void firePartPropertyChange(PropertyChangeEvent event) {
        Object[] l = partPropertyChangeListeners.getListeners();
        for (int i = 0; i < l.length; i++) {
            ((IPropertyChangeListener) l[i]).propertyChange(event);
        }
    }
    
    public void addPartPropertyListener(IPropertyChangeListener listener) {
        partPropertyChangeListeners.add(listener);
    }
    
    public void removePartPropertyListener(IPropertyChangeListener listener) {
        partPropertyChangeListeners.remove(listener);
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#setBounds(dwt.graphics.Rectangle)
     */
    public void setBounds(Rectangle bounds) {
        savedBounds = bounds;
        if (enableInputs && !SwtUtil.isDisposed(part.getControl())) {
            part.setBounds(bounds);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#setVisible(bool)
     */
    public void setVisible(bool isVisible) {
        this.isVisible = isVisible;
        if (enableInputs) {
            part.setVisible(isVisible);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#setFocus()
     */
    public void setFocus() {
        if (!SwtUtil.isDisposed(part.getControl())) {
            if (part.getPage().getActivePart() is part.getPartReference().getPart(false)) { 
                part.setFocus();
            } else {
                part.requestActivation();
            }
        }
    }

    
    private WorkbenchPartReference getPartReference() {
        return (WorkbenchPartReference) part.getPartReference();
    }

    
    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#getName()
     */
    public String getName() {
        if (enableOutputs) {
            return getPartReference().getPartName();
        }
        return name;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#getTitle()
     */
    public String getTitle() {
        return getPartReference().getTitle();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#getTitleStatus()
     */
    public String getTitleStatus() {
        if (enableOutputs) {
            return getPartReference().getContentDescription();
        }

        return titleStatus;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.IPresentablePart#getTitleImage()
     */
    public Image getTitleImage() {
//        
//        return PlatformUI.getWorkbench().getSharedImages().getImage(
//                ISharedImages.IMG_DEF_VIEW); 
//        
        if (enableOutputs) {
            return getPartReference().getTitleImage();
        }

        return PlatformUI.getWorkbench().getSharedImages().getImage(
                ISharedImages.IMG_DEF_VIEW);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.IPresentablePart#getTitleToolTip()
     */
    public String getTitleToolTip() {
        return getPartReference().getTitleToolTip();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#isDirty()
     */
    public bool isDirty() {
        if (enableOutputs) {
            return getPartReference().isDirty();
        }
        return isDirty;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.IPresentablePart#isBusy()
     */
    public bool isBusy() {
        if (enableOutputs) {
            return part.isBusy();
        }
        return isBusy;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.IPresentablePart#getToolBar()
     */
    public Control getToolBar() {
        if (enableOutputs) {
            return getPane().getToolBar();
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.presentations.IPresentablePart#getMenu()
     */
    public IPartMenu getMenu() {
        bool hasMenu;
        
        if (enableOutputs) {
            hasMenu = part.hasViewMenu();
        } else {
            hasMenu = this.hasViewMenu;
        }
        
        if (!hasMenu) {
            return null;
        }

        if (viewMenu is null) {
            viewMenu = new IPartMenu() {
                public void showMenu(Point location) {
                    part.showViewMenu(location);
                }
            };
        }

        return viewMenu;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#isCloseable()
     */
    public bool isCloseable() {        
        return part.isCloseable();
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#getControl()
     */
    public Control getControl() {
        return part.getControl();
    }

    public void enableOutputs(bool isActive) {
        if (isActive is this.enableOutputs) {
            return;
        }
        
        this.enableOutputs = isActive;
        
        if (isActive) {
            if (isBusy !is getPane().isBusy()) {
                firePropertyChange(PROP_BUSY);
            }
            if (isDirty !is isDirty()) {
                firePropertyChange(PROP_DIRTY);
            }
            if (!name.equals(getName())) {
                firePropertyChange(PROP_PART_NAME);
            }
            if (!titleStatus.equals(getTitleStatus())) {
                firePropertyChange(PROP_CONTENT_DESCRIPTION);
            }
            if (hasViewMenu !is getPane().hasViewMenu()) {
                firePropertyChange(PROP_PANE_MENU);
            }
            // Always assume that the toolbar and title has changed (keeping track of this for real
            // would be too expensive)
            firePropertyChange(PROP_TOOLBAR);
            firePropertyChange(PROP_TITLE);
            
            getPane().addPropertyListener(getPropertyListenerProxy());
        } else {
            getPane().removePropertyListener(getPropertyListenerProxy());
            
            WorkbenchPartReference ref = getPartReference();
            isBusy = getPane().isBusy();
            isDirty = ref.isDirty();
            name = ref.getPartName();
            titleStatus = ref.getContentDescription();
            hasViewMenu = getPane().hasViewMenu();
            firePropertyChange(PROP_TITLE);
            firePropertyChange(PROP_TOOLBAR);
        }
    }
    
    public void enableInputs(bool isActive) {
        if (isActive is this.enableInputs) {
            return;
        }
        
        this.enableInputs = isActive;
        
        if (isActive) {
            if (isActive && !SwtUtil.isDisposed(part.getControl())) {
                part.setBounds(savedBounds);
            }
            
            part.setVisible(isVisible);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.ui.presentations.IPresentablePart#getPartProperty(java.lang.String)
     */
    public String getPartProperty(String key) {
        return getPartReference().getPartProperty(key);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.ISizeProvider#computePreferredSize(bool, int, int, int)
     */
    public int computePreferredSize(bool width, int availableParallel,
            int availablePerpendicular, int preferredResult) {

        return getPane().computePreferredSize(width, availableParallel,
                availablePerpendicular, preferredResult);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.ISizeProvider#getSizeFlags(bool)
     */
    public int getSizeFlags(bool width) {
        return getPane().getSizeFlags(width);
    }


}
