/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.ProgressAnimationProcessor;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dwtx.core.runtime.Assert;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.jobs.Job;
import dwtx.ui.PlatformUI;

/**
 * The ProgressAnimationProcessor is the processor for the animation using the
 * system progress.
 */
class ProgressAnimationProcessor implements IAnimationProcessor {

    AnimationManager manager;

    /**
     * Create a new instance of the receiver and listen to the animation
     * manager.
     * 
     * @param animationManager
     */
    ProgressAnimationProcessor(AnimationManager animationManager) {
        manager = animationManager;
    }

    List items = Collections.synchronizedList(new ArrayList());

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#startAnimation(dwtx.core.runtime.IProgressMonitor)
     */
    public void startAnimationLoop(IProgressMonitor monitor) {

        // Create an off-screen image to draw on, and a GC to draw with.
        // Both are disposed after the animation.
        if (items.size() is 0) {
            return;
        }
        if (!PlatformUI.isWorkbenchRunning()) {
            return;
        }

        while (manager.isAnimated() && !monitor.isCanceled()) {
            //Do nothing while animation is happening
        }

        ProgressAnimationItem[] animationItems = getAnimationItems();
        for (int i = 0; i < animationItems.length; i++) {
            animationItems[i].animationDone();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#addItem(dwtx.ui.internal.progress.AnimationItem)
     */
    public void addItem(AnimationItem item) {
        Assert.isTrue(item instanceof ProgressAnimationItem);
        items.add(item);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#removeItem(dwtx.ui.internal.progress.AnimationItem)
     */
    public void removeItem(AnimationItem item) {
        Assert.isTrue(item instanceof ProgressAnimationItem);
        items.remove(item);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#hasItems()
     */
    public bool hasItems() {
        return items.size() > 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#itemsInactiveRedraw()
     */
    public void itemsInactiveRedraw() {
        //Nothing to do here as DWT handles redraw

    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#animationStarted(dwtx.core.runtime.IProgressMonitor)
     */
    public void animationStarted() {
        AnimationItem[] animationItems = getAnimationItems();
        for (int i = 0; i < animationItems.length; i++) {
            animationItems[i].animationStart();
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IAnimationProcessor#getPreferredWidth()
     */
    public int getPreferredWidth() {
        return 30;
    }

    /**
     * Get the animation items currently registered for the receiver.
     * 
     * @return ProgressAnimationItem[]
     */
    private ProgressAnimationItem[] getAnimationItems() {
        ProgressAnimationItem[] animationItems = new ProgressAnimationItem[items
                .size()];
        items.toArray(animationItems);
        return animationItems;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.IAnimationProcessor#animationFinished()
     */
    public void animationFinished() {
        AnimationItem[] animationItems = getAnimationItems();
        for (int i = 0; i < animationItems.length; i++) {
            animationItems[i].animationDone();
        }

    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.progress.IAnimationProcessor#isProcessorJob(dwtx.core.runtime.jobs.Job)
     */
    public bool isProcessorJob(Job job) {
        // We have no jobs
        return false;
    }

}
