/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.ProgressMonitorJobsDialog;

import dwt.dwthelper.utils;

import java.lang.reflect.InvocationTargetException;

import dwt.DWT;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.graphics.Cursor;
import dwt.graphics.Point;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Button;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.Shell;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IProgressMonitorWithBlocking;
import dwtx.core.runtime.IStatus;
import dwtx.jface.dialogs.IDialogConstants;
import dwtx.jface.dialogs.ProgressMonitorDialog;
import dwtx.jface.operation.IRunnableWithProgress;
import dwtx.jface.viewers.Viewer;
import dwtx.jface.viewers.ViewerComparator;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.misc.Policy;

/**
 * The ProgressMonitorJobsDialog is the progress monitor dialog used by the
 * progress service to allow locks to show the current jobs.
 */
public class ProgressMonitorJobsDialog extends ProgressMonitorDialog {
    private DetailedProgressViewer viewer;

    /**
     * The height of the viewer. Set when the details button is selected.
     */
    private int viewerHeight = -1;

    Composite viewerComposite;

    private Button detailsButton;

    private long watchTime = -1;

    protected bool alreadyClosed = false;

    private IProgressMonitor wrapperedMonitor;

    //Cache initial enablement in case the enablement state is set
    //before the button is created
    protected bool enableDetailsButton = false;

    /**
     * Create a new instance of the receiver.
     * 
     * @param parent
     */
    public ProgressMonitorJobsDialog(Shell parent) {
        super(parent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.Dialog#createDialogArea(dwt.widgets.Composite)
     */
    protected Control createDialogArea(Composite parent) {
        Composite top = (Composite) super.createDialogArea(parent);
        createExtendedDialogArea(parent);
        return top;
    }

    /**
     * Create the extensions to the dialog area.
     * @param parent
     */
    protected void createExtendedDialogArea(Composite parent) {
        viewerComposite = new Composite(parent, DWT.NONE);
        GridLayout layout = new GridLayout();
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        viewerComposite.setLayout(layout);
        GridData viewerData = new GridData(GridData.FILL_BOTH);
        viewerData.horizontalSpan = 2;
        viewerData.heightHint = 0;
        viewerComposite.setLayoutData(viewerData);
    }

    /**
     * The details button has been selected. Open or close the progress viewer
     * as appropriate.
     *  
     */
    void handleDetailsButtonSelect() {
        Shell shell = getShell();
        Point shellSize = shell.getSize();
        Composite composite = (Composite) getDialogArea();
        if (viewer !is null) {
            viewer.getControl().dispose();
            viewer = null;
            composite.layout();
            shell.setSize(shellSize.x, shellSize.y - viewerHeight);
            detailsButton.setText(ProgressMessages.ProgressMonitorJobsDialog_DetailsTitle);
        } else {
            //Abort if there are no jobs visible
            if (ProgressManager.getInstance().getRootElements(Policy.DEBUG_SHOW_ALL_JOBS).length is 0) {
                detailsButton.setEnabled(false);
                return;
            }

            viewer = new DetailedProgressViewer(viewerComposite, DWT.MULTI
                    | DWT.H_SCROLL | DWT.V_SCROLL | DWT.BORDER);
            viewer.setComparator(new ViewerComparator() {
                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.jface.viewers.ViewerComparator#compare(dwtx.jface.viewers.Viewer,
                 *      java.lang.Object, java.lang.Object)
                 */
                public int compare(Viewer testViewer, Object e1, Object e2) {
                    return ((Comparable) e1).compareTo(e2);
                }
            });

            viewer.setContentProvider(new ProgressViewerContentProvider(viewer,true,false){
                public Object[] getElements(Object inputElement) {
                    return super.getElements(inputElement);
                }}
            );
            
            viewer.setLabelProvider(new ProgressLabelProvider());
            viewer.setInput(this);
            GridData viewerData = new GridData(GridData.FILL_BOTH);
            viewer.getControl().setLayoutData(viewerData);
            GridData viewerCompositeData = (GridData) viewerComposite.getLayoutData();
            viewerCompositeData.heightHint = convertHeightInCharsToPixels(10);
            viewerComposite.layout(true);
            viewer.getControl().setVisible(true);
            viewerHeight = viewerComposite.computeTrim(0, 0, 0, viewerCompositeData.heightHint).height;
            detailsButton.setText(ProgressMessages.ProgressMonitorJobsDialog_HideTitle); 
            shell.setSize(shellSize.x, shellSize.y + viewerHeight);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.Dialog#createButtonsForButtonBar(dwt.widgets.Composite)
     */
    protected void createButtonsForButtonBar(Composite parent) {
        super.createButtonsForButtonBar(parent);
        createDetailsButton(parent);
    }

    /**
     * Create a spacer label to get the layout to not bunch the widgets.
     * 
     * @param parent
     *            The parent of the new button.
     */
    protected void createSpacer(Composite parent) {
        //Make a label to force the spacing
        Label spacer = new Label(parent, DWT.NONE);
        spacer.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
                | GridData.GRAB_HORIZONTAL));
    }

    /**
     * Create the details button for the receiver.
     * 
     * @param parent
     *            The parent of the new button.
     */
    protected void createDetailsButton(Composite parent) {
        detailsButton = createButton(parent, IDialogConstants.DETAILS_ID,
                ProgressMessages.ProgressMonitorJobsDialog_DetailsTitle, 
                false);
        detailsButton.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see dwt.events.SelectionListener#widgetSelected(dwt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                handleDetailsButtonSelect();
            }
        });
        detailsButton.setCursor(arrowCursor);
        detailsButton.setEnabled(enableDetailsButton);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.IconAndMessageDialog#createButtonBar(dwt.widgets.Composite)
     */
    protected Control createButtonBar(Composite parent) {
        Composite composite = new Composite(parent, DWT.NONE);
        // create a layout with spacing and margins appropriate for the font
        // size.
        GridLayout layout = new GridLayout();
        layout.numColumns = 1; // this is incremented by createButton
        layout.makeColumnsEqualWidth = false;
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
        layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
        composite.setLayout(layout);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.horizontalSpan = 2;
        data.horizontalAlignment = GridData.END;
        data.grabExcessHorizontalSpace = true;
        composite.setLayoutData(data);
        composite.setFont(parent.getFont());
        // Add the buttons to the button bar.
        if (arrowCursor is null) {
            arrowCursor = new Cursor(parent.getDisplay(), DWT.CURSOR_ARROW);
        }
        createButtonsForButtonBar(composite);
        return composite;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.ProgressMonitorDialog#clearCursors()
     */
    protected void clearCursors() {
        if (detailsButton !is null && !detailsButton.isDisposed()) {
            detailsButton.setCursor(null);
        }
        super.clearCursors();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.ProgressMonitorDialog#updateForSetBlocked(dwtx.core.runtime.IStatus)
     */
    protected void updateForSetBlocked(IStatus reason) {
        if(alreadyClosed)
            return;
        
        super.updateForSetBlocked(reason);
        enableDetails(true);
        if (viewer is null) {
            handleDetailsButtonSelect();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.ProgressMonitorDialog#run(bool,
     *      bool, dwtx.jface.operation.IRunnableWithProgress)
     */
    public void run(bool fork, bool cancelable,
            IRunnableWithProgress runnable) throws InvocationTargetException,
            InterruptedException {
        //if it is run in the UI Thread don't do anything.
        if (!fork) {
            enableDetails(false);
        }
        super.run(fork, cancelable, runnable);
    }

    /**
     * Set the enable state of the details button now or when it will be
     * created.
     * 
     * @param enableState
     *            a bool to indicate the preferred' state
     */
    protected void enableDetails(bool enableState) {
        if (detailsButton is null) {
            enableDetailsButton = enableState;
        } else {
            detailsButton.setEnabled(enableState);
        }
    }

    /**
     * Start watching the ticks. When the long operation time has 
     * passed open the dialog.
     */
    public void watchTicks() {
        watchTime = System.currentTimeMillis();
    }

    /**
     * Create a monitor for the receiver that wrappers the superclasses monitor.
     *  
     */
    public void createWrapperedMonitor() {
        wrapperedMonitor = new IProgressMonitorWithBlocking() {

            IProgressMonitor superMonitor = ProgressMonitorJobsDialog.super
                    .getProgressMonitor();

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#beginTask(java.lang.String,
             *      int)
             */
            public void beginTask(String name, int totalWork) {
                superMonitor.beginTask(name, totalWork);
                checkTicking();
            }

            /**
             * Check if we have ticked in the last 800ms.
             */
            private void checkTicking() {
                if (watchTime < 0) {
                    return;
                }
                if ((System.currentTimeMillis() - watchTime) > ProgressManager
                        .getInstance().getLongOperationTime()) {
                    watchTime = -1;
                    openDialog();
                }
            }

            /**
             * Open the dialog in the ui Thread
             */
            private void openDialog() {
                if (!PlatformUI.isWorkbenchRunning()) {
                    return;
                }

                PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
                    /* (non-Javadoc)
                     * @see java.lang.Runnable#run()
                     */
                    public void run() {
                        //Reset the watch if it is not safe to open
                         if (!ProgressManagerUtil.safeToOpen(ProgressMonitorJobsDialog.this,null)){
                              watchTicks();
                              return;
                         }
                             
                        if (!alreadyClosed) {
                            open();
                        }
                    }
                });
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#done()
             */
            public void done() {
                superMonitor.done();
                checkTicking();
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#internalWorked(double)
             */
            public void internalWorked(double work) {
                superMonitor.internalWorked(work);
                checkTicking();
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#isCanceled()
             */
            public bool isCanceled() {
                return superMonitor.isCanceled();
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#setCanceled(bool)
             */
            public void setCanceled(bool value) {
                superMonitor.setCanceled(value);

            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#setTaskName(java.lang.String)
             */
            public void setTaskName(String name) {
                superMonitor.setTaskName(name);
                checkTicking();

            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#subTask(java.lang.String)
             */
            public void subTask(String name) {
                superMonitor.subTask(name);
                checkTicking();
            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitor#worked(int)
             */
            public void worked(int work) {
                superMonitor.worked(work);
                checkTicking();

            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitorWithBlocking#clearBlocked()
             */
            public void clearBlocked() {
                //We want to open on blocking too
                if (superMonitor instanceof IProgressMonitorWithBlocking) {
                    ((IProgressMonitorWithBlocking) superMonitor)
                            .clearBlocked();
                }

            }

            /*
             * (non-Javadoc)
             * 
             * @see dwtx.core.runtime.IProgressMonitorWithBlocking#setBlocked(dwtx.core.runtime.IStatus)
             */
            public void setBlocked(IStatus reason) {
                openDialog();
                if (superMonitor instanceof IProgressMonitorWithBlocking) {
                    ((IProgressMonitorWithBlocking) superMonitor)
                            .setBlocked(reason);
                }

            }

        };
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.ProgressMonitorDialog#getProgressMonitor()
     */
    public IProgressMonitor getProgressMonitor() {
        if (wrapperedMonitor is null) {
            createWrapperedMonitor();
        }
        return wrapperedMonitor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.dialogs.ProgressMonitorDialog#close()
     */
    public bool close() {
        alreadyClosed = true;//As this sometimes delayed cache if it was already closed
        bool result = super.close();
        if (!result) {//If it fails reset the flag
            alreadyClosed = false;
        }
        return result;
    }
    
    /*
     * (non-Javadoc)
     * @see dwtx.jface.dialogs.Dialog#isResizable()
     */
    protected bool isResizable() {
        return true;
    }
}
