/*******************************************************************************
 * Copyright (c) 2003, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.ProgressView;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.layout.GridData;
import dwt.widgets.Composite;
import dwt.widgets.Menu;
import dwtx.jface.action.Action;
import dwtx.jface.action.IMenuListener;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.IToolBarManager;
import dwtx.jface.action.MenuManager;
import dwtx.jface.action.Separator;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.viewers.ISelection;
import dwtx.jface.viewers.ISelectionProvider;
import dwtx.jface.viewers.IStructuredSelection;
import dwtx.ui.IActionBars;
import dwtx.ui.IViewPart;
import dwtx.ui.IWorkbenchActionConstants;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.IWorkbenchHelpContextIds;
import dwtx.ui.internal.WorkbenchImages;
import dwtx.ui.part.ViewPart;
import dwtx.ui.preferences.ViewPreferencesAction;

/**
 * The ProgressView is the class that shows the details of the current workbench
 * progress.
 */
public class ProgressView extends ViewPart implements IViewPart {

    DetailedProgressViewer viewer;

    Action cancelAction;

    Action clearAllAction;


    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWorkbenchPart#createPartControl(dwt.widgets.Composite)
     */
    public void createPartControl(Composite parent) {
        viewer = new DetailedProgressViewer(parent, DWT.MULTI | DWT.H_SCROLL);
        viewer.setComparator(ProgressManagerUtil.getProgressViewerComparator());

        viewer.getControl().setLayoutData(
                new GridData(DWT.FILL, DWT.FILL, true, true));

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent,
                IWorkbenchHelpContextIds.RESPONSIVE_UI);
        
        initContentProvider();
        createClearAllAction();
        createCancelAction();
        initContextMenu();
        initPulldownMenu();
        initToolBar();
        getSite().setSelectionProvider(viewer);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {
        if (viewer !is null) {
            viewer.setFocus();
        }
    }

    /**
     * Sets the content provider for the viewer.
     */
    protected void initContentProvider() {
        ProgressViewerContentProvider provider = new ProgressViewerContentProvider(viewer, true ,true);
        viewer.setContentProvider(provider);
        viewer.setInput(ProgressManager.getInstance());
    }

    /**
     * Initialize the context menu for the receiver.
     */
    private void initContextMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu"); //$NON-NLS-1$
        Menu menu = menuMgr.createContextMenu(viewer.getControl());
        menuMgr.add(cancelAction);
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                JobInfo info = getSelectedInfo();
                if (info is null) {
                    return;
                }
            }
        });
        menuMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
        getSite().registerContextMenu(menuMgr, viewer);
        viewer.getControl().setMenu(menu);
    }

    private void initPulldownMenu() {
        IMenuManager menuMgr = getViewSite().getActionBars().getMenuManager();
        menuMgr.add(clearAllAction);
        menuMgr.add(new ViewPreferencesAction() {
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.ui.internal.preferences.ViewPreferencesAction#openViewPreferencesDialog()
             */
            public void openViewPreferencesDialog() {
                new JobsViewPreferenceDialog(viewer.getControl().getShell())
                        .open();

            }
        });

    }

    private void initToolBar() {
        IActionBars bars = getViewSite().getActionBars();
        IToolBarManager tm = bars.getToolBarManager();
        tm.add(clearAllAction);
    }

    /**
     * Return the selected objects. If any of the selections are not JobInfos or
     * there is no selection then return null.
     * 
     * @return JobInfo[] or <code>null</code>.
     */
    private IStructuredSelection getSelection() {
        // If the provider has not been set yet move on.
        ISelectionProvider provider = getSite().getSelectionProvider();
        if (provider is null) {
            return null;
        }
        ISelection currentSelection = provider.getSelection();
        if (currentSelection instanceof IStructuredSelection) {
            return (IStructuredSelection) currentSelection;
        }
        return null;
    }

    /**
     * Get the currently selected job info. Only return it if it is the only
     * item selected and it is a JobInfo.
     * 
     * @return JobInfo
     */
    JobInfo getSelectedInfo() {
        IStructuredSelection selection = getSelection();
        if (selection !is null && selection.size() is 1) {
            JobTreeElement element = (JobTreeElement) selection
                    .getFirstElement();
            if (element.isJobInfo()) {
                return (JobInfo) element;
            }
        }
        return null;
    }

    /**
     * Create the cancel action for the receiver.
     */
    private void createCancelAction() {
        cancelAction = new Action(ProgressMessages.ProgressView_CancelAction) {
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.jface.action.Action#run()
             */
            public void run() {
                viewer.cancelSelection();
            }
        };

    }

    /**
     * Create the clear all action for the receiver.
     */
    private void createClearAllAction() {
        clearAllAction = new Action(
                ProgressMessages.ProgressView_ClearAllAction) {
            /*
             * (non-Javadoc)
             * 
             * @see dwtx.jface.action.Action#run()
             */
            public void run() {
                FinishedJobs.getInstance().clearAll();
            }
        };
        clearAllAction
                .setToolTipText(ProgressMessages.NewProgressView_RemoveAllJobsToolTip);
        ImageDescriptor id = WorkbenchImages
                .getWorkbenchImageDescriptor("/elcl16/progress_remall.gif"); //$NON-NLS-1$
        if (id !is null) {
            clearAllAction.setImageDescriptor(id);
        }
        id = WorkbenchImages
                .getWorkbenchImageDescriptor("/dlcl16/progress_remall.gif"); //$NON-NLS-1$
        if (id !is null) {
            clearAllAction.setDisabledImageDescriptor(id);
        }
    }

}
