/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.progress.WorkbenchSiteProgressService;

import dwt.dwthelper.utils;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import dwt.DWT;
import dwt.graphics.Cursor;
import dwt.graphics.Image;
import dwt.widgets.Control;
import dwt.widgets.Display;
import dwt.widgets.Shell;
import dwtx.core.runtime.Assert;
import dwtx.core.runtime.IProgressMonitor;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.core.runtime.jobs.IJobChangeEvent;
import dwtx.core.runtime.jobs.IJobChangeListener;
import dwtx.core.runtime.jobs.ISchedulingRule;
import dwtx.core.runtime.jobs.Job;
import dwtx.core.runtime.jobs.JobChangeAdapter;
import dwtx.jface.operation.IRunnableContext;
import dwtx.jface.operation.IRunnableWithProgress;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.util.IPropertyChangeListener;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.PartSite;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.part.WorkbenchPart;
import dwtx.ui.progress.IProgressService;
import dwtx.ui.progress.IWorkbenchSiteProgressService;
import dwtx.ui.progress.WorkbenchJob;

/**
 * The WorkbenchSiteProgressService is the concrete implementation of the
 * WorkbenchSiteProgressService used by the workbench components.
 */
public class WorkbenchSiteProgressService implements
        IWorkbenchSiteProgressService, IJobBusyListener {
    PartSite site;

    private Collection busyJobs = Collections.synchronizedSet(new HashSet());

    private Object busyLock = new Object();

    IJobChangeListener listener;

    IPropertyChangeListener[] changeListeners = new IPropertyChangeListener[0];

    private Cursor waitCursor;

    private SiteUpdateJob updateJob;

    /**
     * Flag that keeps state from calls to {@link #showBusy(bool)}
     */
    private int busyCount = 0;

    private class SiteUpdateJob extends WorkbenchJob {
        private bool busy;

        private bool useWaitCursor = false;

        Object lock = new Object();

        /**
         * Set whether we are updating with the wait or busy cursor.
         * 
         * @param cursorState
         */
        void setBusy(bool cursorState) {
            synchronized (lock) {
                busy = cursorState;
            }
        }

        private SiteUpdateJob() {
            super(ProgressMessages.WorkbenchSiteProgressService_CursorJob);
        }

        /**
         * Get the wait cursor. Initialize it if required.
         * @param display the display to create the cursor on.
         * @return the created cursor
         */
        private Cursor getWaitCursor(Display display) {
            if (waitCursor is null) {
                waitCursor = new Cursor(display, DWT.CURSOR_APPSTARTING);
            }
            return waitCursor;
        }

        /*
         * (non-Javadoc)
         * 
         * @see dwtx.ui.progress.UIJob#runInUIThread(dwtx.core.runtime.IProgressMonitor)
         */
        public IStatus runInUIThread(IProgressMonitor monitor) {
            Control control = site.getPane().getControl();
            if (control is null || control.isDisposed()) {
                return Status.CANCEL_STATUS;
            }
            synchronized (lock) {
                //Update cursors if we are doing that
                if (useWaitCursor) {
                    Cursor cursor = null;
                    if (busy) {
                        cursor = getWaitCursor(control.getDisplay());
                    }
                    control.setCursor(cursor);
                }
                site.getPane().setBusy(busy);
                IWorkbenchPart part = site.getPart();
                 if (part instanceof WorkbenchPart) {
                    ((WorkbenchPart) part).showBusy(busy);
                }
            }
            return Status.OK_STATUS;
        }

        void clearCursors() {
            if (waitCursor !is null) {
                waitCursor.dispose();
                waitCursor = null;
            }
        }
    }

    /**
     * Create a new instance of the receiver with a site of partSite
     * 
     * @param partSite
     *            PartSite.
     */
    public WorkbenchSiteProgressService(final PartSite partSite) {
        site = partSite;
        updateJob = new SiteUpdateJob();
        updateJob.setSystem(true);
    }

    /**
     * Dispose the resources allocated by the receiver.
     *
     */
    public void dispose() {
        if (updateJob !is null) {
            updateJob.cancel();
        }

        ProgressManager.getInstance().removeListener(this);

        if (waitCursor is null) {
            return;
        }
        waitCursor.dispose();
        waitCursor = null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IProgressService#busyCursorWhile(dwtx.jface.operation.IRunnableWithProgress)
     */
    public void busyCursorWhile(IRunnableWithProgress runnable)
            throws InvocationTargetException, InterruptedException {
        getWorkbenchProgressService().busyCursorWhile(runnable);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#schedule(dwtx.core.runtime.jobs.Job,
     *      long, bool)
     */
    public void schedule(Job job, long delay, bool useHalfBusyCursor) {
        job.addJobChangeListener(getJobChangeListener(job, useHalfBusyCursor));
        job.schedule(delay);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#schedule(dwtx.core.runtime.jobs.Job,
     *      int)
     */
    public void schedule(Job job, long delay) {
        schedule(job, delay, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#schedule(dwtx.core.runtime.jobs.Job)
     */
    public void schedule(Job job) {
        schedule(job, 0L, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#showBusyForFamily(java.lang.Object)
     */
    public void showBusyForFamily(Object family) {
        ProgressManager.getInstance().addListenerToFamily(family, this);
    }

    /**
     * Get the job change listener for this site.
     * 
     * @param job
     * @param useHalfBusyCursor
     * @return IJobChangeListener
     */
    public IJobChangeListener getJobChangeListener(final Job job,
            bool useHalfBusyCursor) {
        if (listener is null) {
            updateJob.useWaitCursor = useHalfBusyCursor;
            listener = new JobChangeAdapter() {
                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.runtime.jobs.JobChangeAdapter#aboutToRun(dwtx.core.runtime.jobs.IJobChangeEvent)
                 */
                public void aboutToRun(IJobChangeEvent event) {
                    incrementBusy(event.getJob());
                }

                /*
                 * (non-Javadoc)
                 * 
                 * @see dwtx.core.runtime.jobs.JobChangeAdapter#done(dwtx.core.runtime.jobs.IJobChangeEvent)
                 */
                public void done(IJobChangeEvent event) {
                    decrementBusy(event.getJob());
                }
            };
        }
        return listener;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IJobBusyListener#decrementBusy(dwtx.core.runtime.jobs.Job)
     */
    public void decrementBusy(Job job) {
        synchronized (busyLock) {
            if (!busyJobs.contains(job)) {
                return;
            }
            busyJobs.remove(job);
        }
        try {
            decrementBusy();
        } catch (Exception ex) {
            // protecting against assertion failures
            WorkbenchPlugin.log(ex);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.internal.progress.IJobBusyListener#incrementBusy(dwtx.core.runtime.jobs.Job)
     */
    public void incrementBusy(Job job) {
        synchronized (busyLock) {
            if (busyJobs.contains(job)) {
                return;
            }
            busyJobs.add(job);
        }
        incrementBusy();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#warnOfContentChange()
     */
    public void warnOfContentChange() {
        site.getPane().showHighlight();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.progress.IProgressService#showInDialog(dwt.widgets.Shell,
     *      dwtx.core.runtime.jobs.Job)
     */
    public void showInDialog(Shell shell, Job job) {
        getWorkbenchProgressService().showInDialog(shell, job);
    }

    /**
     * Get the progress service for the workbnech,
     * 
     * @return IProgressService
     */
    private IProgressService getWorkbenchProgressService() {
        return site.getWorkbenchWindow().getWorkbench().getProgressService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.operation.IRunnableContext#run(bool, bool,
     *      dwtx.jface.operation.IRunnableWithProgress)
     */
    public void run(bool fork, bool cancelable,
            IRunnableWithProgress runnable) throws InvocationTargetException,
            InterruptedException {
        getWorkbenchProgressService().run(fork, cancelable, runnable);
    }

    /*
     *  (non-Javadoc)
     * @see dwtx.ui.progress.IProgressService#runInUI(dwtx.jface.operation.IRunnableContext, dwtx.jface.operation.IRunnableWithProgress, dwtx.core.runtime.jobs.ISchedulingRule)
     */
    public void runInUI(IRunnableContext context,
            IRunnableWithProgress runnable, ISchedulingRule rule)
            throws InvocationTargetException, InterruptedException {
        getWorkbenchProgressService().runInUI(context, runnable, rule);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.progress.IProgressService#getLongOperationTime()
     */
    public int getLongOperationTime() {
        return getWorkbenchProgressService().getLongOperationTime();
    }

    /* (non-Javadoc)
     * @see dwtx.ui.progress.IProgressService#registerIconForFamily(dwtx.jface.resource.ImageDescriptor, java.lang.Object)
     */
    public void registerIconForFamily(ImageDescriptor icon, Object family) {
        getWorkbenchProgressService().registerIconForFamily(icon, family);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.progress.IProgressService#getIconFor(dwtx.core.runtime.jobs.Job)
     */
    public Image getIconFor(Job job) {
        return getWorkbenchProgressService().getIconFor(job);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#showBusy(bool)
     */
    public void incrementBusy() {
        synchronized (busyLock) {
            this.busyCount++;
            if (busyCount !is 1) {
                return;
            }
            updateJob.setBusy(true);
        }
        if (PlatformUI.isWorkbenchRunning()) {
            updateJob.schedule(100);
        } else {
            updateJob.cancel();
        }
    }
    /* (non-Javadoc)
     * @see dwtx.ui.progress.IWorkbenchSiteProgressService#showBusy(bool)
     */
    public void decrementBusy() {
        synchronized (busyLock) {
            Assert
                    .isTrue(
                            busyCount > 0,
                            "Ignoring unexpected call to IWorkbenchSiteProgressService.decrementBusy().  This might be due to an earlier call to this method."); //$NON-NLS-1$
            this.busyCount--;
            if (busyCount !is 0) {
                return;
            }
            updateJob.setBusy(false);
        }
        if (PlatformUI.isWorkbenchRunning()) {
            updateJob.schedule(100);
        } else {
            updateJob.cancel();
        }
    }
}
