/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.internal.registry.PreferencePageParameterValues;

import dwt.dwthelper.utils;

import java.util.Map;
import java.util.TreeMap;

import dwtx.core.commands.IParameterValues;
import dwtx.core.runtime.IRegistryChangeEvent;
import dwtx.core.runtime.IRegistryChangeListener;
import dwtx.core.runtime.Platform;
import dwtx.jface.preference.IPreferenceNode;
import dwtx.jface.preference.PreferenceManager;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.WorkbenchMessages;

/**
 * <p>
 * Provides the parameter values for the show preferences command.
 * </p>
 * <p>
 * To disambiguate preference pages with the same local label, names are
 * constructed incorporating the full path of preference page labels. For
 * instance names like <code>General > Appearance</code> and
 * <code>Java > Appearance</code> avoid the problem of trying to put two
 * <code>Appearance</code> keys into the parameter values map.
 * </p>
 * <p>
 * This is only intended for use within the
 * <code>dwtx.ui.workbench</code> plug-in.
 * </p>
 * 
 * @since 3.2
 */
public final class PreferencePageParameterValues implements IParameterValues {

    public PreferencePageParameterValues() {
        Platform.getExtensionRegistry().addRegistryChangeListener(
                new IRegistryChangeListener() {

                    /*
                     * (non-Javadoc)
                     * 
                     * @see dwtx.core.runtime.IRegistryChangeListener#registryChanged(dwtx.core.runtime.IRegistryChangeEvent)
                     */
                    public void registryChanged(IRegistryChangeEvent event) {
                        if (event.getExtensionDeltas(PlatformUI.PLUGIN_ID,
                                IWorkbenchRegistryConstants.PL_PREFERENCES).length > 0) {
                            preferenceMap = null;
                        }
                    }
                });
    }

    private Map preferenceMap;

    /**
     * Iterate through the preference page and build the map of preference page
     * names to ids.
     * 
     * @param values
     *            The Map being populated with parameter values.
     * @param preferenceNodes
     *            An array of <code>IPreferenceNode</code> to process.
     * @param namePrefix
     *            A string incorporating the names of each parent
     *            <code>IPreferenceNode</code> up to the root of the
     *            preference page tree. This will be <code>null</code> for the
     *            root level preference page nodes.
     */
    private final void collectParameterValues(final Map values,
            final IPreferenceNode[] preferenceNodes, final String namePrefix) {

        for (int i = 0; i < preferenceNodes.length; i++) {
            final IPreferenceNode preferenceNode = preferenceNodes[i];

            final String name;
            if (namePrefix is null) {
                name = preferenceNode.getLabelText();
            } else {
                name = namePrefix
                        + WorkbenchMessages.PreferencePageParameterValues_pageLabelSeparator
                        + preferenceNode.getLabelText();
            }
            final String value = preferenceNode.getId();
            values.put(name, value);

            collectParameterValues(values, preferenceNode.getSubNodes(), name);
        }
    }

    public final Map getParameterValues() {
        if (preferenceMap is null) {
            preferenceMap = new TreeMap();

            final PreferenceManager preferenceManager = PlatformUI
                    .getWorkbench().getPreferenceManager();
            collectParameterValues(preferenceMap, preferenceManager
                    .getRootSubNodes(), null);
        }

        return preferenceMap;
    }

}
