/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Markus Alexander Kuppe, Versant Corporation - bug #215797
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.registry.ViewDescriptor;

import dwt.dwthelper.utils;

import java.util.StringTokenizer;

import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtension;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.ui.IPageLayout;
import dwtx.ui.IPluginContribution;
import dwtx.ui.ISharedImages;
import dwtx.ui.IViewPart;
import dwtx.ui.PlatformUI;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.tweaklets.InterceptContributions;
import dwtx.ui.internal.tweaklets.Tweaklets;
import dwtx.ui.plugin.AbstractUIPlugin;
import dwtx.ui.views.IViewDescriptor;

/**
 * Capture the attributes of a view extension.
 */
public class ViewDescriptor implements IViewDescriptor, IPluginContribution {
    private String id;

    private ImageDescriptor imageDescriptor;

    private IConfigurationElement configElement;

    private String[] categoryPath;

    private float fastViewWidthRatio;
    
    /**
     * Create a new <code>ViewDescriptor</code> for an extension.
     * 
     * @param e the configuration element
     * @throws CoreException thrown if there are errors in the configuration
     */
    public ViewDescriptor(IConfigurationElement e)
            throws CoreException {
        configElement = e;
        loadFromExtension();
    }    

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IViewDescriptor#createView()
     */
    public IViewPart createView() throws CoreException {
        Object extension = WorkbenchPlugin.createExtension(
                getConfigurationElement(),
                IWorkbenchRegistryConstants.ATT_CLASS);
        return ((InterceptContributions) Tweaklets
                .get(InterceptContributions.KEY)).tweakView(extension);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IViewDescriptor#getCategoryPath()
     */
    public String[] getCategoryPath() {
        return categoryPath;
    }

    /**
     * Return the configuration element for this descriptor.
     * 
     * @return the configuration element
     */
    public IConfigurationElement getConfigurationElement() {
        return configElement;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IViewDescriptor#getDescription()
     */
    public String getDescription() {
        return RegistryReader.getDescription(configElement);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartDescriptor#getId()
     */
    public String getId() {
        return id;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartDescriptor#getImageDescriptor()
     */
    public ImageDescriptor getImageDescriptor() {
        if (imageDescriptor !is null) {
            return imageDescriptor;
        }
        String iconName = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ICON);
        // If the icon attribute was omitted, use the default one
        if (iconName is null) {
            return PlatformUI.getWorkbench().getSharedImages().getImageDescriptor(
                    ISharedImages.IMG_DEF_VIEW);
        }
        IExtension extension = configElement.getDeclaringExtension();
        String extendingPluginId = extension.getNamespace();
        imageDescriptor = AbstractUIPlugin.imageDescriptorFromPlugin(
                extendingPluginId, iconName);
        // If the icon attribute was invalid, use the error icon
        if (imageDescriptor is null) {
            imageDescriptor = ImageDescriptor.getMissingImageDescriptor();
        }
        
        return imageDescriptor;
    }

    /* (non-Javadoc)
     * @see dwtx.ui.IWorkbenchPartDescriptor#getLabel()
     */
    public String getLabel() {
        return configElement.getAttribute(IWorkbenchRegistryConstants.ATT_NAME);
    }

    /**
     * Return the accelerator attribute.
     * 
     * @return the accelerator attribute
     */
    public String getAccelerator() {
        return configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ACCELERATOR);
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IViewDescriptor#getFastViewWidthRatio()
     */
    public float getFastViewWidthRatio() {
        configElement.getAttribute(IWorkbenchRegistryConstants.ATT_FAST_VIEW_WIDTH_RATIO); // check to ensure the element is still valid - exception thrown if it isn't
        return fastViewWidthRatio;
    }

    /**
     * load a view descriptor from the registry.
     */
    private void loadFromExtension() throws CoreException {     
        id = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ID);
  
        String category = configElement.getAttribute(IWorkbenchRegistryConstants.TAG_CATEGORY);

        // Sanity check.
        if ((configElement.getAttribute(IWorkbenchRegistryConstants.ATT_NAME) is null)
                || (RegistryReader.getClassValue(configElement,
                        IWorkbenchRegistryConstants.ATT_CLASS) is null)) {
            throw new CoreException(new Status(IStatus.ERROR, configElement
                    .getNamespace(), 0,
                    "Invalid extension (missing label or class name): " + id, //$NON-NLS-1$
                    null));
        }
        
        if (category !is null) {
            StringTokenizer stok = new StringTokenizer(category, "/"); //$NON-NLS-1$
            categoryPath = new String[stok.countTokens()];
            // Parse the path tokens and store them
            for (int i = 0; stok.hasMoreTokens(); i++) {
                categoryPath[i] = stok.nextToken();
            }
        }
        
        String ratio = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_FAST_VIEW_WIDTH_RATIO);
        if (ratio !is null) {
            try {
                fastViewWidthRatio = new Float(ratio).floatValue();
                if (fastViewWidthRatio > IPageLayout.RATIO_MAX) {
                    fastViewWidthRatio = IPageLayout.RATIO_MAX;
                }
                if (fastViewWidthRatio < IPageLayout.RATIO_MIN) {
                    fastViewWidthRatio = IPageLayout.RATIO_MIN;
                }
            } catch (NumberFormatException e) {
                fastViewWidthRatio = IPageLayout.DEFAULT_FASTVIEW_RATIO;
            }
        } else {
            fastViewWidthRatio = IPageLayout.DEFAULT_FASTVIEW_RATIO;
        }        
    }

    /**
     * Returns a string representation of this descriptor. For debugging
     * purposes only.
     */
    public String toString() {
        return "View(" + getId() + ")"; //$NON-NLS-2$//$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.activities.support.IPluginContribution#getPluginId()
     */
    public String getPluginId() {
        String pluginId = configElement.getNamespace();
        return pluginId is null ? "" : pluginId; //$NON-NLS-1$    
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.activities.support.IPluginContribution#getLocalId()
     */
    public String getLocalId() {
        return getId() is null ? "" : getId(); //$NON-NLS-1$    
    }

    /* (non-Javadoc)
     * @see dwtx.ui.internal.registry.IViewDescriptor#getAllowMultiple()
     */
    public bool getAllowMultiple() {
        String string = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_ALLOW_MULTIPLE);     
        return string is null ? false : Boolean.valueOf(string).booleanValue();
    }
    
    /* (non-Javadoc)
     * @see dwtx.ui.views.IViewDescriptor#getRestorable()
     */
    public bool isRestorable() {
        String string = configElement.getAttribute(IWorkbenchRegistryConstants.ATT_RESTORABLE);     
        return string is null ? true : Boolean.valueOf(string).booleanValue();
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class adapter) {
        if (adapter.equals(IConfigurationElement.class)) {
            return getConfigurationElement();
        }
        return null;
    }
}
