/*******************************************************************************
 * Copyright (c) 2000, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.internal.registry.WorkingSetRegistry;

import dwt.dwthelper.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import dwtx.core.runtime.Assert;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExtension;
import dwtx.core.runtime.IExtensionPoint;
import dwtx.core.runtime.Platform;
import dwtx.core.runtime.dynamichelpers.ExtensionTracker;
import dwtx.core.runtime.dynamichelpers.IExtensionChangeHandler;
import dwtx.core.runtime.dynamichelpers.IExtensionTracker;
import dwtx.ui.PlatformUI;
import dwtx.ui.dialogs.IWorkingSetPage;

/**
 * Stores working set descriptors for working set extensions.
 */
public class WorkingSetRegistry implements IExtensionChangeHandler {
    // used in Workbench plugin.xml for default workingSet extension
    // @issue this is an IDE specific working set page!
    private static final String DEFAULT_PAGE_ID = "dwtx.ui.resourceWorkingSetPage"; //$NON-NLS-1$

    private HashMap/*<String, WorkingSetDescriptor>*/ workingSetDescriptors = new HashMap();

    /**
     * 
     */
    public WorkingSetRegistry() {
        IExtensionTracker tracker = PlatformUI.getWorkbench()
                .getExtensionTracker();
        tracker.registerHandler(this, ExtensionTracker
                .createExtensionPointFilter(getExtensionPointFilter()));

    }

    /**
     * 
     * @return
     * @since 3.3
     */
    private IExtensionPoint getExtensionPointFilter() {
        return Platform.getExtensionRegistry().getExtensionPoint(
                PlatformUI.PLUGIN_ID,
                IWorkbenchRegistryConstants.PL_WORKINGSETS);
    }
    
    /**
     * Adds a working set descriptor.
     * 
     * @param descriptor working set descriptor to add. Must not 
     *  exist in the registry yet.
     */
    public void addWorkingSetDescriptor(WorkingSetDescriptor descriptor) {
        Assert.isTrue(!workingSetDescriptors.containsValue(descriptor),
                "working set descriptor already registered"); //$NON-NLS-1$
        IExtensionTracker tracker = PlatformUI.getWorkbench()
                .getExtensionTracker();
        tracker.registerObject(descriptor.getConfigurationElement()
                .getDeclaringExtension(), descriptor,
                IExtensionTracker.REF_WEAK);
        workingSetDescriptors.put(descriptor.getId(), descriptor);
    }

    /**
     * Returns the default, resource based, working set page
     * 
     * @return the default working set page.
     */
    public IWorkingSetPage getDefaultWorkingSetPage() {
        // @issue this will return the IDE resource working set page... not good for generic workbench
        WorkingSetDescriptor descriptor = (WorkingSetDescriptor) workingSetDescriptors
                .get(DEFAULT_PAGE_ID);

        if (descriptor !is null) {
            return descriptor.createWorkingSetPage();
        }
        return null;
    }

    /**
     * Returns the working set descriptor with the given id.
     * 
     * @param pageId working set page id
     * @return the working set descriptor with the given id.
     */
    public WorkingSetDescriptor getWorkingSetDescriptor(String pageId) {
        return (WorkingSetDescriptor) workingSetDescriptors.get(pageId);
    }

    /**
     * Returns an array of all working set descriptors.
     * 
     * @return an array of all working set descriptors.
     */
    public WorkingSetDescriptor[] getWorkingSetDescriptors() {
        return (WorkingSetDescriptor[]) workingSetDescriptors.values().toArray(
                new WorkingSetDescriptor[workingSetDescriptors.size()]);
    }
    
    /**
     * Returns an array of all working set descriptors having
     * a page class attribute
     * 
     * @return an array of all working set descriptors having a 
     * page class attribute
     */
    public WorkingSetDescriptor[] getNewPageWorkingSetDescriptors() {
        Collection descriptors= workingSetDescriptors.values();
        List result= new ArrayList(descriptors.size());
        for (Iterator iter= descriptors.iterator(); iter.hasNext();) {
            WorkingSetDescriptor descriptor= (WorkingSetDescriptor)iter.next();
            if (descriptor.getPageClassName() !is null) {
                result.add(descriptor);
            }
        }
        return (WorkingSetDescriptor[])result.toArray(new WorkingSetDescriptor[result.size()]);
    }
    
    /**
     * Returns <code>true</code> if there is a working set descriptor with
     * a page class attribute. Otherwise <code>false</code> is returned.
     * 
     * @return whether a descriptor with a page class attribute exists
     */
    public bool hasNewPageWorkingSetDescriptor() {
        Collection descriptors= workingSetDescriptors.values();
        for (Iterator iter= descriptors.iterator(); iter.hasNext();) {
            WorkingSetDescriptor descriptor= (WorkingSetDescriptor)iter.next();
            if (descriptor.getPageClassName() !is null) {
                return true;
            }
        }
        return false;
    }
    
    public WorkingSetDescriptor[] getUpdaterDescriptorsForNamespace(
            String namespace) {
        if (namespace is null) // fix for Bug 84225
            return new WorkingSetDescriptor[0];
        Collection descriptors = workingSetDescriptors.values();
        List result = new ArrayList();
        for (Iterator iter = descriptors.iterator(); iter.hasNext();) {
            WorkingSetDescriptor descriptor = (WorkingSetDescriptor) iter
                    .next();
            if (namespace.equals(descriptor.getUpdaterNamespace())) {
                result.add(descriptor);
            }
        }
        return (WorkingSetDescriptor[]) result
                .toArray(new WorkingSetDescriptor[result.size()]);
    }
    
    public WorkingSetDescriptor[] getElementAdapterDescriptorsForNamespace(
            String namespace) {
        if (namespace is null) // fix for Bug 84225
            return new WorkingSetDescriptor[0];
        Collection descriptors = workingSetDescriptors.values();
        List result = new ArrayList();
        for (Iterator iter = descriptors.iterator(); iter.hasNext();) {
            WorkingSetDescriptor descriptor = (WorkingSetDescriptor) iter
                    .next();
            if (namespace.equals(descriptor.getDeclaringNamespace())) {
                result.add(descriptor);
            }
        }
        return (WorkingSetDescriptor[]) result
                .toArray(new WorkingSetDescriptor[result.size()]);
    }

    /**
     * Returns the working set page with the given id.
     * 
     * @param pageId working set page id
     * @return the working set page with the given id.
     */
    public IWorkingSetPage getWorkingSetPage(String pageId) {
        WorkingSetDescriptor descriptor = (WorkingSetDescriptor) workingSetDescriptors
                .get(pageId);

        if (descriptor is null) {
            return null;
        }
        return descriptor.createWorkingSetPage();
    }

    /**
     * Loads the working set registry.
     */
    public void load() {
        WorkingSetRegistryReader reader = new WorkingSetRegistryReader();
        reader.readWorkingSets(Platform.getExtensionRegistry(), this);
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.dynamichelpers.IExtensionChangeHandler#addExtension(dwtx.core.runtime.dynamichelpers.IExtensionTracker, dwtx.core.runtime.IExtension)
     */
    public void addExtension(IExtensionTracker tracker, IExtension extension) {
        WorkingSetRegistryReader reader = new WorkingSetRegistryReader(this);
        IConfigurationElement[] elements = extension.getConfigurationElements();
        for (int i = 0; i < elements.length; i++) {
            reader.readElement(elements[i]);
        }
    }

    /* (non-Javadoc)
     * @see dwtx.core.runtime.dynamichelpers.IExtensionChangeHandler#removeExtension(dwtx.core.runtime.IExtension, java.lang.Object[])
     */
    public void removeExtension(IExtension extension, Object[] objects) {
        for (int i = 0; i < objects.length; i++) {
            if (objects[i] instanceof WorkingSetDescriptor) {
                WorkingSetDescriptor desc = (WorkingSetDescriptor) objects[i];
                workingSetDescriptors.remove(desc.getId());
            }
        }
    }
}
