/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.statushandlers.DefaultDetailsArea;

import dwt.dwthelper.utils;

import java.util.Date;

import com.ibm.icu.text.DateFormat;

import dwt.DWT;
import dwt.dnd.Clipboard;
import dwt.dnd.DND;
import dwt.dnd.DragSource;
import dwt.dnd.DragSourceEvent;
import dwt.dnd.DragSourceListener;
import dwt.dnd.TextTransfer;
import dwt.dnd.Transfer;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.layout.GridData;
import dwt.layout.GridLayout;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.List;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IStatus;
import dwtx.jface.dialogs.Dialog;
import dwtx.jface.resource.JFaceResources;
import dwtx.osgi.util.NLS;
import dwtx.ui.internal.progress.ProgressMessages;
import dwtx.ui.statushandlers.AbstractStatusAreaProvider;
import dwtx.ui.statushandlers.IStatusAdapterConstants;
import dwtx.ui.statushandlers.StatusAdapter;
import dwtx.ui.statushandlers.WorkbenchStatusDialogManager;

/**
 * The default details area displaying a tree of statuses.
 * 
 * @since 3.4
 */
public class DefaultDetailsArea extends AbstractStatusAreaProvider {

    private WorkbenchStatusDialogManager workbenchStatusDialog;

    public DefaultDetailsArea(WorkbenchStatusDialogManager wsd){
        this.workbenchStatusDialog = wsd;
    }
    
    /*
     * All statuses should be displayed.
     */
    protected static final int MASK = IStatus.CANCEL | IStatus.ERROR
            | IStatus.INFO | IStatus.WARNING;

    /*
     * New child entry in the list will be shifted by two spaces.
     */
    private static final Object NESTING_INDENT = "  "; //$NON-NLS-1$

    /*
     * Displays statuses.
     */
    private List list;

    private Clipboard clipboard;

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.statushandlers.AbstractStatusAreaProvider#createSupportArea(dwt.widgets.Composite,
     *      dwtx.ui.statushandlers.StatusAdapter)
     */
    public Control createSupportArea(Composite parent,
            StatusAdapter statusAdapter) {
        Composite area = createArea(parent);
        setStatusAdapter(statusAdapter);
        return area;
    }

    protected Composite createArea(Composite parent) {
        parent = new Composite(parent, DWT.NONE);
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(DWT.FILL, DWT.FILL, true, true));
        list = new List(parent, DWT.H_SCROLL | DWT.V_SCROLL | DWT.MULTI
                | DWT.BORDER);
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.grabExcessHorizontalSpace = true;
        gd.grabExcessVerticalSpace = true;
        gd.widthHint = 250;
        gd.heightHint = 100;
        list.setLayoutData(gd);
        list.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                if (clipboard !is null) {
                    clipboard.dispose();
                }
            }
        });
        list.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see dwt.events.SelectionAdapter#widgetSelected(dwt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                list.selectAll();
                super.widgetSelected(e);
            }
        });
        createDNDSource();
        createCopyAction(parent);
        Dialog.applyDialogFont(parent);
        return parent;
    }

    protected void setStatusAdapter(StatusAdapter adapter) {
        list.removeAll();
        populateList(list, adapter.getStatus(), 0);
        if (workbenchStatusDialog.getStatusAdapters().size() is 1) {
            Long timestamp = (Long) adapter
                    .getProperty(IStatusAdapterConstants.TIMESTAMP_PROPERTY);

            if (timestamp !is null) {
                String date = DateFormat.getDateTimeInstance(DateFormat.LONG,
                        DateFormat.LONG)
                        .format(new Date(timestamp.longValue()));
                list.add(NLS.bind(ProgressMessages.JobInfo_Error,
                        (new Object[] { "", date }))); //$NON-NLS-1$
            }
        }
    }

    /**
     * Creates DND source for the list
     */
    private void createDNDSource() {
        DragSource ds = new DragSource(list, DND.DROP_COPY);
        ds.setTransfer(new Transfer[] { TextTransfer.getInstance() });
        ds.addDragListener(new DragSourceListener() {
            public void dragFinished(DragSourceEvent event) {
            }

            public void dragSetData(DragSourceEvent event) {
                if (TextTransfer.getInstance().isSupportedType(event.dataType)) {
                    event.data = prepareCopyString();
                }
            }
            
            public void dragStart(DragSourceEvent event) {
                list.selectAll();
            }
        });
    }

    private void createCopyAction(final Composite parent) {
        Menu menu = new Menu(parent.getShell(), DWT.POP_UP);
        MenuItem copyAction = new MenuItem(menu, DWT.PUSH);
        copyAction.setText(JFaceResources.getString("copy")); //$NON-NLS-1$
        copyAction.addSelectionListener(new SelectionAdapter() {

            /*
             * (non-Javadoc)
             * 
             * @see dwt.events.SelectionAdapter#widgetSelected(dwt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                clipboard = new Clipboard(parent.getDisplay());
                clipboard.setContents(new Object[] { prepareCopyString() },
                        new Transfer[] { TextTransfer.getInstance() });
                super.widgetSelected(e);
            }

        });
        list.setMenu(menu);
    }

    private String prepareCopyString() {
        if (list is null || list.isDisposed()) {
            return ""; //$NON-NLS-1$
        }
        StringBuffer sb = new StringBuffer();
        String newLine = System.getProperty("line.separator"); //$NON-NLS-1$
        for (int i = 0; i < list.getItemCount(); i++) {
            sb.append(list.getItem(i));
            sb.append(newLine);
        }
        return sb.toString();
    }

    private void populateList(List list, IStatus status, int nesting) {
        if (!status.matches(MASK)) {
            return;
        }
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < nesting; i++) {
            buffer.append(NESTING_INDENT);
        }
        buffer.append(status.getMessage());
        list.add(buffer.toString());

        // Look for a nested core exception
        Throwable t = status.getException();
        if (t instanceof CoreException) {
            CoreException ce = (CoreException) t;
            populateList(list, ce.getStatus(), nesting + 1);
        } else if (t !is null) {
            // Include low-level exception message
            buffer = new StringBuffer();
            for (int i = 0; i < nesting; i++) {
                buffer.append(NESTING_INDENT);
            }
            String message = t.getLocalizedMessage();
            if (message is null) {
                message = t.toString();
            }
            buffer.append(message);
            list.add(buffer.toString());
        }

        IStatus[] children = status.getChildren();
        for (int i = 0; i < children.length; i++) {
            populateList(list, children[i], nesting + 1);
        }
    }

    /**
     * @return Returns the list.
     */
    public List getList() {
        return list;
    }
}
