/*******************************************************************************
 * Copyright (c) 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 ******************************************************************************/

module dwtx.ui.internal.statushandlers.StackTraceSupportArea;

import dwt.dwthelper.utils;


import dwt.DWT;
import dwt.dnd.Clipboard;
import dwt.dnd.DND;
import dwt.dnd.DragSource;
import dwt.dnd.DragSourceEvent;
import dwt.dnd.DragSourceListener;
import dwt.dnd.TextTransfer;
import dwt.dnd.Transfer;
import dwt.events.DisposeEvent;
import dwt.events.DisposeListener;
import dwt.events.SelectionAdapter;
import dwt.events.SelectionEvent;
import dwt.layout.GridData;
import dwt.widgets.Composite;
import dwt.widgets.Control;
import dwt.widgets.Label;
import dwt.widgets.List;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwtx.jface.util.Policy;
import dwtx.ui.internal.WorkbenchMessages;
import dwtx.ui.statushandlers.AbstractStatusAreaProvider;
import dwtx.ui.statushandlers.StatusAdapter;
import dwtx.ui.statushandlers.WorkbenchErrorHandler;
import dwtx.ui.statushandlers.WorkbenchStatusDialogManager;

/**
 * This class is responsible for displaying stack trace retrieved from IStatus.
 * It has similar functionality as details area in {@link WorkbenchStatusDialogManager}.
 * This class will be visible only if it is enabled in
 * {@link WorkbenchStatusDialogManager} and no support provider is passed by
 * {@link Policy}
 * 
 * @see Policy#setErrorSupportProvider
 * @see Policy#getErrorSupportProvider()
 * @see WorkbenchStatusDialogManager#enableDefaultSupportArea
 * @see WorkbenchErrorHandler
 * @since 3.4
 */
public class StackTraceSupportArea extends AbstractStatusAreaProvider {

    /*
     * Displays statuses.
     */
    private List list;

    private Clipboard clipboard;

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.statushandlers.AbstractStatusAreaProvider#createSupportArea(dwt.widgets.Composite,
     *      dwtx.ui.statushandlers.StatusAdapter)
     */
    public Control createSupportArea(final Composite parent,
            StatusAdapter statusAdapter) {

        Label label = new Label(parent, DWT.NONE);
        label.setText(WorkbenchMessages.StackTraceSupportArea_Title);

        list = new List(parent, DWT.H_SCROLL | DWT.V_SCROLL | DWT.MULTI);
        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.grabExcessHorizontalSpace = true;
        gd.grabExcessVerticalSpace = true;
        gd.widthHint = 250;
        list.setLayoutData(gd);
        list.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(DisposeEvent e) {
                if (clipboard !is null) {
                    clipboard.dispose();
                }
            }
        });
        list.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see dwt.events.SelectionAdapter#widgetSelected(dwt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                list.selectAll();
                super.widgetSelected(e);
            }
        });
        list.removeAll();
        populateList(statusAdapter.getStatus().getException());
        createDNDSource();
        createCopyAction(parent);
        return parent;
    }

    /**
     * Creates DND source for the list
     */
    private void createDNDSource() {
        DragSource ds = new DragSource(list, DND.DROP_COPY);
        ds.setTransfer(new Transfer[] { TextTransfer.getInstance() });
        ds.addDragListener(new DragSourceListener() {
            public void dragFinished(DragSourceEvent event) {

            }

            public void dragSetData(DragSourceEvent event) {
                if (TextTransfer.getInstance().isSupportedType(event.dataType)) {
                    event.data = prepareCopyString();
                }
            }

            public void dragStart(DragSourceEvent event) {
                list.selectAll();
            }
        });
    }

    private void createCopyAction(final Composite parent) {
        Menu menu = new Menu(parent.getShell(), DWT.POP_UP);
        MenuItem copyAction = new MenuItem(menu, DWT.PUSH);
        copyAction.setText("&Copy"); //$NON-NLS-1$
        copyAction.addSelectionListener(new SelectionAdapter() {
            /*
             * (non-Javadoc)
             * 
             * @see dwt.events.SelectionAdapter#widgetSelected(dwt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                clipboard = new Clipboard(parent.getDisplay());
                clipboard.setContents(new Object[] { prepareCopyString() },
                        new Transfer[] { TextTransfer.getInstance() });
                super.widgetSelected(e);
            }
        });
        list.setMenu(menu);
    }

    private String prepareCopyString() {
        if (list is null || list.isDisposed()) {
            return ""; //$NON-NLS-1$
        }
        StringBuffer sb = new StringBuffer();
        String newLine = System.getProperty("line.separator"); //$NON-NLS-1$
        for (int i = 0; i < list.getItemCount(); i++) {
            sb.append(list.getItem(i));
            sb.append(newLine);
        }
        return sb.toString();
    }

    private void populateList(Throwable t) {
        if (t is null) {
            list.add(WorkbenchMessages.StackTraceSupportArea_NoStackTrace);
            return;
        }
        list.add(t.toString());
        StackTraceElement[] ste = t.getStackTrace();
        for (int i = 0; i < ste.length; i++) {
            list.add(ste[i].toString());
        }
        if (t.getCause() !is null) {
            list.add(WorkbenchMessages.StackTraceSupportArea_CausedBy);
            populateList(t.getCause());
        }
    }

    /**
     * @return Returns the list.
     */
    public List getList() {
        return list;
    }
}
