/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.menus.CommandContributionItem;

import dwt.dwthelper.utils;

import java.util.Map;

import dwt.DWT;
import dwt.graphics.Point;
import dwt.widgets.Display;
import dwt.widgets.Event;
import dwt.widgets.Listener;
import dwt.widgets.Menu;
import dwt.widgets.MenuItem;
import dwt.widgets.ToolBar;
import dwt.widgets.ToolItem;
import dwt.widgets.Widget;
import dwtx.core.commands.Command;
import dwtx.core.commands.CommandEvent;
import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.ICommandListener;
import dwtx.core.commands.NotEnabledException;
import dwtx.core.commands.NotHandledException;
import dwtx.core.commands.ParameterizedCommand;
import dwtx.core.commands.common.NotDefinedException;
import dwtx.jface.action.ContributionItem;
import dwtx.jface.action.IContributionManager;
import dwtx.jface.action.IMenuListener;
import dwtx.jface.action.IMenuManager;
import dwtx.jface.action.MenuManager;
import dwtx.jface.bindings.TriggerSequence;
import dwtx.jface.resource.ImageDescriptor;
import dwtx.jface.resource.JFaceResources;
import dwtx.jface.resource.LocalResourceManager;
import dwtx.ui.IWorkbench;
import dwtx.ui.IWorkbenchPartSite;
import dwtx.ui.IWorkbenchWindow;
import dwtx.ui.commands.ICommandImageService;
import dwtx.ui.commands.ICommandService;
import dwtx.ui.commands.IElementReference;
import dwtx.ui.handlers.IHandlerService;
import dwtx.ui.help.IWorkbenchHelpSystem;
import dwtx.ui.internal.WorkbenchPlugin;
import dwtx.ui.internal.services.IWorkbenchLocationService;
import dwtx.ui.keys.IBindingService;
import dwtx.ui.services.IServiceLocator;

/**
 * A contribution item which delegates to a command. It can be used in {@link
 * AbstractContributionFactory#createContributionItems(IServiceLocator,
 * IContributionRoot)}.
 * <p>
 * It currently supports placement in menus and toolbars.
 * </p>
 * <p>
 * This class may be instantiated; it is not intended to be subclassed.
 * </p>
 * 
 * @since 3.3
 */
public final class CommandContributionItem extends ContributionItem {
    /**
     * A push button tool item or menu item.
     */
    public static final int STYLE_PUSH = DWT.PUSH;

    /**
     * A checked tool item or menu item.
     */
    public static final int STYLE_CHECK = DWT.CHECK;

    /**
     * A radio-button style menu item.
     */
    public static final int STYLE_RADIO = DWT.RADIO;

    /**
     * A ToolBar pulldown item.
     */
    public static final int STYLE_PULLDOWN = DWT.DROP_DOWN;

    /**
     * Mode bit: Show text on tool items or buttons, even if an image is
     * present. If this mode bit is not set, text is only shown on tool items if
     * there is no image present.
     * 
     * @since 3.4
     */
    public static int MODE_FORCE_TEXT = 1;

    private LocalResourceManager localResourceManager;

    private Listener menuItemListener;

    private Widget widget;

    private IMenuService menuService;

    private ICommandService commandService;

    private IHandlerService handlerService;

    private IBindingService bindingService;

    private ParameterizedCommand command;

    private ImageDescriptor icon;

    private String label;

    private String tooltip;

    private ImageDescriptor disabledIcon;

    private ImageDescriptor hoverIcon;

    private String mnemonic;

    private IElementReference elementRef;

    private bool checkedState;

    private int style;

    private ICommandListener commandListener;

    private String dropDownMenuOverride;

    private IWorkbenchHelpSystem workbenchHelpSystem;

    private String helpContextId;

    private int mode = 0;

    /**
     * This is <code>true</code> when the menu contribution's visibleWhen
     * checkEnabled attribute is <code>true</code>.
     */
    private bool visibleEnabled;

    /**
     * Create a CommandContributionItem to place in a ContributionManager.
     * 
     * @param contributionParameters
     *      parameters necessary to render this contribution item.
     * @since 3.4
     */
    public CommandContributionItem(
            CommandContributionItemParameter contributionParameters) {
        super(contributionParameters.id);

        this.icon = contributionParameters.icon;
        this.disabledIcon = contributionParameters.disabledIcon;
        this.hoverIcon = contributionParameters.hoverIcon;
        this.label = contributionParameters.label;
        this.mnemonic = contributionParameters.mnemonic;
        this.tooltip = contributionParameters.tooltip;
        this.style = contributionParameters.style;
        this.helpContextId = contributionParameters.helpContextId;
        this.visibleEnabled = contributionParameters.visibleEnabled;
        this.mode = contributionParameters.mode;

        menuService = (IMenuService) contributionParameters.serviceLocator
                .getService(IMenuService.class);
        commandService = (ICommandService) contributionParameters.serviceLocator
                .getService(ICommandService.class);
        handlerService = (IHandlerService) contributionParameters.serviceLocator
                .getService(IHandlerService.class);
        bindingService = (IBindingService) contributionParameters.serviceLocator
                .getService(IBindingService.class);
        createCommand(contributionParameters.commandId,
                contributionParameters.parameters);

        if (command !is null) {
            try {
                UIElement callback = new UIElement(
                        contributionParameters.serviceLocator) {

                    public void setChecked(bool checked) {
                        CommandContributionItem.this.setChecked(checked);
                    }

                    public void setDisabledIcon(ImageDescriptor desc) {
                        CommandContributionItem.this.setDisabledIcon(desc);
                    }

                    public void setHoverIcon(ImageDescriptor desc) {
                        CommandContributionItem.this.setHoverIcon(desc);
                    }

                    public void setIcon(ImageDescriptor desc) {
                        CommandContributionItem.this.setIcon(desc);
                    }

                    public void setText(String text) {
                        CommandContributionItem.this.setText(text);
                    }

                    public void setTooltip(String text) {
                        CommandContributionItem.this.setTooltip(text);
                    }

                    public void setDropDownId(String id) {
                        dropDownMenuOverride = id;
                    }
                };
                elementRef = commandService.registerElementForCommand(command,
                        callback);
                command.getCommand().addCommandListener(getCommandListener());
                setImages(contributionParameters.serviceLocator,
                        contributionParameters.iconStyle);

                if (contributionParameters.helpContextId is null) {
                    try {
                        this.helpContextId = commandService
                                .getHelpContextId(contributionParameters.commandId);
                    } catch (NotDefinedException e) {
                        // it's OK to not have a helpContextId
                    }
                }
                IWorkbenchLocationService wls = (IWorkbenchLocationService) contributionParameters.serviceLocator
                        .getService(IWorkbenchLocationService.class);
                final IWorkbench workbench = wls.getWorkbench();
                if (workbench !is null && helpContextId !is null) {
                    this.workbenchHelpSystem = workbench.getHelpSystem();
                }
            } catch (NotDefinedException e) {
                WorkbenchPlugin
                        .log("Unable to register menu item \"" + getId() //$NON-NLS-1$
                                + "\", command \"" + contributionParameters.commandId + "\" not defined"); //$NON-NLS-1$ //$NON-NLS-2$
            }
        }

    }

    /**
     * Create a CommandContributionItem to place in a ContributionManager.
     * 
     * @param serviceLocator
     *      a service locator that is most appropriate for this contribution.
     *      Typically the local {@link IWorkbenchWindow} or {@link
     *      IWorkbenchPartSite} will be sufficient.
     * @param id
     *      The id for this item. May be <code>null</code>. Items without an id
     *      cannot be referenced later.
     * @param commandId
     *      A command id for a defined command. Must not be <code>null</code>.
     * @param parameters
     *      A map of strings to strings which represent parameter names to
     *      values. The parameter names must match those in the command
     *      definition.
     * @param icon
     *      An icon for this item. May be <code>null</code>.
     * @param disabledIcon
     *      A disabled icon for this item. May be <code>null</code>.
     * @param hoverIcon
     *      A hover icon for this item. May be <code>null</code>.
     * @param label
     *      A label for this item. May be <code>null</code>.
     * @param mnemonic
     *      A mnemonic for this item to be applied to the label. May be
     *      <code>null</code>.
     * @param tooltip
     *      A tooltip for this item. May be <code>null</code>. Tooltips are
     *      currently only valid for toolbar contributions.
     * @param style
     *      The style of this menu contribution. See the STYLE_* contants.
     * @deprecated create the {@link CommandContributionItemParameter}
     */
    public CommandContributionItem(IServiceLocator serviceLocator, String id,
            String commandId, Map parameters, ImageDescriptor icon,
            ImageDescriptor disabledIcon, ImageDescriptor hoverIcon,
            String label, String mnemonic, String tooltip, int style) {
        this(new CommandContributionItemParameter(serviceLocator, id,
                commandId, parameters, icon, disabledIcon, hoverIcon, label,
                mnemonic, tooltip, style, null, false));
    }

    private void setImages(IServiceLocator locator, String iconStyle) {
        if (icon is null) {
            ICommandImageService service = (ICommandImageService) locator
                    .getService(ICommandImageService.class);
            icon = service.getImageDescriptor(command.getId(),
                    ICommandImageService.TYPE_DEFAULT, iconStyle);
            disabledIcon = service.getImageDescriptor(command.getId(),
                    ICommandImageService.TYPE_DISABLED, iconStyle);
            hoverIcon = service.getImageDescriptor(command.getId(),
                    ICommandImageService.TYPE_HOVER, iconStyle);
        }
    }

    private ICommandListener getCommandListener() {
        if (commandListener is null) {
            commandListener = new ICommandListener() {
                public void commandChanged(CommandEvent commandEvent) {
                    if (commandEvent.isHandledChanged()
                            || commandEvent.isEnabledChanged()
                            || commandEvent.isDefinedChanged()) {
                        updateCommandProperties(commandEvent);
                    }
                }
            };
        }
        return commandListener;
    }

    private void updateCommandProperties(final CommandEvent commandEvent) {
        if (commandEvent.isHandledChanged()) {
            dropDownMenuOverride = null;
        }
        if (widget is null || widget.isDisposed()) {
            return;
        }
        Display display = widget.getDisplay();
        Runnable update = new Runnable() {
            public void run() {
                if (commandEvent.getCommand().isDefined()) {
                    update(null);
                }
                if (commandEvent.isEnabledChanged()
                        || commandEvent.isHandledChanged()) {
                    if (visibleEnabled) {
                        IContributionManager parent = getParent();
                        if (parent !is null) {
                            parent.update(true);
                        }
                    }
                }
            }
        };
        if (display.getThread() is Thread.currentThread()) {
            update.run();
        } else {
            display.asyncExec(update);
        }
    }

    ParameterizedCommand getCommand() {
        return command;
    }

    void createCommand(String commandId, Map parameters) {
        if (commandId is null) {
            WorkbenchPlugin.log("Unable to create menu item \"" + getId() //$NON-NLS-1$
                    + "\", no command id"); //$NON-NLS-1$
            return;
        }
        Command cmd = commandService.getCommand(commandId);
        if (!cmd.isDefined()) {
            WorkbenchPlugin.log("Unable to create menu item \"" + getId() //$NON-NLS-1$
                    + "\", command \"" + commandId + "\" not defined"); //$NON-NLS-1$ //$NON-NLS-2$
            return;
        }
        command = ParameterizedCommand.generateCommand(cmd, parameters);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.jface.action.ContributionItem#fill(dwt.widgets
     * .Menu, int)
     */
    public void fill(Menu parent, int index) {
        if (command is null) {
            return;
        }
        if (widget !is null || parent is null) {
            return;
        }

        // Menus don't support the pulldown style
        int tmpStyle = style;
        if (tmpStyle is STYLE_PULLDOWN)
            tmpStyle = STYLE_PUSH;

        MenuItem item = null;
        if (index >= 0) {
            item = new MenuItem(parent, tmpStyle, index);
        } else {
            item = new MenuItem(parent, tmpStyle);
        }
        item.setData(this);
        if (workbenchHelpSystem !is null) {
            workbenchHelpSystem.setHelp(item, helpContextId);
        }
        item.addListener(DWT.Dispose, getItemListener());
        item.addListener(DWT.Selection, getItemListener());
        widget = item;

        update(null);
        updateIcons();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * dwtx.jface.action.ContributionItem#fill(dwt.widgets
     * .ToolBar, int)
     */
    public void fill(ToolBar parent, int index) {
        if (command is null) {
            return;
        }
        if (widget !is null || parent is null) {
            return;
        }

        ToolItem item = null;
        if (index >= 0) {
            item = new ToolItem(parent, style, index);
        } else {
            item = new ToolItem(parent, style);
        }

        item.setData(this);

        item.addListener(DWT.Selection, getItemListener());
        item.addListener(DWT.Dispose, getItemListener());
        widget = item;

        update(null);
        updateIcons();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.ContributionItem#update()
     */
    public void update() {
        update(null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.ContributionItem#update(java.lang.String)
     */
    public void update(String id) {
        if (widget !is null) {
            if (widget instanceof MenuItem) {
                MenuItem item = (MenuItem) widget;

                String text = label;
                if (text is null) {
                    if (command !is null) {
                        try {
                            text = command.getCommand().getName();
                        } catch (NotDefinedException e) {
                            WorkbenchPlugin.log("Update item failed " //$NON-NLS-1$
                                    + getId(), e);
                        }
                    }
                }
                text = updateMnemonic(text);

                String keyBindingText = null;
                if (command !is null) {
                    TriggerSequence binding = bindingService
                            .getBestActiveBindingFor(command);
                    if (binding !is null) {
                        keyBindingText = binding.format();
                    }
                }
                if (text !is null) {
                    if (keyBindingText is null) {
                        item.setText(text);
                    } else {
                        item.setText(text + '\t' + keyBindingText);
                    }
                }

                if (item.getSelection() !is checkedState) {
                    item.setSelection(checkedState);
                }

                bool shouldBeEnabled = isEnabled();
                if (item.getEnabled() !is shouldBeEnabled) {
                    item.setEnabled(shouldBeEnabled);
                }
            } else if (widget instanceof ToolItem) {
                ToolItem item = (ToolItem) widget;

                String text = label;
                if (text is null) {
                    if (command !is null) {
                        try {
                            text = command.getCommand().getName();
                        } catch (NotDefinedException e) {
                            WorkbenchPlugin.log("Update item failed " //$NON-NLS-1$
                                    + getId(), e);
                        }
                    }
                }

                if ((icon is null || (mode & MODE_FORCE_TEXT) is MODE_FORCE_TEXT)
                        && text !is null) {
                    item.setText(text);
                }

                if (tooltip !is null)
                    item.setToolTipText(tooltip);
                else {
                    if (text !is null) {
                        item.setToolTipText(text);
                    }
                }

                if (item.getSelection() !is checkedState) {
                    item.setSelection(checkedState);
                }

                bool shouldBeEnabled = isEnabled();
                if (item.getEnabled() !is shouldBeEnabled) {
                    item.setEnabled(shouldBeEnabled);
                }
            }
        }
    }

    private String updateMnemonic(String s) {
        if (mnemonic is null || s is null) {
            return s;
        }
        int idx = s.indexOf(mnemonic);
        if (idx is -1) {
            return s;
        }

        return s.substring(0, idx) + '&' + s.substring(idx);
    }

    private void handleWidgetDispose(Event event) {
        if (event.widget is widget) {
            widget.removeListener(DWT.Selection, getItemListener());
            widget.removeListener(DWT.Dispose, getItemListener());
            widget = null;
            disposeOldImages();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.ContributionItem#dispose()
     */
    public void dispose() {
        if (widget !is null) {
            widget.dispose();
            widget = null;
        }
        if (elementRef !is null) {
            commandService.unregisterElement(elementRef);
            elementRef = null;
        }
        if (commandListener !is null) {
            command.getCommand().removeCommandListener(commandListener);
            commandListener = null;
        }
        command = null;
        commandService = null;
        bindingService = null;
        menuService = null;
        handlerService = null;
        disposeOldImages();
        super.dispose();
    }

    private void disposeOldImages() {
        if (localResourceManager !is null) {
            localResourceManager.dispose();
            localResourceManager = null;
        }
    }

    private Listener getItemListener() {
        if (menuItemListener is null) {
            menuItemListener = new Listener() {
                public void handleEvent(Event event) {
                    switch (event.type) {
                    case DWT.Dispose:
                        handleWidgetDispose(event);
                        break;
                    case DWT.Selection:
                        if (event.widget !is null) {
                            handleWidgetSelection(event);
                        }
                        break;
                    }
                }
            };
        }
        return menuItemListener;
    }

    private void handleWidgetSelection(Event event) {
        // Special check for ToolBar dropdowns...
        if (openDropDownMenu(event))
            return;

        if ((style & (DWT.TOGGLE | DWT.CHECK)) !is 0) {
            if (event.widget instanceof ToolItem) {
                checkedState = ((ToolItem) event.widget).getSelection();
            } else if (event.widget instanceof MenuItem) {
                checkedState = ((MenuItem) event.widget).getSelection();
            }
        }

        try {
            handlerService.executeCommand(command, event);
        } catch (ExecutionException e) {
            WorkbenchPlugin.log("Failed to execute item " //$NON-NLS-1$
                    + getId(), e);
        } catch (NotDefinedException e) {
            WorkbenchPlugin.log("Failed to execute item " //$NON-NLS-1$
                    + getId(), e);
        } catch (NotEnabledException e) {
            WorkbenchPlugin.log("Failed to execute item " //$NON-NLS-1$
                    + getId(), e);
        } catch (NotHandledException e) {
            WorkbenchPlugin.log("Failed to execute item " //$NON-NLS-1$
                    + getId(), e);
        }
    }

    /**
     * Determines if the selection was on the dropdown affordance and, if so,
     * opens the drop down menu (populated using the same id as this item...
     * 
     * @param event
     *      The <code>DWT.Selection</code> event to be tested
     * 
     * @return <code>true</code> iff a drop down menu was opened
     */
    private bool openDropDownMenu(Event event) {
        Widget item = event.widget;
        if (item !is null) {
            int style = item.getStyle();
            if ((style & DWT.DROP_DOWN) !is 0) {
                if (event.detail is 4) { // on drop-down button
                    ToolItem ti = (ToolItem) item;

                    final MenuManager menuManager = new MenuManager();
                    Menu menu = menuManager.createContextMenu(ti.getParent());
                    if (workbenchHelpSystem !is null) {
                        workbenchHelpSystem.setHelp(menu, helpContextId);
                    }
                    menuManager.addMenuListener(new IMenuListener() {
                        public void menuAboutToShow(IMenuManager manager) {
                            String id = getId();
                            if (dropDownMenuOverride !is null) {
                                id = dropDownMenuOverride;
                            }
                            menuService.populateContributionManager(
                                    menuManager, "menu:" + id); //$NON-NLS-1$
                        }
                    });

                    // position the menu below the drop down item
                    Point point = ti.getParent().toDisplay(
                            new Point(event.x, event.y));
                    menu.setLocation(point.x, point.y); // waiting for DWT
                    // 0.42
                    menu.setVisible(true);
                    return true; // we don't fire the action
                }
            }
        }

        return false;
    }

    private void setIcon(ImageDescriptor desc) {
        icon = desc;
        updateIcons();
    }

    private void updateIcons() {
        if (widget instanceof MenuItem) {
            MenuItem item = (MenuItem) widget;
            LocalResourceManager m = new LocalResourceManager(JFaceResources
                    .getResources());
            item.setImage(icon is null ? null : m.createImage(icon));
            disposeOldImages();
            localResourceManager = m;
        } else if (widget instanceof ToolItem) {
            ToolItem item = (ToolItem) widget;
            LocalResourceManager m = new LocalResourceManager(JFaceResources
                    .getResources());
            item.setDisabledImage(disabledIcon is null ? null : m
                    .createImage(disabledIcon));
            item.setHotImage(hoverIcon is null ? null : m
                    .createImage(hoverIcon));
            item.setImage(icon is null ? null : m.createImage(icon));
            disposeOldImages();
            localResourceManager = m;
        }
    }

    private void setText(String text) {
        label = text;
        update(null);
    }

    private void setChecked(bool checked) {
        if (checkedState is checked) {
            return;
        }
        checkedState = checked;
        if (widget instanceof MenuItem) {
            ((MenuItem) widget).setSelection(checkedState);
        } else if (widget instanceof ToolItem) {
            ((ToolItem) widget).setSelection(checkedState);
        }
    }

    private void setTooltip(String text) {
        tooltip = text;
        if (widget instanceof ToolItem) {
            ((ToolItem) widget).setToolTipText(text);
        }
    }

    private void setDisabledIcon(ImageDescriptor desc) {
        disabledIcon = desc;
        updateIcons();
    }

    private void setHoverIcon(ImageDescriptor desc) {
        hoverIcon = desc;
        updateIcons();
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.jface.action.ContributionItem#isEnabled()
     */
    public bool isEnabled() {
        if (command !is null) {
            command.getCommand().setEnabled(menuService.getCurrentState());
            return command.getCommand().isEnabled();
        }
        return false;
    }

    /**
     * @since 3.4
     */
    public bool isVisible() {
        if (visibleEnabled) {
            return super.isVisible() && isEnabled();
        }
        return super.isVisible();
    }
}
