/*******************************************************************************
 * Copyright (c) 2005, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.operations.LinearUndoViolationUserApprover;

import dwt.dwthelper.utils;

import dwtx.core.commands.ExecutionException;
import dwtx.core.commands.operations.IOperationHistory;
import dwtx.core.commands.operations.IUndoContext;
import dwtx.core.commands.operations.IUndoableOperation;
import dwtx.core.commands.operations.LinearUndoViolationDetector;
import dwtx.core.runtime.IAdaptable;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.jface.dialogs.MessageDialog;
import dwtx.osgi.util.NLS;
import dwtx.ui.IWorkbenchPart;
import dwtx.ui.IWorkbenchPart2;
import dwtx.ui.internal.Workbench;
import dwtx.ui.internal.WorkbenchMessages;

/**
 * <p>
 * An operation approver that prompts the user to see if linear undo violations
 * are permitted. A linear undo violation is detected when an operation being
 * undone or redone shares an undo context with another operation appearing more
 * recently in the history.
 * </p>
 * <p>
 * This class may be instantiated by clients.
 * </p>
 * 
 * @since 3.1
 */
public final class LinearUndoViolationUserApprover extends
        LinearUndoViolationDetector {

    private IWorkbenchPart part;

    private IUndoContext context;

    /**
     * Create a LinearUndoViolationUserApprover associated with the specified
     * workbench part.
     * 
     * @param context
     *            the undo context with the linear undo violation
     * @param part
     *            the part that should be used for prompting the user
     */
    public LinearUndoViolationUserApprover(IUndoContext context,
            IWorkbenchPart part) {
        super();
        this.part = part;
        this.context = context;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.operations.LinearUndoViolationDetector#allowLinearRedoViolation(dwtx.core.commands.operations.IUndoableOperation,
     *      dwtx.core.commands.operations.IUndoContext,
     *      dwtx.core.commands.operations.IOperationHistory,
     *      dwtx.core.runtime.IAdaptable)
     */
    protected IStatus allowLinearRedoViolation(IUndoableOperation operation,
            IUndoContext context, IOperationHistory history, IAdaptable uiInfo) {

        if (this.context !is context) {
            return Status.OK_STATUS;
        }

        final String message = NLS.bind(
                WorkbenchMessages.Operations_linearRedoViolation,
                getTitle(part), operation.getLabel());
        final bool [] proceed = new bool[1];
        Workbench.getInstance().getDisplay().syncExec(new Runnable() {
            public void run() {
                // Show a dialog.
                part.setFocus();
                proceed[0] = MessageDialog.openQuestion(part.getSite()
                        .getShell(), getTitle(part), message);
            }
        });

        if (proceed[0]) {
            // redo the local changes first
            while (operation !is history.getRedoOperation(context)) {
                try {
                    IStatus status = history.redo(context, null, uiInfo);
                    if (!status.isOK()) {
                        // flush the redo history because the operation
                        // failed
                        history.dispose(context, false, true, false);
                        return Status.CANCEL_STATUS;
                    }
                } catch (ExecutionException e) {
                    // flush the redo history here because it failed.
                    history.dispose(context, false, true, false);
                    return Status.CANCEL_STATUS;
                }
            }
            return Status.OK_STATUS;
        }

        return Status.CANCEL_STATUS;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.core.commands.operations.LinearUndoViolationDetector#allowLinearUndoViolation(dwtx.core.commands.operations.IUndoableOperation,
     *      dwtx.core.commands.operations.IUndoContext,
     *      dwtx.core.commands.operations.IOperationHistory,
     *      dwtx.core.runtime.IAdaptable)
     */
    protected IStatus allowLinearUndoViolation(IUndoableOperation operation,
            IUndoContext context, IOperationHistory history, IAdaptable uiInfo) {

        if (this.context !is context) {
            return Status.OK_STATUS;
        }

        final String message = NLS.bind(
                WorkbenchMessages.Operations_linearUndoViolation,
                getTitle(part), operation.getLabel());
        final bool [] proceed = new bool[1];
        Workbench.getInstance().getDisplay().syncExec(new Runnable() {
            public void run() {
                // Show a dialog.
                part.setFocus();
                proceed[0] = MessageDialog.openQuestion(part.getSite()
                        .getShell(), getTitle(part), message);
            }
        });

        if (proceed[0]) {
            // redo the local changes first
            while (operation !is history.getUndoOperation(context)) {
                try {
                    IStatus status = history.undo(context, null, uiInfo);
                    if (!status.isOK()) {
                        // flush the operation history because the operation
                        // failed.
                        history.dispose(context, true, false, false);
                        return Status.CANCEL_STATUS;
                    }
                } catch (ExecutionException e) {
                    // flush the undo history here because something went wrong.
                    history.dispose(context, true, false, false);
                    return Status.CANCEL_STATUS;
                }
            }
            return Status.OK_STATUS;
        }
        return Status.CANCEL_STATUS;
    }

    /*
     * Get the title for the specified part. Use the newer interface
     * IWorkbenchPart2 if available.
     */
    private String getTitle(IWorkbenchPart part) {
        String title;
        if (part instanceof IWorkbenchPart2) {
            title = ((IWorkbenchPart2) part).getPartName();
        } else {
            title = part.getTitle();
        }
        // Null title is unexpected, but use an empty string if encountered.
        if (title is null) {
            title = ""; //$NON-NLS-1$
        }
        return title;
    }
}
