/*******************************************************************************
 * Copyright (c) 2006, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/

module dwtx.ui.statushandlers.WorkbenchErrorHandler;

import dwt.dwthelper.utils;


import dwt.widgets.Display;
import dwtx.core.runtime.IStatus;
import dwtx.core.runtime.Status;
import dwtx.ui.PlatformUI;
import dwtx.ui.application.WorkbenchAdvisor;
import dwtx.ui.internal.WorkbenchPlugin;

/**
 * This is a default workbench error handler.
 * 
 * @see WorkbenchAdvisor#getWorkbenchErrorHandler()
 * @since 3.3
 */
public class WorkbenchErrorHandler extends AbstractStatusHandler {

    private WorkbenchStatusDialogManager statusDialog;

    /**
     * For testing purposes only. This method must not be used by any other
     * clients.
     * 
     * @param dialog
     *            a new WorkbenchStatusDialog to be set.
     */
    void setStatusDialog(WorkbenchStatusDialogManager dialog) {
        statusDialog = dialog;
    }

    /*
     * (non-Javadoc)
     * 
     * @see dwtx.ui.statushandlers.AbstractStatusHandler#handle(dwtx.ui.statushandlers.StatusAdapter,
     *      int)
     */
    public void handle(final StatusAdapter statusAdapter, int style) {
        if (((style & StatusManager.SHOW) is StatusManager.SHOW)
                || ((style & StatusManager.BLOCK) is StatusManager.BLOCK)) {

            // INFO status is set in the adapter when the passed adapter has OK
            // or CANCEL status
            if (statusAdapter.getStatus().getSeverity() is IStatus.OK
                    || statusAdapter.getStatus().getSeverity() is IStatus.CANCEL) {
                IStatus status = statusAdapter.getStatus();
                statusAdapter.setStatus(new Status(IStatus.INFO, status
                        .getPlugin(), status.getMessage(), status
                        .getException()));
            }

            final bool modal = ((style & StatusManager.BLOCK) is StatusManager.BLOCK);
            if (Display.getCurrent() !is null) {
                getStatusDialogManager().addStatusAdapter(statusAdapter, modal);
            } else {
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        if (!PlatformUI.isWorkbenchRunning()) {
                            // we are shutting down, so just log
                            WorkbenchPlugin.log(statusAdapter.getStatus());
                            return;
                        }
                        getStatusDialogManager().addStatusAdapter(
                                statusAdapter, modal);
                    }
                });
            }
        }

        if ((style & StatusManager.LOG) is StatusManager.LOG) {
            StatusManager.getManager().addLoggedStatus(
                    statusAdapter.getStatus());
            WorkbenchPlugin.getDefault().getLog()
                    .log(statusAdapter.getStatus());
        }
    }

    /**
     * This method returns current {@link WorkbenchStatusDialogManager}.
     * 
     * @return current {@link WorkbenchStatusDialogManager}
     */
    private WorkbenchStatusDialogManager getStatusDialogManager() {
        if (statusDialog is null) {
            initStatusDialogManager();
        }
        return statusDialog;
    }

    /**
     * This methods should be overridden to configure
     * {@link WorkbenchStatusDialogManager} behavior. It is advised to use only
     * following methods of {@link WorkbenchStatusDialogManager}:
     * <ul>
     * <li>{@link WorkbenchStatusDialogManager#enableDefaultSupportArea(bool)}</li>
     * <li>{@link WorkbenchStatusDialogManager#setDetailsAreaProvider(AbstractStatusAreaProvider)}</li>
     * <li>{@link WorkbenchStatusDialogManager#setSupportAreaProvider(AbstractStatusAreaProvider)}</li>
     * </ul>
     * Default configuration does nothing.
     * 
     * @param statusDialog
     *            a status dialog to be configured.
     * @since 3.4
     */
    protected void configureStatusDialog(
            final WorkbenchStatusDialogManager statusDialog) {
        // default configuration does nothing
    }

    /**
     * This method initializes {@link WorkbenchStatusDialogManager} and is called only
     * once.
     */
    private void initStatusDialogManager() {
        statusDialog = new WorkbenchStatusDialogManager(null);
        configureStatusDialog(statusDialog);
    }
}
