/*******************************************************************************
 * Copyright (c) 2004, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwtx.ui.themes.RGBBlendColorFactory;

import dwt.dwthelper.utils;

import java.util.Hashtable;

import dwt.graphics.RGB;
import dwtx.core.runtime.CoreException;
import dwtx.core.runtime.IConfigurationElement;
import dwtx.core.runtime.IExecutableExtension;

/**
 * A resuable <code>IColorFactory</code> that may be used to blend two colors.  
 * The colors to blend are specified as per method number two in 
 * {@link dwtx.core.runtime.IExecutableExtension}.
 * <p>
 * Example usage:
 * <br/>
 * <code>
 * &lt;colorDefinition
 *     label="Red/Blue Blend"
 *     id="example.redblueblend"&gt;
 *     &lt;colorFactory 
 *              plugin="dwtx.ui" 
 *              class="dwtx.ui.themes.RGBBlendColorFactory"&gt;
 *          &lt;parameter name="color1" value="255,0,0" /&gt;
 *          &lt;parameter name="color2" value="COLOR_BLUE" /&gt;
 *     &lt;/colorFactory&gt;
 * &lt;/colorDefinition&gt;
 * </code>
 * </p>
 * 
 * <p>
 * The color values may be specified as RGB triples or as DWT constants.
 * </p>
 * 
 * @see dwt.DWT
 * @since 3.0
 */
public class RGBBlendColorFactory implements IColorFactory,
        IExecutableExtension {

    private String color1, color2;

    /* (non-Javadoc)
     * @see dwtx.ui.themes.IColorFactory#createColor()
     */
    public RGB createColor() {
        if (color1 is null && color2 is null) {
            return new RGB(0, 0, 0);
        } else if (color1 !is null && color2 is null) {
            return ColorUtil.getColorValue(color1);
        } else if (color1 is null && color2 !is null) {
            return ColorUtil.getColorValue(color2);
        } else {
            RGB rgb1 = ColorUtil.getColorValue(color1);
            RGB rgb2 = ColorUtil.getColorValue(color2);
            return ColorUtil.blend(rgb1, rgb2);
        }
    }

    /**
     * This executable extension requires parameters to be explicitly declared 
     * via the second method described in the <code>IExecutableExtension</code> 
     * documentation.  This class expects that there will be two parameters, 
     * <code>color1</code> and <code>color2</code>, that describe the two colors
     * to be blended.  These values may either be RGB triples or DWT constants.
     * 
     * @see dwtx.core.runtime.IExecutableExtension#setInitializationData(dwtx.core.runtime.IConfigurationElement, java.lang.String, java.lang.Object)
     */
    public void setInitializationData(IConfigurationElement config,
            String propertyName, Object data) throws CoreException {

        if (data instanceof Hashtable) {
            Hashtable table = (Hashtable) data;
            color1 = (String) table.get("color1"); //$NON-NLS-1$
            color2 = (String) table.get("color2"); //$NON-NLS-1$            
        }
    }
}
